@@ -9,6 +9,7 @@
 #include "../universe/Planet.h"
 #include "../universe/Fleet.h"
 #include "../universe/Ship.h"
+#include "../universe/System.h"
 #include "../universe/Tech.h"
 #include "../Empire/Empire.h"
 
@@ -17,6 +18,7 @@
 #include "../util/OrderSet.h"
 
 #include <boost/timer.hpp>
+#include <boost/lexical_cast.hpp>
 
 #include <stdexcept>
 #include <string>
@@ -73,24 +75,6 @@ namespace {
         Universe& universe = AIClientApp::GetApp()->GetUniverse();
         universe.InitMeterEstimatesAndDiscrepancies();
     }
-
-    // start of turn initialization for Empire ResourcePools.  determines where supplies can be delivered, and 
-    // between which systems resources can be exchanged (which is necessary to know before resource pools can be
-    // updated
-    // ** not used currently because this info is all provided by Server, and having client recalculate it simply 
-    // risks overwriting with inaccurate information
-    void InitResourcePoolsAndSupply() {
-        EmpireManager& manager = AIClientApp::GetApp()->Empires();
-
-        // determine systems where fleets can delivery supply, and groups of systems that can exchange resources
-        for (EmpireManager::iterator it = manager.begin(); it != manager.end(); ++it) {
-            Empire* empire = it->second;
-            empire->UpdateSupplyUnobstructedSystems();
-            empire->UpdateSystemSupplyRanges();
-            empire->UpdateSupply();
-            empire->InitResourcePools();
-        }
-    }
 }
 
 namespace AIInterface {
@@ -172,19 +156,21 @@ namespace AIInterface {
     int CurrentTurn()
     { return AIClientApp::GetApp()->CurrentTurn(); }
 
+    const GalaxySetupData&  GetGalaxySetupData()
+    { return AIClientApp::GetApp()->GetGalaxySetupData(); }
+
     void InitTurn() {
         boost::timer turn_init_timer;
 
         InitMeterEstimatesAndDiscrepancies();
         UpdateMeterEstimates();
-        //InitResourcePoolsAndSupply(); //unneeded & was overwriting/erasing info provided by Server
         UpdateResourcePools();
 
         Logger().debugStream() << "AIInterface::InitTurn time: " << (turn_init_timer.elapsed() * 1000.0);
     }
 
     void UpdateMeterEstimates(bool pretend_unowned_planets_owned_by_this_ai_empire) {
-        std::vector<Planet*> unowned_planets;
+        std::vector<TemporaryPtr<Planet> > unowned_planets;
         int player_id = -1;
         Universe& universe = AIClientApp::GetApp()->GetUniverse();
         if (pretend_unowned_planets_owned_by_this_ai_empire) {
@@ -198,10 +184,10 @@ namespace AIInterface {
 
             // get all planets the player knows about that aren't yet colonized (aren't owned by anyone).  Add this
             // the current player's ownership to all, while remembering which planets this is done to
-            std::vector<Planet*> all_planets = universe.Objects().FindObjects<Planet>();
+            std::vector<TemporaryPtr<Planet> > all_planets = universe.Objects().FindObjects<Planet>();
             universe.InhibitUniverseObjectSignals(true);
-            for (std::vector<Planet*>::iterator it = all_planets.begin(); it != all_planets.end(); ++it) {
-                 Planet* planet = *it;
+            for (std::vector<TemporaryPtr<Planet> >::iterator it = all_planets.begin(); it != all_planets.end(); ++it) {
+                 TemporaryPtr<Planet> planet = *it;
                  if (planet->Unowned()) {
                      unowned_planets.push_back(planet);
                      planet->SetOwner(player_id);
@@ -214,7 +200,7 @@ namespace AIInterface {
 
         if (pretend_unowned_planets_owned_by_this_ai_empire) {
             // remove temporary ownership added above
-            for (std::vector<Planet*>::iterator it = unowned_planets.begin(); it != unowned_planets.end(); ++it)
+            for (std::vector<TemporaryPtr<Planet> >::iterator it = unowned_planets.begin(); it != unowned_planets.end(); ++it)
                 (*it)->SetOwner(ALL_EMPIRES);
             universe.InhibitUniverseObjectSignals(false);
         }
@@ -247,7 +233,7 @@ namespace AIInterface {
     }
 
     int IssueFleetMoveOrder(int fleet_id, int destination_id) {
-        const Fleet* fleet = GetFleet(fleet_id);
+        TemporaryPtr<const Fleet> fleet = GetFleet(fleet_id);
         if (!fleet) {
             Logger().errorStream() << "AIInterface::IssueFleetMoveOrder : passed an invalid fleet_id";
             return 0;
@@ -278,7 +264,7 @@ namespace AIInterface {
         }
 
         int empire_id = AIClientApp::GetApp()->EmpireID();
-        const UniverseObject* obj = GetUniverseObject(object_id);
+        TemporaryPtr<const UniverseObject> obj = GetUniverseObject(object_id);
 
         if (!obj) {
             Logger().errorStream() << "AIInterface::IssueRenameOrder : passed an invalid object_id";
@@ -304,7 +290,7 @@ namespace AIInterface {
 
             // make sure all objects exist and are owned just by this player
             for (std::vector<int>::const_iterator it = object_ids.begin(); it != object_ids.end(); ++it) {
-                const UniverseObject* obj = GetUniverseObject(*it);
+                TemporaryPtr<const UniverseObject> obj = GetUniverseObject(*it);
 
                 if (!obj) {
                     Logger().errorStream() << "AIInterface::IssueScrapOrder : passed an invalid object_id";
@@ -340,7 +326,7 @@ namespace AIInterface {
         }
 
         int empire_id = AIClientApp::GetApp()->EmpireID();
-        const Ship* ship = 0;
+        TemporaryPtr<const Ship> ship = TemporaryPtr<Ship>();
 
         // make sure all ships exist and are owned just by this player
         for (std::vector<int>::const_iterator it = ship_ids.begin(); it != ship_ids.end(); ++it) {
@@ -364,7 +350,7 @@ namespace AIInterface {
 
         std::vector<int>::const_iterator it = ship_ids.begin();
         for (++it; it != ship_ids.end(); ++it) {
-            const Ship* ship2 = GetShip(*it);
+            TemporaryPtr<const Ship> ship2 = GetShip(*it);
             if (ship2->SystemID() != system_id) {
                 Logger().errorStream() << "AIInterface::IssueNewFleetOrder : passed ship_ids of ships at different locations";
                 return 0;
@@ -387,7 +373,7 @@ namespace AIInterface {
     int IssueFleetTransferOrder(int ship_id, int new_fleet_id) {
         int empire_id = AIClientApp::GetApp()->EmpireID();
 
-        const Ship* ship = GetShip(ship_id);
+        TemporaryPtr<const Ship> ship = GetShip(ship_id);
         if (!ship) {
             Logger().errorStream() << "AIInterface::IssueFleetTransferOrder : passed an invalid ship_id " << ship_id;
             return 0;
@@ -402,7 +388,7 @@ namespace AIInterface {
             return 0;
         }
 
-        const Fleet* fleet = GetFleet(new_fleet_id);
+        TemporaryPtr<const Fleet> fleet = GetFleet(new_fleet_id);
         if (!fleet) {
             Logger().errorStream() << "AIInterface::IssueFleetTransferOrder : passed an invalid new_fleet_id " << new_fleet_id;
             return 0;
@@ -422,15 +408,9 @@ namespace AIInterface {
             return 0;
         }
 
-        int old_fleet_id = ship->FleetID();
-        if (new_fleet_id == old_fleet_id) {
-            Logger().errorStream() << "AIInterface::IssueFleetTransferOrder : ship is already in new fleet id" << new_fleet_id;
-            return 0;
-        }
-
         std::vector<int> ship_ids;
         ship_ids.push_back(ship_id);
-        AIClientApp::GetApp()->Orders().IssueOrder(OrderPtr(new FleetTransferOrder(empire_id, old_fleet_id, new_fleet_id, ship_ids)));
+        AIClientApp::GetApp()->Orders().IssueOrder(OrderPtr(new FleetTransferOrder(empire_id, new_fleet_id, ship_ids)));
 
         return 1;
     }
@@ -439,14 +419,14 @@ namespace AIInterface {
         int empire_id = AIClientApp::GetApp()->EmpireID();
 
         // make sure ship_id is a ship...
-        const Ship* ship = GetShip(ship_id);
+        TemporaryPtr<const Ship> ship = GetShip(ship_id);
         if (!ship) {
             Logger().errorStream() << "AIInterface::IssueColonizeOrder : passed an invalid ship_id";
             return 0;
         }
 
         // get fleet of ship
-        const Fleet* fleet = GetFleet(ship->FleetID());
+        TemporaryPtr<const Fleet> fleet = GetFleet(ship->FleetID());
         if (!fleet) {
             Logger().errorStream() << "AIInterface::IssueColonizeOrder : ship with passed ship_id has invalid fleet_id";
             return 0;
@@ -463,7 +443,7 @@ namespace AIInterface {
         }
 
         // verify that planet exists and is un-occupied.
-        const Planet* planet = GetPlanet(planet_id);
+        TemporaryPtr<const Planet> planet = GetPlanet(planet_id);
         if (!planet) {
             Logger().errorStream() << "AIInterface::IssueColonizeOrder : no planet with passed planet_id";
             return 0;
@@ -492,14 +472,14 @@ namespace AIInterface {
         int empire_id = AIClientApp::GetApp()->EmpireID();
 
         // make sure ship_id is a ship...
-        const Ship* ship = GetShip(ship_id);
+        TemporaryPtr<const Ship> ship = GetShip(ship_id);
         if (!ship) {
             Logger().errorStream() << "AIInterface::IssueInvadeOrder : passed an invalid ship_id";
             return 0;
         }
 
         // get fleet of ship
-        const Fleet* fleet = GetFleet(ship->FleetID());
+        TemporaryPtr<const Fleet> fleet = GetFleet(ship->FleetID());
         if (!fleet) {
             Logger().errorStream() << "AIInterface::IssueInvadeOrder : ship with passed ship_id has invalid fleet_id";
             return 0;
@@ -516,7 +496,7 @@ namespace AIInterface {
         }
 
         // verify that planet exists and is occupied by another empire
-        const Planet* planet = GetPlanet(planet_id);
+        TemporaryPtr<const Planet> planet = GetPlanet(planet_id);
         if (!planet) {
             Logger().errorStream() << "AIInterface::IssueInvadeOrder : no planet with passed planet_id";
             return 0;
@@ -559,6 +539,61 @@ namespace AIInterface {
         return 1;
     }
 
+    int IssueBombardOrder(int ship_id, int planet_id) {
+        int empire_id = AIClientApp::GetApp()->EmpireID();
+
+        // make sure ship_id is a ship...
+        TemporaryPtr<const Ship> ship = GetShip(ship_id);
+        if (!ship) {
+            Logger().errorStream() << "AIInterface::IssueBombardOrder : passed an invalid ship_id";
+            return 0;
+        }
+        if (ship->TotalWeaponsDamage() <= 0) {
+            Logger().errorStream() << "AIInterface::IssueBombardOrder : ship can't attack / bombard";
+            return 0;
+        }
+        if (!ship->OwnedBy(empire_id)) {
+            Logger().errorStream() << "AIInterface::IssueBombardOrder : ship isn't owned by the order-issuing empire";
+            return 0;
+        }
+
+
+        // verify that planet exists and is occupied by another empire
+        TemporaryPtr<const Planet> planet = GetPlanet(planet_id);
+        if (!planet) {
+            Logger().errorStream() << "AIInterface::IssueBombardOrder : no planet with passed planet_id";
+            return 0;
+        }
+        if (planet->OwnedBy(empire_id)) {
+            Logger().errorStream() << "AIInterface::IssueBombardOrder : planet is already owned by the order-issuing empire";
+            return 0;
+        }
+        if (!planet->Unowned() && Empires().GetDiplomaticStatus(planet->Owner(), empire_id) != DIPLO_WAR) {
+            Logger().errorStream() << "AIInterface::IssueBombardOrder : planet owned by an empire not at war with order-issuing empire";
+            return 0;
+        }
+        if (GetUniverse().GetObjectVisibilityByEmpire(planet_id, empire_id) < VIS_BASIC_VISIBILITY) {
+            Logger().errorStream() << "AIInterface::IssueBombardOrder : planet that empire reportedly has insufficient visibility of, but will be allowed to proceed pending investigation";
+            //return;
+        }
+
+
+        int ship_system_id = ship->SystemID();
+        if (ship_system_id == INVALID_OBJECT_ID) {
+            Logger().errorStream() << "AIInterface::IssueBombardOrder : given id of ship not in a system";
+            return 0;
+        }
+        int planet_system_id = planet->SystemID();
+        if (ship_system_id != planet_system_id) {
+            Logger().errorStream() << "AIInterface::IssueBombardOrder : given ids of ship and planet not in the same system";
+            return 0;
+        }
+
+        AIClientApp::GetApp()->Orders().IssueOrder(OrderPtr(new BombardOrder(empire_id, ship_id, planet_id)));
+
+        return 1;
+    }
+
     int IssueDeleteFleetOrder() {
         return 0;
     }
@@ -566,7 +601,7 @@ namespace AIInterface {
     int IssueAggressionOrder(int object_id, bool aggressive) {
         int empire_id = AIClientApp::GetApp()->EmpireID();
 
-        const Fleet* fleet = GetFleet(object_id);
+        TemporaryPtr<const Fleet> fleet = GetFleet(object_id);
         if (!fleet) {
             Logger().errorStream() << "AIInterface::IssueAggressionOrder : no fleet with passed id";
             return 0;
@@ -582,10 +617,69 @@ namespace AIInterface {
         return 1;
     }
 
+    int IssueGiveObjectToEmpireOrder(int object_id, int recipient_id) {
+        int empire_id = AIClientApp::GetApp()->EmpireID();
+
+        if (Empires().Lookup(recipient_id) == 0) {
+            Logger().errorStream() << "AIInterface::IssueGiveObjectToEmpireOrder : given invalid recipient empire id";
+            return 0;
+        }
+
+        if (Empires().GetDiplomaticStatus(empire_id, recipient_id) != DIPLO_PEACE) {
+            Logger().errorStream() << "AIInterface::IssueGiveObjectToEmpireOrder : attempting to give to empire not at peace";
+            return 0;
+        }
+
+        TemporaryPtr<UniverseObject> obj = GetUniverseObject(object_id);
+        if (!obj) {
+            Logger().errorStream() << "AIInterface::IssueGiveObjectToEmpireOrder : passed invalid object id";
+            return 0;
+        }
+
+        if (!obj->OwnedBy(empire_id)) {
+            Logger().errorStream() << "AIInterface::IssueGiveObjectToEmpireOrder : passed object not owned by player";
+            return 0;
+        }
+
+        if (obj->ObjectType() != OBJ_FLEET && obj->ObjectType() != OBJ_PLANET) {
+            Logger().errorStream() << "AIInterface::IssueGiveObjectToEmpireOrder : passed object that is not a fleet or planet";
+            return 0;
+        }
+
+        TemporaryPtr<System> system = GetSystem(obj->SystemID());
+        if (!system) {
+            Logger().errorStream() << "AIInterface::IssueGiveObjectToEmpireOrder : couldn't get system of object";
+            return 0;
+        }
+
+        // can only give to empires with something present to receive the gift
+        bool recipient_has_something_here = false;
+        std::vector<TemporaryPtr<const UniverseObject> > system_objects =
+            Objects().FindObjects<const UniverseObject>(system->ObjectIDs());
+        for (std::vector<TemporaryPtr<const UniverseObject> >::const_iterator it = system_objects.begin();
+                it != system_objects.end(); ++it)
+        {
+            TemporaryPtr<const UniverseObject> obj = *it;
+            if (obj->Owner() == recipient_id) {
+                recipient_has_something_here = true;
+                break;
+            }
+        }
+        if (!recipient_has_something_here) {
+            Logger().errorStream() << "AIInterface::IssueGiveObjectToEmpireOrder : recipient empire has nothing in system";
+            return 0;
+        }
+
+        AIClientApp::GetApp()->Orders().IssueOrder(OrderPtr(
+            new GiveObjectToEmpireOrder(empire_id, object_id, recipient_id)));
+
+        return 1;
+    }
+
     int IssueChangeFocusOrder(int planet_id, const std::string& focus) {
         int empire_id = AIClientApp::GetApp()->EmpireID();
 
-        const Planet* planet = GetPlanet(planet_id);
+        TemporaryPtr<const Planet> planet = GetPlanet(planet_id);
         if (!planet) {
             Logger().errorStream() << "AIInterface::IssueChangeFocusOrder : no planet with passed planet_id "<<planet_id;
             return 0;
@@ -638,7 +732,7 @@ namespace AIInterface {
         int empire_id = AIClientApp::GetApp()->EmpireID();
         Empire* empire = AIClientApp::GetApp()->Empires().Lookup(empire_id);
 
-        if (!empire->BuildableItem(BT_BUILDING, item_name, location_id)) {
+        if (!empire->ProducibleItem(BT_BUILDING, item_name, location_id)) {
             Logger().errorStream() << "AIInterface::IssueEnqueueBuildingProductionOrder : specified item_name and location_id that don't indicate an item that can be built at that location";
             return 0;
         }
@@ -653,7 +747,7 @@ namespace AIInterface {
         int empire_id = AIClientApp::GetApp()->EmpireID();
         Empire* empire = AIClientApp::GetApp()->Empires().Lookup(empire_id);
 
-        if (!empire->BuildableItem(BT_SHIP, design_id, location_id)) {
+        if (!empire->ProducibleItem(BT_SHIP, design_id, location_id)) {
             Logger().errorStream() << "AIInterface::IssueEnqueueShipProductionOrder : specified design_id and location_id that don't indicate a design that can be built at that location";
             return 0;
         }
@@ -750,7 +844,7 @@ namespace AIInterface {
         int current_turn = CurrentTurn();
 
         // create design from stuff chosen in UI
-        ShipDesign* design = new ShipDesign(name, description, current_turn,
+        ShipDesign* design = new ShipDesign(name, description, current_turn, ClientApp::GetApp()->EmpireID(),
                                             hull, parts, icon, model, nameDescInStringTable);
 
         if (!design) {
@@ -760,6 +854,7 @@ namespace AIInterface {
 
         int new_design_id = AIClientApp::GetApp()->GetNewDesignID();
         AIClientApp::GetApp()->Orders().IssueOrder(OrderPtr(new ShipDesignOrder(empire_id, new_design_id, *design)));
+        delete design;
 
         return 1;
     }
