@@ -2,19 +2,19 @@
 #ifndef _Empire_h_
 #define _Empire_h_
 
-#include <GG/Clr.h>
-#include "../util/Export.h"
-#include "../util/AppInterface.h"
-#include "../util/SitRepEntry.h"
 #include "ResourcePool.h"
+#include "../util/Export.h"
 #include "../universe/Meter.h"
 
+#include <GG/Clr.h>
+
 #include <deque>
 #include <string>
 
 struct ItemSpec;
 class ShipDesign;
-class Empire;
+class SitRepEntry;
+extern const int INVALID_GAME_TURN;
 
 class Alignment {
 public:
@@ -42,16 +42,19 @@ struct FO_COMMON_API ResearchQueue {
     struct Element {
         Element() :
             name(),
+            empire_id(ALL_EMPIRES),
             allocated_rp(0.0),
             turns_left(0)
         {}
-        Element(const std::string& name_, double spending_, int turns_left_) :
+        Element(const std::string& name_, int empire_id_, float spending_, int turns_left_) :
             name(name_),
+            empire_id(empire_id_),
             allocated_rp(spending_),
             turns_left(turns_left_)
         {}
         std::string name;
-        double      allocated_rp;
+        int         empire_id;
+        float       allocated_rp;
         int         turns_left;
     private:
         friend class boost::serialization::access;
@@ -77,7 +80,7 @@ struct FO_COMMON_API ResearchQueue {
     /** \name Accessors */ //@{
     bool            InQueue(const std::string& tech_name) const;    ///< Returns true iff \a tech is in this queue.
     int             ProjectsInProgress() const;         ///< Returns the number of research projects currently (perhaps partially) funded.
-    double          TotalRPsSpent() const;              ///< Returns the number of RPs currently spent on the projects in this queue.
+    float           TotalRPsSpent() const;              ///< Returns the number of RPs currently spent on the projects in this queue.
     int             EmpireID() const { return m_empire_id; }
 
     // STL container-like interface
@@ -100,7 +103,7 @@ struct FO_COMMON_API ResearchQueue {
       * precondition of this function that \a RPs must be greater than some
       * epsilon > 0; see the implementation for the actual value used for
       * epsilon. */
-    void            Update(double RPs, const std::map<std::string, double>& research_progress);
+    void            Update(float RPs, const std::map<std::string, float>& research_progress);
 
     // STL container-like interface
     void            push_back(const std::string& tech_name);
@@ -117,13 +120,13 @@ struct FO_COMMON_API ResearchQueue {
       * none exists. */
     iterator        UnderfundedProject();
 
-    mutable boost::signal<void ()> ResearchQueueChangedSignal;
+    mutable boost::signals2::signal<void ()> ResearchQueueChangedSignal;
     //@}
 
 private:
     QueueType   m_queue;
     int         m_projects_in_progress;
-    double      m_total_RPs_spent;
+    float       m_total_RPs_spent;
     int         m_empire_id;
 
     friend class boost::serialization::access;
@@ -138,6 +141,10 @@ struct FO_COMMON_API ProductionQueue {
         ProductionItem(BuildType build_type_, std::string name_);   ///< basic ctor for BuildTypes that use std::string to identify specific items (BuildingTypes)
         ProductionItem(BuildType build_type_, int design_id_);      ///< basic ctor for BuildTypes that use int to indentify the design of the item (ShipDesigns)
 
+        bool CostIsProductionLocationInvariant() const;             ///< indicates whether production location can change the cost of this item. This is useful for cachcing cost results per-location or once for all locations.
+
+        bool operator<(const ProductionItem& rhs) const;
+
         BuildType   build_type;
 
         // only one of these may be valid, depending on BuildType
@@ -153,18 +160,19 @@ struct FO_COMMON_API ProductionQueue {
     /** The type of a single element in the production queue. */
     struct Element {
         Element(); ///< default ctor.
-        Element(ProductionItem item_, int ordered_, int remaining_, int location_); ///< basic ctor.
-        Element(BuildType build_type, std::string name, int ordered_, int remaining_, int location_); ///< basic ctor.
-        Element(BuildType build_type, int design_id, int ordered_, int remaining_, int location_); ///< basic ctor.
+        Element(ProductionItem item_, int empire_id_, int ordered_, int remaining_, int location_);
+        Element(BuildType build_type, std::string name, int empire_id_, int ordered_, int remaining_, int location_);
+        Element(BuildType build_type, int design_id, int empire_id_, int ordered_, int remaining_, int location_);
 
         ProductionItem  item;
+        int             empire_id;
         int             ordered;                    ///< how many of item (blocks) to produce
         int             blocksize;                  ///< size of block to produce (default=1)
         int             remaining;                  ///< how many left to produce
         int             location;                   ///< the ID of the UniverseObject at which this item is being produced
-        double          allocated_pp;               ///< PP allocated to this ProductionQueue Element by Empire production update
-        double          progress;                   ///< PP that has been spent on this production element (will increase by allocation during next turn processing)
-        double          progress_memory;            ///< updated by server turn processing; aides in allowing blocksize changes to be undone in same turn w/o progress loss
+        float           allocated_pp;               ///< PP allocated to this ProductionQueue Element by Empire production update
+        float           progress;                   ///< PP that has been spent on this production element (will increase by allocation during next turn processing)
+        float           progress_memory;            ///< updated by server turn processing; aides in allowing blocksize changes to be undone in same turn w/o progress loss
         int             blocksize_memory;           ///< used along with progress_memory
         int             turns_left_to_next_item;
         int             turns_left_to_completion;
@@ -188,17 +196,17 @@ struct FO_COMMON_API ProductionQueue {
 
     /** \name Accessors */ //@{
     int                             ProjectsInProgress() const;         ///< Returns the number of production projects currently (perhaps partially) funded.
-    double                          TotalPPsSpent() const;              ///< Returns the number of PPs currently spent on the projects in this queue.
+    float                           TotalPPsSpent() const;              ///< Returns the number of PPs currently spent on the projects in this queue.
     int                             EmpireID() const { return m_empire_id; }
 
     /** Returns map from sets of object ids that can share resources to amount
       * of PP available in those groups of objects */
-    std::map<std::set<int>, double>         AvailablePP(const boost::shared_ptr<ResourcePool>& industry_pool) const;
+    std::map<std::set<int>, float>         AvailablePP(const boost::shared_ptr<ResourcePool>& industry_pool) const;
 
     /** Returns map from sets of object ids that can share resources to amount
       * of PP allocated to production queue elements that have build locations
       * in systems in the group. */
-    const std::map<std::set<int>, double>&  AllocatedPP() const;
+    const std::map<std::set<int>, float>&  AllocatedPP() const;
 
     /** Returns sets of object ids that have more available than allocated PP */
     std::set<std::set<int> >                ObjectsWithWastedPP(const boost::shared_ptr<ResourcePool>& industry_pool) const;
@@ -240,13 +248,13 @@ struct FO_COMMON_API ProductionQueue {
     /** Returns an iterator to the underfunded production project, or end() if none exists. */
     iterator                        UnderfundedProject();
 
-    mutable boost::signal<void ()> ProductionQueueChangedSignal;
+    mutable boost::signals2::signal<void ()> ProductionQueueChangedSignal;
     //@}
 
 private:
     QueueType                       m_queue;
     int                             m_projects_in_progress;
-    std::map<std::set<int>, double> m_object_group_allocated_pp;
+    std::map<std::set<int>, float>  m_object_group_allocated_pp;
     int                             m_empire_id;
 
     friend class boost::serialization::access;
@@ -297,19 +305,21 @@ public:
     const std::set<std::string>&    AvailableShipParts() const;                     ///< Returns the set of ship part names this empire that the empire can currently build
     const std::set<std::string>&    AvailableShipHulls() const;                     ///< Returns the set of ship hull names that that the empire can currently build
 
-    const std::string&              TopPriorityEnqueuedTech(bool only_consider_available_techs = false);
-    const std::string&              MostExpensiveEnqueuedTech(bool only_consider_available_techs = false);
-    const std::string&              LeastExpensiveEnqueuedTech(bool only_consider_available_techs = false);
-    const std::string&              MostRPSpentEnqueuedTech(bool only_consider_available_techs = false);
-    const std::string&              MostRPCostLeftEnqueuedTech(bool only_consider_available_techs = false);
+    const std::string&              TopPriorityEnqueuedTech(bool only_consider_available_techs = false) const;
+    const std::string&              MostExpensiveEnqueuedTech(bool only_consider_available_techs = false) const;
+    const std::string&              LeastExpensiveEnqueuedTech(bool only_consider_available_techs = false) const;
+    const std::string&              MostRPSpentEnqueuedTech(bool only_consider_available_techs = false) const;
+    const std::string&              MostRPCostLeftEnqueuedTech(bool only_consider_available_techs = false) const;
 
     const Meter*                                    GetMeter(const std::string& name) const;
     std::map<std::string, Meter>::const_iterator    meter_begin() const { return m_meters.begin(); }
     std::map<std::string, Meter>::const_iterator    meter_end() const   { return m_meters.end(); }
 
+    const ResearchQueue&    GetResearchQueue() const;                   ///< Returns the queue of techs being or queued to be researched.
+    const ProductionQueue&  GetProductionQueue() const;                 ///< Returns the queue of items being or queued to be produced.
+
     bool        ResearchableTech(const std::string& name) const;        ///< Returns true iff \a name is a tech that has not been researched, and has no unresearched prerequisites.
-    const       ResearchQueue& GetResearchQueue() const;                ///< Returns the queue of techs being or queued to be researched.
-    double      ResearchProgress(const std::string& name) const;        ///< Returns the RPs spent towards tech \a name if it has partial research progress, or 0.0 if it is already researched.
+    float       ResearchProgress(const std::string& name) const;        ///< Returns the RPs spent towards tech \a name if it has partial research progress, or 0.0 if it is already researched.
     bool        TechResearched(const std::string& name) const;          ///< Returns true iff this tech has been completely researched.
     TechStatus  GetTechStatus(const std::string& name) const;           ///< Returns the status (researchable, researched, unresearchable) for this tech for this
 
@@ -319,18 +329,19 @@ public:
     bool        ShipPartAvailable(const std::string& name) const;       ///< Returns true iff this ship part can be built by this empire.  If no such ship part exists, returns false
     bool        ShipHullAvailable(const std::string& name) const;       ///< Returns true iff this ship hull can be built by this empire.  If no such ship hull exists, returns false
 
-    const       ProductionQueue& GetProductionQueue() const;            ///< Returns the queue of items being or queued to be produced.
-    double      ProductionStatus(int i) const;                          ///< Returns the PPs spent towards item \a i in the build queue if it has partial progress, -1.0 if there is no such index in the production queue.
+    float       ProductionStatus(int i) const;                          ///< Returns the PPs spent towards item \a i in the build queue if it has partial progress, -1.0 if there is no such index in the production queue.
 
     /** Returns the total cost per item (blocksize 1) and the minimum number of turns
       * required to produce the indicated item, or (-1.0, -1) if the item is
       * unknown, unavailable, or invalid. */
-    std::pair<double, int>  ProductionCostAndTime(const ProductionQueue::Element& element) const;
-    std::pair<double, int>  ProductionCostAndTime(const ProductionQueue::ProductionItem& item, int location_id) const;
+    std::pair<float, int>   ProductionCostAndTime(const ProductionQueue::Element& element) const;
+    std::pair<float, int>   ProductionCostAndTime(const ProductionQueue::ProductionItem& item, int location_id) const;
 
-    bool                    BuildableItem(BuildType build_type, const std::string& name, int location) const;  ///< Returns true iff this empire can produce the specified item at the specified location.
-    bool                    BuildableItem(BuildType build_type, int design_id, int location) const;            ///< Returns true iff this empire can produce the specified item at the specified location.
-    bool                    BuildableItem(const ProductionQueue::ProductionItem& item, int location) const;    ///< Returns true iff this empire can produce the specified item at the specified location.
+    bool                    ProducibleItem(BuildType build_type, const std::string& name, int location) const;  ///< Returns true iff this empire can produce the specified item at the specified location.
+    bool                    ProducibleItem(BuildType build_type, int design_id, int location) const;            ///< Returns true iff this empire can produce the specified item at the specified location.
+    bool                    ProducibleItem(const ProductionQueue::ProductionItem& item, int location) const;    ///< Returns true iff this empire can produce the specified item at the specified location.
+
+    bool                    EnqueuableItem(BuildType build_type, const std::string& name, int location) const;  ///< Returns true iff this empire can enqueue the specified item at the specified location.
 
     bool                    HasExploredSystem(int ID) const;                            ///< returns  true if the given item is in the appropriate list, false if it is not.
 
@@ -377,21 +388,15 @@ public:
     SitRepItr               SitRepBegin() const;                ///< starting iterator for sitrep entries for this empire
     SitRepItr               SitRepEnd() const;                  ///< end iterator for sitreps
 
-    double                  ProductionPoints() const;           ///< Returns the number of production points available to the empire (this is available industry)
-
-    /** Returns amount of trade empire will spend this turn.  Assumes
-      * Empire::UpdateTradeSpending() has previously been called to determine
-      * this number. */
-    double                  TotalTradeSpending() const {return m_maintenance_total_cost;}
+    float                   ProductionPoints() const;           ///< Returns the number of production points available to the empire (this is available industry)
 
     const boost::shared_ptr<ResourcePool>   GetResourcePool(ResourceType resource_type) const;  ///< Returns ResourcePool for \a resource_type or 0 if no such ResourcePool exists
-    double                  ResourceStockpile(ResourceType type) const;         ///< returns current stockpiled amount of resource \a type
-    double                  ResourceMaxStockpile(ResourceType type) const;      ///< returns maximum allowed stockpile of resource \a type
-    double                  ResourceProduction(ResourceType type) const;        ///< returns amount of resource \a type being produced by ResourceCenters
-    double                  ResourceAvailable(ResourceType type) const;         ///< returns amount of resource \a type immediately available.  This = production + stockpile
+    float                   ResourceStockpile(ResourceType type) const;         ///< returns current stockpiled amount of resource \a type
+    float                   ResourceProduction(ResourceType type) const;        ///< returns amount of resource \a type being produced by ResourceCenters
+    float                   ResourceAvailable(ResourceType type) const;         ///< returns amount of resource \a type immediately available.  This = production + stockpile
 
     const PopulationPool&   GetPopulationPool() const;                          ///< Returns PopulationPool
-    double                  Population() const;                                 ///< returns total Population of empire
+    float                   Population() const;                                 ///< returns total Population of empire
     //@}
 
     /** \name Mutators */ //@{
@@ -414,7 +419,7 @@ public:
       * research queue already. */
     void        RemoveTechFromQueue(const std::string& name);
     /** Sets research progress of tech with \a name to \a progress. */
-    void        SetTechResearchProgress(const std::string& name, double progress);
+    void        SetTechResearchProgress(const std::string& name, float progress);
     /** Adds the indicated build to the production queue, placing it before
       * position \a pos.  If \a pos < 0 or queue.size() <= pos, the build is
       * placed at the end of the queue. */
@@ -512,8 +517,7 @@ public:
     void        SetName(const std::string& name);               ///< Mutator for empire name
     void        SetPlayerName(const std::string& player_name);  ///< Mutator for empire's player name
 
-    void        SetResourceStockpile(ResourceType resource_type, double stockpile); ///< Sets current \a stockpile amount of indicated \a resource_type
-    void        SetResourceMaxStockpile(ResourceType resource_type, double max);    ///< Sets \a max amount of stockpile of indicated \a resource_typ
+    void        SetResourceStockpile(ResourceType resource_type, float stockpile); ///< Sets current \a stockpile amount of indicated \a resource_type
 
     /** Determines ResourceCenters that can provide resources for this empire and sets
       * the supply groups used for each ResourcePool as appropriate for each resource.
@@ -546,6 +550,27 @@ public:
 
     /** Resets empire meters. */
     void        ResetMeters();
+
+    std::map<int, int>&         EmpireShipsDestroyed()  { return m_empire_ships_destroyed; }
+    std::map<int, int>&         ShipDesignsDestroyed()  { return m_ship_designs_destroyed; }
+    std::map<std::string, int>& SpeciesShipsDestroyed() { return m_species_ships_destroyed; }
+
+    std::map<std::string, int>& SpeciesPlanetsInvaded() { return m_species_planets_invaded; }
+
+    std::map<std::string, int>& SpeciesShipsProduced()  { return m_species_ships_produced; }
+    std::map<int, int>&         ShipDesignsProduced()   { return m_ship_designs_produced; }
+
+    std::map<std::string, int>& SpeciesShipsLost()      { return m_species_ships_lost; }
+    std::map<int, int>&         ShipDesignsLost()       { return m_ship_designs_lost; }
+
+    std::map<std::string, int>& SpeciesShipsScrapped()  { return m_species_ships_scrapped; }
+    std::map<int, int>&         ShipDesignsScrapped()   { return m_ship_designs_scrapped; }
+
+    std::map<std::string, int>& SpeciesPlanetsDepoped() { return m_species_planets_depoped; }
+    std::map<std::string, int>& SpeciesPlanetsBombed()  { return m_species_planets_bombed; }
+
+    std::map<std::string, int>& BuildingTypesProduced() { return m_building_types_produced; }
+    std::map<std::string, int>& BuildingTypesScrapped() { return m_building_types_scrapped; }
     //@}
 
     /** Processes Builditems on queues of empires other than the indicated
@@ -555,7 +580,7 @@ public:
       * current empire */
     static void ConquerProductionQueueItemsAtLocation(int location_id, int empire_id);
 
-    mutable boost::signal<void ()>  ShipDesignsChangedSignal;
+    mutable boost::signals2::signal<void ()>  ShipDesignsChangedSignal;
 
 private:
     void        Init();
@@ -571,7 +596,7 @@ private:
     std::map<std::string, Meter>    m_meters;                   ///< empire meters, including ratings scales used by species to judge empires
 
     ResearchQueue                   m_research_queue;           ///< the queue of techs being or waiting to be researched
-    std::map<std::string, double>   m_research_progress;        ///< progress of partially-researched techs; fully researched techs are removed
+    std::map<std::string, float>    m_research_progress;        ///< progress of partially-researched techs; fully researched techs are removed
 
     ProductionQueue                 m_production_queue;         ///< the queue of items being or waiting to be built
 
@@ -586,15 +611,23 @@ private:
     std::map<ResourceType, boost::shared_ptr<ResourcePool> >    m_resource_pools;
     PopulationPool                                              m_population_pool;
 
-    /** MAYBE TEMPORARY: Until social projects and/or consequences of unpaid
-      * maintenance is implemented.  Total maintenance on buildings owned by
-      * this empire.  Set by UpdateTradeSpending(), used by
-      * CheckTradeSocialProgress() to deduct maintenance cost from trade
-      * stockpile */
-    double                          m_maintenance_total_cost;
-
     std::map<std::string, int>      m_ship_names_used;                      ///< map from name to number of times used
 
+    std::map<int, int>              m_empire_ships_destroyed;   ///< how many ships of each empire has this empire destroyed?
+    std::map<int, int>              m_ship_designs_destroyed;   ///< how many ships of each design has this empire destroyed?
+    std::map<std::string, int>      m_species_ships_destroyed;  ///< how many ships crewed by each species has this empire destroyed?
+    std::map<std::string, int>      m_species_planets_invaded;  ///< how many planets populated by each species has this empire destroyed?
+    std::map<std::string, int>      m_species_ships_produced;   ///< how many ships crewed by each species has this empire produced?
+    std::map<int, int>              m_ship_designs_produced;    ///< how many ships of each design has this empire produced?
+    std::map<std::string, int>      m_species_ships_lost;       ///< how mahy ships crewed by each species has this empire lost in combat?
+    std::map<int, int>              m_ship_designs_lost;        ///< how many ships of each design has this empire lost in combat?
+    std::map<std::string, int>      m_species_ships_scrapped;   ///< how many ships crewed by each species has this empire scrapped?
+    std::map<int, int>              m_ship_designs_scrapped;    ///< how many ships of each design has this empire scrapped?
+    std::map<std::string, int>      m_species_planets_depoped;  ///< how many planets populated by each species have depopulated while owned by this empire?
+    std::map<std::string, int>      m_species_planets_bombed;   ///< how many planets populated by each species has this empire bombarded?
+    std::map<std::string, int>      m_building_types_produced;  ///< how many buildings of each type has this empire produced?
+    std::map<std::string, int>      m_building_types_scrapped;  ///< how many buildings of each type has this empire scrapped?
+
     // cached calculation results, returned by reference
     std::map<int, int>              m_supply_system_ranges;                 ///< number of starlane jumps away from each system (by id) supply can be conveyed.  This is the number due to a system's contents conveying supply and is computed and set by UpdateSystemSupplyRanges
     std::set<int>                   m_supply_unobstructed_systems;          ///< ids of system that don't block supply from flowing
