@@ -33,6 +33,7 @@
 
 #include <GG/ClrConstants.h>
 #include <GG/TextControl.h>
+#include <GG/Enum.h>
 
 
 namespace GG {
@@ -48,14 +49,15 @@ class GG_API Button : public TextControl
 {
 public:
     /// the states of being for a GG::Button
-    enum ButtonState {
+    GG_CLASS_ENUM(ButtonState,
         BN_PRESSED,    ///< The button is being pressed by the user, and the cursor is over the button
         BN_UNPRESSED,  ///< The button is unpressed
         BN_ROLLOVER    ///< The button has the cursor over it, but is unpressed
-    };
+    )
 
     /** \name Signal Types */ ///@{
-    typedef boost::signal<void ()> ClickedSignalType; ///< Emitted when the button is clicked by the user
+    /** Emitted when the button is clicked by the user */
+    typedef boost::signals2::signal<void ()> ClickedSignalType;
     //@}
 
     /** \name Structors */ ///@{
@@ -71,8 +73,10 @@ public:
     const SubTexture& PressedGraphic() const;    ///< Returns the SubTexture to be used as the image of the button when pressed
     const SubTexture& RolloverGraphic() const;   ///< Returns the SubTexture to be used as the image of the button when it contains the cursor, but is not pressed
 
-    mutable ClickedSignalType LeftClickedSignal; ///< The left clicked signal object for this Button
-    mutable ClickedSignalType RightClickedSignal;///< The right clicked signal object for this Button
+    /** The left clicked signal object for this Button */
+    mutable ClickedSignalType LeftClickedSignal;
+    /** The right clicked signal object for this Button */
+    mutable ClickedSignalType RightClickedSignal;
     //@}
 
     /** \name Mutators */ ///@{
@@ -120,16 +124,6 @@ private:
     SubTexture     m_rollover_graphic;  ///< Graphic used to display button when it's under the mouse and not pressed
 };
 
-// define EnumMap and stream operators for Button::ButtonState
-GG_ENUM_MAP_BEGIN(Button::ButtonState)
-    GG_ENUM_MAP_INSERT(Button::BN_PRESSED)
-    GG_ENUM_MAP_INSERT(Button::BN_UNPRESSED)
-    GG_ENUM_MAP_INSERT(Button::BN_ROLLOVER)
-GG_ENUM_MAP_END
-
-GG_ENUM_STREAM_IN(Button::ButtonState)
-GG_ENUM_STREAM_OUT(Button::ButtonState)
-
 
 /** \brief This is a basic state button control.
 
@@ -146,7 +140,9 @@ class GG_API StateButton : public TextControl
 {
 public:
     /** \name Signal Types */ ///@{
-    typedef boost::signal<void (bool)> CheckedSignalType; ///< Emitted when the StateButton is checked or unchecked; the checked/unchecked status is indicated by the bool parameter
+    /** Emitted when the StateButton is checked or unchecked; the checked or
+        unchecked status is indicated by the bool parameter */
+    typedef boost::signals2::signal<void (bool)> CheckedSignalType;
     //@}
 
     /** \name Structors */ ///@{
@@ -222,7 +218,10 @@ class GG_API RadioButtonGroup : public Control
 {
 public:
     /** \name Signal Types */ ///@{
-    typedef boost::signal<void (std::size_t)> ButtonChangedSignalType; ///< emitted when the currently-selected button has changed; the new selected button's index in the group is provided (this may be NO_BUTTON if no button is currently selected)
+    /** emitted when the currently-selected button has changed; the new
+        selected button's index in the group is provided (this may be
+        NO_BUTTON if no button is currently selected) */
+    typedef boost::signals2::signal<void (std::size_t)> ButtonChangedSignalType;
     //@}
 
     /** \name Structors */ ///@{
@@ -338,8 +337,8 @@ protected:
     {
         ButtonSlot();
         ButtonSlot(StateButton* button_);
-        StateButton*               button;
-        boost::signals::connection connection;
+        StateButton*                button;
+        boost::signals2::connection connection;
     };
 
     /** \name Structors */ ///@{
