@@ -168,17 +168,17 @@ public:
         //@}
 
         /** \name Accessors */ ///@{
-        SortKeyType  SortKey(std::size_t column) const; ///< returns the string by which this row may be sorted
-        std::size_t  size() const;               ///< returns the number of Controls in this Row
-        bool         empty() const;              ///< returns true iff there are 0 Controls in this Row
+        virtual SortKeyType SortKey(std::size_t column) const;  ///< returns the string by which this row may be sorted
+        std::size_t         size() const;                       ///< returns the number of Controls in this Row
+        bool                empty() const;                      ///< returns true iff there are 0 Controls in this Row
 
-        Control*     operator[](std::size_t n) const; ///< returns the Control in the \a nth cell of this Row; not range checked
-        Control*     at(std::size_t n) const;    ///< returns the Control in the \a nth cell of this Row \throw std::range_error throws when size() <= \a n
+        virtual Control*    operator[](std::size_t n) const;    ///< returns the Control in the \a nth cell of this Row; not range checked
+        virtual Control*    at(std::size_t n) const;            ///< returns the Control in the \a nth cell of this Row \throw std::range_error throws when size() <= \a n
 
-        Alignment    RowAlignment() const;       ///< returns the vertical alignment of this Row
+        Alignment    RowAlignment() const;              ///< returns the vertical alignment of this Row
         Alignment    ColAlignment(std::size_t n) const; ///< returns the horizontal alignment of the Control in the \a nth cell of this Row; not range checked
-        X            ColWidth(std::size_t n) const; ///< returns the width of the \a nth cell of this Row; not range checked
-        unsigned int Margin() const;             ///< returns the amount of space left between the contents of adjacent cells, in pixels
+        X            ColWidth(std::size_t n) const;     ///< returns the width of the \a nth cell of this Row; not range checked
+        unsigned int Margin() const;                    ///< returns the amount of space left between the contents of adjacent cells, in pixels
 
         Control*     CreateControl(const std::string& str, const boost::shared_ptr<Font>& font, Clr color) const; ///< creates a "shrink-fit" TextControl from text, font, and color parameters
         Control*     CreateControl(const SubTexture& st) const; ///< creates a "shrink-fit" StaticGraphic Control from a SubTexture parameter
@@ -204,7 +204,7 @@ public:
         void         SetMargin(unsigned int margin); ///< sets the amount of space left between the contents of adjacent cells, in pixels
         //@}
 
-    private:
+    protected:
         void AdjustLayout(bool adjust_for_push_back = false);
 
         std::vector<Control*>  m_cells;          ///< the Controls in this Row (each may be null)
@@ -215,7 +215,7 @@ public:
 
         bool                   m_ignore_adjust_layout;
 
-        friend class DeferAdjustLayout;
+        friend struct DeferAdjustLayout;
     };
 
     typedef std::list<Row*>::iterator iterator;
@@ -226,11 +226,16 @@ public:
     typedef std::set<iterator, RowPtrIteratorLess<std::list<Row*> > > SelectionSet;
 
     /** \name Signal Types */ ///@{
-    typedef boost::signal<void ()>                    ClearedSignalType;    ///< emitted when the list box is cleared
-    typedef boost::signal<void (const SelectionSet&)> SelChangedSignalType; ///< emitted when one or more rows are selected or deselected
-    typedef boost::signal<void (iterator)>            RowSignalType;        ///< the signature of row-change-notification signals
-    typedef boost::signal<void (const_iterator)>      ConstRowSignalType;   ///< the signature of const row-change-notification signals
-    typedef boost::signal<void (iterator, const Pt&)> RowClickSignalType;   ///< the signature of row-click-notification signals
+    /** emitted when the list box is cleared */
+    typedef boost::signals2::signal<void ()>                    ClearedSignalType;
+    /** emitted when one or more rows are selected or deselected */
+    typedef boost::signals2::signal<void (const SelectionSet&)> SelChangedSignalType;
+    /** the signature of row-change-notification signals */
+    typedef boost::signals2::signal<void (iterator)>            RowSignalType;
+    /** the signature of const row-change-notification signals */
+    typedef boost::signals2::signal<void (const_iterator)>      ConstRowSignalType;
+    /** the signature of row-click-notification signals */
+    typedef boost::signals2::signal<void (iterator, const Pt&)> RowClickSignalType;
 
     typedef RowSignalType      BeforeInsertSignalType;   ///< emitted before a row is inserted into the list box
     typedef RowSignalType      AfterInsertSignalType;    ///< emitted after a row is inserted into the list box
@@ -260,20 +265,17 @@ public:
     virtual Pt      ClientUpperLeft() const;
     virtual Pt      ClientLowerRight() const;
 
-    bool            Empty() const;          ///< returns true when the ListBox is empty
-    const_iterator  begin() const;          ///< returns an iterator to the first list row
-    const_iterator  end() const;            ///< returns an iterator to the imaginary row one past the last
-    const_reverse_iterator
-                    rbegin() const;         ///< returns an iterator to the last list row
-    const_reverse_iterator
-                    rend() const;           ///< returns an iterator to the imaginary row one past the first one
-    const Row&      GetRow(std::size_t n) const; ///< returns a const reference to the row at index \a n; not range-checked.  \note This function is O(n).
-    iterator        Caret() const;          ///< returns the row that has the caret
-    const SelectionSet&
-                    Selections() const;     ///< returns a const reference to the set row indexes that is currently selected
-    bool            Selected(iterator it) const; ///< returns true if row \a it is selected
-    Clr             InteriorColor() const;  ///< returns the color painted into the client area of the control
-    Clr             HiliteColor() const;    ///< returns the color behind selected line items
+    bool                    Empty() const;          ///< returns true when the ListBox is empty
+    const_iterator          begin() const;          ///< returns an iterator to the first list row
+    const_iterator          end() const;            ///< returns an iterator to the imaginary row one past the last
+    const_reverse_iterator  rbegin() const;         ///< returns an iterator to the last list row
+    const_reverse_iterator  rend() const;           ///< returns an iterator to the imaginary row one past the first one
+    const Row&              GetRow(std::size_t n) const; ///< returns a const reference to the row at index \a n; not range-checked.  \note This function is O(n).
+    iterator                Caret() const;          ///< returns the row that has the caret
+    const SelectionSet&     Selections() const;     ///< returns a const reference to the set row indexes that is currently selected
+    bool                    Selected(iterator it) const; ///< returns true if row \a it is selected
+    Clr                     InteriorColor() const;  ///< returns the color painted into the client area of the control
+    Clr                     HiliteColor() const;    ///< returns the color behind selected line items
 
     /** Returns the style flags of the listbox \see GG::ListBoxStyle */
     Flags<ListBoxStyle> Style() const;
@@ -318,8 +320,8 @@ public:
     unsigned int    AutoScrollInterval() const;
 
     mutable ClearedSignalType        ClearedSignal;         /// the cleared signal object for this ListBox
-    mutable BeforeInsertSignalType   BeforeInsertSignal;    ///< the before insert signal objet for this ListBox
-    mutable AfterInsertSignalType    AfterInsertSignal;     ///< the after insert signal objet for this ListBox
+    mutable BeforeInsertSignalType   BeforeInsertSignal;    ///< the before insert signal object for this ListBox
+    mutable AfterInsertSignalType    AfterInsertSignal;     ///< the after insert signal object for this ListBox
     mutable SelChangedSignalType     SelChangedSignal;      ///< the selection change signal object for this ListBox
     mutable DroppedSignalType        DroppedSignal;         ///< the dropped signal object for this ListBox
     mutable DropAcceptableSignalType DropAcceptableSignal;  ///< the drop-acceptability signal object for this ListBox
@@ -332,103 +334,118 @@ public:
     //@}
 
     /** \name Mutators */ ///@{
-    virtual void   StartingChildDragDrop(const Wnd* wnd, const GG::Pt& offset);
-    virtual void   AcceptDrops(const std::vector<Wnd*>& wnds, const Pt& pt);
-    virtual void   ChildrenDraggedAway(const std::vector<Wnd*>& wnds, const Wnd* destination);
-    virtual void   Render();
+    virtual void    StartingChildDragDrop(const Wnd* wnd, const GG::Pt& offset);
+    virtual void    AcceptDrops(const std::vector<Wnd*>& wnds, const Pt& pt);
+    virtual void    ChildrenDraggedAway(const std::vector<Wnd*>& wnds, const Wnd* destination);
+    virtual void    Render();
 
-    virtual void   SizeMove(const Pt& ul, const Pt& lr);  ///< resizes the control, then resizes the scrollbars as needed
+    virtual void    SizeMove(const Pt& ul, const Pt& lr);  ///< resizes the control, then resizes the scrollbars as needed
 
-    virtual void   Disable(bool b = true);
-    virtual void   SetColor(Clr c);
+    virtual void    Disable(bool b = true);
+    virtual void    SetColor(Clr c);
 
     /** Insertion sorts \a row into the ListBox if sorted, or inserts into an
         unsorted ListBox before \a it; returns insertion point.  This Row
-        becomes the property of the ListBox. */
-    iterator       Insert(Row* row, iterator it);
+        becomes the property of this ListBox. */
+    iterator        Insert(Row* row, iterator it, bool signal = true);
 
     /** Insertion sorts \a row into the ListBox if sorted, or inserts into an
         unsorted ListBox at the end of the list; returns insertion point.
-        This Row becomes the property of the ListBox. */
-    iterator       Insert(Row* row);
+        This Row becomes the property of this ListBox. */
+    iterator        Insert(Row* row, bool signal = true);
 
-    Row*           Erase(iterator it);                    ///< removes and returns the row that \a it points to from the ListBox, or 0 if no such row exists
-    void           Clear();                               ///< empties the ListBox
-    void           SelectRow(iterator it);                ///< selects row \a it
-    void           DeselectRow(iterator it);              ///< deselects row \a it
-    void           SelectAll();                           ///< selects all rows
-    void           DeselectAll();                         ///< deselects all rows
-    iterator       begin();                               ///< returns an iterator to the first list row
-    iterator       end();                                 ///< returns an iterator to the imaginary row one past the last one
-    reverse_iterator
-                   rbegin();                              ///< returns an iterator to the last list row
-    reverse_iterator
-                   rend();                                ///< returns an iterator to the imaginary row one past the first one
-    Row&           GetRow(std::size_t n);                 ///< returns a reference to the Row at row index \a n; not range-checked.  \note This function is O(n).
+    /** Insertion sorts \a rows into the ListBox if sorted, or inserts into an
+        unsorted ListBox before \a it. The Rows become the property of this
+        ListBox. */
+    void            Insert(const std::vector<Row*>& rows, iterator it, bool signal = true);
 
-    void           SetSelections(const SelectionSet& s);  ///< sets the set of selected rows to \a s
-    void           SetCaret(iterator it);                 ///< sets the position of the caret to \a it
-    void           BringRowIntoView(iterator it);         ///< moves the scrollbars so that row \a it is visible
-    void           SetFirstRowShown(iterator it);         ///< moves the scrollbars so that row \a it is the first visible
+    /** Insertion sorts \a rows into the ListBox if sorted, or inserts into an
+        unsorted ListBox at the end of the list. The Rows become the property
+        of this ListBox. */
+    void            Insert(const std::vector<Row*>& rows, bool signal = true);
 
-    void           SetInteriorColor(Clr c);               ///< sets the color painted into the client area of the control
-    void           SetHiliteColor(Clr c);                 ///< sets the color behind selected line items
+    Row*            Erase(iterator it, bool signal = false);        ///< removes and returns the row that \a it points to from the ListBox, or 0 if no such row exists
+    void            Clear();                                        ///< empties the ListBox
+    void            SelectRow(iterator it, bool signal = false);    ///< selects row \a it
+    void            DeselectRow(iterator it, bool signal = false);  ///< deselects row \a it
+    void            SelectAll(bool signal = false);                 ///< selects all rows
+    void            DeselectAll(bool signal = false);               ///< deselects all rows
+    void            SetSelections(const SelectionSet& s,
+                                  bool signal = false);     ///< sets the set of selected rows to \a s
+
+    iterator            begin();                            ///< returns an iterator to the first list row
+    iterator            end();                              ///< returns an iterator to the imaginary row one past the last one
+    reverse_iterator    rbegin();                           ///< returns an iterator to the last list row
+    reverse_iterator    rend();                             ///< returns an iterator to the imaginary row one past the first one
+
+    Row&            GetRow(std::size_t n);                  ///< returns a reference to the Row at row index \a n; not range-checked.  \note This function is O(n).
+
+    void            SetCaret(iterator it);                  ///< sets the position of the caret to \a it
+    void            BringRowIntoView(iterator it);          ///< moves the scrollbars so that row \a it is visible
+    void            SetFirstRowShown(iterator it);          ///< moves the scrollbars so that row \a it is the first visible
+
+    /** Sets how much to scroll when scrolled using the mousewheel. */
+    void            SetVScrollWheelIncrement(unsigned int increment);
+    void            SetHScrollWheelIncrement(unsigned int increment);
+
+    void            SetInteriorColor(Clr c);                ///< sets the color painted into the client area of the control
+    void            SetHiliteColor(Clr c);                  ///< sets the color behind selected line items
 
     /** sets the style flags for the ListBox to \a s. \see GG::ListBoxStyle */
-    void           SetStyle(Flags<ListBoxStyle> s);
+    void            SetStyle(Flags<ListBoxStyle> s);
 
-    void           SetColHeaders(Row* r);                 ///< sets the row used as headings for the columns; this Row becomes property of the ListBox.
-    void           RemoveColHeaders();                    ///< removes any columns headings set
+    void            SetColHeaders(Row* r);                  ///< sets the row used as headings for the columns; this Row becomes property of the ListBox.
+    void            RemoveColHeaders();                     ///< removes any columns headings set
 
-    void           SetColWidth(std::size_t n, X w);  ///< sets the width of column \n to \a w; not range-checked
-    void           SetNumCols(std::size_t n);             ///< sets the number of columns in the ListBox to \a n; if no column widths exist before this call, proportional widths are calulated and set, otherwise no column widths are set
-    void           SetSortCol(std::size_t n);             ///< sets the index of the column used to sort rows when sorting is enabled; not range-checked
+    void            SetColWidth(std::size_t n, X w);        ///< sets the width of column \n to \a w; not range-checked
+    void            SetNumCols(std::size_t n);              ///< sets the number of columns in the ListBox to \a n; if no column widths exist before this call, proportional widths are calulated and set, otherwise no column widths are set
+    void            SetSortCol(std::size_t n);              ///< sets the index of the column used to sort rows when sorting is enabled; not range-checked
 
     /** Sets the comparison function used to sort a given pair of Rows during
         row sorting.  Note that \a sort_cmp is assumed to produce an ascending
         order when used to sort; setting the LIST_SORTDESCENDING style can be
         used to produce a reverse sort. */
-    void           SetSortCmp(const boost::function<bool (const Row&, const Row&, std::size_t)>& sort_cmp);
+    void            SetSortCmp(const boost::function<bool (const Row&, const Row&, std::size_t)>& sort_cmp);
 
     /** Fixes the column widths; by default, an empty ListBox will take on the
         number of columns of its first added row. \note The number of columns
         and their widths may still be set via SetNumCols() and SetColWidth()
         after this function has been called. */
-    void           LockColWidths();
+    void            LockColWidths();
 
     /** Allows the number of columns to be determined by the first row added to
         an empty ListBox */
-    void           UnLockColWidths();
+    void            UnLockColWidths();
 
     /** Sets the alignment of column \a n to \a align; not range-checked */
-    void           SetColAlignment(std::size_t n, Alignment align);
+    void            SetColAlignment(std::size_t n, Alignment align);
 
     /** Sets the alignment of row \a it to \a align; not range-checked */
-    void           SetRowAlignment(iterator it, Alignment align);
+    void            SetRowAlignment(iterator it, Alignment align);
 
     /** Allows Rows with data type \a str to be dropped over this ListBox when
         drag-and-drop is enabled. \note Passing "" enables all drop types. */
-    void           AllowDropType(const std::string& str);
+    void            AllowDropType(const std::string& str);
 
     /** Disallows Rows with data type \a str to be dropped over this ListBox
         when drag-and-drop is enabled. \note If "" is still an allowed drop
         type, drops of type \a str will still be allowed, even after disallowed
         with a call to this function. */
-    void           DisallowDropType(const std::string& str);
+    void            DisallowDropType(const std::string& str);
 
     /** Set this to determine whether the list should autoscroll when the user
         is attempting to drop an item into a location that is not currently
         visible. */
-    void           AutoScrollDuringDragDrops(bool auto_scroll);
+    void            AutoScrollDuringDragDrops(bool auto_scroll);
 
     /** Sets the thickness of the area around the border of the client area that
         will provoke an auto-scroll, if AutoScrollDuringDragDrops() returns
         true. */
-    void           SetAutoScrollMargin(unsigned int margin);
+    void            SetAutoScrollMargin(unsigned int margin);
 
     /** Sets the number of milliseconds that elapse between row/column scrolls
         when auto-scrolling. */
-    void           SetAutoScrollInterval(unsigned int interval);
+    void            SetAutoScrollInterval(unsigned int interval);
     //@}
 
     /** \brief Sorts two Rows of a ListBox using operator<() on the
@@ -488,22 +505,25 @@ protected:
 
     virtual bool    EventFilter(Wnd* w, const WndEvent& event);
 
-    iterator        Insert(Row* row, iterator it, bool dropped, bool signal); ///< insertion sorts into list, or inserts into an unsorted list before \a it; returns insertion point
+    iterator        Insert(Row* row, iterator it, bool dropped, bool signal);                       ///< insertion sorts into list, or inserts into an unsorted list before \a it; returns insertion point
+    void            Insert(const std::vector<Row*>& rows, iterator it, bool dropped, bool signal);  ///< insertion sorts into list, or inserts into an unsorted list before \a it; returns insertion point
     Row*            Erase(iterator it, bool removing_duplicate, bool signal); ///< erases the row at index \a idx, handling it as a duplicate removal (such as for drag-and-drops within a single ListBox) if indicated
     void            BringCaretIntoView();           ///< makes sure caret is visible when scrolling occurs due to keystrokes etc.
     void            RecreateScrolls();              ///< recreates the vertical and horizontal scrolls as needed.
     void            ResetAutoScrollVars();          ///< resets all variables related to auto-scroll to their initial values
     void            Resort();                       ///< performs a full resort of all rows, using the current sort functor.
+    Row&            ColHeaders();                   ///< returns the row containing the headings for the columns, if any.  If undefined, the returned heading Row will have size() 0. non-const for derivers
     //@}
 
+    void            AdjustScrolls(bool adjust_for_resize);  ///< creates, destroys, or resizes scrolls to reflect size of data in listbox
+
 private:
     void            ConnectSignals();
-    void            ValidateStyle(); ///< reconciles inconsistencies in the style flags
-    void            AdjustScrolls(bool adjust_for_resize); ///< creates, destroys, or resizes scrolls to reflect size of data in listbox
-    void            VScrolled(int tab_low, int tab_high, int low, int high); ///< signals from the vertical scroll bar are caught here
-    void            HScrolled(int tab_low, int tab_high, int low, int high); ///< signals from the horizontal scroll bar are caught here
-    void            ClickAtRow(iterator it, Flags<ModKey> mod_keys); ///< handles to a mouse-click or spacebar-click on \a it, modified by \a keys
-    void            NormalizeRow(Row* row); ///< adjusts a Row so that it has the same number of cells as other rows, and that each cell has the correct width and alignment
+    void            ValidateStyle();                                        ///< reconciles inconsistencies in the style flags
+    void            VScrolled(int tab_low, int tab_high, int low, int high);///< signals from the vertical scroll bar are caught here
+    void            HScrolled(int tab_low, int tab_high, int low, int high);///< signals from the horizontal scroll bar are caught here
+    void            ClickAtRow(iterator it, Flags<ModKey> mod_keys);        ///< handles to a mouse-click or spacebar-click on \a it, modified by \a keys
+    void            NormalizeRow(Row* row);                                 ///< adjusts a Row so that it has the same number of cells as other rows, and that each cell has the correct width and alignment
     iterator        FirstRowShownWhenBottomIs(iterator bottom_row, Y client_height); ///< Returns the first row shown when the last row shown is \a bottom_row
     std::size_t     FirstColShownWhenRightIs(std::size_t right_col, X client_width); ///< Returns the index of the first column shown when the last column shown is \a right_col
 
@@ -511,6 +531,9 @@ private:
 
     Scroll*         m_vscroll;          ///< vertical scroll bar on right
     Scroll*         m_hscroll;          ///< horizontal scroll bar at bottom
+    unsigned int    m_vscroll_wheel_scroll_increment;
+    unsigned int    m_hscroll_wheel_scroll_increment;
+
     iterator        m_caret;            ///< the item currently selected, or the last item selected by the user 
     SelectionSet    m_selections;       ///< vector of indexes of selected items
     iterator        m_old_sel_row;      ///< used to make sure clicks end on the same row where they started
@@ -584,7 +607,7 @@ bool GG::ListBox::RowPtrIteratorLess<Cont>::LessThan(Iter lhs, Iter rhs, Iter en
     return lhs == end ?
         false :
         (rhs == end ?
-         true : (*lhs)->UpperLeft().y < (*rhs)->UpperLeft().y);
+         true : (*lhs)->Top() < (*rhs)->Top());
 }
 
 template <class RowType>
