@@ -50,8 +50,11 @@ class TextControl;
 struct GG_API MenuItem
 {
     /** \name Signal Types */ ///@{
-    typedef boost::signal<void (int)> SelectedIDSignalType; ///< invokes the appropriate functor to handle the menu selection, and passes the ID assigned to the item
-    typedef boost::signal<void ()>    SelectedSignalType;   ///< invokes the appropriate functor to handle the menu selection
+    /** invokes the appropriate functor to handle the menu selection, and
+        passes the ID assigned to the item */
+    typedef boost::signals2::signal<void (int)> SelectedIDSignalType;
+    /** invokes the appropriate functor to handle the menu selection */
+    typedef boost::signals2::signal<void ()>    SelectedSignalType;
     //@}
 
     /** \name Slot Types */ ///@{
@@ -61,7 +64,8 @@ struct GG_API MenuItem
 
     /** \name Structors */ ///@{
     MenuItem(); ///< default ctor
-    MenuItem(const std::string& str, int id, bool disable, bool check); ///< ctor
+    MenuItem(const std::string& str, int id, bool disable, bool check);
+    explicit MenuItem(bool separator);
 
     /** Ctor that allows direct attachment of this item's signal to a "slot"
         function or functor */
@@ -93,6 +97,7 @@ struct GG_API MenuItem
     int                   item_ID;    ///< ID number associated with this menu item
     bool                  disabled;   ///< set to true when this menu item is disabled
     bool                  checked;    ///< set to true when this menu item can be toggled, and is currently on
+    bool                  separator;  ///< set to true to render this menu item as a separator bar, rather than showing its text
     std::vector<MenuItem> next_level; ///< submenu off of this menu item; may be emtpy
 };
 
@@ -123,7 +128,8 @@ class GG_API MenuBar : public Control
 {
 public:
     /** \name Signal Types */ ///@{
-    typedef boost::signal<void (int)> BrowsedSignalType; ///< emits the ID of an item in the menu when the cursor moves over it
+    /** emits the ID of an item in the menu when the cursor moves over it */
+    typedef boost::signals2::signal<void (int)> BrowsedSignalType;
     //@}
 
     /** \name Structors */ ///@{
@@ -232,13 +238,15 @@ class GG_API PopupMenu : public Wnd
 {
 public:
     /** \name Signal Types */ ///@{
-    typedef boost::signal<void (int)> BrowsedSignalType; ///< emits the ID of an item in the menu when the cursor moves over it
+    /** emits the ID of an item in the menu when the cursor moves over it */
+    typedef boost::signals2::signal<void (int)> BrowsedSignalType;
     //@}
 
     /** \name Structors */ ///@{
     /** Ctor.  Parameter \a m should contain the desired menu in its
         next_level member. */
-    PopupMenu(X x, Y y, const boost::shared_ptr<Font>& font, const MenuItem& m, Clr text_color = CLR_WHITE, Clr color = CLR_BLACK, Clr interior = CLR_SHADOW);
+    PopupMenu(X x, Y y, const boost::shared_ptr<Font>& font, const MenuItem& m, Clr text_color = CLR_WHITE,
+              Clr border_color = CLR_BLACK, Clr interior_color = CLR_SHADOW, Clr hilite_color = CLR_GRAY);
     //@}
 
     /** \name Accessors */ ///@{
