@@ -37,7 +37,7 @@ namespace {
     {
         EditedEcho(const std::string& name) : m_name(name) {}
         void operator()(const std::string& str)
-            { std::cerr << "GG SIGNAL : " << m_name << "(str=" << str << ")\n"; }
+        { std::cerr << "GG SIGNAL : " << m_name << "(str=" << str << ")\n"; }
         std::string m_name;
     };
 
@@ -45,7 +45,7 @@ namespace {
     {
         InRange(CPSize value) : m_value(value) {}
         bool operator()(const std::pair<CPSize, CPSize>& p) const
-            { return p.first < m_value && m_value < p.second; }
+        { return p.first < m_value && m_value < p.second; }
         const CPSize m_value;
     };
 
@@ -61,23 +61,29 @@ const int Edit::PIXEL_MARGIN = 5;
 
 Edit::Edit() :
     TextControl(),
-    m_first_char_shown(0),
-    m_recently_edited(false),
+    m_cursor_pos(CP0, CP0),
     m_last_button_down_time(0),
-    m_in_double_click_mode(false)
+    m_in_double_click_mode(false),
+    m_double_click_cursor_pos(CP0, CP0),
+    m_first_char_shown(0),
+    m_int_color(CLR_ZERO),
+    m_hilite_color(CLR_WHITE),
+    m_sel_text_color(CLR_CYAN),
+    m_recently_edited(false)
 {}
 
 Edit::Edit(X x, Y y, X w, const std::string& str, const boost::shared_ptr<Font>& font, Clr color,
            Clr text_color/* = CLR_BLACK*/, Clr interior/* = CLR_ZERO*/, Flags<WndFlag> flags/* = INTERACTIVE*/) :
     TextControl(x, y, w, HeightFromFont(font, PIXEL_MARGIN), str, font, text_color, FORMAT_LEFT | FORMAT_IGNORETAGS, flags),
     m_cursor_pos(CP0, CP0),
+    m_last_button_down_time(0),
+    m_in_double_click_mode(false),
+    m_double_click_cursor_pos(CP0, CP0),
     m_first_char_shown(CP0),
     m_int_color(interior),
     m_hilite_color(CLR_SHADOW),
     m_sel_text_color(CLR_WHITE),
-    m_recently_edited(false),
-    m_last_button_down_time(0),
-    m_in_double_click_mode(false)
+    m_recently_edited(false)
 {
     SetColor(color);
 
@@ -99,6 +105,9 @@ Pt Edit::ClientLowerRight() const
 const std::pair<CPSize, CPSize>& Edit::CursorPosn() const
 { return m_cursor_pos; }
 
+std::string Edit::SelectedText() const
+{ return Text(m_cursor_pos.first, m_cursor_pos.second); }
+
 Clr Edit::InteriorColor() const
 { return m_int_color; }
 
@@ -197,6 +206,13 @@ void Edit::SelectAll()
     AdjustView();
 }
 
+void Edit::DeselectAll()
+{
+    m_cursor_pos.first = CP0;
+    m_cursor_pos.second = CP0;
+    AdjustView();
+}
+
 void Edit::SelectRange(CPSize from, CPSize to)
 {
     if (from < to) {
@@ -215,7 +231,7 @@ void Edit::SetText(const std::string& str)
     m_cursor_pos.second = m_cursor_pos.first; // eliminate any hiliting
 
     // make sure the change in text did not make the cursor or view position invalid
-    if (str.empty() || GetLineData().empty() || GetLineData()[0].char_data.size() < m_cursor_pos.first) {
+    if (Text().empty() || GetLineData().empty() || GetLineData()[0].char_data.size() < m_cursor_pos.first) {
         m_first_char_shown = CP0;
         m_cursor_pos = std::make_pair(CP0, CP0);
     }
@@ -223,6 +239,40 @@ void Edit::SetText(const std::string& str)
     m_recently_edited = true;
 }
 
+void Edit::AcceptPastedText(const std::string& text)
+{
+    if (!Interactive())
+        return;
+    if (!utf8::is_valid(text.begin(), text.end()))
+        return;
+
+    bool modified_text = false;
+
+    if (MultiSelected()) {
+        ClearSelected();
+        modified_text = true;
+        m_cursor_pos.second = m_cursor_pos.first;
+    }
+
+    if (!text.empty()) {
+        Insert(m_cursor_pos.first, text);
+        modified_text = true;
+    }
+
+    if (modified_text) {
+        // moves cursor to end of pasted text
+        CPSize text_span(utf8::distance(text.begin(), text.end()));
+        CPSize new_cursor_pos = std::max(CP0, std::min(Length() - 1, m_cursor_pos.second + text_span));
+        m_cursor_pos.second = new_cursor_pos;
+
+        // ensure nothing is selected after pasting
+        m_cursor_pos.first = m_cursor_pos.second;
+
+        // notify rest of GUI of change to text in this Edit
+        EditedSignal(Text());
+    }
+}
+
 bool Edit::MultiSelected() const
 { return m_cursor_pos.first != m_cursor_pos.second; }
 
@@ -255,7 +305,7 @@ X Edit::FirstCharOffset() const
 X Edit::ScreenPosOfChar(CPSize idx) const
 {
     X first_char_offset = FirstCharOffset();
-    return UpperLeft().x + PIXEL_MARGIN + ((idx ? GetLineData()[0].char_data[Value(idx - 1)].extent : X0) - first_char_offset);
+    return Left() + PIXEL_MARGIN + ((idx ? GetLineData()[0].char_data[Value(idx - 1)].extent : X0) - first_char_offset);
 }
 
 CPSize Edit::LastVisibleChar() const
@@ -280,22 +330,27 @@ std::pair<CPSize, CPSize> Edit::DoubleButtonDownCursorPos() const
 
 void Edit::LButtonDown(const Pt& pt, Flags<ModKey> mod_keys)
 {
-    if (!Disabled()) {
-        X click_xpos = ScreenToWindow(pt).x - PIXEL_MARGIN; // x coord of click within text space
-        CPSize idx = CharIndexOf(click_xpos);
-        m_cursor_pos.first = m_cursor_pos.second = idx;
-        std::pair<CPSize, CPSize> word_indices = GetDoubleButtonDownWordIndices(idx);
-        if (word_indices.first != word_indices.second)
-            m_cursor_pos = word_indices;
-    }
+    if (Disabled())
+        return;
+    //std::cout << "Edit::LButtonDown start" << std::endl;
+    X click_xpos = ScreenToWindow(pt).x - PIXEL_MARGIN; // x coord of click within text space
+    CPSize idx = CharIndexOf(click_xpos);
+    //std::cout << "Edit::LButtonDown got idx: " << idx << std::endl;
+    m_cursor_pos.first = m_cursor_pos.second = idx;
+    std::pair<CPSize, CPSize> word_indices = GetDoubleButtonDownWordIndices(idx);
+    //std::cout << "Edit::LButtonDown got word indices: " << word_indices.first << ", " << word_indices.second << std::endl;
+    if (word_indices.first != word_indices.second)
+        m_cursor_pos = word_indices;
 }
 
 void Edit::LDrag(const Pt& pt, const Pt& move, Flags<ModKey> mod_keys)
 {
-    if (!Disabled()) {
-        X xpos = ScreenToWindow(pt).x - PIXEL_MARGIN; // x coord for mouse position within text space
-        CPSize idx = CharIndexOf(xpos);
-        if (m_in_double_click_mode) {
+    if (Disabled())
+        return;
+
+    X xpos = ScreenToWindow(pt).x - PIXEL_MARGIN; // x coord for mouse position within text space
+    CPSize idx = CharIndexOf(xpos);
+    if (m_in_double_click_mode) {
             std::pair<CPSize, CPSize> word_indices =
                 GetDoubleButtonDownDragWordIndices(idx);
             if (word_indices.first == word_indices.second) {
@@ -323,7 +378,6 @@ void Edit::LDrag(const Pt& pt, const Pt& move, Flags<ModKey> mod_keys)
             if (xpos < 0 || Size().x - 2 * PIXEL_MARGIN < xpos) // if we're dragging past the currently visible text
                 AdjustView();
         }
-    }
 }
 
 void Edit::LButtonUp(const Pt& pt, Flags<ModKey> mod_keys)
@@ -427,7 +481,8 @@ void Edit::KeyPress(Key key, boost::uint32_t key_code_point, Flags<ModKey> mod_k
             std::string translated_code_point;
             GetTranslatedCodePoint(key, key_code_point, mod_keys, translated_code_point);
             if (!translated_code_point.empty() &&
-                !(mod_keys & (MOD_KEY_CTRL | MOD_KEY_ALT | MOD_KEY_META))) {
+                !(mod_keys & (MOD_KEY_CTRL | MOD_KEY_ALT | MOD_KEY_META)))
+            {
                 if (MultiSelected())
                     ClearSelected();
                 Insert(0, m_cursor_pos.first, translated_code_point);
@@ -462,25 +517,22 @@ std::pair<CPSize, CPSize> Edit::GetDoubleButtonDownWordIndices(CPSize char_index
     if (ticks - m_last_button_down_time <= GUI::GetGUI()->DoubleClickInterval())
         m_in_double_click_mode = true;
     m_last_button_down_time = ticks;
-    m_double_click_cursor_pos = std::pair<CPSize, CPSize>(CP0, CP0);
-    if (m_in_double_click_mode) {
-        std::set<std::pair<CPSize, CPSize> > words =
-            GUI::GetGUI()->FindWords(Text());
-        std::set<std::pair<CPSize, CPSize> >::const_iterator it =
-            std::find_if(words.begin(), words.end(), InRange(char_index));
-        if (it != words.end())
-            m_double_click_cursor_pos = *it;
-    }
+
+    m_double_click_cursor_pos = std::pair<CPSize, CPSize>(char_index, char_index);
+    if (m_in_double_click_mode)
+        m_double_click_cursor_pos = GetDoubleButtonDownDragWordIndices(char_index);
+
     return m_double_click_cursor_pos;
 }
 
 std::pair<CPSize, CPSize> Edit::GetDoubleButtonDownDragWordIndices(CPSize char_index)
 {
-    std::pair<CPSize, CPSize> retval(CP0, CP0);
-    std::set<std::pair<CPSize, CPSize> > words =
-        GUI::GetGUI()->FindWords(Text());
+    std::pair<CPSize, CPSize> retval(char_index, char_index);
+
+    std::set<std::pair<CPSize, CPSize> > words = GUI::GetGUI()->FindWords(Text());
     std::set<std::pair<CPSize, CPSize> >::const_iterator it =
         std::find_if(words.begin(), words.end(), InRange(char_index));
+
     if (it != words.end())
         retval = *it;
     return retval;
