@@ -84,13 +84,16 @@ namespace {
     public:
         RowSorter(const boost::function<bool (const ListBox::Row&, const ListBox::Row&, std::size_t)>& cmp,
                   std::size_t col, bool invert) :
-            m_cmp(cmp), m_sort_col(col), m_invert(invert) {}
+            m_cmp(cmp),
+            m_sort_col(col),
+            m_invert(invert)
+        {}
 
         bool operator()(const ListBox::Row* l, const ListBox::Row* r)
-            {
-                bool retval = m_cmp(*l, *r, m_sort_col);
-                return m_invert ? !retval : retval;
-            }
+        {
+            bool retval = m_cmp(*l, *r, m_sort_col);
+            return m_invert ? !retval : retval;
+        }
 
     private:
         boost::function<bool (const ListBox::Row&, const ListBox::Row&, std::size_t)> m_cmp;
@@ -120,9 +123,11 @@ namespace {
     {
         ScopedSet(ListBox::iterator*& var, ListBox::iterator* value) :
             m_var(var = value)
-            {}
+        {}
+
         ~ScopedSet()
-            { m_var = 0; }
+        { m_var = 0; }
+
         ListBox::iterator*& m_var;
     };
 
@@ -163,20 +168,20 @@ namespace {
     bool RegisterListBoxStyles()
     {
         FlagSpec<ListBoxStyle>& spec = FlagSpec<ListBoxStyle>::instance();
-        spec.insert(LIST_NONE, "LIST_NONE", true);
-        spec.insert(LIST_VCENTER, "LIST_VCENTER", true);
-        spec.insert(LIST_TOP, "LIST_TOP", true);
-        spec.insert(LIST_BOTTOM, "LIST_BOTTOM", true);
-        spec.insert(LIST_CENTER, "LIST_CENTER", true);
-        spec.insert(LIST_LEFT, "LIST_LEFT", true);
-        spec.insert(LIST_RIGHT, "LIST_RIGHT", true);
-        spec.insert(LIST_NOSORT, "LIST_NOSORT", true);
-        spec.insert(LIST_SORTDESCENDING, "LIST_SORTDESCENDING", true);
-        spec.insert(LIST_NOSEL, "LIST_NOSEL", true);
-        spec.insert(LIST_SINGLESEL, "LIST_SINGLESEL", true);
-        spec.insert(LIST_QUICKSEL, "LIST_QUICKSEL", true);
-        spec.insert(LIST_USERDELETE, "LIST_USERDELETE", true);
-        spec.insert(LIST_BROWSEUPDATES, "LIST_BROWSEUPDATES", true);
+        spec.insert(LIST_NONE,          "LIST_NONE",            true);
+        spec.insert(LIST_VCENTER,       "LIST_VCENTER",         true);
+        spec.insert(LIST_TOP,           "LIST_TOP",             true);
+        spec.insert(LIST_BOTTOM,        "LIST_BOTTOM",          true);
+        spec.insert(LIST_CENTER,        "LIST_CENTER",          true);
+        spec.insert(LIST_LEFT,          "LIST_LEFT",            true);
+        spec.insert(LIST_RIGHT,         "LIST_RIGHT",           true);
+        spec.insert(LIST_NOSORT,        "LIST_NOSORT",          true);
+        spec.insert(LIST_SORTDESCENDING,"LIST_SORTDESCENDING",  true);
+        spec.insert(LIST_NOSEL,         "LIST_NOSEL",           true);
+        spec.insert(LIST_SINGLESEL,     "LIST_SINGLESEL",       true);
+        spec.insert(LIST_QUICKSEL,      "LIST_QUICKSEL",        true);
+        spec.insert(LIST_USERDELETE,    "LIST_USERDELETE",      true);
+        spec.insert(LIST_BROWSEUPDATES, "LIST_BROWSEUPDATES",   true);
         return true;
     }
     bool dummy = RegisterListBoxStyles();
@@ -262,9 +267,8 @@ void ListBox::Row::push_back(Control* c)
     m_cells.push_back(c);
     m_col_widths.push_back(X(5));
     m_col_alignments.push_back(ALIGN_NONE);
-    if (1 < m_cells.size()) {
+    if (1 < m_cells.size())
         m_col_widths.back() = m_col_widths[m_cells.size() - 1];
-    }
     AdjustLayout();
 }
 
@@ -307,7 +311,10 @@ void ListBox::Row::resize(std::size_t n)
 
 void ListBox::Row::SetCell(std::size_t n, Control* c)
 {
-    assert(c != m_cells[n]);
+    //assert(c != m_cells[n]);  // replaced with following test and return to avoid crashes
+    if (c == m_cells[n])
+        return;
+
     delete m_cells[n];
     m_cells[n] = c;
     AdjustLayout();
@@ -398,9 +405,8 @@ void ListBox::Row::AdjustLayout(bool adjust_for_push_back/* = false*/)
     Layout* layout = GetLayout();
     for (std::size_t i = 0; i < m_cells.size(); ++i) {
         layout->SetMinimumColumnWidth(i, m_col_widths[i]);
-        if (m_cells[i]) {
+        if (m_cells[i])
             layout->Add(m_cells[i], 0, i, m_row_alignment | m_col_alignments[i]);
-        }
     }
 }
 
@@ -415,6 +421,8 @@ ListBox::ListBox() :
     m_rows(),
     m_vscroll(0),
     m_hscroll(0),
+    m_vscroll_wheel_scroll_increment(0),
+    m_hscroll_wheel_scroll_increment(0),
     m_caret(m_rows.end()),
     m_selections(RowPtrIteratorLess<std::list<Row*> >(&m_rows)),
     m_old_sel_row(m_rows.end()),
@@ -451,6 +459,8 @@ ListBox::ListBox(X x, Y y, X w, Y h, Clr color, Clr interior/* = CLR_ZERO*/,
     m_rows(),
     m_vscroll(0),
     m_hscroll(0),
+    m_vscroll_wheel_scroll_increment(0),
+    m_hscroll_wheel_scroll_increment(0),
     m_caret(m_rows.end()),
     m_selections(RowPtrIteratorLess<std::list<Row*> >(&m_rows)),
     m_old_sel_row(m_rows.end()),
@@ -488,18 +498,18 @@ ListBox::ListBox(X x, Y y, X w, Y h, Clr color, Clr interior/* = CLR_ZERO*/,
     InstallEventFilter(this);
 
     if (INSTRUMENT_ALL_SIGNALS) {
-        Connect(ClearedSignal, ListSignalEcho(*this, "ListBox::ClearedSignal"));
-        Connect(BeforeInsertSignal, ListSignalEcho(*this, "ListBox::BeforeInsertSignal"));
-        Connect(AfterInsertSignal, ListSignalEcho(*this, "ListBox::AfterinsertSignal"));
-        Connect(SelChangedSignal, ListSignalEcho(*this, "ListBox::SelChangedSignal"));
-        Connect(DroppedSignal, ListSignalEcho(*this, "ListBox::DroppedSignal"));
-        Connect(DropAcceptableSignal, ListSignalEcho(*this, "ListBox::DropAcceptableSignal"));
-        Connect(LeftClickedSignal, ListSignalEcho(*this, "ListBox::LeftClickedSignal"));
-        Connect(RightClickedSignal, ListSignalEcho(*this, "ListBox::RightClickedSignal"));
-        Connect(DoubleClickedSignal, ListSignalEcho(*this, "ListBox::DoubleClickedSignal"));
-        Connect(BeforeEraseSignal, ListSignalEcho(*this, "ListBox::BeforeEraseSignal"));
-        Connect(AfterEraseSignal, ListSignalEcho(*this, "ListBox::AfterEraseSignal"));
-        Connect(BrowsedSignal, ListSignalEcho(*this, "ListBox::BrowsedSignal"));
+        Connect(ClearedSignal,          ListSignalEcho(*this, "ListBox::ClearedSignal"));
+        Connect(BeforeInsertSignal,     ListSignalEcho(*this, "ListBox::BeforeInsertSignal"));
+        Connect(AfterInsertSignal,      ListSignalEcho(*this, "ListBox::AfterinsertSignal"));
+        Connect(SelChangedSignal,       ListSignalEcho(*this, "ListBox::SelChangedSignal"));
+        Connect(DroppedSignal,          ListSignalEcho(*this, "ListBox::DroppedSignal"));
+        Connect(DropAcceptableSignal,   ListSignalEcho(*this, "ListBox::DropAcceptableSignal"));
+        Connect(LeftClickedSignal,      ListSignalEcho(*this, "ListBox::LeftClickedSignal"));
+        Connect(RightClickedSignal,     ListSignalEcho(*this, "ListBox::RightClickedSignal"));
+        Connect(DoubleClickedSignal,    ListSignalEcho(*this, "ListBox::DoubleClickedSignal"));
+        Connect(BeforeEraseSignal,      ListSignalEcho(*this, "ListBox::BeforeEraseSignal"));
+        Connect(AfterEraseSignal,       ListSignalEcho(*this, "ListBox::AfterEraseSignal"));
+        Connect(BrowsedSignal,          ListSignalEcho(*this, "ListBox::BrowsedSignal"));
     }
 }
 
@@ -627,7 +637,7 @@ bool ListBox::KeepColWidths() const
 { return m_keep_col_widths; }
 
 std::size_t ListBox::SortCol() const
-{ return (m_sort_col < m_col_widths.size() ? m_sort_col : 0); }
+{ return m_sort_col; }
 
 X ListBox::ColWidth(std::size_t n) const
 { return m_col_widths[n]; }
@@ -654,15 +664,24 @@ void ListBox::StartingChildDragDrop(const Wnd* wnd, const Pt& offset)
 {
     if (m_selections.empty())
         return;
+    if (m_rows.empty())
+        return;
+
+    iterator wnd_it = std::find(m_rows.begin(), m_rows.end(), wnd);
+    //assert(wnd_it != m_rows.end());   // replaced with following test and return to avoid crashes
+    if (wnd_it == m_rows.end())
+        return;
+
+    SelectionSet::iterator wnd_sel_it = m_selections.find(wnd_it);
+    //assert(wnd_sel_it != m_selections.end()); // replaced with following test and return to avoid crashes
+    if (wnd_sel_it == m_selections.end())
+        return;
 
     Y vertical_offset = offset.y;
-    iterator wnd_it = std::find(m_rows.begin(), m_rows.end(), wnd);
-    assert(wnd_it != m_rows.end());
-    SelectionSet::iterator wnd_sel_it = m_selections.find(wnd_it);
-    assert(wnd_sel_it != m_selections.end());
+
     for (SelectionSet::iterator sel_it = m_selections.begin(); sel_it != wnd_sel_it; ++sel_it) {
         vertical_offset += (**sel_it)->Height();
-    }    
+    }
     for (SelectionSet::iterator sel_it = m_selections.begin(); sel_it != m_selections.end(); ++sel_it) {
         Wnd* row_wnd = **sel_it;
         if (row_wnd != wnd) {
@@ -686,13 +705,18 @@ void ListBox::AcceptDrops(const std::vector<Wnd*>& wnds, const Pt& pt)
 
 void ListBox::ChildrenDraggedAway(const std::vector<Wnd*>& wnds, const Wnd* destination)
 {
-    if (!MatchesOrContains(this, destination)) {
-        for (std::vector<Wnd*>::const_iterator it = wnds.begin(); it != wnds.end(); ++it) {
-            Row* row = boost::polymorphic_downcast<Row*>(*it);
-            iterator row_it = std::find(m_rows.begin(), m_rows.end(), row);
-            assert(row_it != m_rows.end());
-            Erase(row_it, false, true);
-        }
+    if (MatchesOrContains(this, destination))
+        return;
+
+    // remove dragged-away row from this ListBox
+    for (std::vector<Wnd*>::const_iterator it = wnds.begin(); it != wnds.end(); ++it) {
+        Row* row = boost::polymorphic_downcast<Row*>(*it);
+        iterator row_it = std::find(m_rows.begin(), m_rows.end(), row);
+        //assert(row_it != m_rows.end());   // replaced with following test and continue to avoid crashes
+        if (row_it == m_rows.end())
+            continue;
+
+        Erase(row_it, false, true);
     }
 }
 
@@ -761,8 +785,8 @@ void ListBox::Render()
     // HACK! This gets around the issue of how to render headers and scrolls,
     // which do not fall within the client area.
     if (!m_header_row->empty()) {
-        Rect header_area(Pt(ul.x + static_cast<int>(BORDER_THICK), m_header_row->UpperLeft().y),
-                         Pt(lr.x - static_cast<int>(BORDER_THICK), m_header_row->LowerRight().y));
+        Rect header_area(Pt(ul.x + static_cast<int>(BORDER_THICK), m_header_row->Top()),
+                         Pt(lr.x - static_cast<int>(BORDER_THICK), m_header_row->Bottom()));
         BeginScissorClipping(header_area.ul, header_area.lr);
         GUI::GetGUI()->RenderWindow(m_header_row);
         EndScissorClipping();
@@ -814,14 +838,20 @@ void ListBox::SetColor(Clr c)
         m_hscroll->SetColor(c);
 }
 
-ListBox::iterator ListBox::Insert(Row* row, iterator it)
-{ return Insert(row, it, false, true); }
+ListBox::iterator ListBox::Insert(Row* row, iterator it, bool signal/* = true*/)
+{ return Insert(row, it, false, signal); }
 
-ListBox::iterator ListBox::Insert(Row* row)
-{ return Insert(row, m_rows.end(), false, true); }
+ListBox::iterator ListBox::Insert(Row* row, bool signal/* = true*/)
+{ return Insert(row, m_rows.end(), false, signal); }
 
-ListBox::Row* ListBox::Erase(iterator it)
-{ return Erase(it, false, false); }
+void ListBox::Insert(const std::vector<Row*>& rows, iterator it, bool signal/* = true*/)
+{ Insert(rows, it, false, signal); }
+
+void ListBox::Insert(const std::vector<Row*>& rows, bool signal/* = true*/)
+{ Insert(rows, m_rows.end(), false, signal); }
+
+ListBox::Row* ListBox::Erase(iterator it, bool signal/* = false*/)
+{ return Erase(it, false, signal); }
 
 void ListBox::Clear()
 {
@@ -851,36 +881,71 @@ void ListBox::Clear()
     ClearedSignal();
 }
 
-void ListBox::SelectRow(iterator it)
+void ListBox::SelectRow(iterator it, bool signal/* = false*/)
 {
-    if (it != m_rows.end() && !(m_style & LIST_NOSEL) && m_selections.find(it) == m_selections.end()) {
-        if (m_style & LIST_SINGLESEL)
-            m_selections.clear();
-        m_selections.insert(it);
-    }
+    if (m_style & LIST_NOSEL)
+        return;
+    if (it == m_rows.end())
+        return;
+    if (m_selections.find(it) != m_selections.end())
+        return;
+
+    SelectionSet previous_selections = m_selections;
+
+    if (m_style & LIST_SINGLESEL)
+        m_selections.clear();
+
+    m_selections.insert(it);
+
+    if (signal && previous_selections != m_selections)
+        SelChangedSignal(m_selections);
 }
 
-void ListBox::DeselectRow(iterator it)
+void ListBox::DeselectRow(iterator it, bool signal/* = false*/)
 {
+    SelectionSet previous_selections = m_selections;
+
     if (m_selections.find(it) != m_selections.end())
         m_selections.erase(it);
+
+    if (signal && previous_selections != m_selections)
+        SelChangedSignal(m_selections);
 }
 
-void ListBox::SelectAll()
+void ListBox::SelectAll(bool signal/* = false*/)
 {
-    if (m_selections.size() < m_rows.size()) {
-        for (iterator it = m_rows.begin(); it != m_rows.end(); ++it) {
-            m_selections.insert(it);
+    if (m_style & LIST_NOSEL)
+        return;
+
+    SelectionSet previous_selections = m_selections;
+
+    if (m_style & LIST_SINGLESEL) {
+        if (m_selections.empty() && !m_rows.empty()) {
+            m_selections.insert(m_rows.begin());
+        }
+    } else {
+        if (m_selections.size() != m_rows.size()) {
+            m_selections.clear();
+            for (iterator it = m_rows.begin(); it != m_rows.end(); ++it)
+                m_selections.insert(it);
         }
     }
+
+    if (signal && previous_selections != m_selections)
+        SelChangedSignal(m_selections);
 }
 
-void ListBox::DeselectAll()
+void ListBox::DeselectAll(bool signal/* = false*/)
 {
+    SelectionSet previous_selections = m_selections;
+
     if (!m_selections.empty()) {
         m_selections.clear();
         m_caret = m_rows.end();
     }
+
+    if (signal && previous_selections != m_selections)
+        SelChangedSignal(m_selections);
 }
 
 ListBox::iterator ListBox::begin()
@@ -901,8 +966,18 @@ ListBox::Row& ListBox::GetRow(std::size_t n)
     return **boost::next(m_rows.begin(), n);
 }
 
-void ListBox::SetSelections(const SelectionSet& s)
-{ m_selections = s; }
+void ListBox::SetSelections(const SelectionSet& s, bool signal/* = false*/)
+{
+    if (m_style & LIST_NOSEL)
+        return;
+
+    SelectionSet previous_selections = m_selections;
+
+    m_selections = s;
+
+    if (signal && previous_selections != m_selections)
+        SelChangedSignal(m_selections);
+}
 
 void ListBox::SetCaret(iterator it)
 { m_caret = it; }
@@ -944,6 +1019,18 @@ void ListBox::SetFirstRowShown(iterator it)
     }
 }
 
+void ListBox::SetVScrollWheelIncrement(unsigned int increment)
+{
+    m_vscroll_wheel_scroll_increment = increment;
+    AdjustScrolls(false);
+}
+
+void ListBox::SetHScrollWheelIncrement(unsigned int increment)
+{
+    m_hscroll_wheel_scroll_increment = increment;
+    AdjustScrolls(false);
+}
+
 void ListBox::SetInteriorColor(Clr c)
 { m_int_color = c; }
 
@@ -1143,6 +1230,9 @@ void ListBox::KeyPress(Key key, boost::uint32_t key_code_point, Flags<ModKey> mo
                     }
                     m_selections.clear();
                 }
+            } else {
+                // Pass delete on if we ignored it
+                Control::KeyPress(key, key_code_point, mod_keys);
             }
             break;
 
@@ -1215,27 +1305,15 @@ void ListBox::KeyPress(Key key, boost::uint32_t key_code_point, Flags<ModKey> mo
         }
     } else {
         Control::KeyPress(key, key_code_point, mod_keys);
-   }
+    }
 }
 
 void ListBox::MouseWheel(const Pt& pt, int move, Flags<ModKey> mod_keys)
 {
-    if (m_vscroll) {
-        for (int i = 0; i < move; ++i) {
-            if (m_first_row_shown != m_rows.end() && m_first_row_shown != m_rows.begin()) {
-                m_vscroll->ScrollTo(m_vscroll->PosnRange().first -
-                                    Value((*boost::prior(m_first_row_shown))->Height()));
-                SignalScroll(*m_vscroll, true);
-            }
-        }
-        for (int i = 0; i < -move; ++i) {
-            if (m_first_row_shown != m_rows.end() && m_first_row_shown != --m_rows.end()) {
-                m_vscroll->ScrollTo(m_vscroll->PosnRange().first +
-                                    Value((*m_first_row_shown)->Height()));
-                SignalScroll(*m_vscroll, true);
-            }
-        }
-    }
+    if (Disabled() || !m_vscroll)
+        return;
+    m_vscroll->ScrollLineIncr(-move);
+    SignalScroll(*m_vscroll, true);
 }
 
 void ListBox::DragDropEnter(const Pt& pt, const std::map<Wnd*, Pt>& drag_drop_wnds, Flags<ModKey> mod_keys)
@@ -1292,7 +1370,8 @@ void ListBox::TimerFiring(unsigned int ticks, Timer* timer)
                 iterator last_visible_row = LastVisibleRow();
                 if (last_visible_row != m_rows.end() &&
                     (last_visible_row != --m_rows.end() ||
-                     ClientLowerRight().y < (*last_visible_row)->LowerRight().y)) {
+                     ClientLowerRight().y < (*last_visible_row)->Bottom()))
+                {
                     m_vscroll->ScrollTo(m_vscroll->PosnRange().first +
                                         Value((*m_first_row_shown)->Height()));
                     SignalScroll(*m_vscroll, true);
@@ -1308,7 +1387,7 @@ void ListBox::TimerFiring(unsigned int ticks, Timer* timer)
             if (m_auto_scrolling_right) {
                 std::size_t last_visible_col = LastVisibleCol();
                 if (last_visible_col < m_col_widths.size() - 1 ||
-                    ClientLowerRight().x < m_rows.front()->LowerRight().x) {
+                    ClientLowerRight().x < m_rows.front()->Right()) {
                     m_hscroll->ScrollTo(m_hscroll->PosnRange().first +
                                         Value(m_col_widths[m_first_col_shown]));
                     SignalScroll(*m_hscroll, true);
@@ -1322,120 +1401,122 @@ bool ListBox::EventFilter(Wnd* w, const WndEvent& event)
 {
     assert(w == this || dynamic_cast<Row*>(w));
 
-    if (!Disabled()) {
-        Pt pt = event.Point();
-        Flags<ModKey> mod_keys = event.ModKeys();
+    if (Disabled())
+        return true;
 
-        switch (event.Type()) {
-        case WndEvent::LButtonDown: {
-            m_old_sel_row = RowUnderPt(pt);
-            if (!InClient(pt)) {
-                m_old_sel_row = m_rows.end();
-            } else if (m_old_sel_row != m_rows.end()) {
-                m_old_sel_row_selected = m_selections.find(m_old_sel_row) != m_selections.end();
-                if (!(m_style & LIST_NOSEL) && !m_old_sel_row_selected)
-                    ClickAtRow(m_old_sel_row, mod_keys);
-            }
-            break;
-        }
+    Pt pt = event.Point();
+    Flags<ModKey> mod_keys = event.ModKeys();
 
-        case WndEvent::LButtonUp: {
+    switch (event.Type()) {
+    case WndEvent::LButtonDown: {
+        m_old_sel_row = RowUnderPt(pt);
+        if (!InClient(pt)) {
             m_old_sel_row = m_rows.end();
-            break;
-        }
-
-        case WndEvent::LClick: {
-            if (m_old_sel_row != m_rows.end() && InClient(pt)) {
-                iterator sel_row = RowUnderPt(pt);
-                if (sel_row == m_old_sel_row) {
-                    if (m_style & LIST_NOSEL)
-                        m_caret = sel_row;
-                    else
-                        ClickAtRow(sel_row, mod_keys);
-                    m_lclick_row = sel_row;
-                    LeftClickedSignal(sel_row, pt);
-                }
-            }
-            break;
-        }
-
-        case WndEvent::LDoubleClick: {
-            iterator row = RowUnderPt(pt);
-            if (row != m_rows.end() && row == m_lclick_row && InClient(pt)) {
-                DoubleClickedSignal(row);
-                m_old_sel_row = m_rows.end();
-            } else {
-                LClick(pt, mod_keys);
-            }
-            break;
-        }
-
-        case WndEvent::RButtonDown: {
-            iterator row = RowUnderPt(pt);
-            if (row != m_rows.end() && InClient(pt))
-                m_old_rdown_row = row;
-            else
-                m_old_rdown_row = m_rows.end();
-            break;
-        }
-
-        case WndEvent::RClick: {
-            iterator row = RowUnderPt(pt);
-            if (row != m_rows.end() && row == m_old_rdown_row && InClient(pt)) {
-                m_rclick_row = row;
-                RightClickedSignal(row, pt);
-            }
-            m_old_rdown_row = m_rows.end();
-            break;
-        }
-
-        case WndEvent::MouseEnter: {
-            if (m_style & LIST_BROWSEUPDATES) {
-                iterator sel_row = RowUnderPt(pt);
-                if (m_last_row_browsed != sel_row)
-                    BrowsedSignal(m_last_row_browsed = sel_row);
-            }
-            break;
-        }
-
-        case WndEvent::MouseHere:
-            break;
-
-        case WndEvent::MouseLeave: {
-            if (m_style & LIST_BROWSEUPDATES) {
-                if (m_last_row_browsed != m_rows.end())
-                    BrowsedSignal(m_last_row_browsed = m_rows.end());
-            }
-            break;
-        }
-
-        case WndEvent::GainingFocus: {
-            if (w == this)
-                return false;
-            GUI::GetGUI()->SetFocusWnd(this);
-            break;
-        }
-
-        case WndEvent::MouseWheel:
-            return false;
-
-        case WndEvent::DragDropEnter:
-        case WndEvent::DragDropHere:
-        case WndEvent::DragDropLeave:
-            if (w == this)
-                return false;
-            HandleEvent(event);
-            break;
-
-        case WndEvent::KeyPress:
-        case WndEvent::KeyRelease:
-        case WndEvent::TimerFiring:
-            return false;
-
-        default:
-            break;
+        } else if (m_old_sel_row != m_rows.end()) {
+            m_old_sel_row_selected = m_selections.find(m_old_sel_row) != m_selections.end();
+            if (!(m_style & LIST_NOSEL) && !m_old_sel_row_selected)
+                ClickAtRow(m_old_sel_row, mod_keys);
         }
+        break;
     }
+
+    case WndEvent::LButtonUp: {
+        m_old_sel_row = m_rows.end();
+        break;
+    }
+
+    case WndEvent::LClick: {
+        if (m_old_sel_row != m_rows.end() && InClient(pt)) {
+            iterator sel_row = RowUnderPt(pt);
+            if (sel_row == m_old_sel_row) {
+                if (m_style & LIST_NOSEL)
+                    m_caret = sel_row;
+                else
+                    ClickAtRow(sel_row, mod_keys);
+                m_lclick_row = sel_row;
+                LeftClickedSignal(sel_row, pt);
+            }
+        }
+        break;
+    }
+
+    case WndEvent::LDoubleClick: {
+        iterator row = RowUnderPt(pt);
+        if (row != m_rows.end() && row == m_lclick_row && InClient(pt)) {
+            DoubleClickedSignal(row);
+            m_old_sel_row = m_rows.end();
+        } else {
+            LClick(pt, mod_keys);
+        }
+        break;
+    }
+
+    case WndEvent::RButtonDown: {
+        iterator row = RowUnderPt(pt);
+        if (row != m_rows.end() && InClient(pt))
+            m_old_rdown_row = row;
+        else
+            m_old_rdown_row = m_rows.end();
+        break;
+    }
+
+    case WndEvent::RClick: {
+        iterator row = RowUnderPt(pt);
+        if (row != m_rows.end() && row == m_old_rdown_row && InClient(pt)) {
+            m_rclick_row = row;
+            RightClickedSignal(row, pt);
+        }
+        m_old_rdown_row = m_rows.end();
+        break;
+    }
+
+    case WndEvent::MouseEnter: {
+        if (m_style & LIST_BROWSEUPDATES) {
+            iterator sel_row = RowUnderPt(pt);
+            if (m_last_row_browsed != sel_row)
+                BrowsedSignal(m_last_row_browsed = sel_row);
+        }
+        break;
+    }
+
+    case WndEvent::MouseHere:
+        break;
+
+    case WndEvent::MouseLeave: {
+        if (m_style & LIST_BROWSEUPDATES) {
+            if (m_last_row_browsed != m_rows.end())
+                BrowsedSignal(m_last_row_browsed = m_rows.end());
+        }
+        break;
+    }
+
+    case WndEvent::GainingFocus: {
+        if (w == this)
+            return false;
+        GUI::GetGUI()->SetFocusWnd(this);
+        break;
+    }
+
+    case WndEvent::MouseWheel:
+        return false;
+
+    case WndEvent::DragDropEnter:
+    case WndEvent::DragDropHere:
+    case WndEvent::DragDropLeave:
+        if (w == this)
+            return false;
+        HandleEvent(event);
+        break;
+
+    case WndEvent::KeyPress:
+    case WndEvent::KeyRelease:
+    case WndEvent::TimerFiring:
+        return false;
+
+    default:
+        break;
+    }
+
     return true;
 }
 
@@ -1532,6 +1613,96 @@ ListBox::iterator ListBox::Insert(Row* row, iterator it, bool dropped, bool sign
     return retval;
 }
 
+void ListBox::Insert(const std::vector<Row*>& rows, iterator it, bool dropped, bool signal)
+{
+    if (rows.empty())
+        return;
+
+    if (signal || dropped) {
+        // need to signal or handle dropping for each row, so add individually
+        for (std::vector<Row*>::const_iterator row_it = rows.begin();
+             row_it != rows.end(); ++row_it)
+        { Insert(*row_it, it, dropped, signal); }
+        return;
+    }
+
+    // don't need to signal or handle dropping issues, so can add rows at once
+    // without externally handling after each
+
+
+    // The first row inserted into an empty list box defines the number of
+    // columns, and initializes the column widths and alignments.
+    if (m_col_widths.empty() || !m_keep_col_widths) {
+        Row* row = *rows.begin();
+
+        const GG::X WIDTH = ClientSize().x - SCROLL_WIDTH;
+
+        m_col_widths.resize(row->size());
+        m_col_alignments.resize(row->size());
+        GG::X total = GG::X0;
+        for (std::size_t i = 0; i < row->size(); ++i) {
+            // use the column width from the Row
+            total += row->ColWidth(i);
+
+            // use the column alignment from the Row, if it has been set;
+            // otherwise, use the one dictated by the ListBoxStyle flags
+            Alignment a = row->ColAlignment(i);
+            if (a == ALIGN_NONE)
+                a = AlignmentFromStyle(m_style);
+            m_col_alignments[i] = a;
+        }
+
+        const GG::X_d SCALE_FACTOR = 1.0 * WIDTH / total;
+
+        total = GG::X0;
+        for (std::size_t i = 0; i < row->size(); ++i) {
+            total += (m_col_widths[i] = row->ColWidth(i) * SCALE_FACTOR);
+        }
+        m_col_widths.back() += total - WIDTH;
+
+        if (!m_header_row->empty())
+            NormalizeRow(m_header_row);
+    }
+
+    // housekeeping of rows...
+    for (std::vector<Row*>::const_iterator row_it = rows.begin();
+         row_it != rows.end(); ++row_it)
+    {
+        Row* row = *row_it;
+        row->InstallEventFilter(this);
+        NormalizeRow(row);
+    }
+
+    // add row at requested location (or default end position)
+    m_rows.insert(it, rows.begin(), rows.end());
+
+    // more housekeeping of rows...
+    for (std::vector<Row*>::const_iterator row_it = rows.begin();
+         row_it != rows.end(); ++row_it)
+    {
+        Row* row = *row_it;
+        AttachChild(row);
+    }
+
+    // sort?
+    if (!(m_style & LIST_NOSORT)) {
+        Resort();
+    } else {
+        // reposition rows to account for insertion
+        Y y(0);
+        for (it = m_rows.begin(); it != m_rows.end(); ++it) {
+            Row* row = *it;
+            row->MoveTo(Pt(X0, y));
+            y += row->Height();
+        }
+    }
+
+    if (m_first_row_shown == m_rows.end())
+        m_first_row_shown = m_rows.begin();
+
+    AdjustScrolls(false);
+}
+
 ListBox::Row* ListBox::Erase(iterator it, bool removing_duplicate, bool signal)
 {
     if (it != m_rows.end()) {
@@ -1623,10 +1794,10 @@ void ListBox::Resort()
         selections.insert(**it);
     }
     m_selections.clear();
-    Row* old_sel_row = SafeDeref(m_old_sel_row, m_rows.end());
-    Row* old_rdown_row = SafeDeref(m_old_rdown_row, m_rows.end());
-    Row* lclick_row = SafeDeref(m_lclick_row, m_rows.end());
-    Row* rclick_row = SafeDeref(m_rclick_row, m_rows.end());
+    Row* old_sel_row =      SafeDeref(m_old_sel_row, m_rows.end());
+    Row* old_rdown_row =    SafeDeref(m_old_rdown_row, m_rows.end());
+    Row* lclick_row =       SafeDeref(m_lclick_row, m_rows.end());
+    Row* rclick_row =       SafeDeref(m_rclick_row, m_rows.end());
     Row* last_row_browsed = SafeDeref(m_last_row_browsed, m_rows.end());
 
     std::vector<Row*> rows_vec(m_rows.size());
@@ -1664,6 +1835,9 @@ void ListBox::Resort()
     m_first_row_shown = m_rows.empty() ? m_rows.end() : m_rows.begin();
 }
 
+ListBox::Row& ListBox::ColHeaders()
+{ return *m_header_row; }
+
 void ListBox::ConnectSignals()
 {
     if (m_vscroll)
@@ -1702,12 +1876,15 @@ void ListBox::AdjustScrolls(bool adjust_for_resize)
 {
     // this client area calculation disregards the thickness of scrolls
     Pt cl_sz = (LowerRight() - Pt(X(BORDER_THICK), Y(BORDER_THICK))) -
-        (UpperLeft() + Pt(X(BORDER_THICK), static_cast<int>(BORDER_THICK) + (m_header_row->empty() ? Y0 : m_header_row->Height())));
+        (UpperLeft() + Pt(X(BORDER_THICK), static_cast<int>(BORDER_THICK)
+            + (m_header_row->empty()
+               ? Y0
+               : m_header_row->Height())));
 
     X total_x_extent = std::accumulate(m_col_widths.begin(), m_col_widths.end(), X0);
     Y total_y_extent(0);
     if (!m_rows.empty())
-        total_y_extent = m_rows.back()->LowerRight().y - m_rows.front()->UpperLeft().y;
+        total_y_extent = m_rows.back()->Bottom() - m_rows.front()->Top();
 
     bool vertical_needed =
         m_first_row_shown != m_rows.begin() ||
@@ -1736,17 +1913,27 @@ void ListBox::AdjustScrolls(bool adjust_for_resize)
         if (!vertical_needed) { // remove scroll
             DeleteChild(m_vscroll);
             m_vscroll = 0;
+
         } else { // ensure vertical scroll has the right logical and physical dimensions
             X scroll_x = cl_sz.x - SCROLL_WIDTH;
             Y scroll_y(0);
             if (adjust_for_resize) {
                 m_vscroll->SizeMove(Pt(scroll_x, scroll_y),
-                                    Pt(scroll_x + SCROLL_WIDTH, scroll_y + cl_sz.y - (horizontal_needed ? SCROLL_WIDTH : 0)));
+                                    Pt(scroll_x + SCROLL_WIDTH,
+                                       scroll_y + cl_sz.y - (horizontal_needed ? SCROLL_WIDTH : 0)));
             }
-            int line_size = Value(cl_sz.y / 8);
-            int page_size = Value(cl_sz.y - (horizontal_needed ? SCROLL_WIDTH : 0));
+
+            unsigned int line_size = m_vscroll_wheel_scroll_increment;
+            if (line_size == 0 && !this->Empty()) {
+                const Row* row = *begin();
+                line_size = Value(row->Height());
+            }
+
+            unsigned int page_size = std::abs(Value(cl_sz.y - (horizontal_needed ? SCROLL_WIDTH : 0)));
+
             m_vscroll->SizeScroll(0, Value(total_y_extent - 1),
                                   line_size, std::max(line_size, page_size));
+
             MoveChildUp(m_vscroll);
         }
     } else if (!m_vscroll && vertical_needed) { // if scroll doesn't exist but is needed
@@ -1755,8 +1942,15 @@ void ListBox::AdjustScrolls(bool adjust_for_resize)
                 cl_sz.x - SCROLL_WIDTH, Y0,
                 X(SCROLL_WIDTH), cl_sz.y - (horizontal_needed ? SCROLL_WIDTH : 0),
                 m_color, CLR_SHADOW);
-        int line_size = Value(cl_sz.y / 8);
-        int page_size = Value(cl_sz.y - (horizontal_needed ? SCROLL_WIDTH : 0));
+
+        unsigned int line_size = m_vscroll_wheel_scroll_increment;
+        if (line_size == 0 && !this->Empty()) {
+            const Row* row = *begin();
+            line_size = Value(row->Height());
+        }
+
+        unsigned int page_size = std::abs(Value(cl_sz.y - (horizontal_needed ? SCROLL_WIDTH : 0)));
+
         m_vscroll->SizeScroll(0, Value(total_y_extent - 1),
                               line_size, std::max(line_size, page_size));
         AttachChild(m_vscroll);
@@ -1772,10 +1966,18 @@ void ListBox::AdjustScrolls(bool adjust_for_resize)
             Y scroll_y = cl_sz.y - SCROLL_WIDTH;
             if (adjust_for_resize) {
                 m_hscroll->SizeMove(Pt(scroll_x, scroll_y),
-                                    Pt(scroll_x + cl_sz.x - (vertical_needed ? SCROLL_WIDTH : 0), scroll_y + SCROLL_WIDTH));
+                                    Pt(scroll_x + cl_sz.x - (vertical_needed ? SCROLL_WIDTH : 0),
+                                       scroll_y + SCROLL_WIDTH));
             }
-            int line_size = Value(cl_sz.x / 8);
-            int page_size = Value(cl_sz.x - (vertical_needed ? SCROLL_WIDTH : 0));
+
+            unsigned int line_size = m_hscroll_wheel_scroll_increment;
+            if (line_size == 0 && !this->Empty()) {
+                const Row* row = *begin();
+                line_size = Value(row->Height());
+            }
+
+            unsigned int page_size = std::abs(Value(cl_sz.x - (vertical_needed ? SCROLL_WIDTH : 0)));
+
             m_hscroll->SizeScroll(0, Value(total_x_extent - 1),
                                   line_size, std::max(line_size, page_size));
             MoveChildUp(m_hscroll);
@@ -1786,16 +1988,20 @@ void ListBox::AdjustScrolls(bool adjust_for_resize)
                 X0, cl_sz.y - SCROLL_WIDTH,
                 cl_sz.x - (vertical_needed ? SCROLL_WIDTH : 0), Y(SCROLL_WIDTH),
                 m_color, CLR_SHADOW);
-        int line_size = Value(cl_sz.x / 8);
-        int page_size = Value(cl_sz.x - (vertical_needed ? SCROLL_WIDTH : 0));
+
+        unsigned int line_size = m_hscroll_wheel_scroll_increment;
+        if (line_size == 0 && !this->Empty()) {
+            const Row* row = *begin();
+            line_size = Value(row->Height());
+        }
+
+        unsigned int page_size = std::abs(Value(cl_sz.x - (vertical_needed ? SCROLL_WIDTH : 0)));
+
         m_hscroll->SizeScroll(0, Value(total_x_extent - 1),
                               line_size, std::max(line_size, page_size));
         AttachChild(m_hscroll);
         Connect(m_hscroll->ScrolledSignal, &ListBox::HScrolled, this);
     }
-
-    assert(!m_vscroll || static_cast<int>(m_vscroll->PageSize()) == ClientHeight() || ClientHeight() < 0);
-    assert(!m_hscroll || static_cast<int>(m_hscroll->PageSize()) == ClientWidth() || ClientWidth() < 0);
 }
 
 void ListBox::VScrolled(int tab_low, int tab_high, int low, int high)
@@ -1841,10 +2047,15 @@ void ListBox::HScrolled(int tab_low, int tab_high, int low, int high)
 
 void ListBox::ClickAtRow(iterator it, Flags<ModKey> mod_keys)
 {
-    assert(it != m_rows.end());
-    assert(!m_rows.empty());
+    //assert(it != m_rows.end());   // replaced with following test and return to avoid crashes
+    if (it == m_rows.end())
+        return;
+    //assert(!m_rows.empty());      // replaced with following test and return to avoid crashes
+    if (m_rows.empty())
+        return;
 
     SelectionSet previous_selections = m_selections;
+
     if (m_style & LIST_SINGLESEL) {
         // No special keys are being used; just clear all previous selections,
         // select this row, set the caret here.
