@@ -52,21 +52,32 @@ namespace {
 ////////////////////////////////////////////////
 // GG::MenuItem
 ////////////////////////////////////////////////
-MenuItem::MenuItem() : 
+MenuItem::MenuItem() :
     SelectedIDSignal(new SelectedIDSignalType()),
     SelectedSignal(new SelectedSignalType()),
-    item_ID(0), 
-    disabled(false), 
-    checked(false)
+    item_ID(0),
+    disabled(false),
+    checked(false),
+    separator(false)
 {}
 
-MenuItem::MenuItem(const std::string& str, int id, bool disable, bool check) : 
+MenuItem::MenuItem(bool separator) :
+    SelectedIDSignal(),
+    SelectedSignal(),
+    item_ID(0),
+    disabled(true),
+    checked(false),
+    separator(true)
+{}
+
+MenuItem::MenuItem(const std::string& str, int id, bool disable, bool check) :
     SelectedIDSignal(new SelectedIDSignalType()),
     SelectedSignal(new SelectedSignalType()),
-    label(str), 
-    item_ID(id), 
-    disabled(disable), 
-    checked(check)
+    label(str),
+    item_ID(id),
+    disabled(disable),
+    checked(check),
+    separator(false)
 {
     if (INSTRUMENT_ALL_SIGNALS) {
         Connect(*SelectedIDSignal, MenuSignalEcho("MenuItem::SelectedIDSignal"));
@@ -74,13 +85,14 @@ MenuItem::MenuItem(const std::string& str, int id, bool disable, bool check) :
     }
 }
 
-MenuItem::MenuItem(const std::string& str, int id, bool disable, bool check, const SelectedIDSlotType& slot) : 
+MenuItem::MenuItem(const std::string& str, int id, bool disable, bool check, const SelectedIDSlotType& slot) :
     SelectedIDSignal(new SelectedIDSignalType()),
     SelectedSignal(new SelectedSignalType()),
-    label(str), 
-    item_ID(id), 
-    disabled(disable), 
-    checked(check)
+    label(str),
+    item_ID(id),
+    disabled(disable),
+    checked(check),
+    separator(false)
 {
     SelectedIDSignal->connect(slot);
 
@@ -90,13 +102,14 @@ MenuItem::MenuItem(const std::string& str, int id, bool disable, bool check, con
     }
 }
 
-MenuItem::MenuItem(const std::string& str, int id, bool disable, bool check, const SelectedSlotType& slot) : 
+MenuItem::MenuItem(const std::string& str, int id, bool disable, bool check, const SelectedSlotType& slot) :
     SelectedIDSignal(new SelectedIDSignalType()),
     SelectedSignal(new SelectedSignalType()),
-    label(str), 
-    item_ID(id), 
-    disabled(disable), 
-    checked(check)
+    label(str),
+    item_ID(id),
+    disabled(disable),
+    checked(check),
+    separator(false)
 {
     SelectedSignal->connect(slot);
 
@@ -245,7 +258,7 @@ void MenuBar::LButtonDown(const Pt& pt, Flags<ModKey> mod_keys)
                     (*m_menu_data.next_level[i].SelectedSignal)();
                 } else {
                     MenuItem popup_data;
-                    PopupMenu menu(m_menu_labels[i]->UpperLeft().x, m_menu_labels[i]->LowerRight().y, m_font, m_menu_data.next_level[i], m_text_color, m_border_color, m_int_color);
+                    PopupMenu menu(m_menu_labels[i]->Left(), m_menu_labels[i]->Bottom(), m_font, m_menu_data.next_level[i], m_text_color, m_border_color, m_int_color);
                     menu.SetHiliteColor(m_hilite_color);
                     menu.SetSelectedTextColor(m_sel_text_color);
                     Connect(menu.BrowsedSignal, boost::ref(BrowsedSignal));
@@ -385,12 +398,13 @@ namespace {
 const std::size_t PopupMenu::INVALID_CARET = std::numeric_limits<std::size_t>::max();
 
 PopupMenu::PopupMenu(X x, Y y, const boost::shared_ptr<Font>& font, const MenuItem& m, Clr text_color/* = CLR_WHITE*/,
-                     Clr color/* = CLR_BLACK*/, Clr interior/* = CLR_SHADOW*/) :
+                     Clr border_color/* = CLR_BLACK*/, Clr interior_color/* = CLR_SHADOW*/, Clr hilite_color/* = CLR_GRAY*/) :
     Wnd(X0, Y0, GUI::GetGUI()->AppWidth() - 1, GUI::GetGUI()->AppHeight() - 1, INTERACTIVE | MODAL),
     m_font(font),
-    m_border_color(color),
-    m_int_color(interior),
+    m_border_color(border_color),
+    m_int_color(interior_color),
     m_text_color(text_color),
+    m_hilite_color(hilite_color),
     m_sel_text_color(text_color),
     m_menu_data(m),
     m_open_levels(),
@@ -398,9 +412,6 @@ PopupMenu::PopupMenu(X x, Y y, const boost::shared_ptr<Font>& font, const MenuIt
     m_origin(x, y),
     m_item_selected(0)
 {
-    // use opaque interior color as hilite color
-    interior.a = 255;
-    m_hilite_color = interior;
     m_open_levels.resize(1);
 
     if (INSTRUMENT_ALL_SIGNALS)
@@ -482,7 +493,10 @@ void PopupMenu::Render()
             m_open_levels[i] = r;
 
             // paint caret, if any
-            if (m_caret[i] != INVALID_CARET) {
+            if (m_caret[i] != INVALID_CARET &&
+                !menu.next_level[m_caret[i]].separator &&
+                !menu.next_level[m_caret[i]].disabled)
+            {
                 Rect tmp_r = r;
                 tmp_r.ul.y += static_cast<int>(m_caret[i]) * m_font->Lineskip();
                 tmp_r.lr.y = tmp_r.ul.y + m_font->Lineskip() + 3;
@@ -500,22 +514,45 @@ void PopupMenu::Render()
             line_rect.ul.x += HORIZONTAL_MARGIN;
             line_rect.lr.x -= HORIZONTAL_MARGIN;
             for (std::size_t j = 0; j < menu.next_level.size(); ++j) {
-                Clr clr = (m_caret[i] == j) ?
-                          (menu.next_level[j].disabled ? DisabledColor(m_sel_text_color) : m_sel_text_color) :
-                                  (menu.next_level[j].disabled ? DisabledColor(m_text_color) : m_text_color);
+                Clr clr =   (m_caret[i] == j)
+                                ? (menu.next_level[j].disabled
+                                    ? DisabledColor(m_sel_text_color)
+                                    : m_sel_text_color)
+                                : (menu.next_level[j].disabled
+                                    ? DisabledColor(m_text_color)
+                                    : m_text_color);
                 glColor3ub(clr.r, clr.g, clr.b);
-                m_font->RenderText(line_rect.ul, line_rect.lr, menu.next_level[j].label, fmt);
+
+                if (!menu.next_level[j].separator) {
+                    m_font->RenderText(line_rect.ul, line_rect.lr, menu.next_level[j].label, fmt);
+
+                } else {
+                    glDisable(GL_TEXTURE_2D);
+                    glBegin(GL_LINES);
+                    glVertex(line_rect.ul.x + HORIZONTAL_MARGIN,
+                             line_rect.ul.y + Value(INDICATOR_HEIGHT / 2.0) + INDICATOR_VERTICAL_MARGIN);
+                    glVertex(line_rect.lr.x - HORIZONTAL_MARGIN,
+                             line_rect.ul.y + Value(INDICATOR_HEIGHT / 2.0) + INDICATOR_VERTICAL_MARGIN);
+                    glEnd();
+                    glEnable(GL_TEXTURE_2D);
+                }
+
                 if (menu.next_level[j].checked) {
                     FlatCheck(Pt(line_rect.lr.x - CHECK_WIDTH - HORIZONTAL_MARGIN, line_rect.ul.y + INDICATOR_VERTICAL_MARGIN),
                               Pt(line_rect.lr.x - HORIZONTAL_MARGIN, line_rect.ul.y + INDICATOR_VERTICAL_MARGIN + CHECK_HEIGHT),
                               clr);
                 }
-                if (menu.next_level[j].next_level.size()) {
+
+                // submenu indicator arrow
+                if (menu.next_level[j].next_level.size() > 0u) {
                     glDisable(GL_TEXTURE_2D);
                     glBegin(GL_TRIANGLES);
-                    glVertex(line_rect.lr.x - Value(INDICATOR_HEIGHT / 2.0) - HORIZONTAL_MARGIN, line_rect.ul.y + INDICATOR_VERTICAL_MARGIN);
-                    glVertex(line_rect.lr.x - Value(INDICATOR_HEIGHT / 2.0) - HORIZONTAL_MARGIN, line_rect.ul.y + m_font->Lineskip() - INDICATOR_VERTICAL_MARGIN);
-                    glVertex(line_rect.lr.x - HORIZONTAL_MARGIN,                                 line_rect.ul.y + m_font->Lineskip() / 2.0);
+                    glVertex(line_rect.lr.x - Value(INDICATOR_HEIGHT / 2.0) - HORIZONTAL_MARGIN,
+                             line_rect.ul.y + INDICATOR_VERTICAL_MARGIN);
+                    glVertex(line_rect.lr.x - Value(INDICATOR_HEIGHT / 2.0) - HORIZONTAL_MARGIN,
+                             line_rect.ul.y + m_font->Lineskip() - INDICATOR_VERTICAL_MARGIN);
+                    glVertex(line_rect.lr.x - HORIZONTAL_MARGIN,
+                             line_rect.ul.y + m_font->Lineskip() / 2.0);
                     glEnd();
                     glEnable(GL_TEXTURE_2D);
                 }
@@ -529,14 +566,19 @@ void PopupMenu::LButtonUp(const Pt& pt, Flags<ModKey> mod_keys)
 {
     if (m_caret[0] != INVALID_CARET) {
         MenuItem* menu_ptr = &m_menu_data;
-        for (std::size_t i = 0; i < m_caret.size(); ++i)
-            menu_ptr = &menu_ptr->next_level[m_caret[i]];
-        if (!menu_ptr->disabled) {
-            m_item_selected = menu_ptr;
+        for (std::size_t i = 0; i < m_caret.size(); ++i) {
+            if (m_caret[i] != INVALID_CARET) {
+                menu_ptr = &menu_ptr->next_level[m_caret[i]];
+            }
         }
+        if (!menu_ptr->disabled && !menu_ptr->separator) {
+            m_item_selected = menu_ptr;
+            m_done = true;
+        }
+    } else {
+        m_done = true;
     }
     BrowsedSignal(0);
-    m_done = true;
 }
 
 void PopupMenu::LClick(const Pt& pt, Flags<ModKey> mod_keys)
@@ -553,7 +595,8 @@ void PopupMenu::LDrag(const Pt& pt, const Pt& move, Flags<ModKey> mod_keys)
         MenuItem& menu = *menu_ptr;
 
         if (pt.x >= m_open_levels[i].ul.x && pt.x <= m_open_levels[i].lr.x &&
-                pt.y >= m_open_levels[i].ul.y && pt.y <= m_open_levels[i].lr.y) {
+            pt.y >= m_open_levels[i].ul.y && pt.y <= m_open_levels[i].lr.y)
+        {
             std::size_t row_selected = Value((pt.y - m_open_levels[i].ul.y) / m_font->Lineskip());
             if (row_selected == m_caret[i]) {
                 cursor_is_in_menu = true;
@@ -562,7 +605,7 @@ void PopupMenu::LDrag(const Pt& pt, const Pt& move, Flags<ModKey> mod_keys)
                 m_open_levels.resize(i + 1);
                 m_caret.resize(i + 1);
                 if (!menu.next_level[row_selected].disabled && menu.next_level[row_selected].next_level.size()) {
-                    m_caret.push_back(0);
+                    m_caret.push_back(INVALID_CARET);
                     m_open_levels.push_back(Rect());
                 }
                 cursor_is_in_menu = true;
