@@ -11,6 +11,7 @@
 #include "InfoPanels.h"
 #include "../util/i18n.h"
 #include "../util/Logger.h"
+#include "../util/ScopedTimer.h"
 #include "../universe/UniverseObject.h"
 #include "../Empire/Empire.h"
 #include "../Empire/EmpireManager.h"
@@ -33,7 +34,7 @@ namespace {
     public:
         ProductionItemPanel(GG::X w, GG::Y h, const ProductionQueue::ProductionItem& item,
                             int empire_id, int location_id) :
-            Control(GG::X0, GG::Y0, w, h, GG::Flags<GG::WndFlag>()),
+            Control(GG::X0, GG::Y0, w, h, GG::NO_WND_FLAGS),
             m_initialized(false),
             m_item(item),
             m_empire_id(empire_id),
@@ -68,7 +69,6 @@ namespace {
             if (!m_initialized)
                 return;
 
-            const GG::Y ICON_HEIGHT(ClientHeight());
             const GG::X ICON_WIDTH(Value(ClientHeight()));
             const GG::X ITEM_NAME_WIDTH(ClientUI::Pts() * 16);
             const GG::X COST_WIDTH(ClientUI::Pts() * 4);
@@ -76,7 +76,6 @@ namespace {
             const GG::X DESC_WIDTH(ClientUI::Pts() * 18);
 
             GG::X left(GG::X0);
-            GG::Y top(GG::Y0);
             GG::Y bottom(ClientHeight());
 
             m_icon->SizeMove(GG::Pt(left, GG::Y0), GG::Pt(left + ICON_WIDTH, bottom));
@@ -173,10 +172,10 @@ namespace {
         GG::TextControl*                m_desc;
     };
 
-    const UniverseObject* GetSourceObjectForEmpire(int empire_id) {
+    TemporaryPtr<const UniverseObject> GetSourceObjectForEmpire(int empire_id) {
         // get a source object, which is owned by the empire,
         // preferably the capital
-        const UniverseObject* source = 0;
+        TemporaryPtr<const UniverseObject> source;
         if (empire_id == ALL_EMPIRES)
             return source;
         const Empire* empire = Empires().Lookup(empire_id);
@@ -207,7 +206,7 @@ namespace {
         location_conditions.push_back(&bld_avail_cond);
         if (const BuildingType* building_type = GetBuildingType(building_name))
             location_conditions.push_back(building_type->Location());
-        const UniverseObject* source = GetSourceObjectForEmpire(empire_id);
+        TemporaryPtr<const UniverseObject> source = GetSourceObjectForEmpire(empire_id);
         return ConditionDescription(location_conditions, GetUniverseObject(candidate_object_id), source);
     }
 
@@ -235,7 +234,7 @@ namespace {
                     location_conditions.push_back(part_type->Location());
             }
         }
-        const UniverseObject* source = GetSourceObjectForEmpire(empire_id);
+        TemporaryPtr<const UniverseObject> source = GetSourceObjectForEmpire(empire_id);
         return ConditionDescription(location_conditions, GetUniverseObject(candidate_object_id), source);
     }
 
@@ -286,13 +285,13 @@ namespace {
             push_back(m_panel);
 
             if (const Empire* empire = Empires().Lookup(m_empire_id)) {
-                if (!empire->BuildableItem(m_item, m_location_id)) {
+                if (!empire->ProducibleItem(m_item, m_location_id)) {
                     this->Disable(true);
                     m_panel->Disable(true);
                 }
             }
 
-            m_panel->SetBrowseModeTime(GetOptionsDB().Get<int>("UI.tooltip-delay"));
+            SetBrowseModeTime(GetOptionsDB().Get<int>("UI.tooltip-delay"));
             SetBrowseInfoWnd(ProductionItemRowBrowseWnd(m_item, m_location_id, m_empire_id));
 
             //std::cout << "ProductionItemRow(building) height: " << Value(Height()) << std::endl;
@@ -333,6 +332,8 @@ namespace {
             SetNumCols(1);
             SetColWidth(0, GG::X0);
             LockColWidths();
+
+            SetVScrollWheelIncrement(Value(ListRowHeight())*3);
         }
 
         virtual void    SizeMove(const GG::Pt& ul, const GG::Pt& lr) {
@@ -414,10 +415,10 @@ public:
     void    HideAvailability(bool available, bool refresh_list = true);
     //@}
 
-    mutable boost::signal<void (const BuildingType*)>                   DisplayBuildingTypeSignal;
-    mutable boost::signal<void (BuildType, const std::string&, int)>    RequestNamedBuildItemSignal;
-    mutable boost::signal<void (const ShipDesign*)>                     DisplayShipDesignSignal;
-    mutable boost::signal<void (BuildType, int, int)>                   RequestIDedBuildItemSignal;
+    mutable boost::signals2::signal<void (const BuildingType*)>                DisplayBuildingTypeSignal;
+    mutable boost::signals2::signal<void (BuildType, const std::string&, int)> RequestNamedBuildItemSignal;
+    mutable boost::signals2::signal<void (const ShipDesign*)>                  DisplayShipDesignSignal;
+    mutable boost::signals2::signal<void (BuildType, int, int)>                RequestIDedBuildItemSignal;
 
 private:
     static const GG::X TEXT_MARGIN_X;
@@ -432,12 +433,15 @@ private:
       * filter settings. */
     void    PopulateList();
 
-    /** respond to the user single-click to select item on the queue */
-    void    BuildItemClicked(GG::ListBox::iterator it, const GG::Pt& pt);
+    /** respond to the user single-clicking a producible item in the build selector */
+    void    BuildItemLeftClicked(GG::ListBox::iterator it, const GG::Pt& pt);
 
-    /** respond to the user double-clicking an item on the queue */
+    /** respond to the user double-clicking a producible item in the build selector */
     void    BuildItemDoubleClicked(GG::ListBox::iterator it);
 
+    /** respond to the user right-clicking a producible item in the build selector */
+    void    BuildItemRightClicked(GG::ListBox::iterator it, const GG::Pt& pt);
+
     std::map<BuildType, CUIButton*>         m_build_type_buttons;
     std::vector<CUIButton*>                 m_availability_buttons;
 
@@ -450,7 +454,7 @@ private:
     int                                     m_production_location;
     int                                     m_empire_id;
 
-    mutable boost::signals::connection      m_empire_ship_designs_changed_signal;
+    mutable boost::signals2::connection     m_empire_ship_designs_changed_signal;
 
     friend class BuildDesignatorWnd;        // so BuildDesignatorWnd can access buttons
 };
@@ -459,29 +463,31 @@ const GG::Y BuildDesignatorWnd::BuildSelector::TEXT_MARGIN_Y(3);
 
 BuildDesignatorWnd::BuildSelector::BuildSelector(GG::X w, GG::Y h) :
     CUIWnd(UserString("PRODUCTION_WND_BUILD_ITEMS_TITLE"), GG::X1, GG::Y1, w - 1, h - 1,
-           GG::INTERACTIVE | GG::DRAGABLE | GG::RESIZABLE | GG::ONTOP),
+           GG::INTERACTIVE | GG::DRAGABLE | GG::RESIZABLE | GG::ONTOP | PINABLE ),
     m_buildable_items(new BuildableItemsListBox(GG::X0, GG::Y0, GG::X1, GG::Y1)),
     m_production_location(INVALID_OBJECT_ID),
     m_empire_id(ALL_EMPIRES)
 {
     // create build type toggle buttons (ship, building, all)
-    m_build_type_buttons[BT_BUILDING] = new CUIButton(GG::X0, GG::Y0, GG::X1, UserString("PRODUCTION_WND_CATEGORY_BT_BUILDING"));
+    m_build_type_buttons[BT_BUILDING] = new CUIButton(UserString("PRODUCTION_WND_CATEGORY_BT_BUILDING"));
     AttachChild(m_build_type_buttons[BT_BUILDING]);
-    m_build_type_buttons[BT_SHIP] = new CUIButton(GG::X0, GG::Y0, GG::X1, UserString("PRODUCTION_WND_CATEGORY_BT_SHIP"));
+    m_build_type_buttons[BT_SHIP] = new CUIButton(UserString("PRODUCTION_WND_CATEGORY_BT_SHIP"));
     AttachChild(m_build_type_buttons[BT_SHIP]);
 
     // create availability toggle buttons (available, not available)
-    m_availability_buttons.push_back(new CUIButton(GG::X0, GG::Y0, GG::X1, UserString("PRODUCTION_WND_AVAILABILITY_AVAILABLE")));
+    m_availability_buttons.push_back(new CUIButton(UserString("PRODUCTION_WND_AVAILABILITY_AVAILABLE")));
     AttachChild(m_availability_buttons.back());
-    m_availability_buttons.push_back(new CUIButton(GG::X0, GG::Y0, GG::X1, UserString("PRODUCTION_WND_AVAILABILITY_UNAVAILABLE")));
+    m_availability_buttons.push_back(new CUIButton(UserString("PRODUCTION_WND_AVAILABILITY_UNAVAILABLE")));
     AttachChild(m_availability_buttons.back());
 
     // selectable list of buildable items
     AttachChild(m_buildable_items);
     GG::Connect(m_buildable_items->LeftClickedSignal,
-                &BuildDesignatorWnd::BuildSelector::BuildItemClicked, this);
+                &BuildDesignatorWnd::BuildSelector::BuildItemLeftClicked, this);
     GG::Connect(m_buildable_items->DoubleClickedSignal,
                 &BuildDesignatorWnd::BuildSelector::BuildItemDoubleClicked, this);
+    GG::Connect(m_buildable_items->RightClickedSignal,
+                &BuildDesignatorWnd::BuildSelector::BuildItemRightClicked, this);
 
     //GG::ListBox::Row* header = new GG::ListBox::Row();
     //boost::shared_ptr<GG::Font> font = ClientUI::GetFont();
@@ -567,13 +573,13 @@ void BuildDesignatorWnd::BuildSelector::SetEmpireID(int empire_id/* = ALL_EMPIRE
             m_empire_ship_designs_changed_signal = GG::Connect(empire->ShipDesignsChangedSignal,
                                                                &BuildDesignatorWnd::BuildSelector::Refresh,
                                                                this,
-                                                               boost::signals::at_front);
+                                                               boost::signals2::at_front);
     }
 }
 
 void BuildDesignatorWnd::BuildSelector::Refresh() {
-    Logger().debugStream() << "BuildDesignatorWnd::BuildSelector::Refresh()";
-    if (const UniverseObject* prod_loc = GetUniverseObject(this->m_production_location))
+    ScopedTimer timer("BuildDesignatorWnd::BuildSelector::Refresh()");
+    if (TemporaryPtr<const UniverseObject> prod_loc = GetUniverseObject(this->m_production_location))
         this->SetName(boost::io::str(FlexibleFormat(UserString("PRODUCTION_WND_BUILD_ITEMS_TITLE_LOCATION")) % prod_loc->Name()));
     else
         this->SetName(UserString("PRODUCTION_WND_BUILD_ITEMS_TITLE"));
@@ -583,7 +589,7 @@ void BuildDesignatorWnd::BuildSelector::Refresh() {
         m_empire_ship_designs_changed_signal = GG::Connect(empire->ShipDesignsChangedSignal,
                                                            &BuildDesignatorWnd::BuildSelector::Refresh,
                                                            this,
-                                                           boost::signals::at_front);
+                                                           boost::signals2::at_front);
     PopulateList();
 }
 
@@ -649,7 +655,9 @@ void BuildDesignatorWnd::BuildSelector::HideAvailability(bool available, bool re
     }
 }
 
-bool BuildDesignatorWnd::BuildSelector::BuildableItemVisible(BuildType build_type, const std::string& name) {
+bool BuildDesignatorWnd::BuildSelector::BuildableItemVisible(BuildType build_type,
+                                                             const std::string& name)
+{
     if (build_type != BT_BUILDING)
         throw std::invalid_argument("BuildableItemVisible was passed an invalid build type with a name");
 
@@ -664,7 +672,7 @@ bool BuildDesignatorWnd::BuildSelector::BuildableItemVisible(BuildType build_typ
     if (!empire)
         return true;
 
-    bool producible_here = empire->BuildableItem(BT_BUILDING, name, m_production_location);
+    bool producible_here = empire->ProducibleItem(BT_BUILDING, name, m_production_location);
 
     if (producible_here)
         return m_availabilities_shown.first;
@@ -687,7 +695,7 @@ bool BuildDesignatorWnd::BuildSelector::BuildableItemVisible(BuildType build_typ
     if (!empire)
         return true;
 
-    bool producible_here = empire->BuildableItem(BT_SHIP, design_id, m_production_location);
+    bool producible_here = empire->ProducibleItem(BT_SHIP, design_id, m_production_location);
 
     if (producible_here)
         return m_availabilities_shown.first;
@@ -730,23 +738,29 @@ void BuildDesignatorWnd::BuildSelector::PopulateList() {
     //Logger().debugStream() << "BuildDesignatorWnd::BuildSelector::PopulateList() : Adding Buildings ";
     if (m_build_types_shown.find(BT_BUILDING) != m_build_types_shown.end()) {
         BuildingTypeManager& manager = GetBuildingTypeManager();
-
+        // craete and insert rows...
+        std::vector<GG::ListBox::Row*> rows;
+        rows.reserve(std::distance(manager.begin(), manager.end()));
         for (BuildingTypeManager::iterator it = manager.begin(); it != manager.end(); ++it, ++i) {
             const std::string name = it->first;
-
-            if (!BuildableItemVisible(BT_BUILDING, name)) continue;
-
+            if (!BuildableItemVisible(BT_BUILDING, name))
+                continue;
             ProductionItemRow* item_row = new ProductionItemRow(row_size.x, row_size.y,
                                                                 ProductionQueue::ProductionItem(BT_BUILDING, name),
                                                                 m_empire_id, m_production_location);
-
-            GG::ListBox::iterator row_it = m_buildable_items->Insert(item_row);
-            if (item_row->DragDropDataType() == initial_first_row_name)
-                new_first_row_it = row_it;
-
-            item_row->Resize(row_size);
+            rows.push_back(item_row);
+        }
+        m_buildable_items->Insert(rows, false);
+        // resize inserted rows and record first row to show
+        for (BuildableItemsListBox::iterator it = m_buildable_items->begin();
+            it != m_buildable_items->end(); ++it)
+        {
+            if ((*it)->DragDropDataType() == initial_first_row_name)
+                new_first_row_it = it;
+            (*it)->Resize(row_size);
         }
     }
+
     // populate with ship designs
     //Logger().debugStream() << "BuildDesignatorWnd::BuildSelector::PopulateList() : Adding ship designs";
     if (m_build_types_shown.find(BT_SHIP) != m_build_types_shown.end()) {
@@ -759,25 +773,29 @@ void BuildDesignatorWnd::BuildSelector::PopulateList() {
             for (Universe::ship_design_iterator it = GetUniverse().beginShipDesigns(); it != GetUniverse().endShipDesigns(); ++it)
                 design_ids.push_back(it->first);
 
+        // craete and insert rows...
+        std::vector<GG::ListBox::Row*> rows;
+        rows.reserve(design_ids.size());
         for (std::vector<int>::const_iterator it = design_ids.begin(); it != design_ids.end(); ++it, ++i) {
             int ship_design_id = *it;
-
-            if (!BuildableItemVisible(BT_SHIP, ship_design_id)) continue;
-
+            if (!BuildableItemVisible(BT_SHIP, ship_design_id))
+                continue;
             const ShipDesign* ship_design = GetShipDesign(ship_design_id);
-            if (!ship_design) continue;
-
-            // add build item panel
-
+            if (!ship_design)
+                continue;
             ProductionItemRow* item_row = new ProductionItemRow(row_size.x, row_size.y,
                                                                 ProductionQueue::ProductionItem(BT_SHIP, ship_design_id),
                                                                 m_empire_id, m_production_location);
-
-            GG::ListBox::iterator row_it = m_buildable_items->Insert(item_row);
-            if (item_row->DragDropDataType() == initial_first_row_name)
-                new_first_row_it = row_it;
-
-            item_row->Resize(row_size);
+            rows.push_back(item_row);
+        }
+        m_buildable_items->Insert(rows, false);
+        // resize inserted rows and record first row to show
+        for (BuildableItemsListBox::iterator it = m_buildable_items->begin();
+            it != m_buildable_items->end(); ++it)
+        {
+            if ((*it)->DragDropDataType() == initial_first_row_name)
+                new_first_row_it = it;
+            (*it)->Resize(row_size);
         }
     }
 
@@ -792,8 +810,9 @@ void BuildDesignatorWnd::BuildSelector::PopulateList() {
     //Logger().debugStream() << "Done";
 }
 
-void BuildDesignatorWnd::BuildSelector::BuildItemClicked(GG::ListBox::iterator it, const GG::Pt& pt) {
-
+void BuildDesignatorWnd::BuildSelector::BuildItemLeftClicked(GG::ListBox::iterator it,
+                                                             const GG::Pt& pt)
+{
     ProductionItemRow* item_row = dynamic_cast<ProductionItemRow*>(*it);
     if (!item_row)
         return;
@@ -834,6 +853,30 @@ void BuildDesignatorWnd::BuildSelector::BuildItemDoubleClicked(GG::ListBox::iter
         RequestIDedBuildItemSignal(BT_SHIP, item.design_id, 1);
 }
 
+void BuildDesignatorWnd::BuildSelector::BuildItemRightClicked(GG::ListBox::iterator it,
+                                                              const GG::Pt& pt)
+{
+    if ((*it)->Disabled())
+        return;
+
+    GG::MenuItem menu_contents;
+    menu_contents.next_level.push_back(GG::MenuItem(UserString("PRODUCTION_DETAIL_ADD_TO_QUEUE"),   1, false, false));
+
+    GG::PopupMenu popup(pt.x, pt.y, ClientUI::GetFont(), menu_contents, ClientUI::TextColor(),
+                        ClientUI::WndOuterBorderColor(), ClientUI::WndColor(), ClientUI::EditHiliteColor());
+
+    if (popup.Run()) {
+        switch (popup.MenuID()) {
+        case 1: { // move item to top
+            BuildItemDoubleClicked(it);
+            break;
+        }
+        default:
+            break;
+        }
+    }
+}
+
 //////////////////////////////////////////////////
 // BuildDesignatorWnd
 //////////////////////////////////////////////////
@@ -845,7 +888,7 @@ BuildDesignatorWnd::BuildDesignatorWnd(GG::X w, GG::Y h) :
     const GG::Y DETAIL_PANEL_HEIGHT =   GG::Y(240);
     const GG::Y BUILD_SELECTOR_HEIGHT = DETAIL_PANEL_HEIGHT;
 
-    m_enc_detail_panel = new EncyclopediaDetailPanel(CHILD_WIDTHS, DETAIL_PANEL_HEIGHT);
+    m_enc_detail_panel = new EncyclopediaDetailPanel(CHILD_WIDTHS, DETAIL_PANEL_HEIGHT, GG::ONTOP | GG::INTERACTIVE | GG::DRAGABLE | GG::RESIZABLE | PINABLE );
 
     m_side_panel = new SidePanel(Width() - SIDEPANEL_WIDTH, GG::Y0, Height());
     m_side_panel->EnableSelection();
@@ -855,10 +898,10 @@ BuildDesignatorWnd::BuildDesignatorWnd(GG::X w, GG::Y h) :
     m_build_selector->MoveTo(GG::Pt(GG::X0, h - BUILD_SELECTOR_HEIGHT));
 
 
-    GG::Connect(m_build_selector->DisplayBuildingTypeSignal,    &EncyclopediaDetailPanel::SetItem,          m_enc_detail_panel);
-    GG::Connect(m_build_selector->DisplayShipDesignSignal,      &EncyclopediaDetailPanel::SetItem,          m_enc_detail_panel);
-    GG::Connect(m_build_selector->RequestNamedBuildItemSignal,  &BuildDesignatorWnd::BuildItemRequested,    this);
-    GG::Connect(m_build_selector->RequestIDedBuildItemSignal,   &BuildDesignatorWnd::BuildItemRequested,    this);
+    GG::Connect(m_build_selector->DisplayBuildingTypeSignal,    static_cast<void (EncyclopediaDetailPanel::*)(const BuildingType*)>(&EncyclopediaDetailPanel::SetItem),                 m_enc_detail_panel);
+    GG::Connect(m_build_selector->DisplayShipDesignSignal,      static_cast<void (EncyclopediaDetailPanel::*)(const ShipDesign*)>(&EncyclopediaDetailPanel::SetItem),                   m_enc_detail_panel);
+    GG::Connect(m_build_selector->RequestNamedBuildItemSignal,  static_cast<void (BuildDesignatorWnd::*)(BuildType, const std::string&, int)>(&BuildDesignatorWnd::BuildItemRequested), this);
+    GG::Connect(m_build_selector->RequestIDedBuildItemSignal,   static_cast<void (BuildDesignatorWnd::*)(BuildType, int, int)>(&BuildDesignatorWnd::BuildItemRequested),                this);
 
     GG::Connect(m_side_panel->PlanetSelectedSignal, PlanetSelectedSignal);
     GG::Connect(m_side_panel->SystemSelectedSignal, SystemSelectedSignal);
@@ -909,7 +952,7 @@ void BuildDesignatorWnd::CenterOnBuild(int queue_idx) {
     const ProductionQueue& queue = empire->GetProductionQueue();
     if (0 <= queue_idx && queue_idx < static_cast<int>(queue.size())) {
         int location_id = queue[queue_idx].location;
-        if (const UniverseObject* build_location = objects.Object(location_id)) {
+        if (TemporaryPtr<const UniverseObject> build_location = objects.Object(location_id)) {
             // centre map on system of build location
             int system_id = build_location->SystemID();
             MapWnd* map = ClientUI::GetClientUI()->GetMapWnd();
@@ -1027,8 +1070,7 @@ void BuildDesignatorWnd::HideAllTypes(bool refresh_list) {
     m_build_selector->m_build_type_buttons[BT_SHIP]->MarkNotSelected();
 }
 
-void BuildDesignatorWnd::ToggleType(BuildType type, bool refresh_list)
-{
+void BuildDesignatorWnd::ToggleType(BuildType type, bool refresh_list) {
     if (type == BT_BUILDING || type == BT_SHIP) {
         const std::set<BuildType>& types_shown = m_build_selector->GetBuildTypesShown();
         if (types_shown.find(type) == types_shown.end())
@@ -1037,11 +1079,10 @@ void BuildDesignatorWnd::ToggleType(BuildType type, bool refresh_list)
             HideType(type, refresh_list);
     } else {
         throw std::invalid_argument("BuildDesignatorWnd::ShowType was passed an invalid BuildType");
-    } 
+    }
 }
 
-void BuildDesignatorWnd::ToggleAllTypes(bool refresh_list)
-{
+void BuildDesignatorWnd::ToggleAllTypes(bool refresh_list) {
     const std::set<BuildType>& types_shown = m_build_selector->GetBuildTypesShown();
     if (types_shown.size() == NUM_BUILD_TYPES - 1)  // -1 because there are no buttons for BuildType::BT_NOT_BUILDING
         HideAllTypes(refresh_list);
@@ -1049,8 +1090,7 @@ void BuildDesignatorWnd::ToggleAllTypes(bool refresh_list)
         ShowAllTypes(refresh_list);
 }
 
-void BuildDesignatorWnd::ShowAvailability(bool available, bool refresh_list)
-{
+void BuildDesignatorWnd::ShowAvailability(bool available, bool refresh_list) {
     m_build_selector->ShowAvailability(available, refresh_list);
     if (available)
         m_build_selector->m_availability_buttons.at(0)->MarkSelectedGray();
@@ -1058,8 +1098,7 @@ void BuildDesignatorWnd::ShowAvailability(bool available, bool refresh_list)
         m_build_selector->m_availability_buttons.at(1)->MarkSelectedGray();
 }
 
-void BuildDesignatorWnd::HideAvailability(bool available, bool refresh_list)
-{
+void BuildDesignatorWnd::HideAvailability(bool available, bool refresh_list) {
     m_build_selector->HideAvailability(available, refresh_list);
     if (available)
         m_build_selector->m_availability_buttons.at(0)->MarkNotSelected();
@@ -1067,8 +1106,7 @@ void BuildDesignatorWnd::HideAvailability(bool available, bool refresh_list)
         m_build_selector->m_availability_buttons.at(1)->MarkNotSelected();
 }
 
-void BuildDesignatorWnd::ToggleAvailabilitly(bool available, bool refresh_list)
-{
+void BuildDesignatorWnd::ToggleAvailabilitly(bool available, bool refresh_list) {
     const std::pair<bool, bool>& avail_shown = m_build_selector->GetAvailabilitiesShown();
     if (available) {
         if (avail_shown.first)
@@ -1084,43 +1122,36 @@ void BuildDesignatorWnd::ToggleAvailabilitly(bool available, bool refresh_list)
 }
 
 void BuildDesignatorWnd::ShowBuildingTypeInEncyclopedia(const std::string& building_type)
-{
-    m_enc_detail_panel->SetBuildingType(building_type);
-}
+{ m_enc_detail_panel->SetBuildingType(building_type); }
 
 void BuildDesignatorWnd::ShowShipDesignInEncyclopedia(int design_id)
-{
-    m_enc_detail_panel->SetDesign(design_id);
-}
+{ m_enc_detail_panel->SetDesign(design_id); }
 
 int BuildDesignatorWnd::BuildLocation() const
-{
-    return m_side_panel->SelectedPlanetID();
-}
+{ return m_side_panel->SelectedPlanetID(); }
 
-void BuildDesignatorWnd::BuildItemRequested(BuildType build_type, const std::string& item, int num_to_build)
+void BuildDesignatorWnd::BuildItemRequested(BuildType build_type, const std::string& item,
+                                            int num_to_build)
 {
     //std::cout << "BuildDesignatorWnd::BuildItemRequested item name: " << item << std::endl;
     const Empire* empire = Empires().Lookup(HumanClientApp::GetApp()->EmpireID());
-    if (empire && empire->BuildableItem(build_type, item, BuildLocation()))
+    if (empire && empire->EnqueuableItem(build_type, item, BuildLocation()))
         AddNamedBuildToQueueSignal(build_type, item, num_to_build, BuildLocation());
 }
 
-void BuildDesignatorWnd::BuildItemRequested(BuildType build_type, int design_id, int num_to_build)
+void BuildDesignatorWnd::BuildItemRequested(BuildType build_type, int design_id,
+                                            int num_to_build)
 {
     //std::cout << "BuildDesignatorWnd::BuildItemRequested design id: " << design_id << std::endl;
     const Empire* empire = Empires().Lookup(HumanClientApp::GetApp()->EmpireID());
-    if (empire && empire->BuildableItem(build_type, design_id, BuildLocation()))
+    if (empire && empire->ProducibleItem(build_type, design_id, BuildLocation()))
         AddIDedBuildToQueueSignal(build_type, design_id, num_to_build, BuildLocation());
 }
 
 void BuildDesignatorWnd::BuildQuantityChanged(int queue_idx, int quantity)
-{
-    BuildQuantityChangedSignal(queue_idx, quantity);
-}
+{ BuildQuantityChangedSignal(queue_idx, quantity); }
 
-void BuildDesignatorWnd::SelectDefaultPlanet()
-{
+void BuildDesignatorWnd::SelectDefaultPlanet() {
     int system_id = SidePanel::SystemID();
     if (system_id == INVALID_OBJECT_ID) {
         this->SelectPlanet(INVALID_OBJECT_ID);
@@ -1142,13 +1173,13 @@ void BuildDesignatorWnd::SelectDefaultPlanet()
     // couldn't reselect stored default, so need to find a reasonable other
     // planet to select.  attempt to find one owned by this client's player
 
-    const System* sys = GetSystem(system_id);   // only checking visible objects for this clients empire (and not the latest known objects) as an empire shouldn't be able to use a planet or system it can't currently see as a production location
+    TemporaryPtr<const System> sys = GetSystem(system_id);   // only checking visible objects for this clients empire (and not the latest known objects) as an empire shouldn't be able to use a planet or system it can't currently see as a production location
     if (!sys) {
         Logger().errorStream() << "BuildDesignatorWnd::SelectDefaultPlanet couldn't get system with id " << system_id;
         return;
     }
 
-    std::vector<int> planets = sys->FindObjectIDs<Planet>();
+    std::vector<TemporaryPtr<const Planet> > planets = Objects().FindObjects<const Planet>(sys->PlanetIDs());
 
     if (planets.empty()) {
         this->SelectPlanet(INVALID_OBJECT_ID);
@@ -1160,12 +1191,8 @@ void BuildDesignatorWnd::SelectDefaultPlanet()
     int best_planet_id = INVALID_OBJECT_ID; // id of selected planet
     double best_planet_pop = -99999.9;                      // arbitrary negative number, so any planet's pop will be better
 
-    for (std::vector<int>::iterator it = planets.begin(); it != planets.end(); ++it) {
-        const Planet* planet = GetPlanet(*it);
-        if (!planet) {
-            Logger().errorStream() << "BuildDesignatorWnd::SetDefaultPlanet couldn't get planet with id " << *it;
-            continue;
-        }
+    for (std::vector<TemporaryPtr<const Planet> >::const_iterator it = planets.begin(); it != planets.end(); ++it) {
+        TemporaryPtr<const Planet> planet = *it;
         int planet_id = planet->ID();
         if (!m_side_panel->PlanetSelectable(planet_id))
             continue;
