@@ -17,6 +17,7 @@
 
 #include <boost/lexical_cast.hpp>
 #include <boost/format.hpp>
+#include <boost/regex.hpp>
 
 #include <limits>
 
@@ -25,9 +26,6 @@ namespace {
     void PlayButtonClickSound()
     { Sound::GetSound().PlaySound(GetOptionsDB().Get<std::string>("UI.sound.button-click"), true); }
 
-    void PlayTurnButtonClickSound()
-    { Sound::GetSound().PlaySound(GetOptionsDB().Get<std::string>("UI.sound.turn-button-click"), true); }
-
     struct PlayButtonCheckSound {
         PlayButtonCheckSound(bool play_only_when_checked) : m_play_only_when_checked(play_only_when_checked) {}
         void operator()(bool checked) const {
@@ -49,9 +47,6 @@ namespace {
     void PlayTextTypingSound(const std::string&)
     { Sound::GetSound().PlaySound(GetOptionsDB().Get<std::string>("UI.sound.text-typing"), true); }
 
-    boost::shared_ptr<GG::Font> FontOrDefaultFont(const boost::shared_ptr<GG::Font>& font)
-    { return font ? font : ClientUI::GetFont(); }
-
     const double ARROW_BRIGHTENING_SCALE_FACTOR = 1.5;
     const double STATE_BUTTON_BRIGHTENING_SCALE_FACTOR = 1.25;
     const double TAB_BRIGHTENING_SCALE_FACTOR = 1.25;
@@ -65,13 +60,10 @@ namespace {
     const int CUIBUTTON_ANGLE_OFFSET = 5;
 }
 
-CUIButton::CUIButton(GG::X x, GG::Y y, GG::X w, const std::string& str, const boost::shared_ptr<GG::Font>& font/* = boost::shared_ptr<GG::Font>()*/,
-                     GG::Clr color/* = ClientUI::CtrlColor()*/,
-                     GG::Clr border/* = ClientUI::CtrlBorderColor()*/, int thick/* = 2*/, 
-                     GG::Clr text_color/* = ClientUI::TextColor()*/, GG::Flags<GG::WndFlag> flags/* = GG::INTERACTIVE*/) :
-    Button(x, y, w, FontOrDefaultFont(font)->Lineskip() + 6, str, FontOrDefaultFont(font), color, text_color, flags),
-    m_border_color(border),
-    m_border_thick(thick)
+CUIButton::CUIButton(const std::string& str, GG::X x, GG::Y y, GG::X w, GG::Clr color/* = ClientUI::CtrlColor()*/) :
+    Button(x, y, w, ClientUI::GetFont()->Lineskip() + 6, str, ClientUI::GetFont(), color, ClientUI::TextColor(), GG::INTERACTIVE),
+    m_border_color(ClientUI::CtrlBorderColor()),
+    m_border_thick(1)
 { GG::Connect(LeftClickedSignal, &PlayButtonClickSound, -1); }
 
 bool CUIButton::InWindow(const GG::Pt& pt) const {
@@ -88,6 +80,17 @@ void CUIButton::MouseHere(const GG::Pt& pt, GG::Flags<GG::ModKey> mod_keys) {
     }
 }
 
+GG::Pt CUIButton::MinUsableSize() const {
+    GG::Pt result = GG::Button::MinUsableSize();
+    const int CUIBUTTON_HPADDING = 10;
+    const int CUIBUTTON_VPADDING = 3;
+
+    result.x += CUIBUTTON_HPADDING * 2;
+    result.y += CUIBUTTON_VPADDING * 2;
+
+    return result;
+}
+
 void CUIButton::SetBorderColor(GG::Clr clr)
 { m_border_color = clr; }
 
@@ -168,14 +171,9 @@ void CUIButton::MarkSelectedTechCategoryColor(std::string category) {
 ///////////////////////////////////////
 // class SettableInWindowCUIButton
 ///////////////////////////////////////
-SettableInWindowCUIButton::SettableInWindowCUIButton(GG::X x, GG::Y y, GG::X w, const std::string& str,
-                                                     const boost::shared_ptr<GG::Font>& font/* = boost::shared_ptr<GG::Font>()*/,
-                                                     GG::Clr color/* = ClientUI::CtrlColor()*/,
-                                                     GG::Clr border/* = ClientUI::CtrlBorderColor()*/,
-                                                     int thick/* = 2*/, 
-                                                     GG::Clr text_color/* = ClientUI::TextColor()*/,
-                                                     GG::Flags<GG::WndFlag> flags/* = GG::INTERACTIVE*/) :
-    CUIButton(x, y, w, str, font, color, border, thick, text_color, flags)
+SettableInWindowCUIButton::SettableInWindowCUIButton(const std::string& str, GG::X x, GG::Y y, GG::X w,
+                                                     GG::Clr color/* = ClientUI::CtrlColor()*/) :
+    CUIButton(str, x, y, w, color)
 {}
 
 bool SettableInWindowCUIButton::InWindow(const GG::Pt& pt) const {
@@ -189,22 +187,12 @@ void SettableInWindowCUIButton::SetInWindow(boost::function<bool(const GG::Pt&)>
 { m_in_window_func = in_window_function; }
 
 
-///////////////////////////////////////
-// class CUITurnButton
-///////////////////////////////////////
-CUITurnButton::CUITurnButton(GG::X x, GG::Y y, GG::X w, const std::string& str, const boost::shared_ptr<GG::Font>& font/* = boost::shared_ptr<GG::Font>()*/,
-                             GG::Clr color/* = ClientUI::WndColor()*/, 
-                             GG::Clr border/* = ClientUI::CtrlBorderColor()*/, int thick/* = 2*/, 
-                             GG::Clr text_color/* = ClientUI::TextColor()*/, GG::Flags<GG::WndFlag> flags/* = GG::INTERACTIVE*/) : 
-    CUIButton(x, y, w, str, FontOrDefaultFont(font), color, border, thick, text_color, flags)
-{ GG::Connect(LeftClickedSignal, &PlayTurnButtonClickSound, -1); }
-
-
 ///////////////////////////////////////
 // class CUIArrowButton
 ///////////////////////////////////////
-CUIArrowButton::CUIArrowButton(GG::X x, GG::Y y, GG::X w, GG::Y h, ShapeOrientation orientation, GG::Clr color, GG::Flags<GG::WndFlag> flags/* = GG::INTERACTIVE*/) :
-    Button(x, y, w, h, "", boost::shared_ptr<GG::Font>(), color, GG::CLR_ZERO, flags),
+CUIArrowButton::CUIArrowButton(GG::X x, GG::Y y, GG::X w, GG::Y h, ShapeOrientation orientation,
+                               GG::Flags<GG::WndFlag> flags/* = GG::INTERACTIVE*/) :
+    Button(x, y, w, h, "", boost::shared_ptr<GG::Font>(), ClientUI::DropDownListArrowColor(), GG::CLR_ZERO, flags),
     m_orientation(orientation),
     m_fill_background_with_wnd_color (false)
 { GG::Connect(LeftClickedSignal, &PlayButtonClickSound, -1); }
@@ -265,17 +253,14 @@ void CUIArrowButton::RenderUnpressed() {
 ///////////////////////////////////////
 // class CUIStateButton
 ///////////////////////////////////////
-CUIStateButton::CUIStateButton(GG::X x, GG::Y y, GG::X w, GG::Y h, const std::string& str, GG::Flags<GG::TextFormat> format, GG::StateButtonStyle style/* = GG::SBSTYLE_3D_CHECKBOX*/,
-                               GG::Clr color/* = ClientUI::CtrlColor()*/, const boost::shared_ptr<GG::Font>& font/* = boost::shared_ptr<GG::Font>()*/,
-                               GG::Clr text_color/* = ClientUI::TextColor()*/, GG::Clr interior/* = GG::CLR_ZERO*/,
-                               GG::Clr border/* = ClientUI::CtrlBorderColor()*/, GG::Flags<GG::WndFlag> flags/* = GG::INTERACTIVE*/) :
-    StateButton(x, y, w, h, str, FontOrDefaultFont(font), format, color, text_color, interior, style, flags),
-    m_border_color(border),
+CUIStateButton::CUIStateButton(const std::string& str, GG::X x, GG::Y y, GG::X w, GG::Y h, GG::Flags<GG::TextFormat> format, GG::StateButtonStyle style/* = GG::SBSTYLE_3D_CHECKBOX*/) :
+    StateButton(x, y, w, h, str, ClientUI::GetFont(), format, ClientUI::StateButtonColor(), ClientUI::TextColor(), GG::CLR_ZERO, style, GG::INTERACTIVE),
+    m_border_color(ClientUI::CtrlBorderColor()),
     m_mouse_here(false)
 {
     if (style == GG::SBSTYLE_3D_TOP_DETACHED_TAB || style == GG::SBSTYLE_3D_TOP_ATTACHED_TAB) {
         SetColor(ClientUI::WndColor());
-        SetTextColor(DarkColor(text_color));
+        SetTextColor(DarkColor(ClientUI::TextColor()));
     }
     // HACK! radio buttons should only emit sounds when they are checked, and *not* when they are unchecked; currently, there's no 
     // other way to detect the difference between these two kinds of CUIStateButton within the CUIStateButton ctor other than
@@ -332,9 +317,8 @@ void CUIStateButton::Render() {
                 glVertex(bn_lr.x, bn_lr.y - OFFSET);
                 glVertex(bn_lr.x, bn_ul.y);
                 glEnd();
-                glPolygonMode(GL_FRONT_AND_BACK, GL_LINE);
                 glColor(outside_color);
-                glBegin(GL_POLYGON);
+                glBegin(GL_LINE_STRIP);
                 glVertex(bn_lr.x, bn_ul.y);
                 glVertex(bn_ul.x + OFFSET, bn_ul.y);
                 glVertex(bn_ul.x, bn_ul.y + OFFSET);
@@ -344,7 +328,6 @@ void CUIStateButton::Render() {
                 glVertex(bn_lr.x, bn_lr.y - OFFSET);
                 glVertex(bn_lr.x, bn_ul.y);
                 glEnd();
-                glPolygonMode(GL_FRONT_AND_BACK, GL_FILL);
                 glEnable(GL_TEXTURE_2D);
             } else {
                 GG::Clr inside_color = border_color_to_use;
@@ -553,11 +536,9 @@ void CUIScroll::ScrollTab::MouseLeave() {
 ///////////////////////////////////////
 // class CUIScroll
 ///////////////////////////////////////
-CUIScroll::CUIScroll(GG::X x, GG::Y y, GG::X w, GG::Y h, GG::Orientation orientation,
-                     GG::Clr border_color/* = ClientUI::CtrlBorderColor()*/, GG::Clr interior_color/* = ClientUI::CtrlColor()*/, 
-                     GG::Flags<GG::WndFlag> flags/* = INTERACTIVE | REPEAT_BUTTON_DOWN*/) :
-    Scroll(x, y, w, h, orientation, interior_color, interior_color, flags),
-    m_border_color(border_color)
+CUIScroll::CUIScroll(GG::X x, GG::Y y, GG::X w, GG::Y h, GG::Orientation orientation) :
+    Scroll(x, y, w, h, orientation, ClientUI::CtrlColor(), ClientUI::CtrlColor(), GG::INTERACTIVE | GG::REPEAT_BUTTON_DOWN),
+    m_border_color(ClientUI::CtrlBorderColor())
 {}
 
 void CUIScroll::Render() {
@@ -581,9 +562,8 @@ void CUIScroll::SizeMove(const GG::Pt& ul, const GG::Pt& lr) {
 ///////////////////////////////////////
 // class CUIListBox
 ///////////////////////////////////////
-CUIListBox::CUIListBox(GG::X x, GG::Y y, GG::X w, GG::Y h, GG::Clr border_color/* = ClientUI::CtrlBorderColor()*/, 
-                       GG::Clr interior_color/* = ClientUI::CtrlColor()*/, GG::Flags<GG::WndFlag> flags/* = INTERACTIVE*/) : 
-    ListBox(x, y, w, h, border_color, interior_color, flags)
+CUIListBox::CUIListBox(GG::X x, GG::Y y, GG::X w, GG::Y h):
+    ListBox(x, y, w, h, ClientUI::CtrlBorderColor(), ClientUI::CtrlColor(), GG::INTERACTIVE)
 {
     RecreateScrolls();
     GG::Connect(SelChangedSignal,   &PlayListSelectSound,   -1);
@@ -609,7 +589,7 @@ namespace {
 }
 
 CUIDropDownList::CUIDropDownList(GG::X x, GG::Y y, GG::X w, GG::Y h, GG::Y drop_ht, GG::Clr border_color/* = ClientUI::CtrlBorderColor()*/,
-                                 GG::Clr interior/* = ClientUI::WndColor()*/, GG::Flags<GG::WndFlag> flags/* = INTERACTIVE*/) : 
+                                 GG::Clr interior/* = ClientUI::WndColor()*/) :
     DropDownList(x, y, w, h, drop_ht, border_color),
     m_render_drop_arrow(true),
     m_mouse_here(false)
@@ -675,16 +655,23 @@ void CUIDropDownList::EnableDropArrow()
 ///////////////////////////////////////
 // class CUIEdit
 ///////////////////////////////////////
-CUIEdit::CUIEdit(GG::X x, GG::Y y, GG::X w, const std::string& str, const boost::shared_ptr<GG::Font>& font/* = boost::shared_ptr<GG::Font>()*/,
-                 GG::Clr border_color/* = ClientUI::CtrlBorderColor()*/, 
-                 GG::Clr text_color/* = ClientUI::TextColor()*/, GG::Clr interior/* = ClientUI::CtrlColor()*/, 
-                 GG::Flags<GG::WndFlag> flags/* = INTERACTIVE*/) : 
-    Edit(x, y, w, str, FontOrDefaultFont(font), border_color, text_color, interior, flags)
+CUIEdit::CUIEdit(GG::X x, GG::Y y, GG::X w, const std::string& str, GG::Clr border_color/* = ClientUI::CtrlBorderColor()*/, GG::Clr interior/* = ClientUI::CtrlColor()*/) :
+    Edit(x, y, w, str, ClientUI::GetFont(), border_color, ClientUI::TextColor(), interior, GG::INTERACTIVE | GG::REPEAT_KEY_PRESS)
 {
     GG::Connect(EditedSignal, &PlayTextTypingSound, -1);
     SetHiliteColor(ClientUI::EditHiliteColor());
 }
 
+void CUIEdit::GainingFocus() {
+    GG::Edit::GainingFocus();
+    GainingFocusSignal();
+}
+
+void CUIEdit::LosingFocus() {
+    GG::Edit::LosingFocus();
+    LosingFocusSignal();
+}
+
 void CUIEdit::Render() {
     GG::Clr color = Color();
     GG::Clr border_color = Disabled() ? DisabledColor(color) : color;
@@ -703,11 +690,8 @@ void CUIEdit::Render() {
 ///////////////////////////////////////
 // class CUIMultiEdit
 ///////////////////////////////////////
-CUIMultiEdit::CUIMultiEdit(GG::X x, GG::Y y, GG::X w, GG::Y h, const std::string& str, GG::Flags<GG::MultiEditStyle> style/* = MULTI_LINEWRAP*/,
-                           const boost::shared_ptr<GG::Font>& font/* = boost::shared_ptr<GG::Font>()*/,
-                           GG::Clr border_color/* = ClientUI::CtrlBorderColor()*/, GG::Clr text_color/* = ClientUI::TextColor()*/,
-                           GG::Clr interior/* = ClientUI::CtrlColor()*/, GG::Flags<GG::WndFlag> flags/* = INTERACTIVE*/) :
-    MultiEdit(x, y, w, h, str, FontOrDefaultFont(font), border_color, style, text_color, interior, flags)
+CUIMultiEdit::CUIMultiEdit(GG::X x, GG::Y y, GG::X w, GG::Y h, const std::string& str, GG::Flags<GG::MultiEditStyle> style/* = MULTI_LINEWRAP*/) :
+    MultiEdit(x, y, w, h, str, ClientUI::GetFont(), ClientUI::CtrlBorderColor(), style, ClientUI::TextColor(), ClientUI::CtrlColor(), GG::INTERACTIVE | GG::REPEAT_KEY_PRESS)
 {
     RecreateScrolls();
     SetHiliteColor(ClientUI::EditHiliteColor());
@@ -730,11 +714,8 @@ void CUIMultiEdit::Render() {
 ///////////////////////////////////////
 // class CUILinkTextMultiEdit
 ///////////////////////////////////////
-CUILinkTextMultiEdit::CUILinkTextMultiEdit(GG::X x, GG::Y y, GG::X w, GG::Y h, const std::string& str, GG::Flags<GG::MultiEditStyle> style,
-                                           const boost::shared_ptr<GG::Font>& font,
-                                           GG::Clr border_color, GG::Clr text_color, 
-                                           GG::Clr interior, GG::Flags<GG::WndFlag> flags) :
-    CUIMultiEdit(x, y, w, h, str, style, font, border_color, text_color, interior, flags),
+CUILinkTextMultiEdit::CUILinkTextMultiEdit(GG::X x, GG::Y y, GG::X w, GG::Y h, const std::string& str, GG::Flags<GG::MultiEditStyle> style) :
+    CUIMultiEdit(x, y, w, h, str, style),
     TextLinker(),
     m_already_setting_text_so_dont_link(false),
     m_raw_text(str)
@@ -793,9 +774,19 @@ void CUILinkTextMultiEdit::SizeMove(const GG::Pt& ul, const GG::Pt& lr) {
     if (Style() & GG::MULTI_INTEGRAL_HEIGHT)
         lower_right.y -= ((lr.y - ul.y) - (2 * PIXEL_MARGIN)) % GetFont()->Lineskip();
     bool resized = lower_right - ul != Size();
+
+    // need to restore scroll position after SetText call below, so that
+    // resizing this control doesn't reset the scroll position to the top.
+    // just calling PreserveTextPositionOnNextSetText() before the SetText
+    // call doesn't work as that leaves the scrollbars unadjusted for the resize
+    GG::Pt initial_scroll_pos = ScrollPosition();
+
     GG::Edit::SizeMove(ul, lower_right);
-    if (resized)
-        SetText(RawText());
+
+    if (resized) {
+        SetText(RawText());                     // this line is the primary difference between this and MultiEdit::SizeMove
+        SetScrollPosition(initial_scroll_pos);  // restores scroll position
+    }
 }
 
 void CUILinkTextMultiEdit::SetText(const std::string& str) {
@@ -846,9 +837,8 @@ namespace {
 }
 
 StatisticIcon::StatisticIcon(GG::X x, GG::Y y, GG::X w, GG::Y h, const boost::shared_ptr<GG::Texture> texture,
-                             double value, int digits, bool showsign,
-                             GG::Flags<GG::WndFlag> flags/* = GG::INTERACTIVE*/) :
-    GG::Control(x, y, w, h, flags),
+                             double value, int digits, bool showsign) :
+    GG::Control(x, y, w, h, GG::INTERACTIVE),
     m_num_values(1),
     m_values(std::vector<double>(1, value)), m_digits(std::vector<int>(1, digits)),
     m_show_signs(std::vector<bool>(1, showsign)),
@@ -858,13 +848,19 @@ StatisticIcon::StatisticIcon(GG::X x, GG::Y y, GG::X w, GG::Y h, const boost::sh
     // arrange child controls horizontally if icon is wider than it is high, or vertically otherwise
     if (w >= Value(h)) {
         m_icon = new GG::StaticGraphic(GG::X0, GG::Y0, GG::X(Value(h)), h, texture, GG::GRAPHIC_FITGRAPHIC);
-        m_text = new GG::TextControl(GG::X(Value(h) + STAT_ICON_PAD), GG::Y0, w - Value(h) - STAT_ICON_PAD, GG::Y(std::max(font_space, Value(h))), "", ClientUI::GetFont(), ClientUI::TextColor(), GG::FORMAT_LEFT | GG::FORMAT_VCENTER);
+        m_text = new GG::TextControl(GG::X(Value(h) + STAT_ICON_PAD), GG::Y0,
+                                     w - Value(h) - STAT_ICON_PAD, GG::Y(std::max(font_space, Value(h))),
+                                     "", ClientUI::GetFont(), ClientUI::TextColor(),
+                                     GG::FORMAT_LEFT | GG::FORMAT_VCENTER);
     } else {
         // need vertical space for text, but don't want icon to be larger than available horizontal space
         int icon_height = std::min(Value(w), std::max(Value(h - font_space - STAT_ICON_PAD), 1));
         int icon_left = Value(w - icon_height)/2;
-        m_icon = new GG::StaticGraphic(GG::X(icon_left), GG::Y0, GG::X(icon_height), GG::Y(icon_height), texture, GG::GRAPHIC_FITGRAPHIC);
-        m_text = new GG::TextControl(GG::X0, GG::Y(icon_height + STAT_ICON_PAD), w, GG::Y(font_space), "", ClientUI::GetFont(), ClientUI::TextColor(), GG::FORMAT_CENTER | GG::FORMAT_TOP);
+        m_icon = new GG::StaticGraphic(GG::X(icon_left), GG::Y0, GG::X(icon_height),
+                                       GG::Y(icon_height), texture, GG::GRAPHIC_FITGRAPHIC);
+        m_text = new GG::TextControl(GG::X0, GG::Y(icon_height + STAT_ICON_PAD), w, GG::Y(font_space),
+                                     "", ClientUI::GetFont(), ClientUI::TextColor(),
+                                     GG::FORMAT_CENTER | GG::FORMAT_TOP);
     }
 
     SetBrowseModeTime(GetOptionsDB().Get<int>("UI.tooltip-delay"));
@@ -876,9 +872,8 @@ StatisticIcon::StatisticIcon(GG::X x, GG::Y y, GG::X w, GG::Y h, const boost::sh
 
 StatisticIcon::StatisticIcon(GG::X x, GG::Y y, GG::X w, GG::Y h, const boost::shared_ptr<GG::Texture> texture,
                              double value0, double value1, int digits0, int digits1,
-                             bool showsign0, bool showsign1,
-                             GG::Flags<GG::WndFlag> flags/* = GG::INTERACTIVE*/) :
-    GG::Control(x, y, w, h, flags),
+                             bool showsign0, bool showsign1) :
+    GG::Control(x, y, w, h, GG::INTERACTIVE),
     m_num_values(2),
     m_values(std::vector<double>(2, 0.0)), m_digits(std::vector<int>(2, 2)),
     m_show_signs(std::vector<bool>(2, false)),
@@ -888,10 +883,14 @@ StatisticIcon::StatisticIcon(GG::X x, GG::Y y, GG::X w, GG::Y h, const boost::sh
     // arrange child controls horizontally if icon is wider than it is high, or vertically otherwise
     if (w >= Value(h)) {
         m_icon = new GG::StaticGraphic(GG::X0, GG::Y0, GG::X(Value(h)), h, texture, GG::GRAPHIC_FITGRAPHIC);
-        m_text = new GG::TextControl(GG::X(Value(h) + STAT_ICON_PAD), GG::Y0, w - Value(h) - STAT_ICON_PAD, std::max(GG::Y(ClientUI::Pts()*3/2), h), "", ClientUI::GetFont(), ClientUI::TextColor(), GG::FORMAT_LEFT | GG::FORMAT_VCENTER);
+        m_text = new GG::TextControl(GG::X(Value(h) + STAT_ICON_PAD), GG::Y0,
+                                     w - Value(h) - STAT_ICON_PAD, std::max(GG::Y(ClientUI::Pts()*3/2), h),
+                                     "", ClientUI::GetFont(), ClientUI::TextColor(), GG::FORMAT_LEFT | GG::FORMAT_VCENTER);
     } else {
         m_icon = new GG::StaticGraphic(GG::X0, GG::Y0, w, GG::Y(Value(w)), texture, GG::GRAPHIC_FITGRAPHIC);
-        m_text = new GG::TextControl(GG::X0, GG::Y(Value(w) + STAT_ICON_PAD), w, GG::Y(ClientUI::Pts()*3/2), "", ClientUI::GetFont(), ClientUI::TextColor(), GG::FORMAT_CENTER | GG::FORMAT_BOTTOM);
+        m_text = new GG::TextControl(GG::X0, GG::Y(Value(w) + STAT_ICON_PAD),
+                                     w, GG::Y(ClientUI::Pts()*3/2),
+                                     "", ClientUI::GetFont(), ClientUI::TextColor(), GG::FORMAT_CENTER | GG::FORMAT_BOTTOM);
     }
 
     m_values[0] = value0;
@@ -934,6 +933,16 @@ void StatisticIcon::LButtonDown(const GG::Pt& pt, GG::Flags<GG::ModKey> mod_keys
 void StatisticIcon::RButtonDown(const GG::Pt& pt, GG::Flags<GG::ModKey> mod_keys)
 { ForwardEventToParent(); }
 
+void StatisticIcon::LClick(const GG::Pt& pt, GG::Flags<GG::ModKey> mod_keys) {
+    if (!Disabled())
+        LeftClickedSignal();
+}
+
+void StatisticIcon::RClick(const GG::Pt& pt, GG::Flags<GG::ModKey> mod_keys) {
+    if (!Disabled())
+        RightClickedSignal();
+}
+
 void StatisticIcon::MouseWheel(const GG::Pt& pt, int move, GG::Flags<GG::ModKey> mod_keys)
 { ForwardEventToParent(); }
 
@@ -995,7 +1004,7 @@ namespace {
     // row type used in the SpeciesSelector
     struct SpeciesRow : public GG::ListBox::Row {
         SpeciesRow(const Species* species, GG::X w, GG::Y h) :
-            GG::ListBox::Row(w, h, "SpeciesRow", GG::ALIGN_VCENTER, 0)
+            GG::ListBox::Row(w, h, "", GG::ALIGN_VCENTER, 0)
         {
             if (!species)
                 return;
@@ -1113,7 +1122,7 @@ namespace {
     struct ColorRow : public GG::ListBox::Row {
         struct ColorSquare : GG::Control {
             ColorSquare(const GG::Clr& color, GG::Y h) :
-                GG::Control(GG::X0, GG::Y0, COLOR_SELECTOR_WIDTH - 40, h, GG::Flags<GG::WndFlag>())
+                GG::Control(GG::X0, GG::Y0, COLOR_SELECTOR_WIDTH - 40, h, GG::NO_WND_FLAGS)
             {
                 SetColor(color);
             }
@@ -1122,7 +1131,7 @@ namespace {
             }
         };
         ColorRow(const GG::Clr& color, GG::Y h) :
-            GG::ListBox::Row(GG::X(Value(h)), h, "ColorRow")
+            GG::ListBox::Row(GG::X(Value(h)), h, "")
         {
             push_back(new ColorSquare(color, h));
         }
@@ -1202,7 +1211,7 @@ void ColorSelector::RClick(const GG::Pt& pt, GG::Flags<GG::ModKey> mod_keys) {
     menu_contents.next_level.push_back(GG::MenuItem(UserString("RESET"), 1, false, false));
 
     GG::PopupMenu popup(pt.x, pt.y, ClientUI::GetFont(), menu_contents, ClientUI::TextColor(),
-                        ClientUI::WndOuterBorderColor(), ClientUI::WndColor());
+                        ClientUI::WndOuterBorderColor(), ClientUI::WndColor(), ClientUI::EditHiliteColor());
 
     if (popup.Run()) {
         switch (popup.MenuID()) {
@@ -1250,7 +1259,7 @@ const GG::Y ProductionInfoPanel::VERTICAL_SECTION_GAP(4);
 
 ProductionInfoPanel::ProductionInfoPanel(GG::X w, GG::Y h, const std::string& title, const std::string& points_str,
                                          float border_thickness, const GG::Clr& color, const GG::Clr& text_and_border_color) :
-    GG::Wnd(GG::X0, GG::Y0, w, h, GG::Flags<GG::WndFlag>()),
+    GG::Wnd(GG::X0, GG::Y0, w, h, GG::NO_WND_FLAGS),
     m_border_thickness(border_thickness),
     m_color(color),
     m_text_and_border_color(text_and_border_color)
@@ -1268,21 +1277,21 @@ ProductionInfoPanel::ProductionInfoPanel(GG::X w, GG::Y h, const std::string& ti
     m_center_gap = std::make_pair(Value(LABEL_TEXT_WIDTH + 2), Value(LABEL_TEXT_WIDTH + 2 + CENTERLINE_GAP));
 
     m_title = new GG::TextControl(GG::X(2), GG::Y(4), Width() - 4, GG::Y(RESEARCH_TITLE_PTS + 4), title, ClientUI::GetFont(RESEARCH_TITLE_PTS), TEXT_COLOR);
-    m_total_points_label = new GG::TextControl(LEFT_TEXT_X, m_title->LowerRight().y + VERTICAL_SECTION_GAP + 4, LABEL_TEXT_WIDTH, GG::Y(STAT_TEXT_PTS + 4), UserString("PRODUCTION_INFO_TOTAL_PS_LABEL"), ClientUI::GetFont(STAT_TEXT_PTS), TEXT_COLOR, GG::FORMAT_RIGHT);
-    m_total_points = new GG::TextControl(RIGHT_TEXT_X, m_title->LowerRight().y + VERTICAL_SECTION_GAP + 4, VALUE_TEXT_WIDTH, GG::Y(STAT_TEXT_PTS + 4), "", ClientUI::GetFont(STAT_TEXT_PTS), TEXT_COLOR, GG::FORMAT_LEFT);
-    m_total_points_P_label = new GG::TextControl(P_LABEL_X, m_title->LowerRight().y + VERTICAL_SECTION_GAP + 4, P_LABEL_WIDTH, GG::Y(STAT_TEXT_PTS + 4), points_str, ClientUI::GetFont(STAT_TEXT_PTS), TEXT_COLOR, GG::FORMAT_LEFT);
-    m_wasted_points_label = new GG::TextControl(LEFT_TEXT_X, m_total_points_label->LowerRight().y, LABEL_TEXT_WIDTH, GG::Y(STAT_TEXT_PTS + 4), UserString("PRODUCTION_INFO_WASTED_PS_LABEL"), ClientUI::GetFont(STAT_TEXT_PTS), TEXT_COLOR, GG::FORMAT_RIGHT);
-    m_wasted_points = new GG::TextControl(RIGHT_TEXT_X, m_total_points_label->LowerRight().y, VALUE_TEXT_WIDTH, GG::Y(STAT_TEXT_PTS + 4), "", ClientUI::GetFont(STAT_TEXT_PTS), TEXT_COLOR, GG::FORMAT_LEFT);
-    m_wasted_points_P_label = new GG::TextControl(P_LABEL_X, m_total_points_label->LowerRight().y, P_LABEL_WIDTH, GG::Y(STAT_TEXT_PTS + 4), points_str, ClientUI::GetFont(STAT_TEXT_PTS), TEXT_COLOR, GG::FORMAT_LEFT);
-    m_projects_in_progress_label = new GG::TextControl(LEFT_TEXT_X, m_wasted_points_label->LowerRight().y + VERTICAL_SECTION_GAP + 4, LABEL_TEXT_WIDTH, GG::Y(STAT_TEXT_PTS + 4), UserString("PRODUCTION_INFO_PROJECTS_IN_PROGRESS_LABEL"), ClientUI::GetFont(STAT_TEXT_PTS), TEXT_COLOR, GG::FORMAT_RIGHT);
-    m_projects_in_progress = new GG::TextControl(RIGHT_TEXT_X, m_wasted_points_label->LowerRight().y + VERTICAL_SECTION_GAP + 4, VALUE_TEXT_WIDTH, GG::Y(STAT_TEXT_PTS + 4), "", ClientUI::GetFont(STAT_TEXT_PTS), TEXT_COLOR, GG::FORMAT_LEFT);
-    m_points_to_underfunded_projects_label = new GG::TextControl(LEFT_TEXT_X, m_projects_in_progress_label->LowerRight().y, LABEL_TEXT_WIDTH, GG::Y(STAT_TEXT_PTS + 4), UserString("PRODUCTION_INFO_PS_TO_UNDERFUNDED_PROJECTS_LABEL"), ClientUI::GetFont(STAT_TEXT_PTS), TEXT_COLOR, GG::FORMAT_RIGHT);
-    m_points_to_underfunded_projects = new GG::TextControl(RIGHT_TEXT_X, m_projects_in_progress_label->LowerRight().y, VALUE_TEXT_WIDTH, GG::Y(STAT_TEXT_PTS + 4), "", ClientUI::GetFont(STAT_TEXT_PTS), TEXT_COLOR, GG::FORMAT_LEFT);
-    m_points_to_underfunded_projects_P_label = new GG::TextControl(P_LABEL_X, m_projects_in_progress_label->LowerRight().y, P_LABEL_WIDTH, GG::Y(STAT_TEXT_PTS + 4), points_str, ClientUI::GetFont(STAT_TEXT_PTS), TEXT_COLOR, GG::FORMAT_LEFT);
-    m_projects_in_queue_label = new GG::TextControl(LEFT_TEXT_X, m_points_to_underfunded_projects_label->LowerRight().y, LABEL_TEXT_WIDTH, GG::Y(STAT_TEXT_PTS + 4), UserString("PRODUCTION_INFO_PROJECTS_IN_QUEUE_LABEL"), ClientUI::GetFont(STAT_TEXT_PTS), TEXT_COLOR, GG::FORMAT_RIGHT);
-    m_projects_in_queue = new GG::TextControl(RIGHT_TEXT_X, m_points_to_underfunded_projects_label->LowerRight().y, VALUE_TEXT_WIDTH, GG::Y(STAT_TEXT_PTS + 4), "", ClientUI::GetFont(STAT_TEXT_PTS), TEXT_COLOR, GG::FORMAT_LEFT);
+    m_total_points_label = new GG::TextControl(LEFT_TEXT_X, m_title->Bottom() + VERTICAL_SECTION_GAP + 4, LABEL_TEXT_WIDTH, GG::Y(STAT_TEXT_PTS + 4), UserString("PRODUCTION_INFO_TOTAL_PS_LABEL"), ClientUI::GetFont(STAT_TEXT_PTS), TEXT_COLOR, GG::FORMAT_RIGHT);
+    m_total_points = new GG::TextControl(RIGHT_TEXT_X, m_title->Bottom() + VERTICAL_SECTION_GAP + 4, VALUE_TEXT_WIDTH, GG::Y(STAT_TEXT_PTS + 4), "", ClientUI::GetFont(STAT_TEXT_PTS), TEXT_COLOR, GG::FORMAT_LEFT);
+    m_total_points_P_label = new GG::TextControl(P_LABEL_X, m_title->Bottom() + VERTICAL_SECTION_GAP + 4, P_LABEL_WIDTH, GG::Y(STAT_TEXT_PTS + 4), points_str, ClientUI::GetFont(STAT_TEXT_PTS), TEXT_COLOR, GG::FORMAT_LEFT);
+    m_wasted_points_label = new GG::TextControl(LEFT_TEXT_X, m_total_points_label->Bottom(), LABEL_TEXT_WIDTH, GG::Y(STAT_TEXT_PTS + 4), UserString("PRODUCTION_INFO_WASTED_PS_LABEL"), ClientUI::GetFont(STAT_TEXT_PTS), TEXT_COLOR, GG::FORMAT_RIGHT);
+    m_wasted_points = new GG::TextControl(RIGHT_TEXT_X, m_total_points_label->Bottom(), VALUE_TEXT_WIDTH, GG::Y(STAT_TEXT_PTS + 4), "", ClientUI::GetFont(STAT_TEXT_PTS), TEXT_COLOR, GG::FORMAT_LEFT);
+    m_wasted_points_P_label = new GG::TextControl(P_LABEL_X, m_total_points_label->Bottom(), P_LABEL_WIDTH, GG::Y(STAT_TEXT_PTS + 4), points_str, ClientUI::GetFont(STAT_TEXT_PTS), TEXT_COLOR, GG::FORMAT_LEFT);
+    m_projects_in_progress_label = new GG::TextControl(LEFT_TEXT_X, m_wasted_points_label->Bottom() + VERTICAL_SECTION_GAP + 4, LABEL_TEXT_WIDTH, GG::Y(STAT_TEXT_PTS + 4), UserString("PRODUCTION_INFO_PROJECTS_IN_PROGRESS_LABEL"), ClientUI::GetFont(STAT_TEXT_PTS), TEXT_COLOR, GG::FORMAT_RIGHT);
+    m_projects_in_progress = new GG::TextControl(RIGHT_TEXT_X, m_wasted_points_label->Bottom() + VERTICAL_SECTION_GAP + 4, VALUE_TEXT_WIDTH, GG::Y(STAT_TEXT_PTS + 4), "", ClientUI::GetFont(STAT_TEXT_PTS), TEXT_COLOR, GG::FORMAT_LEFT);
+    m_points_to_underfunded_projects_label = new GG::TextControl(LEFT_TEXT_X, m_projects_in_progress_label->Bottom(), LABEL_TEXT_WIDTH, GG::Y(STAT_TEXT_PTS + 4), UserString("PRODUCTION_INFO_PS_TO_UNDERFUNDED_PROJECTS_LABEL"), ClientUI::GetFont(STAT_TEXT_PTS), TEXT_COLOR, GG::FORMAT_RIGHT);
+    m_points_to_underfunded_projects = new GG::TextControl(RIGHT_TEXT_X, m_projects_in_progress_label->Bottom(), VALUE_TEXT_WIDTH, GG::Y(STAT_TEXT_PTS + 4), "", ClientUI::GetFont(STAT_TEXT_PTS), TEXT_COLOR, GG::FORMAT_LEFT);
+    m_points_to_underfunded_projects_P_label = new GG::TextControl(P_LABEL_X, m_projects_in_progress_label->Bottom(), P_LABEL_WIDTH, GG::Y(STAT_TEXT_PTS + 4), points_str, ClientUI::GetFont(STAT_TEXT_PTS), TEXT_COLOR, GG::FORMAT_LEFT);
+    m_projects_in_queue_label = new GG::TextControl(LEFT_TEXT_X, m_points_to_underfunded_projects_label->Bottom(), LABEL_TEXT_WIDTH, GG::Y(STAT_TEXT_PTS + 4), UserString("PRODUCTION_INFO_PROJECTS_IN_QUEUE_LABEL"), ClientUI::GetFont(STAT_TEXT_PTS), TEXT_COLOR, GG::FORMAT_RIGHT);
+    m_projects_in_queue = new GG::TextControl(RIGHT_TEXT_X, m_points_to_underfunded_projects_label->Bottom(), VALUE_TEXT_WIDTH, GG::Y(STAT_TEXT_PTS + 4), "", ClientUI::GetFont(STAT_TEXT_PTS), TEXT_COLOR, GG::FORMAT_LEFT);
 
-    Resize(GG::Pt(Width(), m_projects_in_queue_label->LowerRight().y + 5));
+    Resize(GG::Pt(Width(), m_projects_in_queue_label->Bottom() + 5));
 
     AttachChild(m_title);
     AttachChild(m_total_points_label);
@@ -1327,16 +1336,16 @@ void ProductionInfoPanel::Draw(GG::Clr clr, bool fill) {
     GG::Pt square_3(GG::X(3), GG::Y(3));
     GG::Pt ul = UpperLeft() + square_3, lr = LowerRight() - square_3;
     glColor(clr);
-    PartlyRoundedRect(ul, GG::Pt(lr.x, m_title->LowerRight().y + 2),
+    PartlyRoundedRect(ul, GG::Pt(lr.x, m_title->Bottom() + 2),
                       CORNER_RADIUS, true, true, false, false, fill);
     std::pair<GG::X, GG::X> gap_to_use(m_center_gap.first + ul.x, m_center_gap.second + ul.x);
-    PartlyRoundedRect(GG::Pt(ul.x, m_total_points_label->UpperLeft().y - 2), GG::Pt(gap_to_use.first, m_wasted_points_label->LowerRight().y + 2),
+    PartlyRoundedRect(GG::Pt(ul.x, m_total_points_label->Top() - 2), GG::Pt(gap_to_use.first, m_wasted_points_label->Bottom() + 2),
                       CORNER_RADIUS, false, false, false, false, fill);
-    PartlyRoundedRect(GG::Pt(gap_to_use.second, m_total_points_label->UpperLeft().y - 2), GG::Pt(lr.x, m_wasted_points_label->LowerRight().y + 2),
+    PartlyRoundedRect(GG::Pt(gap_to_use.second, m_total_points_label->Top() - 2), GG::Pt(lr.x, m_wasted_points_label->Bottom() + 2),
                       CORNER_RADIUS, false, false, false, false, fill);
-    PartlyRoundedRect(GG::Pt(ul.x, m_projects_in_progress_label->UpperLeft().y - 2), GG::Pt(gap_to_use.first, m_projects_in_queue_label->LowerRight().y + 2),
+    PartlyRoundedRect(GG::Pt(ul.x, m_projects_in_progress_label->Top() - 2), GG::Pt(gap_to_use.first, m_projects_in_queue_label->Bottom() + 2),
                       CORNER_RADIUS, false, false, true, false, fill);
-    PartlyRoundedRect(GG::Pt(gap_to_use.second, m_projects_in_progress_label->UpperLeft().y - 2), GG::Pt(lr.x, m_projects_in_queue_label->LowerRight().y + 2),
+    PartlyRoundedRect(GG::Pt(gap_to_use.second, m_projects_in_progress_label->Top() - 2), GG::Pt(lr.x, m_projects_in_queue_label->Bottom() + 2),
                       CORNER_RADIUS, false, false, false, true, fill);
 }
 
@@ -1346,7 +1355,7 @@ void ProductionInfoPanel::Draw(GG::Clr clr, bool fill) {
 MultiTurnProgressBar::MultiTurnProgressBar(GG::X w, GG::Y h, int total_turns, double turns_completed,
                                            const GG::Clr& bar_color, const GG::Clr& background,
                                            const GG::Clr& outline_color) :
-    Control(GG::X0, GG::Y0, w, h, GG::Flags<GG::WndFlag>()),
+    Control(GG::X0, GG::Y0, w, h, GG::NO_WND_FLAGS),
     m_total_turns(std::max(1, total_turns)),
     m_turns_completed(std::max(0.0, std::min<double>(turns_completed, m_total_turns))),
     m_bar_color(bar_color),
@@ -1408,41 +1417,78 @@ void FPSIndicator::UpdateEnabled()
 //////////////////////////////////////////////////
 // ShadowedTextControl
 //////////////////////////////////////////////////
+
+// Create a regex that recognizes a tag
+boost::regex TagRegex(const std::string& tag){
+   return boost::regex(std::string("<")+tag+"[^<>]*>(.*)</"+tag+">");
+}
+
+
+std::string RemoveRGB(const std::string& text){
+    static boost::regex rx("<rgba[^<>]*>|</rgba>");// = TagRegex("rgba");
+    return boost::regex_replace(text, rx, "", boost::match_default | boost::format_all);
+}
+
 ShadowedTextControl::ShadowedTextControl(GG::X x, GG::Y y, GG::X w, GG::Y h, const std::string& str,
                                          const boost::shared_ptr<GG::Font>& font,
-                                         GG::Clr color, GG::Flags<GG::TextFormat> format,
-                                         GG::Flags<GG::WndFlag> flags) :
-    GG::TextControl(x, y, w, h, str, font, color, format, flags)
-{}
+                                         GG::Clr color, GG::Flags<GG::TextFormat> format) :
+    GG::Control(x, y, w, h, GG::NO_WND_FLAGS),
+    shadow_text(new GG::TextControl(GG::X0, GG::Y0, w, h, RemoveRGB(str), font, GG::CLR_BLACK, format, GG::NO_WND_FLAGS)),
+    main_text(new GG::TextControl(GG::X0, GG::Y1, w, h, str, font, color, format, GG::NO_WND_FLAGS))
+{
+    AttachChild(shadow_text);
+    AttachChild(main_text);
+}
 
 ShadowedTextControl::ShadowedTextControl(GG::X x, GG::Y y, const std::string& str,
                                          const boost::shared_ptr<GG::Font>& font,
-                                         GG::Clr color, GG::Flags<GG::TextFormat> format,
-                                         GG::Flags<GG::WndFlag> flags) :
-    GG::TextControl(x, y, str, font, color, format, flags)
-{}
+                                         GG::Clr color, GG::Flags<GG::TextFormat> format) :
+    GG::Control(x, y, GG::X1, GG::Y1, GG::NO_WND_FLAGS),
+    shadow_text(new GG::TextControl(GG::X0, GG::Y0, RemoveRGB(str), font, GG::CLR_BLACK, format, GG::NO_WND_FLAGS)),
+    main_text(new GG::TextControl(GG::X0, GG::Y0, str, font, color, format, GG::NO_WND_FLAGS))
+{
+    Resize(main_text->Size());
+    AttachChild(shadow_text);
+    AttachChild(main_text);
+}
+
+GG::Pt ShadowedTextControl::MinUsableSize() const {
+    return main_text->MinUsableSize();
+}
+
+void ShadowedTextControl::SetText(const std::string& str){
+    shadow_text->SetText(str);
+    main_text->SetText(str);
+}
+
+void ShadowedTextControl::SizeMove(const GG::Pt& ul, const GG::Pt& lr){
+    GG::Control::SizeMove(ul, lr);
+    main_text->Resize(Size());
+    shadow_text->Resize(Size());
+}
+
+void ShadowedTextControl::SetColor(GG::Clr c){
+    main_text->SetColor(c);
+}
+
+void ShadowedTextControl::SetTextColor ( GG::Clr c ) {
+    main_text->SetTextColor(c);
+}
 
 void ShadowedTextControl::Render() {
-    GG::Clr text_colour = TextColor();          // save original colour
+    shadow_text->OffsetMove(GG::Pt(-GG::X1, GG::Y(0)));      // shadow to left
+    shadow_text->Render();
 
-    SetTextColor(GG::CLR_BLACK);                // render shadows in opaque black
+    shadow_text->OffsetMove(GG::Pt(GG::X1, GG::Y1));         // up
+    shadow_text->Render();
 
-    OffsetMove(GG::Pt(-GG::X1, GG::Y(0)));      // shadow to left
-    TextControl::Render();
+    shadow_text->OffsetMove(GG::Pt(GG::X1, -GG::Y1));        // right
+    shadow_text->Render();
 
-    OffsetMove(GG::Pt(GG::X1, GG::Y1));         // up
-    TextControl::Render();
-
-    OffsetMove(GG::Pt(GG::X1, -GG::Y1));        // right
-    TextControl::Render();
-
-    OffsetMove(GG::Pt(-GG::X1, -GG::Y1));       // down
-    TextControl::Render();
-
-    SetTextColor(text_colour);                  // restore original colour
-
-    OffsetMove(GG::Pt(GG::X(0), GG::Y1));       // render main coloured text
-    TextControl::Render();
+    shadow_text->OffsetMove(GG::Pt(-GG::X1, -GG::Y1));       // down
+    shadow_text->Render();
+    
+    shadow_text->OffsetMove(GG::Pt(GG::X0, GG::Y1));       // center
 }
 
 
@@ -1455,8 +1501,8 @@ void ShadowedTextControl::Render() {
       * if \a styles is not specified or contains fewer entres than \a textures, entries in \a textures without 
       * associated styles use the style GRAPHIC_NONE. */
 MultiTextureStaticGraphic::MultiTextureStaticGraphic(GG::X x, GG::Y y, GG::X w, GG::Y h, const std::vector<boost::shared_ptr<GG::Texture> >& textures,
-                                                     const std::vector<GG::Flags<GG::GraphicStyle> >& styles, GG::Flags<GG::WndFlag> flags) :
-    GG::Control(x, y, w, h, flags),
+                                                     const std::vector<GG::Flags<GG::GraphicStyle> >& styles) :
+    GG::Control(x, y, w, h, GG::NO_WND_FLAGS),
     m_graphics(),
     m_styles(styles)
 {
@@ -1466,8 +1512,8 @@ MultiTextureStaticGraphic::MultiTextureStaticGraphic(GG::X x, GG::Y y, GG::X w,
 }
 
 MultiTextureStaticGraphic::MultiTextureStaticGraphic(GG::X x, GG::Y y, GG::X w, GG::Y h, const std::vector<GG::SubTexture>& subtextures,
-                                                     const std::vector<GG::Flags<GG::GraphicStyle> >& styles, GG::Flags<GG::WndFlag> flags) :
-    GG::Control(x, y, w, h, flags),
+                                                     const std::vector<GG::Flags<GG::GraphicStyle> >& styles) :
+    GG::Control(x, y, w, h, GG::NO_WND_FLAGS),
     m_graphics(subtextures),
     m_styles(styles)
 { Init(); }
