@@ -30,12 +30,12 @@
 class CUIButton : public GG::Button {
 public:
     /** \name Structors */ //@{
-    CUIButton(GG::X x, GG::Y y, GG::X w, const std::string& str, const boost::shared_ptr<GG::Font>& font = boost::shared_ptr<GG::Font>(), 
-              GG::Clr color = ClientUI::CtrlColor(), GG::Clr border = ClientUI::CtrlBorderColor(), int thick = 1, 
-              GG::Clr text_color = ClientUI::TextColor(), GG::Flags<GG::WndFlag> flags = GG::INTERACTIVE); ///< basic ctor
+    CUIButton(const std::string& str, GG::X x = GG::X0, GG::Y y = GG::Y0, GG::X w = GG::X1, GG::Clr color = ClientUI::CtrlColor()); ///< basic ctor
     //@}
 
     /** \name Accessors */ //@{
+    virtual GG::Pt MinUsableSize() const;
+
     GG::Clr      BorderColor() const {return m_border_color;} ///< returns the color used to render the border of the button
     int          BorderThickness() const {return m_border_thick;} ///< returns the width used to render the border of the button
 
@@ -68,11 +68,8 @@ private:
 class SettableInWindowCUIButton : public CUIButton {
 public:
     /** \name Structors */ //@{
-    SettableInWindowCUIButton(GG::X x, GG::Y y, GG::X w, const std::string& str,
-                              const boost::shared_ptr<GG::Font>& font = boost::shared_ptr<GG::Font>(),
-                              GG::Clr color = ClientUI::CtrlColor(), GG::Clr border = ClientUI::CtrlBorderColor(),
-                              int thick = 1,  GG::Clr text_color = ClientUI::TextColor(),
-                              GG::Flags<GG::WndFlag> flags = GG::INTERACTIVE); ///< basic ctor
+    SettableInWindowCUIButton(const std::string& str, GG::X x, GG::Y y, GG::X w,
+                              GG::Clr color = ClientUI::CtrlColor()); ///< basic ctor
     //@}
 
     /** \name Accessors */ //@{
@@ -87,24 +84,12 @@ private:
     boost::function<bool(const GG::Pt&)>    m_in_window_func;
 };
 
-/** a FreeOrion next-turn button control */
-class CUITurnButton : public CUIButton {
-public:
-    /** \name Structors */ //@{
-    CUITurnButton(GG::X x, GG::Y y, GG::X w, const std::string& str,
-                  const boost::shared_ptr<GG::Font>& font = boost::shared_ptr<GG::Font>(), 
-                  GG::Clr color = ClientUI::CtrlColor(), GG::Clr border = ClientUI::CtrlBorderColor(),
-                  int thick = 1,  GG::Clr text_color = ClientUI::TextColor(),
-                  GG::Flags<GG::WndFlag> flags = GG::INTERACTIVE); ///< basic ctor
-    //@}
-};
-
 /** a FreeOrion triangular arrow button */
 class CUIArrowButton : public GG::Button {
 public:
     /** \name Structors */ //@{
     CUIArrowButton(GG::X x, GG::Y y, GG::X w, GG::Y h, ShapeOrientation orientation,
-                   GG::Clr color, GG::Flags<GG::WndFlag> flags = GG::INTERACTIVE); ///< basic ctor
+                   GG::Flags<GG::WndFlag> flags = GG::INTERACTIVE); ///< basic ctor
     //@}
 
     /** \name Accessors */ //@{
@@ -133,10 +118,7 @@ private:
 class CUIStateButton : public GG::StateButton {
 public:
     /** \name Structors */ //@{
-    CUIStateButton(GG::X x, GG::Y y, GG::X w, GG::Y h, const std::string& str, GG::Flags<GG::TextFormat> format, GG::StateButtonStyle style = GG::SBSTYLE_3D_CHECKBOX,
-                   GG::Clr color = ClientUI::StateButtonColor(), const boost::shared_ptr<GG::Font>& font = boost::shared_ptr<GG::Font>(),
-                   GG::Clr text_color = ClientUI::TextColor(), GG::Clr interior = GG::CLR_ZERO,
-                   GG::Clr border = ClientUI::CtrlBorderColor(), GG::Flags<GG::WndFlag> flags = GG::INTERACTIVE); ///< ctor
+    CUIStateButton(const std::string& str, GG::X x, GG::Y y, GG::X w, GG::Y h, GG::Flags<GG::TextFormat> format, GG::StateButtonStyle style = GG::SBSTYLE_3D_CHECKBOX); ///< ctor
     //@}
 
     /** \name Accessors */ //@{
@@ -191,9 +173,7 @@ public:
     };
 
     /** \name Structors */ //@{
-    CUIScroll(GG::X x, GG::Y y, GG::X w, GG::Y h, GG::Orientation orientation,
-              GG::Clr border_color = ClientUI::CtrlBorderColor(), GG::Clr interior_color = ClientUI::CtrlColor(),
-              GG::Flags<GG::WndFlag> flags = GG::INTERACTIVE | GG::REPEAT_BUTTON_DOWN); ///< basic ctor
+    CUIScroll(GG::X x, GG::Y y, GG::X w, GG::Y h, GG::Orientation orientation); ///< basic ctor
     //@}
 
     /** \name Mutators */ //@{
@@ -209,8 +189,7 @@ protected:
 class CUIListBox : public GG::ListBox {
 public:
     /** \name Structors */ //@{
-    CUIListBox(GG::X x, GG::Y y, GG::X w, GG::Y h, GG::Clr border_color = ClientUI::CtrlBorderColor(), GG::Clr interior_color = ClientUI::CtrlColor(),
-               GG::Flags<GG::WndFlag> flags = GG::INTERACTIVE); ///< basic ctor
+    CUIListBox(GG::X x, GG::Y y, GG::X w, GG::Y h); ///< basic ctor
     //@}
 
     /** \name Mutators */ //@{
@@ -218,33 +197,12 @@ public:
     //@}
 };
 
-/** a ListBox with user-sortable columns */
-class CUISortListBox : public GG::ListBox {
-public:
-    /** \name Structors */ //@{
-    CUISortListBox(GG::X x, GG::Y y, GG::X w, GG::Y h, GG::Clr color = ClientUI::CtrlBorderColor(), GG::Clr interior = GG::CLR_ZERO,
-                   GG::Flags<GG::WndFlag> flags = GG::INTERACTIVE);
-    //@}
-
-    /** \name Mutators */ //@{
-    virtual void Render();
-
-    void SetSortCol(int n);                 ///< sets column used to sort rows, updates sort-button appearance
-    void SetSortDirecion(bool ascending);   ///< sets whether to sort ascending (true) or descending (false), updates sort-button appearance
-    void SetColHeaders(std::map<int, boost::shared_ptr<GG::Texture> > textures_map);    ///< indexed by column number, provides texture with which to label column
-    void SetColHeaders(std::map<int, std::string> label_text_map);    ///< indexed by column number, provides text with which to label column
-    //@}
-
-private:
-    Row* m_header_row;  ///< need to keep own pointer to header row, since ListBox doesn't provide non-const access to its header row pointer
-};
-
 /** a FreeOrion DropDownList control */
 class CUIDropDownList : public GG::DropDownList {
 public:
     /** \name Structors */ //@{
     CUIDropDownList(GG::X x, GG::Y y, GG::X w, GG::Y h, GG::Y drop_ht, GG::Clr border_color = ClientUI::CtrlBorderColor(),
-                    GG::Clr interior = ClientUI::CtrlColor(), GG::Flags<GG::WndFlag> flags = GG::INTERACTIVE); ///< basic ctor
+                    GG::Clr interior = ClientUI::CtrlColor()); ///< basic ctor
     //@}
 
     /** \name Mutators */ //@{
@@ -266,24 +224,24 @@ private:
 class CUIEdit : public GG::Edit {
 public:
     /** \name Structors */ //@{
-    CUIEdit(GG::X x, GG::Y y, GG::X w, const std::string& str, const boost::shared_ptr<GG::Font>& font = boost::shared_ptr<GG::Font>(),
-            GG::Clr border_color = ClientUI::CtrlBorderColor(), GG::Clr text_color = ClientUI::TextColor(),
-            GG::Clr interior = ClientUI::CtrlColor(), GG::Flags<GG::WndFlag> flags = GG::INTERACTIVE); ///< basic ctor
+    CUIEdit(GG::X x, GG::Y y, GG::X w, const std::string& str, GG::Clr border_color = ClientUI::CtrlBorderColor(), GG::Clr interior = ClientUI::CtrlColor()); ///< basic ctor
     //@}
 
     /** \name Mutators */ //@{
+    virtual void GainingFocus();
+    virtual void LosingFocus();
     virtual void Render();
     //@}
+
+    mutable boost::signals2::signal<void ()> GainingFocusSignal;
+    mutable boost::signals2::signal<void ()> LosingFocusSignal;
 };
 
 /** a FreeOrion MultiEdit control */
 class CUIMultiEdit : public GG::MultiEdit {
 public:
     /** \name Structors */ //@{
-    CUIMultiEdit(GG::X x, GG::Y y, GG::X w, GG::Y h, const std::string& str, GG::Flags<GG::MultiEditStyle> style = GG::MULTI_LINEWRAP,
-                 const boost::shared_ptr<GG::Font>& font = boost::shared_ptr<GG::Font>(),
-                 GG::Clr border_color = ClientUI::CtrlBorderColor(), GG::Clr text_color = ClientUI::TextColor(),
-                 GG::Clr interior = ClientUI::CtrlColor(), GG::Flags<GG::WndFlag> flags = GG::INTERACTIVE); ///< basic ctor
+    CUIMultiEdit(GG::X x, GG::Y y, GG::X w, GG::Y h, const std::string& str, GG::Flags<GG::MultiEditStyle> style = GG::MULTI_LINEWRAP); ///< basic ctor
     //@}
 
     /** \name Mutators */ //@{
@@ -295,10 +253,7 @@ public:
 class CUILinkTextMultiEdit : public CUIMultiEdit, public TextLinker {
 public:
     /** \name Structors */ //@{
-    CUILinkTextMultiEdit(GG::X x, GG::Y y, GG::X w, GG::Y h, const std::string& str, GG::Flags<GG::MultiEditStyle> style = GG::MULTI_LINEWRAP,
-                         const boost::shared_ptr<GG::Font>& font = boost::shared_ptr<GG::Font>(),
-                         GG::Clr border_color = ClientUI::CtrlBorderColor(), GG::Clr text_color = ClientUI::TextColor(),
-                         GG::Clr interior = ClientUI::CtrlColor(), GG::Flags<GG::WndFlag> flags = GG::INTERACTIVE); ///< basic ctor
+    CUILinkTextMultiEdit(GG::X x, GG::Y y, GG::X w, GG::Y h, const std::string& str, GG::Flags<GG::MultiEditStyle> style = GG::MULTI_LINEWRAP); ///< basic ctor
     //@}
     /** \name Accessors */ //@{
     virtual const std::vector<GG::Font::LineData>&  GetLineData() const;
@@ -315,7 +270,7 @@ public:
     virtual void    RClick(const GG::Pt& pt, GG::Flags<GG::ModKey> mod_keys);
     virtual void    MouseHere(const GG::Pt& pt, GG::Flags<GG::ModKey> mod_keys);
     virtual void    MouseLeave();
-    virtual void    SizeMove(const GG::Pt& ul, const GG::Pt& lr);
+    virtual void    SizeMove(const GG::Pt& ul, const GG::Pt& lr);   // needed primarily so the SetText call will take a RawText
 
     /** sets the text to \a str; may resize the window.  If the window was
         constructed to fit the size of the text (i.e. if the second ctor type
@@ -348,13 +303,11 @@ class StatisticIcon : public GG::Control {
 public:
     /** \name Structors */ //@{
     StatisticIcon(GG::X x, GG::Y y, GG::X w, GG::Y h, const boost::shared_ptr<GG::Texture> texture,
-                  double value, int digits, bool showsign,
-                  GG::Flags<GG::WndFlag> flags = GG::INTERACTIVE); ///< initializes with one value
+                  double value, int digits, bool showsign); ///< initializes with one value
 
     StatisticIcon(GG::X x, GG::Y y, GG::X w, GG::Y h, const boost::shared_ptr<GG::Texture> texture,
                   double value0, double value1, int digits0, int digits1,
-                  bool showsign0, bool showsign1,
-                  GG::Flags<GG::WndFlag> flags = GG::INTERACTIVE);  ///< initializes with two values
+                  bool showsign0, bool showsign1);  ///< initializes with two values
     //@}
 
     /** \name Accessors */ //@{
@@ -366,11 +319,16 @@ public:
 
     virtual void    LButtonDown(const GG::Pt& pt, GG::Flags<GG::ModKey> mod_keys);
     virtual void    RButtonDown(const GG::Pt& pt, GG::Flags<GG::ModKey> mod_keys);
+    virtual void    LClick(const GG::Pt& pt, GG::Flags<GG::ModKey> mod_keys);
+    virtual void    RClick(const GG::Pt& pt, GG::Flags<GG::ModKey> mod_keys);
     virtual void    MouseWheel(const GG::Pt& pt, int move, GG::Flags<GG::ModKey> mod_keys);
 
     void            SetValue(double value, int index = 0);  ///< sets displayed \a value with \a index
     //@}
 
+    mutable boost::signals2::signal<void ()>    LeftClickedSignal;
+    mutable boost::signals2::signal<void ()>    RightClickedSignal;
+
 private:
     void            Refresh();
     GG::Clr         ValueColor(int index) const;        ///< returns colour in which to draw value
@@ -419,7 +377,7 @@ public:
     void SetSpecies(const std::vector<std::string>& species_names);         ///< sets the species that can be selected
     //@}
 
-    mutable boost::signal<void (const std::string&)> SpeciesChangedSignal;
+    mutable boost::signals2::signal<void (const std::string&)> SpeciesChangedSignal;
 
 private:
     void SelectionChanged(GG::DropDownList::iterator it);
@@ -440,7 +398,7 @@ public:
     void SelectColor(const GG::Clr& clr);
     //@}
 
-    mutable boost::signal<void (const GG::Clr&)> ColorChangedSignal;
+    mutable boost::signals2::signal<void (const GG::Clr&)> ColorChangedSignal;
 
 private:
     void SelectionChanged(GG::DropDownList::iterator it);
@@ -459,7 +417,7 @@ public:
     virtual void RClick(const GG::Pt& pt, GG::Flags<GG::ModKey> mod_keys);
     //@}
 
-    mutable boost::signal<void (const GG::Clr&)> ColorChangedSignal;
+    mutable boost::signals2::signal<void (const GG::Clr&)> ColorChangedSignal;
 
 private:
     GG::Clr m_default_color;
@@ -548,17 +506,28 @@ private:
 
 /** Acts like a normal TextControl, but renders extra black copy / copies of text behind to create a
   * drop-shadow effect, impriving text readability. */
-class ShadowedTextControl : public GG::TextControl {
+class ShadowedTextControl : public GG::Control {
 public:
     ShadowedTextControl(GG::X x, GG::Y y, GG::X w, GG::Y h, const std::string& str, const boost::shared_ptr<GG::Font>& font,
-                        GG::Clr color = GG::CLR_BLACK, GG::Flags<GG::TextFormat> format = GG::FORMAT_NONE,
-                        GG::Flags<GG::WndFlag> flags = GG::Flags<GG::WndFlag>());
+                        GG::Clr color = GG::CLR_BLACK, GG::Flags<GG::TextFormat> format = GG::FORMAT_NONE);
 
     ShadowedTextControl(GG::X x, GG::Y y, const std::string& str, const boost::shared_ptr<GG::Font>& font,
-                        GG::Clr color = GG::CLR_BLACK, GG::Flags<GG::TextFormat> format = GG::FORMAT_NONE,
-                        GG::Flags<GG::WndFlag> flags = GG::Flags<GG::WndFlag>());
+                        GG::Clr color = GG::CLR_BLACK, GG::Flags<GG::TextFormat> format = GG::FORMAT_NONE);
+    
+    virtual GG::Pt MinUsableSize() const;
+    
+    virtual void SetText(const std::string& str);
+    
+    virtual void SizeMove(const GG::Pt& ul, const GG::Pt& lr);
+    
+    virtual void SetColor(GG::Clr c);
+    
+    virtual void SetTextColor(GG::Clr c);
  
     virtual void Render();
+private:
+    GG::TextControl* shadow_text;
+    GG::TextControl* main_text;
 };
 
 /** Functions like a StaticGraphic, except can have multiple textures that are rendered bottom to top
@@ -572,16 +541,14 @@ public:
       * if \a styles is not specified or contains fewer entres than \a textures, entries in \a textures without 
       * associated styles use the style GRAPHIC_CENTER. */
     MultiTextureStaticGraphic(GG::X x, GG::Y y, GG::X w, GG::Y h, const std::vector<boost::shared_ptr<GG::Texture> >& textures,
-                              const std::vector<GG::Flags<GG::GraphicStyle> >& styles = std::vector<GG::Flags<GG::GraphicStyle> >(),
-                              GG::Flags<GG::WndFlag> flags = GG::Flags<GG::WndFlag>());
+                              const std::vector<GG::Flags<GG::GraphicStyle> >& styles = std::vector<GG::Flags<GG::GraphicStyle> >());
 
     /** creates a MultiTextureStaticGraphic from multiple pre-existing SubTextures which are rendered back-to-front in the
       * order they are specified in \a subtextures with GraphicStyles specified in the same-indexed value of \a styles.
       * if \a styles is not specified or contains fewer entres than \a subtextures, entries in \a subtextures without 
       * associated styles use the style GRAPHIC_CENTER. */
     MultiTextureStaticGraphic(GG::X x, GG::Y y, GG::X w, GG::Y h, const std::vector<GG::SubTexture>& subtextures,
-                              const std::vector<GG::Flags<GG::GraphicStyle> >& styles = std::vector<GG::Flags<GG::GraphicStyle> >(),
-                              GG::Flags<GG::WndFlag> flags = GG::Flags<GG::WndFlag>());
+                              const std::vector<GG::Flags<GG::GraphicStyle> >& styles = std::vector<GG::Flags<GG::GraphicStyle> >());
     //@}
 
     /** \name Mutators */ ///@{
