@@ -9,6 +9,7 @@
 #include "PlayerListWnd.h"
 #include "MultiplayerLobbyWnd.h"
 #include "Sound.h"
+#include "Hotkeys.h"
 
 #undef int64_t
 
@@ -45,9 +46,8 @@
 
 const Tech* GetTech(const std::string& name);
 
-bool FNComp(const boost::shared_ptr<GG::Texture> t1, const boost::shared_ptr<GG::Texture> t2) {
-    return t1 && t2 && t1->Filename() < t2->Filename();
-}
+bool TextureFileNameCompare(const boost::shared_ptr<GG::Texture> t1, const boost::shared_ptr<GG::Texture> t2)
+{ return t1 && t2 && t1->Filename() < t2->Filename(); }
 
 namespace fs = boost::filesystem;
 
@@ -178,6 +178,7 @@ boost::shared_ptr<GG::Texture> ClientUI::MeterIcon(MeterType meter_type) {
     case METER_MAX_FUEL:
         icon_filename = "fuel.png";         break;
     case METER_SUPPLY:
+    case METER_MAX_SUPPLY:
         icon_filename = "supply.png";       break;
     case METER_STEALTH:
         icon_filename = "stealth.png";      break;
@@ -430,28 +431,28 @@ namespace {
         db.Add<std::string>("UI.sound.bg-music",                UserStringNop("OPTIONS_DB_BG_MUSIC"),                          (GetRootDataDir() / "default" / "data" / "sound" / "artificial_intelligence_v3.ogg").string());
         db.Add("UI.sound.music-volume",                         UserStringNop("OPTIONS_DB_MUSIC_VOLUME"),                      127,                    RangedValidator<int>(1, 255));
         db.Add("UI.sound.volume",                               UserStringNop("OPTIONS_DB_UI_SOUND_VOLUME"),                   255,                    RangedValidator<int>(0, 255));
-        db.Add<std::string>("UI.sound.button-rollover",         UserStringNop("OPTIONS_DB_UI_SOUND_BUTTON_ROLLOVER"),          (GetRootDataDir() / "default" / "data" / "sound" / "button_rollover.wav").string());
-        db.Add<std::string>("UI.sound.button-click",            UserStringNop("OPTIONS_DB_UI_SOUND_BUTTON_CLICK"),             (GetRootDataDir() / "default" / "data" / "sound" / "button_click.wav").string());
-        db.Add<std::string>("UI.sound.turn-button-click",       UserStringNop("OPTIONS_DB_UI_SOUND_TURN_BUTTON_CLICK"),        (GetRootDataDir() / "default" / "data" / "sound" / "turn_button_click.wav").string());
-        db.Add<std::string>("UI.sound.list-select",             UserStringNop("OPTIONS_DB_UI_SOUND_LIST_SELECT"),              (GetRootDataDir() / "default" / "data" / "sound" / "list_select.wav").string());
-        db.Add<std::string>("UI.sound.item-drop",               UserStringNop("OPTIONS_DB_UI_SOUND_ITEM_DROP"),                (GetRootDataDir() / "default" / "data" / "sound" / "item_drop.wav").string());
-        db.Add<std::string>("UI.sound.list-pulldown",           UserStringNop("OPTIONS_DB_UI_SOUND_LIST_PULLDOWN"),            (GetRootDataDir() / "default" / "data" / "sound" / "list_pulldown.wav").string());
-        db.Add<std::string>("UI.sound.text-typing",             UserStringNop("OPTIONS_DB_UI_SOUND_TEXT_TYPING"),              (GetRootDataDir() / "default" / "data" / "sound" / "text_typing.wav").string());
-        db.Add<std::string>("UI.sound.window-maximize",         UserStringNop("OPTIONS_DB_UI_SOUND_WINDOW_MAXIMIZE"),          (GetRootDataDir() / "default" / "data" / "sound" / "window_maximize.wav").string());
-        db.Add<std::string>("UI.sound.window-minimize",         UserStringNop("OPTIONS_DB_UI_SOUND_WINDOW_MINIMIZE"),          (GetRootDataDir() / "default" / "data" / "sound" / "window_minimize.wav").string());
-        db.Add<std::string>("UI.sound.window-close",            UserStringNop("OPTIONS_DB_UI_SOUND_WINDOW_CLOSE"),             (GetRootDataDir() / "default" / "data" / "sound" / "window_close.wav").string());
-        db.Add<std::string>("UI.sound.alert",                   UserStringNop("OPTIONS_DB_UI_SOUND_ALERT"),                    (GetRootDataDir() / "default" / "data" / "sound" / "alert.wav").string());
-        db.Add<std::string>("UI.sound.planet-button-click",     UserStringNop("OPTIONS_DB_UI_SOUND_PLANET_BUTTON_CLICK"),      (GetRootDataDir() / "default" / "data" / "sound" / "button_click.wav").string());
-        db.Add<std::string>("UI.sound.fleet-button-rollover",   UserStringNop("OPTIONS_DB_UI_SOUND_FLEET_BUTTON_ROLLOVER"),    (GetRootDataDir() / "default" / "data" / "sound" / "fleet_button_rollover.wav").string());
-        db.Add<std::string>("UI.sound.fleet-button-click",      UserStringNop("OPTIONS_DB_UI_SOUND_FLEET_BUTTON_CLICK"),       (GetRootDataDir() / "default" / "data" / "sound" / "fleet_button_click.wav").string());
-        db.Add<std::string>("UI.sound.system-icon-rollover",    UserStringNop("OPTIONS_DB_UI_SOUND_SYSTEM_ICON_ROLLOVER"),     (GetRootDataDir() / "default" / "data" / "sound" / "fleet_button_rollover.wav").string());
-        db.Add<std::string>("UI.sound.sidepanel-open",          UserStringNop("OPTIONS_DB_UI_SOUND_SIDEPANEL_OPEN"),           (GetRootDataDir() / "default" / "data" / "sound" / "sidepanel_open.wav").string());
-        db.Add<std::string>("UI.sound.farming-focus",           UserStringNop("OPTIONS_DB_UI_SOUND_FARMING_FOCUS"),            (GetRootDataDir() / "default" / "data" / "sound" / "farm_select.wav").string());
-        db.Add<std::string>("UI.sound.industry-focus",          UserStringNop("OPTIONS_DB_UI_SOUND_INDUSTRY_FOCUS"),           (GetRootDataDir() / "default" / "data" / "sound" / "industry_select.wav").string());
-        db.Add<std::string>("UI.sound.research-focus",          UserStringNop("OPTIONS_DB_UI_SOUND_RESEARCH_FOCUS"),           (GetRootDataDir() / "default" / "data" / "sound" / "research_select.wav").string());
-        db.Add<std::string>("UI.sound.mining-focus",            UserStringNop("OPTIONS_DB_UI_SOUND_MINING_FOCUS"),             (GetRootDataDir() / "default" / "data" / "sound" / "mining_select.wav").string());
-        db.Add<std::string>("UI.sound.trade-focus",             UserStringNop("OPTIONS_DB_UI_SOUND_TRADE_FOCUS"),              (GetRootDataDir() / "default" / "data" / "sound" / "trade_select.wav").string());
-        db.Add<std::string>("UI.sound.balanced-focus",          UserStringNop("OPTIONS_DB_UI_SOUND_BALANCED_FOCUS"),           (GetRootDataDir() / "default" / "data" / "sound" / "balanced_select.wav").string());
+        db.Add<std::string>("UI.sound.button-rollover",         UserStringNop("OPTIONS_DB_UI_SOUND_BUTTON_ROLLOVER"),          (GetRootDataDir() / "default" / "data" / "sound" / "button_rollover.ogg").string());
+        db.Add<std::string>("UI.sound.button-click",            UserStringNop("OPTIONS_DB_UI_SOUND_BUTTON_CLICK"),             (GetRootDataDir() / "default" / "data" / "sound" / "button_click.ogg").string());
+        db.Add<std::string>("UI.sound.turn-button-click",       UserStringNop("OPTIONS_DB_UI_SOUND_TURN_BUTTON_CLICK"),        (GetRootDataDir() / "default" / "data" / "sound" / "turn_button_click.ogg").string());
+        db.Add<std::string>("UI.sound.list-select",             UserStringNop("OPTIONS_DB_UI_SOUND_LIST_SELECT"),              (GetRootDataDir() / "default" / "data" / "sound" / "list_select.ogg").string());
+        db.Add<std::string>("UI.sound.item-drop",               UserStringNop("OPTIONS_DB_UI_SOUND_ITEM_DROP"),                (GetRootDataDir() / "default" / "data" / "sound" / "list_select.ogg").string());//TODO: replace with dedicated 'item_drop' sound
+        db.Add<std::string>("UI.sound.list-pulldown",           UserStringNop("OPTIONS_DB_UI_SOUND_LIST_PULLDOWN"),            (GetRootDataDir() / "default" / "data" / "sound" / "list_pulldown.ogg").string());
+        db.Add<std::string>("UI.sound.text-typing",             UserStringNop("OPTIONS_DB_UI_SOUND_TEXT_TYPING"),              (GetRootDataDir() / "default" / "data" / "sound" / "text_typing.ogg").string());
+        db.Add<std::string>("UI.sound.window-maximize",         UserStringNop("OPTIONS_DB_UI_SOUND_WINDOW_MAXIMIZE"),          (GetRootDataDir() / "default" / "data" / "sound" / "window_maximize.ogg").string());
+        db.Add<std::string>("UI.sound.window-minimize",         UserStringNop("OPTIONS_DB_UI_SOUND_WINDOW_MINIMIZE"),          (GetRootDataDir() / "default" / "data" / "sound" / "window_minimize.ogg").string());
+        db.Add<std::string>("UI.sound.window-close",            UserStringNop("OPTIONS_DB_UI_SOUND_WINDOW_CLOSE"),             (GetRootDataDir() / "default" / "data" / "sound" / "window_close.ogg").string());
+        db.Add<std::string>("UI.sound.alert",                   UserStringNop("OPTIONS_DB_UI_SOUND_ALERT"),                    (GetRootDataDir() / "default" / "data" / "sound" / "alert.ogg").string());
+        db.Add<std::string>("UI.sound.planet-button-click",     UserStringNop("OPTIONS_DB_UI_SOUND_PLANET_BUTTON_CLICK"),      (GetRootDataDir() / "default" / "data" / "sound" / "button_click.ogg").string());
+        db.Add<std::string>("UI.sound.fleet-button-rollover",   UserStringNop("OPTIONS_DB_UI_SOUND_FLEET_BUTTON_ROLLOVER"),    (GetRootDataDir() / "default" / "data" / "sound" / "fleet_button_rollover.ogg").string());
+        db.Add<std::string>("UI.sound.fleet-button-click",      UserStringNop("OPTIONS_DB_UI_SOUND_FLEET_BUTTON_CLICK"),       (GetRootDataDir() / "default" / "data" / "sound" / "fleet_button_click.ogg").string());
+        db.Add<std::string>("UI.sound.system-icon-rollover",    UserStringNop("OPTIONS_DB_UI_SOUND_SYSTEM_ICON_ROLLOVER"),     (GetRootDataDir() / "default" / "data" / "sound" / "fleet_button_rollover.ogg").string());
+        db.Add<std::string>("UI.sound.sidepanel-open",          UserStringNop("OPTIONS_DB_UI_SOUND_SIDEPANEL_OPEN"),           (GetRootDataDir() / "default" / "data" / "sound" / "sidepanel_open.ogg").string());
+        db.Add<std::string>("UI.sound.farming-focus",           UserStringNop("OPTIONS_DB_UI_SOUND_FARMING_FOCUS"),            (GetRootDataDir() / "default" / "data" / "sound" / "farm_select.ogg").string());
+        db.Add<std::string>("UI.sound.industry-focus",          UserStringNop("OPTIONS_DB_UI_SOUND_INDUSTRY_FOCUS"),           (GetRootDataDir() / "default" / "data" / "sound" / "industry_select.ogg").string());
+        db.Add<std::string>("UI.sound.research-focus",          UserStringNop("OPTIONS_DB_UI_SOUND_RESEARCH_FOCUS"),           (GetRootDataDir() / "default" / "data" / "sound" / "research_select.ogg").string());
+        db.Add<std::string>("UI.sound.mining-focus",            UserStringNop("OPTIONS_DB_UI_SOUND_MINING_FOCUS"),             (GetRootDataDir() / "default" / "data" / "sound" / "mining_select.ogg").string());
+        db.Add<std::string>("UI.sound.trade-focus",             UserStringNop("OPTIONS_DB_UI_SOUND_TRADE_FOCUS"),              (GetRootDataDir() / "default" / "data" / "sound" / "trade_select.ogg").string());
+        db.Add<std::string>("UI.sound.balanced-focus",          UserStringNop("OPTIONS_DB_UI_SOUND_BALANCED_FOCUS"),           (GetRootDataDir() / "default" / "data" / "sound" / "balanced_select.ogg").string());
 
         // fonts
         db.Add<std::string>("UI.font",          UserStringNop("OPTIONS_DB_UI_FONT"),                       (GetRootDataDir() / "default" / "DejaVuSans.ttf").string());
@@ -518,13 +519,15 @@ ClientUI::ClientUI() :
     m_multiplayer_lobby_wnd(0)
 {
     s_the_UI = this;
+    Hotkey::ReadFromOptions(GetOptionsDB());
 
-    m_message_wnd =             new MessageWnd(GG::X0,                           GG::GUI::GetGUI()->AppHeight() - PANEL_HEIGHT, PANEL_WIDTH,             PANEL_HEIGHT);
-    m_player_list_wnd =         new PlayerListWnd(m_message_wnd->LowerRight().x, GG::GUI::GetGUI()->AppHeight() - PANEL_HEIGHT, PLAYER_LIST_PANEL_WIDTH, PANEL_HEIGHT);
+    m_message_wnd =             new MessageWnd(GG::X0,                       GG::GUI::GetGUI()->AppHeight() - PANEL_HEIGHT, PANEL_WIDTH,             PANEL_HEIGHT);
+    m_player_list_wnd =         new PlayerListWnd(m_message_wnd->Right(),    GG::GUI::GetGUI()->AppHeight() - PANEL_HEIGHT, PLAYER_LIST_PANEL_WIDTH, PANEL_HEIGHT);
     m_map_wnd =                 new MapWnd();
     m_intro_screen =            new IntroScreen();
     m_multiplayer_lobby_wnd =   new MultiPlayerLobbyWnd();
 
+
     if (GetOptionsDB().Get<bool>("tech-demo")) {
         if (HumanClientApp* app = HumanClientApp::GetApp()) {
             m_combat_wnd =  new CombatWnd(app->SceneManager(), app->Camera(), app->Viewport());
@@ -578,7 +581,7 @@ bool ClientUI::ZoomToObject(int id) {
 
 bool ClientUI::ZoomToPlanet(int id) {
     // this just zooms to the appropriate system, until we create a planet window of some kind
-    if (const Planet* planet = GetPlanet(id))
+    if (TemporaryPtr<const Planet> planet = GetPlanet(id))
         return ZoomToSystem(planet->SystemID());
     return false;
 }
@@ -590,7 +593,7 @@ bool ClientUI::ZoomToPlanetPedia(int id) {
 }
 
 bool ClientUI::ZoomToSystem(int id) {
-    if (const System* system = GetSystem(id)) {
+    if (TemporaryPtr<const System> system = GetSystem(id)) {
         ZoomToSystem(system);
         return true;
     }
@@ -598,7 +601,7 @@ bool ClientUI::ZoomToSystem(int id) {
 }
 
 bool ClientUI::ZoomToFleet(int id) {
-    if (const Fleet* fleet = GetFleet(id)) {
+    if (TemporaryPtr<const Fleet> fleet = GetFleet(id)) {
         ZoomToFleet(fleet);
         return true;
     }
@@ -606,13 +609,13 @@ bool ClientUI::ZoomToFleet(int id) {
 }
 
 bool ClientUI::ZoomToShip(int id) {
-    if (const Ship* ship = GetShip(id))
+    if (TemporaryPtr<const Ship> ship = GetShip(id))
         return ZoomToFleet(ship->FleetID());
     return false;
 }
 
 bool ClientUI::ZoomToBuilding(int id) {
-    if (const Building* building = GetBuilding(id)) {
+    if (TemporaryPtr<const Building> building = GetBuilding(id)) {
         ZoomToBuildingType(building->BuildingTypeName());
         return ZoomToPlanet(building->PlanetID());
     }
@@ -634,7 +637,7 @@ bool ClientUI::ZoomToCombatLog(int id) {
     return false;
 }
 
-void ClientUI::ZoomToSystem(const System* system) {
+void ClientUI::ZoomToSystem(TemporaryPtr<const System> system) {
     if (!system)
         return;
 
@@ -642,7 +645,7 @@ void ClientUI::ZoomToSystem(const System* system) {
     m_map_wnd->SelectSystem(system->ID());
 }
 
-void ClientUI::ZoomToFleet(const Fleet* fleet) {
+void ClientUI::ZoomToFleet(TemporaryPtr<const Fleet> fleet) {
     if (!fleet)
         return;
 
@@ -765,7 +768,7 @@ bool ClientUI::ZoomToEncyclopediaEntry(const std::string& str) {
 }
 
 void ClientUI::DumpObject(int object_id) {
-    const UniverseObject* obj = GetUniverseObject(object_id);
+    TemporaryPtr<const UniverseObject> obj = GetUniverseObject(object_id);
     if (!obj)
         return;
     m_message_wnd->HandleLogMessage(obj->Dump() + "\n");
@@ -806,7 +809,7 @@ void ClientUI::MessageBox(const std::string& message, bool play_alert_sound/* =
                            WndColor(), WndOuterBorderColor(), CtrlColor(), TextColor(), 1,
                            UserString("OK"));
     if (play_alert_sound)
-        Sound::GetSound().PlaySound(SoundDir() / "alert.wav", true);
+        Sound::GetSound().PlaySound(SoundDir() / "alert.ogg", true);
     dlg.Run();
 }
 
@@ -891,7 +894,7 @@ ClientUI::TexturesAndDist ClientUI::PrefixedTexturesAndDist(const boost::filesys
             }
         }
         rand_int.reset(new SmallIntDistType(SmallIntDist(0, textures.size() - 1)));
-        std::sort(textures.begin(), textures.end(), FNComp);
+        std::sort(textures.begin(), textures.end(), TextureFileNameCompare);
     }
     return prefixed_textures_it->second;
 }
@@ -941,144 +944,3 @@ std::istream& operator>>(std::istream& is, StreamableColor& clr) {
     return is;
 }
 
-const double SMALL_UI_DISPLAY_VALUE = 1.0e-6;
-const double LARGE_UI_DISPLAY_VALUE = 9.99999999e+9;
-const double UNKNOWN_UI_DISPLAY_VALUE = std::numeric_limits<double>::infinity();
-
-int EffectiveSign(double val) {
-    if (val == UNKNOWN_UI_DISPLAY_VALUE)
-        return 0;
-
-    if (std::abs(val) >= SMALL_UI_DISPLAY_VALUE) {
-        if (val >= 0)
-            return 1;
-        else
-            return -1;
-    }
-    else
-        return 0;
-}
-
-std::string DoubleToString(double val, int digits, bool always_show_sign) {
-    std::string text = "";
-
-    // minimum digits is 2.  If digits was 1, then 30 couldn't be displayed,
-    // as 0.1k is too much and 9 is too small and just 30 is 2 digits
-    digits = std::max(digits, 2);
-
-    // default result for sentinel value
-    if (val == UNKNOWN_UI_DISPLAY_VALUE)
-        return UserString("UNKNOWN_VALUE_SYMBOL");
-
-    double mag = std::abs(val);
-
-    // early termination if magnitude is 0
-    if (mag == 0.0) {
-        std::string format;
-        format += "%1." + boost::lexical_cast<std::string>(digits - 1) + "f";
-        text += (boost::format(format) % mag).str();
-        return text;
-    }
-
-    // prepend signs if neccessary
-    int effectiveSign = EffectiveSign(val);
-    if (effectiveSign == -1) {
-        text += "-";
-    } else {
-        if (always_show_sign) text += "+";
-    }
-
-    if (mag > LARGE_UI_DISPLAY_VALUE) mag = LARGE_UI_DISPLAY_VALUE;
-
-    // if value is effectively 0, avoid unnecessary later processing
-    if (effectiveSign == 0) {
-        text = "0.0";
-        for (int n = 2; n < digits; ++n)
-            text += "0";  // fill in 0's to required number of digits
-        return text;
-    }
-
-
-    //std::cout << std::endl << "DoubleToString val: " << val << " digits: " << digits << std::endl;
-
-    // power of 10 of highest valued digit in number
-    int pow10 = static_cast<int>(floor(log10(mag))); // = 2 (100's) for 234.4,  = 4 (10000's) for 45324
-    //std::cout << "magnitude power of 10: " << pow10 << std::endl;
-
-    // determine base unit for number: the next lower power of 10^3 from the number (inclusive)
-    int pow10_digits_above_pow1000 = 0;
-    if (pow10 >= 0)
-        pow10_digits_above_pow1000 = pow10 % 3;
-    else
-        pow10_digits_above_pow1000 = (pow10 % 3) + 3;   // +3 ensures positive result of mod
-    int unit_pow10 = pow10 - pow10_digits_above_pow1000;
-    if (unit_pow10 < 0)
-        unit_pow10 = 0;
-    //std::cout << "unit power of 10: " << unit_pow10 << std::endl;
-
-    // if not enough digits to include most significant digit and next lower
-    // power of 10, add extra digits. this still uses less space than using the
-    // next higher power of 10 and adding a 0. out front.  for example, 240 with
-    // 2 digits is better shown as "240" than "0.24k"
-    digits = std::max(digits, pow10_digits_above_pow1000 + 1);
-    //std::cout << "adjusted digits: " << digits << std::endl;
-
-    int lowest_digit_pow10 = pow10 - digits + 1;
-    //std::cout << "lowest_digit_pow10: " << lowest_digit_pow10 << std::endl;
-
-    // fraction digits:
-    int fractionDigits = std::max(0, std::min(digits - 1, unit_pow10 - lowest_digit_pow10));
-    //std::cout << "fractionDigits: " << fractionDigits << std::endl;
-
-
-    /* round number down at lowest digit to be displayed, to prevent lexical_cast from rounding up
-       in cases like 0.998k with 2 digits -> 1.00k  instead of  0.99k  (as it should be) */
-    double roundingFactor = pow(10.0, static_cast<double>(pow10 - digits + 1));
-    mag /= roundingFactor;
-    mag = floor(mag);
-    mag *= roundingFactor;
-
-    // scale number by unit power of 10
-    mag /= pow(10.0, static_cast<double>(unit_pow10));  // if mag = 45324 and unitPow = 3, get mag = 45.324
-
-
-    std::string format;
-    format += "%" + boost::lexical_cast<std::string>(digits) + "." +
-                    boost::lexical_cast<std::string>(fractionDigits) + "f";
-    text += (boost::format(format) % mag).str();
-
-    // append base scale SI prefix (as postfix)
-    switch (unit_pow10) {
-    case -15:
-        text += "f";        // femto
-        break;
-    case -12:
-        text += "p";        // pico
-        break;
-    case -9:
-        text += "n";        // nano
-        break;
-    case -6:
-        text += "\xC2\xB5"; // micro.  mu in UTF-8
-        break;
-    case -3:
-        text += "m";        // milli
-        break;
-    case 3:
-        text += "k";        // kilo
-        break;
-    case 6:
-        text += "M";        // Mega
-        break;
-    case 9:
-        text += "G";        // Giga
-        break;
-    case 12:
-        text += "T";        // Terra
-        break;
-    default:
-        break;
-    }
-    return text;
-}
-
