@@ -8,6 +8,7 @@
 #include "CUIControls.h"
 #include "EntityRenderer.h" // TODO: Remove; for testing only.
 #include "InGameMenu.h"
+#include "Hotkeys.h"
 #include "../combat/CombatFighter.h"
 #include "../combat/CombatShip.h"
 #include "../combat/Missile.h"
@@ -27,8 +28,8 @@
 
 // TODO: Remove these once the obstacle test code is removed.
 #include "../combat/AsteroidBeltObstacle.h"
-#include "Obstacle.h"
-#include "SimpleVehicle.h"
+#include <OpenSteer/Obstacle.h>
+#include <OpenSteer/SimpleVehicle.h>
 
 // TODO: Remove this.  It is only here for prototyping.
 #include "../universe/Fleet.h"
@@ -38,19 +39,9 @@
 #include "PagedGeometry.h"
 #include "TreeLoader3D.h"
 
-#include <OgreBillboard.h>
-#include <OgreBillboardSet.h>
 #include <OgreCompositorManager.h>
 #include <OgreConfigFile.h>
-#include <OgreEntity.h>
-#include <OgreMaterialManager.h>
-#include <OgreMeshManager.h>
-#include <OgreRoot.h>
 #include <OgreRenderQueueListener.h>
-#include <OgreRenderSystem.h>
-#include <OgreRenderTarget.h>
-#include <OgreSceneQuery.h>
-#include <OgreSubEntity.h>
 
 #include <btBulletCollisionCommon.h>
 
@@ -187,10 +178,10 @@ namespace {
         return retval;
     }
 
-    std::pair<std::string, int> PlanetLightsChannel(const std::string& base_name, const Planet& planet) {
+    std::pair<std::string, int> PlanetLightsChannel(const std::string& base_name, TemporaryPtr<const Planet> planet) {
         std::pair<std::string, int> retval;
 
-        double pop = planet.CurrentMeterValue(METER_POPULATION);
+        double pop = planet->CurrentMeterValue(METER_POPULATION);
         unsigned int lights_level = NO_CITY_LIGHTS;
         const double MIN_POP_FOR_LIGHTS = 5.0;
         if (MIN_POP_FOR_LIGHTS < pop)
@@ -377,11 +368,11 @@ namespace {
         }
     }
 
-    std::string ShipIDString(const Ship& ship)
-    { return "ship_" + boost::lexical_cast<std::string>(ship.ID()) + "_"; }
+    std::string ShipIDString(TemporaryPtr<const Ship> ship)
+    { return "ship_" + boost::lexical_cast<std::string>(ship->ID()) + "_"; }
 
-    std::string ShipMeshName(const Ship& ship)
-    { return ship.Design()->Model() + ".mesh"; }
+    std::string ShipMeshName(TemporaryPtr<const Ship> ship)
+    { return ship->Design()->Model() + ".mesh"; }
 
     bool CloseTo(const GG::Pt& p1, const GG::Pt& p2) {
         const int EPSILON = 5;
@@ -413,6 +404,20 @@ namespace {
         db.Add("combat.enable-skybox",      UserStringNop("OPTIONS_DB_COMBAT_ENABLE_SKYBOX"),      true, Validator<bool>());
         db.Add("combat.enable-lens-flare",  UserStringNop("OPTIONS_DB_COMBAT_ENABLE_LENS_FLARE"),  true, Validator<bool>());
         db.Add("combat.filled-selection",   UserStringNop("OPTIONS_DB_COMBAT_FILLED_SELECTION"),   false, Validator<bool>());
+
+        // We also register shortcut names/default values, for the
+        // context "map".
+        Hotkey::AddHotkey("combat.open_chat",           GG::GGK_RETURN);
+        Hotkey::AddHotkey("combat.end_turn",            GG::GGK_RETURN, GG::MOD_KEY_CTRL);
+        Hotkey::AddHotkey("combat.menu",                GG::GGK_F10);
+        Hotkey::AddHotkey("combat.zoom_in",             GG::GGK_e);
+        Hotkey::AddHotkey("combat.zoom_in_alt",         GG::GGK_KP_PLUS);
+        Hotkey::AddHotkey("combat.zoom_out",            GG::GGK_r);
+        Hotkey::AddHotkey("combat.zoom_out_alt",        GG::GGK_KP_MINUS);
+        Hotkey::AddHotkey("combat.zoom_prev_unit",      GG::GGK_v);
+        Hotkey::AddHotkey("combat.zoom_next_unit",      GG::GGK_b);
+        Hotkey::AddHotkey("combat.zoom_prev_idle_unit", GG::GGK_f);
+        Hotkey::AddHotkey("combat.zoom_next_idle_unit", GG::GGK_g);
     }
     bool temp_bool = RegisterOptions(&AddOptions);
 }
@@ -532,21 +537,33 @@ public:
             render_system->setStencilBufferParams(
                 Ogre::CMPF_ALWAYS_PASS,
                 OUTLINE_SELECTION_HILITING_STENCIL_VALUE, 0xFFFFFFFF,
+#if (OGRE_VERSION >= ((1 << 16 | 9 << 8)))
+                0xFFFFFFFF,
+#endif
                 Ogre::SOP_KEEP, Ogre::SOP_KEEP, Ogre::SOP_REPLACE, false);
         } else if (queue_group_id == SELECTION_HILITING_OUTLINED_RENDER_QUEUE) { // outline object's selection hiliting
             render_system->setStencilBufferParams(
                 Ogre::CMPF_NOT_EQUAL,
                 OUTLINE_SELECTION_HILITING_STENCIL_VALUE, 0xFFFFFFFF,
+#if (OGRE_VERSION >= ((1 << 16 | 9 << 8)))
+                0xFFFFFFFF,
+#endif
                 Ogre::SOP_KEEP, Ogre::SOP_KEEP, Ogre::SOP_REPLACE, false);
         } else if (queue_group_id == SELECTION_HILITING_FILLED_1_RENDER_QUEUE) { // fully-hilited object's stencil-writing pass
             render_system->setStencilBufferParams(
                 Ogre::CMPF_ALWAYS_PASS,
                 FULL_SELECTION_HILITING_STENCIL_VALUE, 0xFFFFFFFF,
+#if (OGRE_VERSION >= ((1 << 16 | 9 << 8)))
+                0xFFFFFFFF,
+#endif
                 Ogre::SOP_KEEP, Ogre::SOP_KEEP, Ogre::SOP_REPLACE, false);
         } else if (queue_group_id == SELECTION_HILITING_FILLED_2_RENDER_QUEUE) { // fully-hilited object's rendering pass
             render_system->setStencilBufferParams(
                 Ogre::CMPF_EQUAL,
                 FULL_SELECTION_HILITING_STENCIL_VALUE, 0xFFFFFFFF,
+#if (OGRE_VERSION >= ((1 << 16 | 9 << 8)))
+                0xFFFFFFFF,
+#endif
                 Ogre::SOP_KEEP, Ogre::SOP_KEEP, Ogre::SOP_ZERO, false);
         }
     }
@@ -624,7 +641,7 @@ CombatWnd::CombatWnd(Ogre::SceneManager* scene_manager,
     m_stencil_op_frame_listener(new StencilOpQueueListener),
     m_fps_text(new FPSIndicator(GG::X(5), GG::Y(5))),
     m_menu_showing(false),
-    m_end_turn_button(new CUIButton(GG::X0, GG::Y0, GG::X(75), UserString("TURN"))),
+    m_end_turn_button(new CUIButton(UserString("TURN"), GG::X0, GG::Y0, GG::X(75))),
     m_time_since_last_turn_update(0.0),
     m_exit(false)
 {
@@ -816,28 +833,18 @@ CombatWnd::CombatWnd(Ogre::SceneManager* scene_manager,
 #endif
 
         // a sample system
-        std::vector<Planet*> planets;
-        planets.push_back(new Planet(planet_types[0], planet_sizes[0]));
-        planets.push_back(new Planet(planet_types[1], planet_sizes[1]));
-        planets.push_back(new Planet(planet_types[2], planet_sizes[2]));
-        planets.push_back(new Planet(planet_types[3], planet_sizes[3]));
-        planets.push_back(new Planet(planet_types[4], planet_sizes[4]));
-        planets.push_back(new Planet(planet_types[5], planet_sizes[5]));
-        planets.push_back(new Planet(planet_types[6], planet_sizes[6]));
-        planets.push_back(new Planet(planet_types[7], planet_sizes[7]));
-        planets.push_back(new Planet(planet_types[8], planet_sizes[8]));
-        planets.push_back(new Planet(planet_types[9], planet_sizes[9]));
+        std::vector<TemporaryPtr<Planet> > planets;
+        for (unsigned int i = 0; i <= 9; ++i)
+            planets.push_back(GetUniverse().CreatePlanet(planet_types[0], planet_sizes[0]));
 
         CombatData* combat_data = new CombatData;
-        combat_data->m_system = new System(star_type, planets.size(), "Sample", 0.0, 0.0);
-        System& system = *combat_data->m_system;
-        std::map<int, UniverseObject*>& combat_universe = combat_data->m_combat_universe;
+        combat_data->m_system = GetUniverse().CreateSystem(star_type, "Sample", 0.0, 0.0);
+        std::map<int, TemporaryPtr<UniverseObject> >& combat_universe = combat_data->m_combat_universe;
         for (std::size_t i = 0; i < planets.size(); ++i) {
-            Planet* planet = planets[i];
-            GetUniverse().InsertID(planet, planet_ids[i]);
+            TemporaryPtr<Planet> planet = planets[i];;
             combat_universe[planet_ids[i]] = planet;
-            system.Insert(planet_ids[i], i);
-            assert(system.Contains(i));
+            combat_data->m_system->Insert(planet, i);   // just created planets above, and know they have no buildings to insert
+            assert(combat_data->m_system->Contains(i));
         }
 
         std::vector<CombatSetupGroup> setup_groups;
@@ -846,10 +853,10 @@ CombatWnd::CombatWnd(Ogre::SceneManager* scene_manager,
         // TODO: For prototyping only.
         GG::X width(150);
         CUIButton* done_button =
-            new CUIButton(GG::GUI::GetGUI()->AppWidth() - width - GG::X(5),
+            new CUIButton("End Combat",
+                          GG::GUI::GetGUI()->AppWidth() - width - GG::X(5),
                           GG::GUI::GetGUI()->AppHeight() - GG::Y(25),
-                          width,
-                          "End Combat");
+                          width);
         GG::Connect(done_button->LeftClickedSignal, &EndCombatButtonClicked);
         AttachChild(done_button);
 
@@ -925,17 +932,26 @@ void CombatWnd::InitCombat(CombatData& combat_data, const std::vector<CombatSetu
     CreateAsteroidEntities(asteroid_entities, m_scene_manager);
 
     // create planets
-    for (System::const_orbit_iterator it = m_combat_data->m_system->begin();
-         it != m_combat_data->m_system->end(); ++it)
+    std::set<int> planet_ids = m_combat_data->m_system->PlanetIDs();
+
+    for (std::set<int>::const_iterator it = planet_ids.begin();
+         it != planet_ids.end(); ++it)
     {
-        const Planet* planet = 0;
-        planet = universe_object_cast<Planet*>(m_combat_data->m_combat_universe[it->second]);
-        if (!planet) continue;
+        int planet_id = *it;
+
+        TemporaryPtr<const UniverseObject> obj = m_combat_data->m_combat_universe[planet_id];
+        if (!obj)
+            continue;
+        TemporaryPtr<const Planet> planet = boost::dynamic_pointer_cast<const Planet>(obj);
+        if (!planet)
+            continue;
+
+        int orbit = m_combat_data->m_system->OrbitOfPlanet(planet_id);
 
         std::string material_name = PlanetNodeMaterial(planet->Type());
         if (material_name != "asteroid") {
             std::string planet_name =
-                "orbit " + boost::lexical_cast<std::string>(it->first) + " planet";
+                "orbit " + boost::lexical_cast<std::string>(orbit) + " planet";
 
             Ogre::SceneNode* node =
                 m_scene_manager->getRootSceneNode()->createChildSceneNode(
@@ -943,7 +959,7 @@ void CombatWnd::InitCombat(CombatData& combat_data, const std::vector<CombatSetu
             Ogre::Real planet_radius = PlanetRadius(planet->Size());
             node->setScale(planet_radius, planet_radius, planet_radius);
             node->yaw(Ogre::Degree(planet->AxialTilt()));
-            double orbit_radius = OrbitalRadius(it->first);
+            double orbit_radius = OrbitalRadius(orbit);
             double rads =
                 planet->OrbitalPositionOnTurn(ClientApp::GetApp()->CurrentTurn());
             Ogre::Vector3 position(orbit_radius * std::cos(rads),
@@ -988,11 +1004,11 @@ void CombatWnd::InitCombat(CombatData& combat_data, const std::vector<CombatSetu
                 entity->setVisibilityFlags(REGULAR_OBJECTS_MASK);
                 entity->setQueryFlags(UNSELECTABLE_OBJECT_MASK);
                 std::string new_material_name =
-                    material_name + "_" + boost::lexical_cast<std::string>(it->first);
+                    material_name + "_" + boost::lexical_cast<std::string>(orbit);
                 Ogre::MaterialPtr material =
                     Ogre::MaterialManager::getSingleton().getByName(material_name);
                 material = material->clone(new_material_name);
-                m_planet_assets[it->first].second.push_back(material);
+                m_planet_assets[orbit].second.push_back(material);
                 material->getTechnique(0)->getPass(0)->getTextureUnitState(0)->
                     setTextureName(base_name + ".png");
                 entity->setMaterialName(new_material_name);
@@ -1002,14 +1018,14 @@ void CombatWnd::InitCombat(CombatData& combat_data, const std::vector<CombatSetu
                     m_scene_manager->createEntity(planet_name, "sphere.mesh");
                 entity->setVisibilityFlags(REGULAR_OBJECTS_MASK);
                 std::string new_material_name =
-                    material_name + "_" + boost::lexical_cast<std::string>(it->first);
+                    material_name + "_" + boost::lexical_cast<std::string>(orbit);
                 Ogre::MaterialPtr material =
                     Ogre::MaterialManager::getSingleton().getByName(
                         material_name == "planet" ?
                         PlanetMaterialName(base_name) :
                         material_name);
                 material = material->clone(new_material_name);
-                m_planet_assets[it->first].second.push_back(material);
+                m_planet_assets[orbit].second.push_back(material);
                 assert(entity->getNumSubEntities() == 1u);
                 material->getTechnique(0)->getPass(0)->getTextureUnitState(0)->
                     setTextureName(base_name + "Day.png");
@@ -1034,13 +1050,13 @@ void CombatWnd::InitCombat(CombatData& combat_data, const std::vector<CombatSetu
                         Ogre::MaterialManager::getSingleton().getByName(
                             AtmosphereMaterialName(base_name));
                     entity->setMaterialName(material->getName());
-                    m_planet_assets[it->first].second.push_back(material);
+                    m_planet_assets[orbit].second.push_back(material);
                     node->attachObject(entity);
                 } else {
                     assert(material_name == "atmosphereless_planet");
                     material->getTechnique(0)->getPass(0)->getTextureUnitState(2)->
                         setTextureName(base_name + "Normal.png");
-                    std::pair<std::string, int> lights_channel = PlanetLightsChannel(base_name, *planet);
+                    std::pair<std::string, int> lights_channel = PlanetLightsChannel(base_name, planet);
                     material->getTechnique(0)->getPass(0)->getTextureUnitState(3)->
                         setTextureName(lights_channel.first);
                     material->getTechnique(0)->getPass(0)->getFragmentProgramParameters()->
@@ -1048,22 +1064,22 @@ void CombatWnd::InitCombat(CombatData& combat_data, const std::vector<CombatSetu
                 }
             }
 
-            m_planet_assets[it->first].first = node;
+            m_planet_assets[orbit].first = node;
         } else {
             SetupPagedGeometry(m_paged_geometry, m_paged_geometry_loader, m_ogre_camera);
 
             const int ASTEROIDS_IN_BELT_AT_FOURTH_ORBIT = 1000;
-            const int ORBITAL_RADIUS = OrbitalRadius(it->first);
+            const int ORBITAL_RADIUS = OrbitalRadius(orbit);
             const int ASTEROIDS =
                 ORBITAL_RADIUS / OrbitalRadius(3) * ASTEROIDS_IN_BELT_AT_FOURTH_ORBIT;
 
             std::string planet_name =
-                "orbit " + boost::lexical_cast<std::string>(it->first) + " planet";
+                "orbit " + boost::lexical_cast<std::string>(orbit) + " planet";
 
             const Ogre::Real DELTA_THETA = Ogre::Math::TWO_PI / ASTEROIDS;
             Ogre::Real theta = 0.0;
             for (int i = 0; i < ASTEROIDS; ++i, theta += DELTA_THETA) {
-                const Ogre::Real THICKNESS = AsteroidBeltRadius() * 2.0;
+                const Ogre::Real THICKNESS = static_cast<Ogre::Real>(AsteroidBeltRadius() * 2.0f);
                 Ogre::Radian yaw(Ogre::Math::TWO_PI * RandZeroToOne());
                 Ogre::Vector3 position;
                 position.z = THICKNESS * (RandZeroToOne() - 0.5);
@@ -1081,8 +1097,8 @@ void CombatWnd::InitCombat(CombatData& combat_data, const std::vector<CombatSetu
     }
 
     // create starlane entrance points
-    for (System::const_lane_iterator it = m_combat_data->m_system->begin_lanes();
-         it != m_combat_data->m_system->end_lanes(); ++it)
+    for (std::map<int, bool>::const_iterator it = m_combat_data->m_system->StarlanesWormholes().begin();
+         it != m_combat_data->m_system->StarlanesWormholes().end(); ++it)
     {
         // this will break if/when we add support for wormholes, so we'll know to fix this code
         assert(!it->second);
@@ -1328,7 +1344,7 @@ void CombatWnd::RClick(const GG::Pt& pt, GG::Flags<GG::ModKey> mod_keys) {
                         CombatShipPtr combat_ship = boost::static_pointer_cast<CombatShip>(combat_object);
                         m_combat_order_set.push_back(
                             CombatOrder(
-                                combat_ship->GetShip().ID(),
+                                combat_ship->GetShip()->ID(),
                                 ShipMission(attack ?
                                             ShipMission::ATTACK_THIS : ShipMission::DEFEND_THIS,
                                             *target),
@@ -1372,7 +1388,7 @@ void CombatWnd::RClick(const GG::Pt& pt, GG::Flags<GG::ModKey> mod_keys) {
                         CombatShipPtr combat_ship = boost::static_pointer_cast<CombatShip>(combat_object);
                         m_combat_order_set.push_back(
                             CombatOrder(
-                                combat_ship->GetShip().ID(),
+                                combat_ship->GetShip()->ID(),
                                 ShipMission(patrol ? ShipMission::PATROL_TO : ShipMission::MOVE_TO,
                                             ToOpenSteer(intersection.second)),
                                 append));
@@ -1524,8 +1540,8 @@ void CombatWnd::ApplyUpdateFromServer() {
         assert(boost::dynamic_pointer_cast<CombatShip>(*it));
         CombatShipPtr combat_ship = boost::static_pointer_cast<CombatShip>(*it);
         combat_ship->SetListener(*this);
-        Ship& ship = combat_ship->GetShip();
-        std::map<int, ShipData>::iterator ship_data_it = m_ship_assets.find(ship.ID());
+        TemporaryPtr<Ship> ship = combat_ship->GetShip();
+        std::map<int, ShipData>::iterator ship_data_it = m_ship_assets.find(ship->ID());
         if (ship_data_it == m_ship_assets.end()) {
             AddCombatShip(combat_ship);
         } else {
@@ -1814,13 +1830,13 @@ Ogre::MovableObject* CombatWnd::GetObjectUnderPt(const GG::Pt& pt) {
 void CombatWnd::DeselectAll()
 { m_current_selections.clear(); }
 
-const Ogre::MaterialPtr& CombatWnd::GetShipMaterial(const Ship& ship) {
-    assert(ship.Design());
-    const ShipDesign& ship_design = *ship.Design();
+const Ogre::MaterialPtr& CombatWnd::GetShipMaterial(TemporaryPtr<const Ship> ship) {
+    assert(ship->Design());
+    const ShipDesign& ship_design = *ship->Design();
 
     Ogre::MaterialPtr ship_material =
         Ogre::MaterialManager::getSingleton().getByName("ship");
-    std::string modified_material_name = ShipMaterialName(ship_design, ship.Owner());
+    std::string modified_material_name = ShipMaterialName(ship_design, ship->Owner());
     Ogre::MaterialPtr& modified_material = m_ship_materials[modified_material_name];
     if (!modified_material.get()) {
         modified_material = ship_material->clone(modified_material_name);
@@ -1839,8 +1855,8 @@ const Ogre::MaterialPtr& CombatWnd::GetShipMaterial(const Ship& ship) {
     modified_material->getTechnique(0)->getPass(1)->getFragmentProgramParameters()->
         setNamedConstant("skybox_light_color", GetSystemColor("sky_box_1"));
 
-    assert(!ship.Unowned());
-    GG::Clr color = Empires().Lookup(ship.Owner())->Color();
+    assert(!ship->Unowned());
+    GG::Clr color = Empires().Lookup(ship->Owner())->Color();
     modified_material->getTechnique(0)->getPass(1)->getFragmentProgramParameters()->
         setNamedConstant("decal_color", Ogre::Vector3(color.r / 255.0, color.g / 255.0, color.b / 255.0));
 
@@ -1888,8 +1904,8 @@ void CombatWnd::UpdateObjectPosition(const CombatObjectPtr& combat_object) {
     if (combat_object->IsShip()) {
         assert(boost::dynamic_pointer_cast<CombatShip>(combat_object));
         CombatShipPtr combat_ship = boost::static_pointer_cast<CombatShip>(combat_object);
-        Ship& ship = combat_ship->GetShip();
-        std::map<int, ShipData>::iterator ship_data_it = m_ship_assets.find(ship.ID());
+        TemporaryPtr<Ship> ship = combat_ship->GetShip();
+        std::map<int, ShipData>::iterator ship_data_it = m_ship_assets.find(ship->ID());
         assert(ship_data_it != m_ship_assets.end());
 
         ShipData& ship_data = ship_data_it->second;
@@ -1918,7 +1934,7 @@ void CombatWnd::RemoveShip(int ship_id) {
 }
 
 void CombatWnd::AddCombatShip(const CombatShipPtr& combat_ship) {
-    const Ship& ship = combat_ship->GetShip();
+    TemporaryPtr<const Ship> ship = combat_ship->GetShip();
 
     const Ogre::MaterialPtr& material = GetShipMaterial(ship);
     Ogre::Entity* entity = CreateShipEntity(m_scene_manager, ship, material);
@@ -1928,11 +1944,11 @@ void CombatWnd::AddCombatShip(const CombatShipPtr& combat_ship) {
 
     SetNodePositionAndOrientation(node, combat_ship);
 
-    AddShipNode(ship.ID(), node, entity, material);
+    AddShipNode(ship->ID(), node, entity, material);
 }
 
 void CombatWnd::RemoveCombatShip(const CombatShipPtr& combat_ship) {
-    ShipData& ship_data = m_ship_assets[combat_ship->GetShip().ID()];
+    ShipData& ship_data = m_ship_assets[combat_ship->GetShip()->ID()];
     m_scene_manager->destroySceneNode(ship_data.m_node);
     m_collision_world->getCollisionObjectArray().remove(ship_data.m_bt_object);
     delete ship_data.m_bt_mesh;
@@ -1940,7 +1956,7 @@ void CombatWnd::RemoveCombatShip(const CombatShipPtr& combat_ship) {
     delete ship_data.m_bt_object;
     m_collision_shapes.erase(ship_data.m_bt_shape);
     m_collision_objects.erase(ship_data.m_bt_object);
-    m_ship_assets.erase(combat_ship->GetShip().ID());
+    m_ship_assets.erase(combat_ship->GetShip()->ID());
 }
 
 bool CombatWnd::OpenChatWindow() {
@@ -2006,122 +2022,50 @@ bool CombatWnd::ZoomToNextUnit() {
 }
 
 void CombatWnd::ConnectKeyboardAcceleratorSignals() {
-    m_keyboard_accelerator_signals.insert(
-        GG::Connect(GG::GUI::GetGUI()->AcceleratorSignal(GG::GGK_RETURN),
-                    &CombatWnd::OpenChatWindow, this));
-    m_keyboard_accelerator_signals.insert(
-        GG::Connect(GG::GUI::GetGUI()->AcceleratorSignal(GG::GGK_KP_ENTER),
-                    &CombatWnd::OpenChatWindow, this));
+    HotkeyManager * hkm = HotkeyManager::GetManager();
 
-    m_keyboard_accelerator_signals.insert(
-        GG::Connect(GG::GUI::GetGUI()->AcceleratorSignal(GG::GGK_RETURN, GG::MOD_KEY_CTRL),
-                    &CombatWnd::EndTurn, this));
-    m_keyboard_accelerator_signals.insert(
-        GG::Connect(GG::GUI::GetGUI()->AcceleratorSignal(GG::GGK_KP_ENTER, GG::MOD_KEY_CTRL),
-                    &CombatWnd::EndTurn, this));
+    // disable combat hotkeys for now to prevent potential crashes.
+    // need conditions specified when connecting these...
 
-    m_keyboard_accelerator_signals.insert(
-        GG::Connect(GG::GUI::GetGUI()->AcceleratorSignal(GG::GGK_F10),
-                    &CombatWnd::ShowMenu, this));
+    //hkm->Connect(this, &CombatWnd::OpenChatWindow,      "combat.open_chat");
+    //hkm->Connect(this, &CombatWnd::ShowMenu,            "combat.menu");
+    //hkm->Connect(this, &CombatWnd::KeyboardZoomIn,      "combat.zoom_in");
+    //hkm->Connect(this, &CombatWnd::KeyboardZoomIn,      "combat.zoom_in_alt");
+    //hkm->Connect(this, &CombatWnd::KeyboardZoomOut,     "combat.zoom_out");
+    //hkm->Connect(this, &CombatWnd::KeyboardZoomOut,     "combat.zoom_out_alt");
+    //hkm->Connect(this, &CombatWnd::ZoomToPrevUnit,      "combat.zoom_prev_unit");
+    //hkm->Connect(this, &CombatWnd::ZoomToNextUnit,      "combat.zoom_next_unit");
+    //hkm->Connect(this, &CombatWnd::ZoomToPrevIdleUnit,  "combat.zoom_prev_idle_unit");
+    //hkm->Connect(this, &CombatWnd::ZoomToNextIdleUnit,  "combat.zoom_next_idle_unit");
 
-    // Keys for zooming
-    m_keyboard_accelerator_signals.insert(
-        GG::Connect(GG::GUI::GetGUI()->AcceleratorSignal(GG::GGK_e),
-                    &CombatWnd::KeyboardZoomIn, this));
-    m_keyboard_accelerator_signals.insert(
-        GG::Connect(GG::GUI::GetGUI()->AcceleratorSignal(GG::GGK_KP_PLUS),
-                    &CombatWnd::KeyboardZoomIn, this));
-    m_keyboard_accelerator_signals.insert(
-        GG::Connect(GG::GUI::GetGUI()->AcceleratorSignal(GG::GGK_r),
-                    &CombatWnd::KeyboardZoomOut, this));
-    m_keyboard_accelerator_signals.insert(
-        GG::Connect(GG::GUI::GetGUI()->AcceleratorSignal(GG::GGK_KP_MINUS),
-                    &CombatWnd::KeyboardZoomOut, this));
-
-    // Keys for showing units
-    m_keyboard_accelerator_signals.insert(
-        GG::Connect(GG::GUI::GetGUI()->AcceleratorSignal(GG::GGK_f),
-                    &CombatWnd::ZoomToPrevIdleUnit, this));
-    m_keyboard_accelerator_signals.insert(
-        GG::Connect(GG::GUI::GetGUI()->AcceleratorSignal(GG::GGK_g),
-                    &CombatWnd::ZoomToNextIdleUnit, this));
-    m_keyboard_accelerator_signals.insert(
-        GG::Connect(GG::GUI::GetGUI()->AcceleratorSignal(GG::GGK_v),
-                    &CombatWnd::ZoomToPrevUnit, this));
-    m_keyboard_accelerator_signals.insert(
-        GG::Connect(GG::GUI::GetGUI()->AcceleratorSignal(GG::GGK_b),
-                    &CombatWnd::ZoomToNextUnit, this));
+    hkm->RebuildShortcuts();
 }
 
 void CombatWnd::SetAccelerators() {
-    GG::GUI::GetGUI()->SetAccelerator(GG::GGK_RETURN);
-    GG::GUI::GetGUI()->SetAccelerator(GG::GGK_KP_ENTER);
-
-    GG::GUI::GetGUI()->SetAccelerator(GG::GGK_RETURN, GG::MOD_KEY_CTRL);
-    GG::GUI::GetGUI()->SetAccelerator(GG::GGK_KP_ENTER, GG::MOD_KEY_CTRL);
-
-    GG::GUI::GetGUI()->SetAccelerator(GG::GGK_F10);
-
-    // Keys for zooming
-    GG::GUI::GetGUI()->SetAccelerator(GG::GGK_e);
-    GG::GUI::GetGUI()->SetAccelerator(GG::GGK_r);
-    GG::GUI::GetGUI()->SetAccelerator(GG::GGK_KP_PLUS);
-    GG::GUI::GetGUI()->SetAccelerator(GG::GGK_KP_MINUS);
-
-    // Keys for showing units
-    GG::GUI::GetGUI()->SetAccelerator(GG::GGK_f);
-    GG::GUI::GetGUI()->SetAccelerator(GG::GGK_g);
-    GG::GUI::GetGUI()->SetAccelerator(GG::GGK_v);
-    GG::GUI::GetGUI()->SetAccelerator(GG::GGK_b);
-
     ConnectKeyboardAcceleratorSignals();
 }
 
 void CombatWnd::RemoveAccelerators() {
+    // This is mostly pointless, now, isn't it ?
     GG::GUI::accel_iterator i = GG::GUI::GetGUI()->accel_begin();
     while (i != GG::GUI::GetGUI()->accel_end()) {
         GG::GUI::GetGUI()->RemoveAccelerator(i);
         i = GG::GUI::GetGUI()->accel_begin();
     }
     m_disabled_accels_list.clear();
-
-    for (std::set<boost::signals::connection>::iterator it =
-             m_keyboard_accelerator_signals.begin();
-         it != m_keyboard_accelerator_signals.end();
-         ++it) {
-        it->disconnect();
-    }
-    m_keyboard_accelerator_signals.clear();
 }
 
-void CombatWnd::DisableAlphaNumAccels() {
-    for (GG::GUI::const_accel_iterator i = GG::GUI::GetGUI()->accel_begin();
-         i != GG::GUI::GetGUI()->accel_end(); ++i) {
-        if (i->second != 0) // we only want to disable mod_keys without modifiers
-            continue; 
-        GG::Key key = i->first;
-        if ((key >= GG::GGK_a && key <= GG::GGK_z) || 
-            (key >= GG::GGK_0 && key <= GG::GGK_9)) {
-            m_disabled_accels_list.insert(key);
-        }
-    }
-    for (std::set<GG::Key>::iterator i = m_disabled_accels_list.begin();
-         i != m_disabled_accels_list.end(); ++i) {
-        GG::GUI::GetGUI()->RemoveAccelerator(*i);
-    }
+void CombatWnd::DisableTypingUnsafeAccels() {
+    HotkeyManager::GetManager()->DisableTypingUnsafeHotkeys();
 }
 
-void CombatWnd::EnableAlphaNumAccels() {
-    for (std::set<GG::Key>::iterator i = m_disabled_accels_list.begin();
-         i != m_disabled_accels_list.end(); ++i) {
-        GG::GUI::GetGUI()->SetAccelerator(*i);
-    }
-    m_disabled_accels_list.clear();
+void CombatWnd::EnableTypingUnsafeAccels() {
+    HotkeyManager::GetManager()->DisableTypingUnsafeHotkeys();
 }
 
 void CombatWnd::ChatMessageSentSlot() {
     if (!m_disabled_accels_list.empty()) {
-        EnableAlphaNumAccels();
+        EnableTypingUnsafeAccels();
         GG::GUI::GetGUI()->SetFocusWnd(this);
     }
 }
@@ -2138,12 +2082,12 @@ bool IsVisible(const Ogre::SceneNode& node) {
     return retval;
 }
 
-Ogre::SceneNode* CreateShipSceneNode(Ogre::SceneManager* scene_manager, const Ship& ship) {
+Ogre::SceneNode* CreateShipSceneNode(Ogre::SceneManager* scene_manager, TemporaryPtr<const Ship> ship) {
     return scene_manager->getRootSceneNode()->createChildSceneNode(
         ShipIDString(ship) + ShipMeshName(ship) + "_node");
 }
 
-Ogre::Entity* CreateShipEntity(Ogre::SceneManager* scene_manager, const Ship& ship,
+Ogre::Entity* CreateShipEntity(Ogre::SceneManager* scene_manager, TemporaryPtr<const Ship> ship,
                                const Ogre::MaterialPtr& material)
 {
     std::string mesh_name = ShipMeshName(ship);
