@@ -133,7 +133,7 @@ namespace {
 FleetButton::FleetButton(const std::vector<int>& fleet_IDs, SizeType size_type) :
     GG::Button(GG::X0, GG::Y0, GG::X1, GG::Y1, "", boost::shared_ptr<GG::Font>(), GG::CLR_ZERO),
     m_fleets(),
-    m_head_icon(),
+    m_head_icons(),
     m_size_icon(),
     m_selection_texture(),
     m_vertex_components(),
@@ -145,7 +145,7 @@ FleetButton::FleetButton(const std::vector<int>& fleet_IDs, SizeType size_type)
 FleetButton::FleetButton(int fleet_id, SizeType size_type) :
     GG::Button(GG::X0, GG::Y0, GG::X1, GG::Y1, "", boost::shared_ptr<GG::Font>(), GG::CLR_ZERO),
     m_fleets(),
-    m_head_icon(),
+    m_head_icons(),
     m_size_icon(),
     m_selection_texture(),
     m_vertex_components(),
@@ -166,9 +166,9 @@ void FleetButton::Init(const std::vector<int>& fleet_IDs, SizeType size_type) {
     }
 
     // get fleets
-    std::vector<const Fleet*> fleets;
+    std::vector<TemporaryPtr<const Fleet> > fleets;
     for (std::vector<int>::const_iterator it = fleet_IDs.begin(); it != fleet_IDs.end(); ++it) {
-        const Fleet* fleet = GetFleet(*it);
+        TemporaryPtr<const Fleet> fleet = GetFleet(*it);
         if (!fleet) {
             Logger().errorStream() << "FleetButton::FleetButton couldn't get fleet with id " << *it;
             continue;
@@ -188,8 +188,8 @@ void FleetButton::Init(const std::vector<int>& fleet_IDs, SizeType size_type) {
     } else {
         owner_id = (*fleets.begin())->Owner();
         // use ALL_EMPIRES if there are multiple owners (including no owner and an owner)
-        for (std::vector<const Fleet*>::const_iterator it = fleets.begin(); it != fleets.end(); ++it) {
-            const Fleet* fleet = *it;
+        for (std::vector<TemporaryPtr<const Fleet> >::const_iterator it = fleets.begin(); it != fleets.end(); ++it) {
+            TemporaryPtr<const Fleet> fleet = *it;
             if (fleet->Owner() != owner_id) {
                 owner_id = ALL_EMPIRES;
                 multiple_owners = true;
@@ -206,12 +206,12 @@ void FleetButton::Init(const std::vector<int>& fleet_IDs, SizeType size_type) {
         // all ships owned by now empire
         bool monsters = true;
         // find if any ship in fleets in button is not a monster
-        for (std::vector<const Fleet*>::const_iterator it = fleets.begin(); it != fleets.end(); ++it) {
-            const Fleet* fleet = *it;
+        for (std::vector<TemporaryPtr<const Fleet> >::const_iterator it = fleets.begin(); it != fleets.end(); ++it) {
+            TemporaryPtr<const Fleet> fleet = *it;
             for (std::set<int>::const_iterator ship_it = fleet->ShipIDs().begin();
                  ship_it != fleet->ShipIDs().end(); ++ship_it)
             {
-                if (const Ship* ship = GetShip(*ship_it)) {
+                if (TemporaryPtr<const Ship> ship = GetShip(*ship_it)) {
                     if (!ship->IsMonster()) {
                         monsters = false;
                         break;
@@ -234,35 +234,23 @@ void FleetButton::Init(const std::vector<int>& fleet_IDs, SizeType size_type) {
     }
 
 
-    // select icon(s) for fleet(s), and get a fleet for use later
-    const Fleet* first_fleet = 0;
-    if (m_fleets.size() != 1) {
-        first_fleet = *(fleets.begin());
-
-        // if there are multiple fleets, default to generic fleet icon by
-        // passing null pointer (0) to FleetHeadIcon.  TODO: instead, check if
-        // all fleets would use the same icon, and if so, use that icon, and
-        // only default to generic icon in cases where there are multiple
-        // different icons for the various fleets being represented
-        m_head_icon = FleetHeadIcon(0, size_type);
-        int num_ships = 0;
-        for (std::vector<const Fleet*>::const_iterator it = fleets.begin(); it != fleets.end(); ++it)
-            num_ships += (*it)->NumShips();
-        m_size_icon = FleetSizeIcon(num_ships, size_type);
-
-    } else if (!m_fleets.empty()) {
-        first_fleet = *fleets.begin();
-        m_head_icon = FleetHeadIcon(first_fleet, size_type);
-        m_size_icon = FleetSizeIcon(first_fleet, size_type);
+    // select icon(s) for fleet(s)
+    int num_ships = 0;
+    for (std::vector<TemporaryPtr<const Fleet> >::const_iterator it = fleets.begin(); it != fleets.end(); ++it) {
+        TemporaryPtr<const Fleet> fleet = *it;
+        if (fleet)
+            num_ships += fleet->NumShips();
     }
+    m_size_icon = FleetSizeIcon(num_ships, size_type);
+    m_head_icons = FleetHeadIcons(fleets, size_type);
 
     // resize to fit icon by first determining necessary size, and then resizing
     GG::X texture_width(0);
     GG::Y texture_height(0);
 
-    if (m_head_icon) {
-        texture_width = m_head_icon->DefaultWidth();
-        texture_height = m_head_icon->DefaultHeight();
+    if (!m_head_icons.empty()) {
+        texture_width = m_head_icons.front()->DefaultWidth();
+        texture_height = m_head_icons.front()->DefaultHeight();
     }
     if (m_size_icon) {
         texture_width = std::max(texture_width, m_size_icon->DefaultWidth());
@@ -274,9 +262,13 @@ void FleetButton::Init(const std::vector<int>& fleet_IDs, SizeType size_type) {
     // a starlane, which is the case if the fleet is not in a system and has a valid next system
     GG::Pt direction_vector(GG::X(0), GG::Y(1));    // default, unrotated button orientation
 
+    TemporaryPtr<const Fleet> first_fleet;
+    if (!m_fleets.empty())
+        first_fleet = *fleets.begin();
+    
     if (first_fleet && first_fleet->SystemID() == INVALID_OBJECT_ID) {
         int next_sys_id = first_fleet->NextSystemID();
-        if (const UniverseObject* obj = GetUniverseObject(next_sys_id)) {
+        if (TemporaryPtr<const UniverseObject> obj = GetUniverseObject(next_sys_id)) {
             // fleet is not in a system and has a valid next destination, so can orient it in that direction
             // fleet icons might not appear on the screen in the exact place corresponding to their 
             // actual universe position, but if they're moving along a starlane, this code will assume
@@ -360,8 +352,8 @@ void FleetButton::RenderUnpressed() {
     if (m_vertex_components.empty()) {
         if (m_size_icon)
             m_size_icon->OrthoBlit(ul);
-        if (m_head_icon)
-            m_head_icon->OrthoBlit(ul);
+        for (std::vector<boost::shared_ptr<GG::Texture> >::iterator it = m_head_icons.begin(); it != m_head_icons.end(); ++it)
+            (*it)->OrthoBlit(ul);
     } else {
         std::vector<double> vertsXY;
         vertsXY.push_back(midX + m_vertex_components[0]);
@@ -373,7 +365,8 @@ void FleetButton::RenderUnpressed() {
         vertsXY.push_back(midX + m_vertex_components[6]);
         vertsXY.push_back(midY + m_vertex_components[7]);
 
-        RenderTexturedQuad(vertsXY, m_head_icon);
+        for (std::vector<boost::shared_ptr<GG::Texture> >::iterator it = m_head_icons.begin(); it != m_head_icons.end(); ++it)
+            RenderTexturedQuad(vertsXY, *it);
         RenderTexturedQuad(vertsXY, m_size_icon);
     }
 
@@ -429,34 +422,57 @@ void FleetButton::PlayFleetButtonOpenSound()
 /////////////////////
 // Free Functions
 /////////////////////
-boost::shared_ptr<GG::Texture> FleetHeadIcon(const Fleet* fleet, FleetButton::SizeType size_type) {
+std::vector<boost::shared_ptr<GG::Texture> > FleetHeadIcons(TemporaryPtr<const Fleet> fleet, FleetButton::SizeType size_type) {
+    std::vector< TemporaryPtr<const Fleet> > fleets(1U, fleet);
+    return FleetHeadIcons(fleets, size_type);
+}
+
+std::vector<boost::shared_ptr<GG::Texture> > FleetHeadIcons(const std::vector< TemporaryPtr<const Fleet> >& fleets, FleetButton::SizeType size_type) {
     if (size_type == FleetButton::FLEET_BUTTON_NONE || size_type == FleetButton::FLEET_BUTTON_TINY)
-        return boost::shared_ptr<GG::Texture>();
+        return std::vector<boost::shared_ptr<GG::Texture> >();
 
     // get file name prefix for appropriate size of icon
     std::string size_prefix = FleetIconSizePrefix(size_type);
     if (size_prefix.empty())
-        return boost::shared_ptr<GG::Texture>();
+        return std::vector<boost::shared_ptr<GG::Texture> >();
 
+    // the set of fleets is treated like a fleet that contains all the ships
+    bool hasColonyShips = false; bool hasOutpostShips = false; bool hasTroopShips = false; bool hasMonsters = false; bool hasArmedShips = false;
+    for (std::vector< TemporaryPtr<const Fleet> >::const_iterator fleet_it = fleets.begin(); fleet_it != fleets.end(); ++fleet_it) {
+        const TemporaryPtr<const Fleet> fleet = *fleet_it;
+        if (!fleet) 
+            continue;
+        
+        hasColonyShips  = hasColonyShips  || fleet->HasColonyShips();
+        hasOutpostShips = hasOutpostShips || fleet->HasOutpostShips();
+        hasTroopShips   = hasTroopShips   || fleet->HasTroopShips();
+        hasMonsters     = hasMonsters     || fleet->HasMonsters();
+        hasArmedShips   = hasArmedShips   || fleet->HasArmedShips();
+    }
+    
     // get file name main part depending on type of fleet
-    std::string main_filename = "head-scout.png";
-    if (fleet && fleet->HasColonyShips())
-        main_filename = "head-colony.png";
-    else if (fleet && fleet->HasOutpostShips())
-        main_filename = "head-outpost.png";
-    else if (fleet && fleet->HasTroopShips())
-        main_filename = "head-lander.png";
-    else if (fleet && fleet->HasMonsters() && fleet->HasArmedShips())
-        main_filename = "head-monster.png";
-    else if (fleet && fleet->HasMonsters() && !fleet->HasArmedShips())
-        main_filename = "head-monster-harmless.png";
-    else if (fleet && fleet->HasArmedShips())
-        main_filename = "head-warship.png";
-
-    return ClientUI::GetTexture(ClientUI::ArtDir() / "icons" / "fleet" / (size_prefix + main_filename), false);
+    // symbol type prioritized by the ship type arbitrarily deemed "most important"
+    std::vector<std::string> main_filenames;
+    if (hasMonsters) {
+        if (hasArmedShips)   { main_filenames.push_back("head-monster.png"); }
+        else                 { main_filenames.push_back("head-monster-harmless.png"); }
+    } else {
+        if (hasArmedShips)   { main_filenames.push_back("head-warship.png"); }
+        if (hasColonyShips)  { main_filenames.push_back("head-colony.png");  }
+        if (hasOutpostShips) { main_filenames.push_back("head-outpost.png"); }
+        if (hasTroopShips)   { main_filenames.push_back("head-lander.png");  }
+    }
+    if (main_filenames.empty()) { main_filenames.push_back("head-scout.png"); }
+    
+    std::vector<boost::shared_ptr<GG::Texture> > result;
+    for (std::vector<std::string>::const_iterator it = main_filenames.begin(); it != main_filenames.end(); ++it) {
+        result.push_back(ClientUI::GetTexture(ClientUI::ArtDir() / "icons" / "fleet" / (size_prefix + *it), false));
+    }
+    
+    return result;
 }
 
-boost::shared_ptr<GG::Texture> FleetSizeIcon(const Fleet* fleet, FleetButton::SizeType size_type) {
+boost::shared_ptr<GG::Texture> FleetSizeIcon(TemporaryPtr<const Fleet> fleet, FleetButton::SizeType size_type) {
     if (!fleet)
         return FleetSizeIcon(1u, size_type);
     return FleetSizeIcon(fleet->NumShips(), size_type);
