@@ -85,6 +85,8 @@ public:
       * ModeratorActionsWnd. */
     ModeratorActionSetting      GetModeratorActionSetting() const;
 
+    bool                        AutoEndTurnEnabled() const;
+
     /** returns the position on the screen that corresponds to the specified
       * universe X and Y coordinates. */
     GG::Pt                      ScreenCoordsFromUniversePosition(double universe_x, double universe_y) const;
@@ -101,6 +103,8 @@ public:
     virtual void    LClick(const GG::Pt& pt, GG::Flags<GG::ModKey> mod_keys);
     virtual void    RClick(const GG::Pt& pt, GG::Flags<GG::ModKey> mod_keys);
     virtual void    MouseWheel(const GG::Pt& pt, int move, GG::Flags<GG::ModKey> mod_keys);
+    virtual void    KeyPress(GG::Key key, boost::uint32_t key_code_point, GG::Flags<GG::ModKey> mod_keys);
+    virtual void    KeyRelease(GG::Key key, boost::uint32_t key_code_point, GG::Flags<GG::ModKey> mod_keys);
 
     void            DoLayout();
 
@@ -113,14 +117,14 @@ public:
     void            ShowSystemNames();                                      //!< enables the system name text
     void            HideSystemNames();                                      //!< disables the system name text
 
-    mutable boost::signal<void (int)>       SystemLeftClickedSignal;
-    mutable boost::signal<void (int)>       SystemRightClickedSignal;
-    mutable boost::signal<void (int)>       SystemBrowsedSignal;
-    mutable boost::signal<void (double)>    ZoomedSignal;
+    mutable boost::signals2::signal<void (int)>    SystemLeftClickedSignal;
+    mutable boost::signals2::signal<void (int)>    SystemRightClickedSignal;
+    mutable boost::signals2::signal<void (int)>    SystemBrowsedSignal;
+    mutable boost::signals2::signal<void (double)> ZoomedSignal;
 
     void            CenterOnMapCoord(double x, double y);                   //!< centers the map on map position (x, y)
     void            CenterOnObject(int id);                                 //!< centers the map on object with id \a id
-    void            CenterOnObject(const UniverseObject* obj);              //!< centers the map on object \a id
+    void            CenterOnObject(TemporaryPtr<const UniverseObject> obj);              //!< centers the map on object \a id
 
     void            ShowPlanet(int planet_id);                              //!< brings up encyclopedia panel and displays info about the planet
     void            ShowCombatLog(int log_id);                              //!< brings up encyclopedia panel and displays info about the combat
@@ -139,7 +143,7 @@ public:
     void            ReselectLastSystem();                                   //!< re-selects the most recently selected system, if a valid one exists
     void            SelectPlanet(int planetID);                             //!< programatically selects planets on sidepanels.  catches signals from production wnd or sidepanel for when the user changes the selected planet
     void            SelectFleet(int fleetID);                               //!< programatically selects fleets by ID
-    void            SelectFleet(Fleet* fleet);                              //!< programatically selects fleets
+    void            SelectFleet(TemporaryPtr<Fleet> fleet);                              //!< programatically selects fleets
     void            ReselectLastFleet();                                    //!< re-selects the most recent selected fleet, if a valid one exists
 
     void            SetFleetMovementLine(const FleetButton* fleet_button);  //!< creates fleet movement lines for all fleets in the given FleetButton to indicate where (and whether) they are moving.  Move lines originate from the FleetButton.
@@ -178,13 +182,7 @@ private:
     void            RefreshPopulationIndicator();
     void            RefreshDetectionIndicator();
 
-    void            UpdateMetersAndResourcePools();                                                     ///< update meter estimates and resource pool amounts for this client's empire
-    void            UpdateMetersAndResourcePools(const std::vector<int>& objects_vec);                  ///< update meter estimates for indicated objects, and resource pool amounts for this client's empire
-    void            UpdateMetersAndResourcePools(int object_id, bool update_contained_objects = false); ///< update meter esimtates for indiacted objects, and resource pool amounts for this client's empire
     void            UpdateSidePanelSystemObjectMetersAndResourcePools();                                ///< update meter estimates for objects contained within the current system shown in the sidepanel, or all objects if there is no system shown
-    void            UpdateMeterEstimates();                                                             ///< re-estimates meter values of all known objects based on orders given
-    void            UpdateMeterEstimates(int object_id, bool update_contained_objects = false);         ///< re-estimates meter values of specified objects
-    void            UpdateMeterEstimates(const std::vector<int>& objects_vec);                          ///< re-estimates meter values of specified objects
     void            UpdateEmpireResourcePools();                                                        ///< recalculates production and predicted changes of player's empire's resource and population pools
 
     /** contains information necessary to render a single fleet movement line on the main map. also
@@ -211,10 +209,10 @@ private:
 
     void            RefreshFleetButtons();                      //!< removes old / existing and creates new fleet buttons
     void            RefreshFleetButtonSelectionIndicators();    //!< marks (only) selected fleets' buttons as selected
-    void            FleetAddedOrRemoved(Fleet& fleet);
+    void            FleetsAddedOrRemoved(const std::vector<TemporaryPtr<Fleet> >& fleets);
 
     void            DoFleetButtonsLayout();                     //!< does layout of fleet buttons
-    std::pair<double, double>   MovingFleetMapPositionOnLane(const Fleet* fleet) const; //!< returns position on map where a moving fleet should be displayed.  This is different from the fleet's actual universe position due to the squishing of fleets moving along a lane into the space between the system circles at the ends of the lane
+    std::pair<double, double>   MovingFleetMapPositionOnLane(TemporaryPtr<const Fleet> fleet) const; //!< returns position on map where a moving fleet should be displayed.  This is different from the fleet's actual universe position due to the squishing of fleets moving along a lane into the space between the system circles at the ends of the lane
 
     void            DoSystemIconsLayout();                      //!< does layout of system icons
     void            DoFieldIconsLayout();                       //!< does layout of field icons
@@ -267,46 +265,58 @@ private:
 
     void            SystemDoubleClicked(int system_id);
     void            SystemLeftClicked(int system_id);
-    void            SystemRightClicked(int system_id);
-    void            MouseEnteringSystem(int system_id);
+    void            SystemRightClicked(int system_id, GG::Flags< GG::ModKey > mod_keys);
+    void            MouseEnteringSystem(int system_id, GG::Flags< GG::ModKey > mod_keys);
     void            MouseLeavingSystem(int system_id);
 
     void            PlanetRightClicked(int planet_id);
     void            BuildingRightClicked(int building_id);
 
-    void            PlotFleetMovement(int system_id, bool execute_move);   //!< issues fleet move orders to appropriate fleets in active FleetWnd
+    void            ReplotProjectedFleetMovement(bool append); //!< Find any projected movement plots and replots them with the current append state
+    void            PlotFleetMovement(int system_id, bool execute_move, bool append);   //!< issues fleet move orders to appropriate fleets in active FleetWnd
 
     void            FleetButtonLeftClicked(const FleetButton* fleet_btn);
     void            FleetButtonRightClicked(const FleetButton* fleet_btn);
     void            FleetRightClicked(int fleet_id);
     void            FleetsRightClicked(const std::vector<int>& fleet_ids);
+    
+    void            ShipRightClicked(int fleet_id);
+    void            ShipsRightClicked(const std::vector<int>& fleet_ids);
 
-    void            ShipRightClicked(int ship_id);
-    void            ShipsRightClicked(const std::vector<int>& ship_ids);
-
-    void            UniverseObjectDeleted(const UniverseObject *obj);
+    void            UniverseObjectDeleted(TemporaryPtr<const UniverseObject> obj);
 
     bool            ReturnToMap();
-    bool            OpenChatWindow();
 
     bool            EndTurn();
+    void            ToggleAutoEndTurn();
 
     bool            ToggleModeratorActions();
     void            ShowModeratorActions();
     void            HideModeratorActions();
 
-    bool            ToggleObjects();
-    void            ShowObjects();
-    void            HideObjects();
+    bool            ToggleMessages();
+    void            ShowMessages();
+    bool            OpenMessages();
+    void            HideMessages();
 
     bool            ToggleSitRep();
     void            ShowSitRep();
     void            HideSitRep();
 
+    bool            ToggleEmpires();
+    void            ShowEmpires();
+    void            HideEmpires();
+
+    bool            ToggleObjects();
+    void            ShowObjects();
+    void            HideObjects();
+
     bool            TogglePedia();
     void            ShowPedia();
     void            HidePedia();
 
+    void            ShowGraphs();
+
     void            HideSidePanel();
     void            RestoreSidePanel();
 
@@ -339,17 +349,15 @@ private:
     bool            ZoomToSystemWithWastedPP();
 
     void            ConnectKeyboardAcceleratorSignals();        //!< connects signals from keyboard accelerators to various GUI responses
-    void            DisconnectKeyboardAcceleratorSignals();     //!< disconnects signals
-    void            SetAccelerators();                          //!< tells the GUI which keypress combinations to track and emit signals for when they occur
 
     /** Disables keyboard accelerators that use an alphanumeric key without
       * modifiers. This is useful if a keyboard input is required, so that the
       * keys aren't interpreted as an accelerator.
-      * @note Repeated calls of DisableAlphaNumAccels have to be followed by
-      * the same number of calls to EnableAlphaNumAccels to re-enable the
+      * @note Repeated calls of DisableTypingUnsafeAccels have to be followed by
+      * the same number of calls to EnableTypingUnsafeAccels to re-enable the
       * accelerators. */
-    void            DisableAlphaNumAccels();
-    void            EnableAlphaNumAccels();                     //!< Re-enable accelerators disabled by DisableAlphaNumAccels
+    void            DisableTypingUnsafeAccels();
+    void            EnableTypingUnsafeAccels();                     //!< Re-enable accelerators disabled by DisableTypingUnsafeAccels
 
     void            ChatMessageSentSlot();
 
@@ -362,7 +370,7 @@ private:
 
     void            HandleEmpireElimination(int empire_id);             //!< cleans up internal storage of now-invalidated empire ID
 
-    std::set<GG::Key>           m_disabled_accels_list;                 //!< the list of Accelerators disabled by \a DisableAlphaNumAccels
+    std::set<GG::Key>           m_disabled_accels_list;                 //!< the list of Accelerators disabled by \a DisableTypingUnsafeAccels
 
     std::vector<boost::shared_ptr<GG::Texture> >    m_backgrounds;      //!< starfield backgrounds
     std::vector<double>                             m_bg_scroll_rate;   //!< array, the rates at which each background scrolls
@@ -389,10 +397,8 @@ private:
     std::set<FleetButton*>                          m_moving_fleet_buttons;                 //!< icons representing fleets not at a system
     std::map<int, FleetButton*>                     m_fleet_buttons;                        //!< fleet icons, index by fleet
 
-    std::map<int, boost::signals::connection>               m_fleet_state_change_signals;
-    std::map<int, std::vector<boost::signals::connection> > m_system_fleet_insert_remove_signals;
-
-    std::set<boost::signals::connection>            m_keyboard_accelerator_signals;         //!< signals connecting keyboard accelerators to GUI responses
+    std::map<int, boost::signals2::connection>               m_fleet_state_change_signals;
+    std::map<int, std::vector<boost::signals2::connection> > m_system_fleet_insert_remove_signals;
 
     std::map<int, MovementLineData>                 m_fleet_lines;                          //!< lines used for moving fleets in the main map
     std::map<int, MovementLineData>                 m_projected_fleet_lines;                //!< lines that show the projected path of the active fleet in the FleetWnd
@@ -414,7 +420,9 @@ private:
 
     GG::Pt                      m_drag_offset;      //!< distance the cursor is from the upper-left corner of the window during a drag ((-1, -1) if no drag is occurring)
     bool                        m_dragged;          //!< tracks whether or not a drag occurs during a left button down sequence of events
-    CUITurnButton*              m_turn_update;      //!< button that updates player's turn
+    CUIButton*                  m_btn_turn;         //!< button that updates player's turn
+    GG::Button*                 m_btn_auto_turn;    //!< button that toggles whether to automatically end turns
+    bool                        m_auto_end_turn;    //!< should turns be ended automatically by this client?
     std::list<MapWndPopup*>     m_popups;           //!< list of currently active popup windows
     bool                        m_menu_showing;     //!< set during ShowMenu() to prevent reentrency
     int                         m_current_owned_system;
@@ -426,7 +434,10 @@ private:
     CUIToolBar*                 m_toolbar;
     StatisticIcon               *m_trade, *m_population, *m_research, *m_industry, *m_detection, *m_fleet;
     GG::Button                  *m_industry_wasted, *m_research_wasted;
-    SettableInWindowCUIButton   *m_btn_moderator, *m_btn_siterep, *m_btn_research, *m_btn_production, *m_btn_design, *m_btn_pedia, *m_btn_objects, *m_btn_menu;
+    SettableInWindowCUIButton   *m_btn_moderator, *m_btn_messages, *m_btn_empires,
+                                *m_btn_siterep, *m_btn_research, *m_btn_production,
+                                *m_btn_design, *m_btn_pedia, *m_btn_graphs,
+                                *m_btn_objects, *m_btn_menu;
     FPSIndicator*               m_FPS;
 
     MapScaleLine*               m_scale_line;       //!< indicates the on-screen distance that reprensents an in-universe distance
@@ -434,7 +445,10 @@ private:
 
     std::set<int>               m_fleets_exploring;
 
-    friend class PlayingTurn;
+    friend struct IntroMenu;
+    friend struct WaitingForGameStart;
+    friend struct PlayingTurn;
+    friend struct PlayingGame;
 };
 
 
