@@ -393,7 +393,7 @@ void MarkupBox::MarkupSurface::Refresh() {
         control->MoveTo(GG::Pt(left, top));
         top += control->Height() + EDGE_PAD;
 
-        std::cout << "MarkupSurface::Refresh control (x,y): " << control->UpperLeft().x << ", " << control->UpperLeft().y <<
+        std::cout << "MarkupSurface::Refresh control (x,y): " << control->Left() << ", " << control->Top() <<
                                                    " (w,h): " << control->Width() << ", " << control->Height() << std::endl;
     }
 
@@ -453,44 +453,7 @@ const std::string& MarkupBox::Text() const {
 
 void MarkupBox::Render() {
     // Draw outline and background...
-
-    // copied from CUIWnd
-    GG::Pt ul = UpperLeft();
-    GG::Pt lr = LowerRight();
-    GG::Pt cl_ul = ClientUpperLeft();
-    GG::Pt cl_lr = ClientLowerRight();
-
-    // use GL to draw the lines
-    glDisable(GL_TEXTURE_2D);
-    GLint initial_modes[2];
-    glGetIntegerv(GL_POLYGON_MODE, initial_modes);
-
-    // draw background
-    glPolygonMode(GL_BACK, GL_FILL);
-    glBegin(GL_POLYGON);
-        glColor(ClientUI::WndColor());
-        glVertex(ul.x, ul.y);
-        glVertex(lr.x, ul.y);
-        glVertex(lr.x, lr.y);
-        glVertex(ul.x, lr.y);
-        glVertex(ul.x, ul.y);
-    glEnd();
-
-    // draw outer border on pixel inside of the outer edge of the window
-    glPolygonMode(GL_BACK, GL_LINE);
-    glBegin(GL_POLYGON);
-        glColor(ClientUI::WndOuterBorderColor());
-        glVertex(ul.x, ul.y);
-        glVertex(lr.x, ul.y);
-        glVertex(lr.x, lr.y);
-        glVertex(ul.x, lr.y);
-        glVertex(ul.x, ul.y);
-    glEnd();
-
-    // reset this to whatever it was initially
-    glPolygonMode(GL_BACK, initial_modes[1]);
-
-    glEnable(GL_TEXTURE_2D);
+    GG::FlatRectangle(UpperLeft(), LowerRight(), ClientUI::WndColor(), ClientUI::WndOuterBorderColor(), 1);
 }
 
 void MarkupBox::SizeMove(const GG::Pt& ul, const GG::Pt& lr) {
