@@ -4,6 +4,7 @@
 #include "CUIControls.h"
 #include "../util/i18n.h"
 #include "../util/OptionsDB.h"
+#include "../util/AppInterface.h"
 #include "../Empire/EmpireManager.h"
 #include "../Empire/Empire.h"
 #include "InfoPanels.h"
@@ -20,7 +21,7 @@ namespace {
 // ModeratorActionsWnd
 ////////////////////////////////////////////////
 ModeratorActionsWnd::ModeratorActionsWnd(GG::X w, GG::Y h) :
-    CUIWnd(UserString("MODERATOR"), GG::X1, GG::Y1, w - 1, h - 1, GG::ONTOP | GG::INTERACTIVE | GG::DRAGABLE | GG::RESIZABLE | CLOSABLE),
+    CUIWnd(UserString("MODERATOR"), GG::X1, GG::Y1, w - 1, h - 1, GG::ONTOP | GG::INTERACTIVE | GG::DRAGABLE | GG::RESIZABLE | CLOSABLE | PINABLE ),
     m_actions_enabled(true),
     m_selected_action(MAS_NoAction),
     m_no_action_button(0),
@@ -67,7 +68,7 @@ ModeratorActionsWnd::ModeratorActionsWnd(GG::X w, GG::Y h) :
             ClientUI::ArtDir() / "stars", ClientUI::StarTypeFilePrefixes()[star_type], 0);
         GG::DropDownList::Row* row = new GG::DropDownList::Row();
         row->push_back(new GG::StaticGraphic(GG::X0, GG::Y0, CONTROL_WIDTH, CONTROL_HEIGHT,
-                                             disc_texture, style, GG::Flags<GG::WndFlag>()));
+                                             disc_texture, style, GG::NO_WND_FLAGS));
         m_star_type_drop->Insert(row);
     }
     m_star_type_drop->Select(m_star_type_drop->begin());        // default select first type
@@ -89,7 +90,7 @@ ModeratorActionsWnd::ModeratorActionsWnd(GG::X w, GG::Y h) :
         boost::shared_ptr<GG::Texture> texture = ClientUI::PlanetIcon(planet_type);
         GG::DropDownList::Row* row = new GG::DropDownList::Row();
         row->push_back(new GG::StaticGraphic(GG::X0, GG::Y0, CONTROL_WIDTH, CONTROL_HEIGHT,
-                                             texture, style, GG::Flags<GG::WndFlag>()));
+                                             texture, style, GG::NO_WND_FLAGS));
         m_planet_type_drop->Insert(row);
     }
     m_planet_type_drop->Select(m_planet_type_drop->begin());    // default select first type
@@ -100,7 +101,7 @@ ModeratorActionsWnd::ModeratorActionsWnd(GG::X w, GG::Y h) :
         boost::shared_ptr<GG::Texture> texture = ClientUI::PlanetSizeIcon(planet_size);
         GG::DropDownList::Row* row = new GG::DropDownList::Row();
         row->push_back(new GG::StaticGraphic(GG::X0, GG::Y0, CONTROL_WIDTH, CONTROL_HEIGHT,
-                                             texture, style, GG::Flags<GG::WndFlag>()));
+                                             texture, style, GG::NO_WND_FLAGS));
         m_planet_size_drop->Insert(row);
     }
     GG::DropDownList::iterator it = m_planet_size_drop->begin();
