@@ -29,7 +29,6 @@ namespace {
     boost::shared_ptr<GG::Texture> CombatIcon()     { return ClientUI::GetTexture(ClientUI::ArtDir() / "icons" / "combat.png"); }
     boost::shared_ptr<GG::Texture> WarIcon()        { return ClientUI::GetTexture(ClientUI::ArtDir() / "icons" / "war.png"); }
     boost::shared_ptr<GG::Texture> PeaceIcon()      { return ClientUI::GetTexture(ClientUI::ArtDir() / "icons" / "peace.png"); }
-    boost::shared_ptr<GG::Texture> DeadIcon()       { return ClientUI::GetTexture(ClientUI::ArtDir() / "icons" / "sitrep" / "empire_eliminated.png"); }
     boost::shared_ptr<GG::Texture> UnknownIcon()    { return ClientUI::GetTexture(ClientUI::ArtDir() / "icons" / "unknown.png"); }
     boost::shared_ptr<GG::Texture> ShipIcon()       { return ClientUI::GetTexture(ClientUI::ArtDir() / "icons" / "sitrep" / "fleet_arrived.png"); }
     boost::shared_ptr<GG::Texture> ProductionIcon() { return ClientUI::GetTexture(ClientUI::ArtDir() / "icons" / "meter" / "industry.png"); }
@@ -44,7 +43,7 @@ namespace {
     class PlayerDataPanel : public GG::Control {
     public:
         PlayerDataPanel(GG::X w, GG::Y h, int player_id) :
-            Control(GG::X0, GG::Y0, w, h, GG::Flags<GG::WndFlag>()),
+            Control(GG::X0, GG::Y0, w, h, GG::NO_WND_FLAGS),
             m_player_id(player_id),
             //m_player_name_text(0),
             m_empire_name_text(0),
@@ -111,7 +110,7 @@ namespace {
             if (Disabled())
                 border_colour = DisabledColor(border_colour);
 
-            GG::FlatRectangle(UpperLeft(), LowerRight(), background_colour,  border_colour, DATA_PANEL_BORDER);
+            GG::FlatRectangle(UpperLeft(), LowerRight(), background_colour, border_colour, DATA_PANEL_BORDER);
 
             glColor(GG::CLR_WHITE);
 
@@ -163,10 +162,8 @@ namespace {
                 HostIcon()->OrthoBlit(UpperLeft() + m_host_icon_ul, UpperLeft() + m_host_icon_ul + ICON_SIZE);
         }
 
-        void            Select(bool b) {
-            if (m_selected != b)
-                m_selected = b;
-        }
+        void            Select(bool b)
+        { m_selected = b; }
 
         virtual void    SizeMove(const GG::Pt& ul, const GG::Pt& lr) {
             const GG::Pt old_size = Size();
@@ -220,7 +217,7 @@ namespace {
             const std::set<int>& this_client_stale_object_info       = GetUniverse().EmpireStaleKnowledgeObjectIDs(HumanClientApp::GetApp()->EmpireID());
 
             for (ObjectMap::const_iterator<Ship> ship_it = objects.const_begin<Ship>(); ship_it != objects.const_end<Ship>(); ++ship_it) {
-                const Ship* ship = *ship_it;
+                TemporaryPtr<const Ship> ship = *ship_it;
                 if (empire) {
                     if (ship->Owner() == empire->EmpireID()
                         && this_client_known_destroyed_objects.find(ship->ID()) == this_client_known_destroyed_objects.end()
@@ -231,7 +228,7 @@ namespace {
             }
 
             for (ObjectMap::const_iterator<Planet> planet_it = objects.const_begin<Planet>(); planet_it != objects.const_end<Planet>(); ++planet_it) {
-                const Planet* planet = *planet_it;
+                TemporaryPtr<const Planet> planet = *planet_it;
                 if (empire) {
                     if (planet->Owner() == empire->EmpireID()) {
                         empires_planet_count      += 1;
@@ -380,7 +377,6 @@ namespace {
         {
             SetName("PlayerRow");
             SetChildClippingMode(ClipToClient);
-            SetDragDropDataType("PLAYER_ROW");
             m_panel = new PlayerDataPanel(w, h, m_player_id);
             push_back(m_panel);
         }
@@ -454,12 +450,12 @@ public:
 //  PlayerListWnd  //
 /////////////////////
 PlayerListWnd::PlayerListWnd(GG::X x, GG::Y y, GG::X w, GG::Y h) :
-    CUIWnd(UserString("PLAYERS_LIST_PANEL_TITLE"), x, y, w, h, GG::INTERACTIVE | GG::DRAGABLE | GG::ONTOP | GG::RESIZABLE | CLOSABLE),
+    CUIWnd(UserString("PLAYERS_LIST_PANEL_TITLE"), x, y, w, h, GG::INTERACTIVE | GG::DRAGABLE | GG::ONTOP | GG::RESIZABLE | CLOSABLE | PINABLE ),
     m_player_list(0)
 {
     m_player_list = new PlayerListBox(GG::X0, GG::Y0, ClientWidth(), ClientHeight());
     m_player_list->SetHiliteColor(GG::CLR_ZERO);
-    m_player_list->SetStyle(GG::LIST_QUICKSEL | GG::LIST_NOSORT);
+    m_player_list->SetStyle(GG::LIST_NOSORT);
     GG::Connect(m_player_list->SelChangedSignal,            &PlayerListWnd::PlayerSelectionChanged, this);
     GG::Connect(m_player_list->DoubleClickedSignal,         &PlayerListWnd::PlayerDoubleClicked,    this);
     GG::Connect(m_player_list->RightClickedSignal,          &PlayerListWnd::PlayerRightClicked,     this);
@@ -578,6 +574,9 @@ void PlayerListWnd::SizeMove(const GG::Pt& ul, const GG::Pt& lr) {
 void PlayerListWnd::DoLayout()
 { m_player_list->SizeMove(GG::Pt(), GG::Pt(ClientWidth(), ClientHeight() - GG::Y(INNER_BORDER_ANGLE_OFFSET))); }
 
+void PlayerListWnd::CloseClicked()
+{ ClosingSignal(); }
+
 void PlayerListWnd::PlayerSelectionChanged(const GG::ListBox::SelectionSet& rows) {
     // mark as selected all PlayerDataPanel that are in \a rows and mark as not
     // selected all PlayerDataPanel that aren't in \a rows
@@ -629,10 +628,6 @@ void PlayerListWnd::PlayerRightClicked(GG::ListBox::iterator it, const GG::Pt& p
     if (client_player_id == Networking::INVALID_PLAYER_ID)
         return;
     int client_empire_id = app->EmpireID();
-    if (client_empire_id == ALL_EMPIRES)
-        return;
-    if (clicked_player_id == client_player_id)
-        return;
 
     // get empire id of clicked player
     const std::map<int, PlayerInfo>& players = app->Players();
@@ -644,8 +639,6 @@ void PlayerListWnd::PlayerRightClicked(GG::ListBox::iterator it, const GG::Pt& p
     const PlayerInfo& clicked_player_info = clicked_player_it->second;
     int clicked_empire_id = clicked_player_info.empire_id;
 
-    if (clicked_empire_id == ALL_EMPIRES)
-        return;
     if (!Empires().Lookup(clicked_empire_id)) {
         Logger().errorStream() << "PlayerListWnd::PlayerRightClicked tried to look up empire id "
                                << clicked_empire_id << " for player " << clicked_player_id
@@ -653,37 +646,43 @@ void PlayerListWnd::PlayerRightClicked(GG::ListBox::iterator it, const GG::Pt& p
         return;
     }
 
-    // get diplomatic status between client and clicked empires
-    DiplomaticStatus diplo_status = Empires().GetDiplomaticStatus(clicked_empire_id, client_empire_id);
-    if (diplo_status == INVALID_DIPLOMATIC_STATUS) {
-        Logger().errorStream() << "PlayerListWnd::PlayerRightClicked found invalid diplomatic status between client and clicked empires.";
-        return;
-    }
-    DiplomaticMessage existing_message = Empires().GetDiplomaticMessage(clicked_empire_id, client_empire_id);
-
-    // create popup menu with diplomacy options in it
     GG::MenuItem menu_contents;
-    if (diplo_status == DIPLO_WAR) {
-        if (existing_message.GetType() == DiplomaticMessage::PEACE_PROPOSAL) {
-            // who sent message?
-            if (existing_message.SenderEmpireID() == client_empire_id)
-                menu_contents.next_level.push_back(GG::MenuItem(UserString("PEACE_PROPOSAL_CANEL"), 4, false, false));
-            else if (existing_message.SenderEmpireID() == clicked_empire_id)
-                menu_contents.next_level.push_back(GG::MenuItem(UserString("PEACE_ACCEPT"),         3, false, false));
-
-        } else if (existing_message.GetType() == DiplomaticMessage::INVALID_DIPLOMATIC_MESSAGE_TYPE) {
-            menu_contents.next_level.push_back(GG::MenuItem(UserString("PEACE_PROPOSAL"),           2, false, false));
+    if (app->GetClientType() == Networking::CLIENT_TYPE_HUMAN_PLAYER) {
+        // get diplomatic status between client and clicked empires
+        DiplomaticStatus diplo_status = Empires().GetDiplomaticStatus(clicked_empire_id, client_empire_id);
+        if (diplo_status == INVALID_DIPLOMATIC_STATUS && clicked_player_id != client_player_id) {
+            Logger().errorStream() << "PlayerListWnd::PlayerRightClicked found invalid diplomatic status between client and clicked empires.";
+            return;
         }
+        DiplomaticMessage existing_message = Empires().GetDiplomaticMessage(clicked_empire_id, client_empire_id);
 
-    } else if (diplo_status == DIPLO_PEACE) {
-        if (existing_message.GetType() == DiplomaticMessage::INVALID_DIPLOMATIC_MESSAGE_TYPE)
-            menu_contents.next_level.push_back(GG::MenuItem(UserString("WAR_DECLARATION"),          1, false, false));
+        // create popup menu with diplomacy options in it
+        if ( client_empire_id != ALL_EMPIRES) {
+            if (diplo_status == DIPLO_WAR) {
+                if (existing_message.GetType() == DiplomaticMessage::PEACE_PROPOSAL) {
+                    // who sent message?
+                    if (existing_message.SenderEmpireID() == client_empire_id)
+                        menu_contents.next_level.push_back(GG::MenuItem(UserString("PEACE_PROPOSAL_CANEL"), 4, false, false));
+                    else if (existing_message.SenderEmpireID() == clicked_empire_id)
+                        menu_contents.next_level.push_back(GG::MenuItem(UserString("PEACE_ACCEPT"),         3, false, false));
+
+                } else if (existing_message.GetType() == DiplomaticMessage::INVALID_DIPLOMATIC_MESSAGE_TYPE) {
+                    menu_contents.next_level.push_back(GG::MenuItem(UserString("PEACE_PROPOSAL"),           2, false, false));
+                }
+
+            } else if (diplo_status == DIPLO_PEACE) {
+                if (existing_message.GetType() == DiplomaticMessage::INVALID_DIPLOMATIC_MESSAGE_TYPE)
+                    menu_contents.next_level.push_back(GG::MenuItem(UserString("WAR_DECLARATION"),          1, false, false));
+            }
+        }
     }
 
+    menu_contents.next_level.push_back(GG::MenuItem(str(FlexibleFormat(UserString("ENC_LOOKUP")) % Empires().Lookup(clicked_empire_id)->Name()), 5, false, false));
+
     ClientNetworking& net = HumanClientApp::GetApp()->Networking();
 
     GG::PopupMenu popup(pt.x, pt.y, ClientUI::GetFont(), menu_contents, ClientUI::TextColor(),
-                        ClientUI::WndOuterBorderColor(), ClientUI::WndColor());
+                        ClientUI::WndOuterBorderColor(), ClientUI::WndColor(), ClientUI::EditHiliteColor());
     if (popup.Run()) {
         switch (popup.MenuID()) {
         case 1: {   // WAR_DECLARATION
@@ -706,6 +705,10 @@ void PlayerListWnd::PlayerRightClicked(GG::ListBox::iterator it, const GG::Pt& p
                                              CancelDiplomaticMessage(client_empire_id, clicked_empire_id)));
             break;
         }
+        case 5: { // Pedia lookup
+            ClientUI::GetClientUI()->ZoomToEmpire(clicked_empire_id);
+            break;
+        }
         default:
             break;
         }
