@@ -7,6 +7,7 @@
 #include "../universe/Tech.h"
 #include "../util/i18n.h"
 #include "../util/Order.h"
+#include "../util/OptionsDB.h"
 #include "../UI/TechTreeWnd.h"
 #include "../client/human/HumanClientApp.h"
 
@@ -18,16 +19,14 @@
 #include <cmath>
 
 namespace {
-    const GG::X RESEARCH_INFO_AND_QUEUE_WIDTH(250);
     const float OUTER_LINE_THICKNESS = 2.0f;
 
-    //////////////////////////////////////////////////
-    // QueueRow
-    //////////////////////////////////////////////////
-    struct QueueRow : GG::ListBox::Row {
-        QueueRow(GG::X w, const ResearchQueue::Element& queue_element);
-        std::string tech_name;
-    };
+    void AddOptions(OptionsDB& db) {
+        // queue width used also on production screen. prevent double-adding...
+        if (!db.OptionExists("UI.queue-width"))
+            db.Add("UI.queue-width",    UserStringNop("OPTIONS_DB_UI_QUEUE_WIDTH"), 300,    RangedValidator<int>(200, 500));
+    }
+    bool temp_bool = RegisterOptions(&AddOptions);
 
     //////////////////////////////////////////////////
     // QueueTechPanel
@@ -35,7 +34,7 @@ namespace {
     class QueueTechPanel : public GG::Control {
     public:
         QueueTechPanel(GG::X w, const std::string& tech_name, double allocated_rp,
-                       int turns_left, double turns_completed);
+                       int turns_left, double turns_completed, int empire_id);
         virtual void Render();
 
     private:
@@ -49,45 +48,52 @@ namespace {
         MultiTurnProgressBar*   m_progress_bar;
         bool                    m_in_progress;
         int                     m_total_turns;
-        double                  m_turns_completed;
+        int                     m_empire_id;
     };
 
     //////////////////////////////////////////////////
-    // QueueRow implementation
+    // QueueRow
     //////////////////////////////////////////////////
-    QueueRow::QueueRow(GG::X w, const ResearchQueue::Element& queue_element) :
-        GG::ListBox::Row(),
-        tech_name(queue_element.name)
-    {
-        int empire_id = HumanClientApp::GetApp()->EmpireID();
-        const Empire* empire = Empires().Lookup(empire_id);
+    struct QueueRow : GG::ListBox::Row {
+        QueueRow(GG::X w, const ResearchQueue::Element& queue_element) :
+            GG::ListBox::Row(),
+            tech_name(queue_element.name)
+        {
+            const Empire* empire = Empires().Lookup(queue_element.empire_id);
 
-        const Tech* tech = GetTech(tech_name);
-        double per_turn_cost = tech ? tech->PerTurnCost(empire_id) : 1;
-        double progress = 0.0;
-        if (empire && empire->TechResearched(tech_name))
-            progress = tech->ResearchCost(empire_id);
-        else if (empire)
-            progress = empire->ResearchProgress(tech_name);
+            const Tech* tech = GetTech(tech_name);
+            double per_turn_cost = tech ? tech->PerTurnCost(queue_element.empire_id) : 1;
+            double progress = 0.0;
+            if (empire && empire->TechResearched(tech_name))
+                progress = tech->ResearchCost(queue_element.empire_id);
+            else if (empire)
+                progress = empire->ResearchProgress(tech_name);
 
-        GG::Control* panel = new QueueTechPanel(w, tech_name, queue_element.allocated_rp,
-                                                queue_element.turns_left, progress / per_turn_cost);
-        Resize(panel->Size());
-        push_back(panel);
+            GG::Control* panel = new QueueTechPanel(w, tech_name, queue_element.allocated_rp,
+                                                    queue_element.turns_left, progress / per_turn_cost,
+                                                    queue_element.empire_id);
+            Resize(panel->Size());
+            push_back(panel);
 
-        SetDragDropDataType("RESEARCH_QUEUE_ROW");
-    }
+            SetDragDropDataType("RESEARCH_QUEUE_ROW");
+
+            SetBrowseModeTime(GetOptionsDB().Get<int>("UI.tooltip-delay"));
+            SetBrowseInfoWnd(TechPanelRowBrowseWnd(queue_element.name, queue_element.empire_id));
+        }
+
+        std::string tech_name;
+    };
 
     //////////////////////////////////////////////////
     // QueueTechPanel implementation
     //////////////////////////////////////////////////
     QueueTechPanel::QueueTechPanel(GG::X w, const std::string& tech_name, double turn_spending,
-                                   int turns_left, double turns_completed) :
-        GG::Control(GG::X0, GG::Y0, w, GG::Y(10), GG::Flags<GG::WndFlag>()),
+                                   int turns_left, double turns_completed, int empire_id) :
+        GG::Control(GG::X0, GG::Y0, w, GG::Y(10), GG::NO_WND_FLAGS),
         m_tech_name(tech_name),
         m_in_progress(turn_spending),
         m_total_turns(1),
-        m_turns_completed(turns_completed)
+        m_empire_id(empire_id)
     {
         const int MARGIN = 2;
 
@@ -104,11 +110,13 @@ namespace {
 
         const Tech* tech = GetTech(m_tech_name);
         if (tech)
-            m_total_turns = tech->ResearchTime(HumanClientApp::GetApp()->EmpireID());
+            m_total_turns = tech->ResearchTime(m_empire_id);
 
         Resize(GG::Pt(w, HEIGHT));
 
-        GG::Clr clr = m_in_progress ? GG::LightColor(ClientUI::ResearchableTechTextAndBorderColor()) : ClientUI::ResearchableTechTextAndBorderColor();
+        GG::Clr clr = m_in_progress
+                        ? GG::LightColor(ClientUI::ResearchableTechTextAndBorderColor())
+                        : ClientUI::ResearchableTechTextAndBorderColor();
         boost::shared_ptr<GG::Font> font = ClientUI::GetFont();
 
         GG::Y top(MARGIN);
@@ -119,23 +127,20 @@ namespace {
                                        ClientUI::TechIcon(m_tech_name),
                                        GG::GRAPHIC_FITGRAPHIC);
         m_icon->SetColor(tech ? ClientUI::CategoryColor(tech->Category()) : GG::Clr());
-
         left += m_icon->Width() + MARGIN;
 
         m_name_text = new GG::TextControl(left, top, NAME_WIDTH, GG::Y(FONT_PTS + 2*MARGIN),
                                           UserString(m_tech_name),
                                           font, clr, GG::FORMAT_TOP | GG::FORMAT_LEFT);
         m_name_text->ClipText(true);
+        top += m_name_text->Height();
 
-        top += m_name_text->Height();    // not sure why I need two margins here... otherwise the progress bar appears over the bottom of the text
-
-        m_progress_bar = new MultiTurnProgressBar(METER_WIDTH, METER_HEIGHT, tech ? tech->ResearchTime(HumanClientApp::GetApp()->EmpireID()) : 1,
+        m_progress_bar = new MultiTurnProgressBar(METER_WIDTH, METER_HEIGHT, tech ? tech->ResearchTime(m_empire_id) : 1,
                                                   turns_completed,
                                                   GG::LightColor(ClientUI::TechWndProgressBarBackgroundColor()),
                                                   ClientUI::TechWndProgressBarColor(),
                                                   m_in_progress ? ClientUI::ResearchableTechFillColor() : GG::LightColor(ClientUI::ResearchableTechFillColor()) );
         m_progress_bar->MoveTo(GG::Pt(left, top));
-
         top += m_progress_bar->Height() + MARGIN;
 
         using boost::io::str;
@@ -152,7 +157,6 @@ namespace {
                                                      turns_left_text, font, clr, GG::FORMAT_RIGHT);
         m_turns_remaining_text->ClipText(true);
 
-
         AttachChild(m_name_text);
         AttachChild(m_RPs_and_turns_text);
         AttachChild(m_turns_remaining_text);
@@ -193,8 +197,11 @@ ResearchWnd::ResearchWnd(GG::X w, GG::Y h) :
     m_tech_tree_wnd(0),
     m_enabled(false)
 {
-    m_research_info_panel = new ProductionInfoPanel(RESEARCH_INFO_AND_QUEUE_WIDTH, GG::Y(200), UserString("RESEARCH_INFO_PANEL_TITLE"), UserString("RESEARCH_INFO_RP"),
-                                                    OUTER_LINE_THICKNESS, ClientUI::KnownTechFillColor(), ClientUI::KnownTechTextAndBorderColor());
+    m_research_info_panel = new ProductionInfoPanel(GG::X(GetOptionsDB().Get<int>("UI.queue-width")), GG::Y(200),
+                                                    UserString("RESEARCH_INFO_PANEL_TITLE"),
+                                                    UserString("RESEARCH_INFO_RP"),
+                                                    OUTER_LINE_THICKNESS, ClientUI::KnownTechFillColor(),
+                                                    ClientUI::KnownTechTextAndBorderColor());
 
     m_queue_lb = new QueueListBox(GG::X(2),
                                   m_research_info_panel->Height(),
@@ -202,15 +209,18 @@ ResearchWnd::ResearchWnd(GG::X w, GG::Y h) :
                                   ClientSize().y - 4 - m_research_info_panel->Height(),
                                   "RESEARCH_QUEUE_ROW",
                                   UserString("RESEARCH_QUEUE_PROMPT"));
-    GG::Connect(m_queue_lb->QueueItemMoved, &ResearchWnd::QueueItemMoved, this);
     m_queue_lb->SetStyle(GG::LIST_NOSORT | GG::LIST_NOSEL | GG::LIST_USERDELETE);
 
+    GG::Connect(m_queue_lb->QueueItemMovedSignal,       &ResearchWnd::QueueItemMoved,               this);
+    GG::Connect(m_queue_lb->LeftClickedSignal,          &ResearchWnd::QueueItemClickedSlot,         this);
+    GG::Connect(m_queue_lb->DoubleClickedSignal,        &ResearchWnd::QueueItemDoubleClickedSlot,   this);
+
+
     GG::Pt tech_tree_wnd_size = ClientSize() - GG::Pt(m_research_info_panel->Width(), GG::Y0);
     m_tech_tree_wnd = new TechTreeWnd(tech_tree_wnd_size.x, tech_tree_wnd_size.y);
 
     GG::Connect(m_tech_tree_wnd->AddTechsToQueueSignal, &ResearchWnd::AddTechsToQueueSlot,          this);
-    GG::Connect(m_queue_lb->LeftClickedSignal,          &ResearchWnd::QueueItemClickedSlot,         this);
-    GG::Connect(m_queue_lb->DoubleClickedSignal,        &ResearchWnd::QueueItemDoubleClickedSlot,   this);
+
 
     AttachChild(m_research_info_panel);
     AttachChild(m_queue_lb);
@@ -308,8 +318,10 @@ void ResearchWnd::UpdateQueue() {
     m_queue_lb->Clear();
     const GG::X QUEUE_WIDTH = m_queue_lb->Width() - 8 - 14;
 
-    for (ResearchQueue::const_iterator it = queue.begin(); it != queue.end(); ++it)
-        m_queue_lb->Insert(new QueueRow(QUEUE_WIDTH, *it));
+    for (ResearchQueue::const_iterator it = queue.begin(); it != queue.end(); ++it) {
+        const ResearchQueue::Element& elem = *it;
+        m_queue_lb->Insert(new QueueRow(QUEUE_WIDTH, elem));
+    }
 
     if (!m_queue_lb->Empty())
         m_queue_lb->BringRowIntoView(--m_queue_lb->end());
