@@ -17,6 +17,10 @@
 
 
 namespace {
+    GG::X ICON_WIDTH(16);
+    GG::X ICON_RIGHT_MARGIN(3);
+    GG::Y ITEM_VERTICAL_PADDING(2);
+
     /** Adds options related to SitRepPanel to Options DB. */
     void AddOptions(OptionsDB& db)
     { db.Add("verbose-sitrep", UserStringNop("OPTIONS_DB_VERBOSE_SITREP_DESC"),  false,  Validator<bool>()); }
@@ -70,26 +74,43 @@ namespace {
 
     std::vector<std::string> OrderedSitrepTemplateStrings() {
         // determine sitrep order
-        //Logger().debugStream() << "Sitrep Order: " << UserString("SITREP_PRIORITY_ORDER");
         std::istringstream template_stream(UserString("SITREP_PRIORITY_ORDER"));
         std::vector<std::string> sitrep_order;
-        std::copy(std::istream_iterator<std::string>(template_stream), 
-                    std::istream_iterator<std::string>(),
+        std::copy(std::istream_iterator<std::string>(template_stream),
+                  std::istream_iterator<std::string>(),
                   std::back_inserter<std::vector<std::string> >(sitrep_order));
         return sitrep_order;
     }
 
-    std::vector<std::string> AllSitRepTemplateStrings() {
+    std::set<std::string> EmpireSitRepTemplateStrings(int empire_id) {
         std::set<std::string> template_set;
-        std::vector<std::string> retval;
-        Empire *empire = HumanClientApp::GetApp()->Empires().Lookup(HumanClientApp::GetApp()->EmpireID());
+
+        Empire *empire = HumanClientApp::GetApp()->Empires().Lookup(empire_id);
         if (!empire)
-            return retval;
+            return template_set;
+
         for (Empire::SitRepItr sitrep_it = empire->SitRepBegin();
              sitrep_it != empire->SitRepEnd(); ++sitrep_it)
         { template_set.insert(sitrep_it->GetTemplateString()); }
-        std::vector<std::string> ordered_templates = OrderedSitrepTemplateStrings();
+
+        return template_set;
+    }
+
+    std::vector<std::string> AllSitRepTemplateStrings() {
+        std::set<std::string> template_set;
+
+        // get templates for each empire
+        for (EmpireManager::iterator it = Empires().begin();
+             it != Empires().end(); ++it)
+        {
+            std::set<std::string> empire_strings = EmpireSitRepTemplateStrings(it->first);
+            template_set.insert(empire_strings.begin(), empire_strings.end());
+        }
+
+        std::vector<std::string> retval;
+
         // only use those ordered templates actually in the current set of sitrep templates
+        std::vector<std::string> ordered_templates = OrderedSitrepTemplateStrings();
         for (std::vector<std::string>::iterator it = ordered_templates.begin(); 
              it!=ordered_templates.end(); it++) 
         {
@@ -97,20 +118,35 @@ namespace {
                  (std::find(retval.begin(), retval.end(), *it) == retval.end()) )
             { retval.push_back(*it); }
         }
+
         //now add the current templates that did not have a specified order
         for (std::set<std::string>::iterator it = template_set.begin(); it!= template_set.end(); it++)
             if (std::find(retval.begin(), retval.end(), *it) == retval.end())
                 retval.push_back(*it);
+
         return retval;
     }
 
+    class ColorEmpire : public LinkDecorator {
+    public:
+        virtual std::string Decorate ( const std::string& target, const std::string& content ) const{
+            GG::Clr color = ClientUI::DefaultLinkColor();
+            int id = try_to_int(target);
+            Empire* empire = Empires().Lookup(id);
+            if( empire ) {
+                color = empire->Color();
+            }
+            return GG::RgbaTag(color) + content + "</rgba>";
+        }
+    };
+
     //////////////////////////////////
     // SitRepDataPanel
     //////////////////////////////////
     class SitRepDataPanel : public GG::Control {
     public:
         SitRepDataPanel(GG::X w, GG::Y h, const SitRepEntry& sitrep) :
-            Control(GG::X0, GG::Y0, w, h, GG::Flags<GG::WndFlag>()),
+            Control(GG::X0, GG::Y0, w, h, GG::NO_WND_FLAGS),
             m_initialized(false),
             m_sitrep_entry(sitrep),
             m_icon(0),
@@ -123,14 +159,25 @@ namespace {
             if (!m_initialized)
                 Init();
             GG::Clr background_clr = this->Disabled() ? ClientUI::WndColor() : ClientUI::CtrlColor();
-            GG::FlatRectangle(UpperLeft(), LowerRight(), background_clr, ClientUI::WndOuterBorderColor(), 1u);
+            GG::FlatRectangle(UpperLeft(), LowerRight() - GG::Pt(GG::X0, GG::Y(2)), background_clr, ClientUI::WndOuterBorderColor(), 1u);
         }
 
         virtual void        SizeMove(const GG::Pt& ul, const GG::Pt& lr) {
+            Init();
             const GG::Pt old_size = Size();
             GG::Control::SizeMove(ul, lr);
-            if (old_size != Size())
+            if (old_size != Size()) {
                 DoLayout();
+            }
+            if (m_link_text) {
+                // Use the height of the text as our height.
+                // DoLayout reflowed the text.
+                GG::Pt text_size = m_link_text->TextLowerRight() - m_link_text->TextUpperLeft();
+                text_size.y += ITEM_VERTICAL_PADDING*2; // Text centers, so this puts padding on both above and below
+                text_size.x = lr.x - ul.x; // Ignore the width of the text, use whatever was requested.
+                GG::Control::SizeMove(ul, ul + text_size );
+                DoLayout();
+            }
         }
 
         const SitRepEntry&  GetSitRepEntry() const { return m_sitrep_entry; }
@@ -139,15 +186,13 @@ namespace {
         void            DoLayout() {
             if (!m_initialized)
                 return;
-            const GG::Y ICON_HEIGHT(ClientHeight());
-            const GG::X ICON_WIDTH(Value(ClientHeight()));
+            const GG::Y ICON_HEIGHT(Value(ICON_WIDTH));
 
             GG::X left(GG::X0);
-            GG::Y top(GG::Y0);
             GG::Y bottom(ClientHeight());
 
-            m_icon->SizeMove(GG::Pt(left, GG::Y0), GG::Pt(left + ICON_WIDTH, bottom));
-            left += ICON_WIDTH + GG::X(3);
+            m_icon->SizeMove(GG::Pt(left, bottom/2 - ICON_HEIGHT/2), GG::Pt(left + ICON_WIDTH, bottom/2 + ICON_HEIGHT/2));
+            left += ICON_WIDTH + ICON_RIGHT_MARGIN;
 
             m_link_text->SizeMove(GG::Pt(left, GG::Y0), GG::Pt(ClientWidth(), bottom));
         }
@@ -167,7 +212,8 @@ namespace {
 
             m_link_text = new LinkText(m_icon->Width(), GG::Y0, Width() - m_icon->Width(),
                                        m_sitrep_entry.GetText() + " ", ClientUI::GetFont(),
-                                       GG::FORMAT_LEFT | GG::FORMAT_VCENTER, ClientUI::TextColor());
+                                       GG::FORMAT_LEFT | GG::FORMAT_VCENTER | GG::FORMAT_WORDBREAK, ClientUI::TextColor());
+            m_link_text->SetDecorator(VarText::EMPIRE_ID_TAG, new ColorEmpire());
             AttachChild(m_link_text);
 
             GG::Connect(m_link_text->LinkClickedSignal,       &HandleLinkClick);
@@ -203,9 +249,13 @@ namespace {
 
         void SizeMove(const GG::Pt& ul, const GG::Pt& lr) {
             const GG::Pt old_size = Size();
-            GG::ListBox::Row::SizeMove(ul, lr);
-            if (!empty() && old_size != Size() && m_panel)
-                m_panel->Resize(Size());
+            GG::Pt new_size = lr - ul;
+            new_size.x -= 7; // Avoid allowing the scrollbar to hide the very rightmost pixels.
+            if (!empty() && m_panel && old_size != new_size) {
+                m_panel->Resize(new_size);
+                new_size.y = m_panel->Size().y;
+            }
+            GG::ListBox::Row::SizeMove(ul, ul + new_size);
         }
 
         const SitRepEntry&  GetSitRepEntry() const { return m_panel->GetSitRepEntry(); }
@@ -216,7 +266,7 @@ namespace {
 }
 
 SitRepPanel::SitRepPanel(GG::X x, GG::Y y, GG::X w, GG::Y h) :
-    CUIWnd(UserString("SITREP_PANEL_TITLE"), x, y, w, h, GG::ONTOP | GG::INTERACTIVE | GG::DRAGABLE | GG::RESIZABLE | CLOSABLE),
+    CUIWnd(UserString("SITREP_PANEL_TITLE"), x, y, w, h, GG::ONTOP | GG::INTERACTIVE | GG::DRAGABLE | GG::RESIZABLE | CLOSABLE | PINABLE ),
     m_sitreps_lb(0),
     m_prev_turn_button(0),
     m_next_turn_button(0),
@@ -228,15 +278,16 @@ SitRepPanel::SitRepPanel(GG::X x, GG::Y y, GG::X w, GG::Y h) :
 
     m_sitreps_lb = new CUIListBox(GG::X0, GG::Y0, GG::X1, GG::Y1);
     m_sitreps_lb->SetStyle(GG::LIST_NOSORT | GG::LIST_NOSEL);
+    m_sitreps_lb->SetVScrollWheelIncrement(ClientUI::Pts()*4.5);
     AttachChild(m_sitreps_lb);
 
-    m_prev_turn_button = new CUIButton(GG::X0, GG::Y0, GG::X(20), UserString("BACK"));
+    m_prev_turn_button = new CUIButton(UserString("BACK"));
     AttachChild(m_prev_turn_button);
-    m_next_turn_button = new CUIButton(GG::X0, GG::Y0, GG::X(20), UserString("NEXT"));
+    m_next_turn_button = new CUIButton(UserString("NEXT"));
     AttachChild(m_next_turn_button);
-    m_last_turn_button = new CUIButton(GG::X0, GG::Y0, GG::X(20), UserString("LAST"));
+    m_last_turn_button = new CUIButton(UserString("LAST"));
     AttachChild(m_last_turn_button);
-    m_filter_button = new CUIButton(GG::X0, GG::Y0, GG::X(30), UserString("FILTERS"));
+    m_filter_button = new CUIButton(UserString("FILTERS"));
     AttachChild(m_filter_button);
 
     GG::Connect(m_prev_turn_button->LeftClickedSignal,  &SitRepPanel::PrevClicked,          this);
@@ -263,7 +314,7 @@ void SitRepPanel::DoLayout() {
     m_prev_turn_button->SizeMove(button_ul, button_ul + GG::Pt(BUTTON_WIDTH, BUTTON_HEIGHT));
     button_ul -= GG::Pt(BUTTON_WIDTH + GG::X(PAD), GG::Y0);
 
-    m_sitreps_lb->SizeMove(GG::Pt(GG::X0, GG::Y0), GG::Pt(ClientWidth(), button_ul.y));
+    m_sitreps_lb->SizeMove(GG::Pt(GG::X0, GG::Y0), GG::Pt(ClientWidth() - 1, button_ul.y));
 
     m_filter_button->SizeMove(GG::Pt(GG::X0, button_ul.y), GG::Pt(BUTTON_WIDTH*2, button_ul.y + BUTTON_HEIGHT));
 
@@ -276,9 +327,8 @@ void SitRepPanel::KeyPress(GG::Key key, boost::uint32_t key_code_point,
     switch (key) {
     case GG::GGK_RETURN:
     case GG::GGK_KP_ENTER:
-    case GG::GGK_ESCAPE:
-    case GG::GGK_F2: {
-        Hide();
+    case GG::GGK_ESCAPE:{
+        CloseClicked();
         break;
     }
     default:
@@ -321,6 +371,8 @@ void SitRepPanel::FilterClicked() {
     std::map<int, std::string> menu_index_templates;
     std::map<int, bool> menu_index_checked;
     int index = 1;
+    bool all_checked = true;
+    int ALL_INDEX = 9999;
 
     GG::MenuItem menu_contents;
     for (std::vector<std::string>::const_iterator it = all_templates.begin();
@@ -328,33 +380,87 @@ void SitRepPanel::FilterClicked() {
     {
         menu_index_templates[index] = *it;
         bool check = true;
-        if (m_hidden_sitrep_templates.find(*it) != m_hidden_sitrep_templates.end())
+        if (m_hidden_sitrep_templates.find(*it) != m_hidden_sitrep_templates.end()) {
             check = false;
+            all_checked = false;
+        }
         menu_index_checked[index] = check;
         const std::string& menu_label = UserString(*it + "_LABEL");
         menu_contents.next_level.push_back(GG::MenuItem(menu_label, index, false, check));
     }
+    menu_contents.next_level.push_back(GG::MenuItem((all_checked ? UserString("NONE") : UserString("ALL")),
+                                       ALL_INDEX, false, false));
 
-    GG::PopupMenu popup(m_filter_button->UpperLeft().x, m_filter_button->LowerRight().y,
+    GG::PopupMenu popup(m_filter_button->Left(), m_filter_button->Bottom(),
                         ClientUI::GetFont(), menu_contents, ClientUI::TextColor(),
-                        ClientUI::WndOuterBorderColor(), ClientUI::WndColor());
+                        ClientUI::WndOuterBorderColor(), ClientUI::WndColor(),
+                        ClientUI::EditHiliteColor());
     if (!popup.Run())
         return;
     int selected_menu_item = popup.MenuID();
     if (selected_menu_item == 0)
-        return;
-    const std::string& selected_template_string = menu_index_templates[selected_menu_item];
-    if (menu_index_checked[selected_menu_item]) {
-        // disable showing this template string
-        m_hidden_sitrep_templates.insert(selected_template_string);
+        return; // nothing was selected
+
+    if (selected_menu_item == ALL_INDEX) {
+        // select / deselect all templates
+        if (all_checked) {
+            // deselect all
+            for (std::vector<std::string>::const_iterator it = all_templates.begin();
+                 it != all_templates.end(); ++it, ++index)
+            { m_hidden_sitrep_templates.insert(*it); }
+        } else {
+            // select all
+            m_hidden_sitrep_templates.clear();
+        }
     } else {
-        // re-enabled showing this template string
-        m_hidden_sitrep_templates.erase(selected_template_string);
+        // select / deselect the chosen template
+        const std::string& selected_template_string = menu_index_templates[selected_menu_item];
+        if (menu_index_checked[selected_menu_item]) {
+            // disable showing this template string
+            m_hidden_sitrep_templates.insert(selected_template_string);
+        } else {
+            // re-enabled showing this template string
+            m_hidden_sitrep_templates.erase(selected_template_string);
+        }
     }
     Update();
 }
 
+void SitRepPanel::GetTurnSitrepsFromEmpire(std::list<SitRepEntry>& append_list, int empire_id) const {
+    // get id(s) of empires to get sitreps for
+    std::vector<int> empire_ids;
+    if (empire_id != ALL_EMPIRES) {
+        if (Empires().Lookup(empire_id))
+            empire_ids.push_back(empire_id);
+    } else {
+        for (EmpireManager::iterator it = Empires().begin(); it != Empires().end(); ++it)
+            empire_ids.push_back(it->first);
+    }
+
+    for (std::vector<int>::const_iterator it = empire_ids.begin(); it != empire_ids.end(); ++it) {
+        const Empire* empire = Empires().Lookup(*it);
+        if (!empire)
+            continue;
+
+        // loop through sitreps and add to current list
+        for (Empire::SitRepItr sitrep_it = empire->SitRepBegin();
+             sitrep_it != empire->SitRepEnd(); ++sitrep_it)
+        {
+            if (!GetOptionsDB().Get<bool>("verbose-sitrep")) {
+                if (!sitrep_it->Validate())
+                    continue;
+            }
+            if (m_showing_turn != INVALID_GAME_TURN && m_showing_turn != sitrep_it->GetTurn())
+                continue;
+            if (m_hidden_sitrep_templates.find(sitrep_it->GetTemplateString()) != m_hidden_sitrep_templates.end())
+                continue;
+            append_list.push_back(*sitrep_it);
+        }
+    }
+}
+
 void SitRepPanel::Update() {
+    Logger().debugStream() << "SitRepPanel::Update()";
     m_sitreps_lb->Clear();
 
     if (m_showing_turn == INVALID_GAME_TURN)
@@ -362,25 +468,15 @@ void SitRepPanel::Update() {
     else
         this->SetName(boost::io::str(FlexibleFormat(UserString("SITREP_PANEL_TITLE_TURN")) % m_showing_turn));
 
-    Empire *empire = HumanClientApp::GetApp()->Empires().Lookup(HumanClientApp::GetApp()->EmpireID());
-    if (!empire)
-        return;
-
-    GG::X width = m_sitreps_lb->Width() - 8;
 
+    // get sitrep entries for this client's player empire, or for all empires
+    // if this client is an observer or moderator.
+    // todo: double check that no-empire players are actually moderator or
+    //       observers, instead of just passing the client empire id.
     std::list<SitRepEntry> currentTurnSitreps;
-    // loop through sitreps and add to current list
-    for (Empire::SitRepItr sitrep_it = empire->SitRepBegin(); sitrep_it != empire->SitRepEnd(); ++sitrep_it) {
-        if (!GetOptionsDB().Get<bool>("verbose-sitrep")) {
-            if (!sitrep_it->Validate())
-                continue;
-        }
-        if (m_showing_turn != INVALID_GAME_TURN && m_showing_turn != sitrep_it->GetTurn())
-            continue;
-        if (m_hidden_sitrep_templates.find(sitrep_it->GetTemplateString()) != m_hidden_sitrep_templates.end())
-            continue;
-        currentTurnSitreps.push_back(*sitrep_it);
-    }
+    GetTurnSitrepsFromEmpire(currentTurnSitreps, HumanClientApp::GetApp()->EmpireID());
+
+
     // order sitreps for display
     std::vector<SitRepEntry> orderedSitreps;
     std::vector<std::string> ordered_templates = OrderedSitrepTemplateStrings();
@@ -398,20 +494,26 @@ void SitRepPanel::Update() {
             }
         }
     }
-    //Logger().debugStream()<< "copying over any remaining currentTurnSitreps, total of " << currentTurnSitreps.size();
+
+    // copy remaining unordered sitreps
     for (std::list<SitRepEntry>::iterator sitrep_it = currentTurnSitreps.begin();
          sitrep_it != currentTurnSitreps.end(); sitrep_it++)
     { orderedSitreps.push_back(*sitrep_it); }
-    for (std::vector<SitRepEntry>::iterator sitrep_it = orderedSitreps.begin(); 
+
+    // create UI rows for all sitrps
+    GG::X width = m_sitreps_lb->Width() - 8;
+    for (std::vector<SitRepEntry>::iterator sitrep_it = orderedSitreps.begin();
          sitrep_it != orderedSitreps.end(); sitrep_it++)
     { m_sitreps_lb->Insert(new SitRepRow(width, GG::Y(ClientUI::Pts()*2), *sitrep_it)); }
 
+    // if at first turn, disable back button
     if (CurrentTurn() >= 1 && m_showing_turn > 1) {
         m_prev_turn_button->Disable(false);
     } else {
         m_prev_turn_button->Disable();
     }
 
+    // if at current turn, disable forward button
     if (CurrentTurn() >= 1 && m_showing_turn < CurrentTurn()) {
         m_next_turn_button->Disable(false);
         m_last_turn_button->Disable(false);
@@ -434,17 +536,31 @@ void SitRepPanel::SetHiddenSitRepTemplates(const std::set<std::string>& template
 int SitRepPanel::NumVisibleSitrepsThisTurn() const {
     int count = 0;
 
-    Empire *empire = HumanClientApp::GetApp()->Empires().Lookup(HumanClientApp::GetApp()->EmpireID());
-    if (!empire)
-        return count;
+    int client_empire_id = HumanClientApp::GetApp()->EmpireID();
 
-    for (Empire::SitRepItr sitrep_it = empire->SitRepBegin(); sitrep_it != empire->SitRepEnd(); ++sitrep_it) {
-        if (CurrentTurn() != sitrep_it->GetTurn())
-            continue;
-        if (m_hidden_sitrep_templates.find(sitrep_it->GetTemplateString()) != m_hidden_sitrep_templates.end())
-            continue;
-        ++count;
+    // which empires to count sitreps for?
+    std::vector<int> empire_ids;
+    if (HumanClientApp::GetApp()->Empires().Lookup(client_empire_id)) {
+        empire_ids.push_back(client_empire_id);
+    } else {
+        for (EmpireManager::const_iterator it = Empires().begin(); it != Empires().end(); ++it)
+            empire_ids.push_back(it->first);
     }
 
+    for (std::vector<int>::const_iterator empire_it = empire_ids.begin();
+         empire_it != empire_ids.end(); ++empire_it)
+    {
+        const Empire* empire = HumanClientApp::GetApp()->Empires().Lookup(*empire_it);
+        if (!empire)
+            continue;
+
+        for (Empire::SitRepItr sitrep_it = empire->SitRepBegin(); sitrep_it != empire->SitRepEnd(); ++sitrep_it) {
+            if (CurrentTurn() != sitrep_it->GetTurn())
+                continue;
+            if (m_hidden_sitrep_templates.find(sitrep_it->GetTemplateString()) != m_hidden_sitrep_templates.end())
+                continue;
+            ++count;
+        }
+    }
     return count;
 }
