@@ -20,6 +20,8 @@
 #include <GG/DrawUtil.h>
 #include <GG/StaticGraphic.h>
 #include <GG/TextControl.h>
+#include <GG/utf8/core.h>
+#include <GG/utf8/checked.h>
 
 #include <boost/format.hpp>
 #include <boost/lexical_cast.hpp>
@@ -38,6 +40,45 @@ namespace {
             ClientUI::GetClientUI()->GetPrefixedTextures(ClientUI::ArtDir() / "misc" / "system_selection_tiny", "system_selection_tiny", false);
         return tiny_textures;
     }
+    
+    // Wrap content int an rgba tag with color color. Opacity ignored.
+    std::string ColorTag(const std::string& content, GG::Clr color){
+        boost::format templ("<rgba %d %d %d 255>%s</rgba>");
+        return (templ % static_cast<int>(color.r) % static_cast<int>(color.g) % static_cast<int>(color.b) % content).str();
+    }
+    
+    /// Adds color tags to name_o according to the empires in owner_empire_ids
+    std::string ColorNameByOwners(const std::string& name_o, std::set<int>& owner_empire_ids, const EmpireManager& empires) {
+        if (owner_empire_ids.size() < 1) {
+            return name_o;
+        } else if(owner_empire_ids.size() == 1) {
+            return ColorTag(name_o, empires.Lookup(*owner_empire_ids.begin())->Color());
+        } else {
+            // We will split the name into pieces.
+            // To avoid splitting multi-byte glyphs, we need to convert it to 32-bit form,
+            // where a single element always corresponds to a single glyph
+            std::vector<utf8::uint32_t> name;
+            utf8::utf8to32(name_o.begin(), name_o.end(), std::back_inserter(name));
+            const unsigned owner_count = owner_empire_ids.size();
+            const unsigned piece_length = name.size() / owner_count;
+            unsigned extra = name.size() - owner_count*piece_length; // letters that would be left over
+            std::string retval;
+            int start = 0;
+            for (std::set<int>::iterator it = owner_empire_ids.begin(); it != owner_empire_ids.end(); ++it) {
+                int current_length = piece_length;
+                if (extra > 0) { // Use left over letters as long as we have them
+                    ++current_length;
+                    --extra;
+                }
+                // Now we convert a piece of the name back into utf8 and wrap it in tags.
+                std::string  piece;
+                utf8::utf32to8(name.begin() + start, name.begin() + start + current_length, std::back_inserter(piece));
+                retval += ColorTag(piece, empires.Lookup(*it)->Color());
+                start += current_length;
+            }
+            return retval;
+        }
+    }
 
     const double        PI = 3.1415926535;
     const unsigned int  MAX_TRIES = 128;     // most allowed unique fleetbutton locations
@@ -47,7 +88,7 @@ namespace {
 // OwnerColoredSystemName
 ////////////////////////////////////////////////
 OwnerColoredSystemName::OwnerColoredSystemName(int system_id, int font_size, bool blank_unexplored_and_none) :
-    Control(GG::X0, GG::Y0, GG::X1, GG::Y1, GG::Flags<GG::WndFlag>())
+    Control(GG::X0, GG::Y0, GG::X1, GG::Y1, GG::NO_WND_FLAGS)
 {
     // TODO: Have this make a single call per color.
     // Set up texture coord and vertex buffers (quads) for the glyphs.
@@ -55,7 +96,7 @@ OwnerColoredSystemName::OwnerColoredSystemName(int system_id, int font_size, boo
 
     int client_empire_id = HumanClientApp::GetApp()->EmpireID();
 
-    const System* system = GetSystem(system_id);
+    TemporaryPtr<const System> system = GetSystem(system_id);
     if (!system)
         return;
 
@@ -63,7 +104,6 @@ OwnerColoredSystemName::OwnerColoredSystemName(int system_id, int font_size, boo
     if (known_destroyed_object_ids.find(system_id) != known_destroyed_object_ids.end())
         return;
 
-    const ObjectMap& objects = Objects();
     const SpeciesManager& species_manager = GetSpeciesManager();
     const EmpireManager& empire_manager = Empires();
 
@@ -74,37 +114,40 @@ OwnerColoredSystemName::OwnerColoredSystemName(int system_id, int font_size, boo
     // or have a shipyard, or have neutral population
     bool capital = false, homeworld = false, has_shipyard = false, has_neutrals = false, has_player_planet = false;
 
-    std::vector<const Planet*> system_planets;
-    std::vector<const Planet*> planets = objects.FindObjects<Planet>();
-    for (std::vector<const Planet*>::const_iterator it = planets.begin(); it != planets.end(); ++it) {
-        const Planet* planet = *it;
-        if (planet->SystemID() != system_id)
-            continue;
-        if (known_destroyed_object_ids.find(planet->ID()) != known_destroyed_object_ids.end())
-            continue;
-        system_planets.push_back(*it);
-    }
-
     std::set<int> owner_empire_ids;
-    for (std::vector<const Planet*>::const_iterator it = system_planets.begin(); it != system_planets.end(); ++it) {
-        const Planet* planet = *it;
+    std::vector<TemporaryPtr<const Planet> > system_planets = Objects().FindObjects<const Planet>(system->PlanetIDs());
+
+    for (std::vector<TemporaryPtr<const Planet> >::const_iterator planet_it = system_planets.begin();
+         planet_it != system_planets.end(); ++planet_it)
+    {
+        TemporaryPtr<const Planet> planet = *planet_it;
         int planet_id = planet->ID();
 
+        if (known_destroyed_object_ids.find(planet_id) != known_destroyed_object_ids.end())
+            continue;
+
         // is planet a capital?
         if (!capital) {
-            for (EmpireManager::const_iterator empire_it = empire_manager.begin(); empire_it != empire_manager.end(); ++empire_it) {
-                if (empire_it->second->CapitalID() == planet_id)
+            for (EmpireManager::const_iterator empire_it = empire_manager.begin();
+                 empire_it != empire_manager.end(); ++empire_it)
+            {
+                if (empire_it->second->CapitalID() == planet_id) {
                     capital = true;
+                    break;
+                }
             }
         }
 
         // is planet a homeworld? (for any species)
         if (!homeworld) {
-            for (SpeciesManager::iterator species_it = species_manager.begin(); species_it != species_manager.end(); ++species_it) {
+            for (SpeciesManager::iterator species_it = species_manager.begin();
+                 species_it != species_manager.end(); ++species_it)
+            {
                 if (const Species* species = species_it->second) {
                     const std::set<int>& homeworld_ids = species->Homeworlds();
                     if (homeworld_ids.find(planet_id) != homeworld_ids.end()) {
                         homeworld = true;
+                        break;
                     }
                 }
             }
@@ -112,30 +155,34 @@ OwnerColoredSystemName::OwnerColoredSystemName(int system_id, int font_size, boo
 
         // does planet contain a shipyard?
         if (!has_shipyard) {
-            const std::set<int>& buildings = planet->Buildings();
-            for (std::set<int>::const_iterator building_it = buildings.begin(); building_it != buildings.end(); ++building_it) {
-                if (known_destroyed_object_ids.find(*building_it) != known_destroyed_object_ids.end())
-                    continue;
-                const Building* building = objects.Object<Building>(*building_it);
-                if (!building)
+            std::vector<TemporaryPtr<const Building> > buildings = Objects().FindObjects<const Building>(planet->BuildingIDs());
+            for (std::vector<TemporaryPtr<const Building> >::const_iterator building_it = buildings.begin();
+                 building_it != buildings.end(); ++building_it)
+            {
+                TemporaryPtr<const Building> building = *building_it;
+                int building_id = building->ID();
+
+                if (known_destroyed_object_ids.find(building_id) != known_destroyed_object_ids.end())
                     continue;
+
                 // annoying hard-coded building name here... not sure how better to deal with it
                 if (building->BuildingTypeName() == "BLD_SHIPYARD_BASE") {
                     has_shipyard = true;
+                    break;
                 }
             }
         }
 
+        // is planet populated by neutral species
         if (!has_neutrals) {
             if (planet->Unowned() && !planet->SpeciesName().empty() && planet->CurrentMeterValue(METER_POPULATION) > 0.0)
                 has_neutrals = true;
         }
 
-        if (!has_player_planet) {
-            if (!planet->Unowned()) {
-                has_player_planet = true;
-                owner_empire_ids.insert(planet->Owner());
-            }
+        // remember if this system has a player-owned planet
+        if (!planet->Unowned()) {
+            has_player_planet = true;
+            owner_empire_ids.insert(planet->Owner());
         }
     }
 
@@ -147,7 +194,7 @@ OwnerColoredSystemName::OwnerColoredSystemName(int system_id, int font_size, boo
     // need to check all empires for homeworld or capitals
 
     // wrap with formatting tags
-    std::string wrapped_system_name = system_name;
+    std::string wrapped_system_name = ColorNameByOwners(system_name, owner_empire_ids, empire_manager);
     if (homeworld)
         wrapped_system_name = "<i>" + wrapped_system_name + "</i>";
     if (has_shipyard)
@@ -163,10 +210,10 @@ OwnerColoredSystemName::OwnerColoredSystemName(int system_id, int font_size, boo
         if (owner_empire_ids.size() == 1)
             text_color = Empires().Lookup(*owner_empire_ids.begin())->Color();
     } else if (has_neutrals) {
-            text_color = ClientUI::TextColor();
+        text_color = ClientUI::TextColor();
     }
 
-    GG::TextControl* text = new ShadowedTextControl(GG::X0, GG::Y0, wrapped_system_name, font, text_color);
+    GG::Control* text = new ShadowedTextControl(GG::X0, GG::Y0, wrapped_system_name, font, text_color);
     AttachChild(text);
     Resize(GG::Pt(text->Width(), text->Height()));
 }
@@ -190,7 +237,7 @@ SystemIcon::SystemIcon(GG::X x, GG::Y y, GG::X w, int system_id) :
     m_showing_name(false)
 {
     ClientUI* ui = ClientUI::GetClientUI();
-    if (const System* system = GetSystem(m_system_id)) {
+    if (TemporaryPtr<const System> system = GetSystem(m_system_id)) {
         StarType star_type = system->GetStarType();
         m_disc_texture = ui->GetModuloTexture(ClientUI::ArtDir() / "stars",
                                               ClientUI::StarTypeFilePrefixes()[star_type],
@@ -499,7 +546,7 @@ void SystemIcon::LClick(const GG::Pt& pt, GG::Flags<GG::ModKey> mod_keys)
 { if (!Disabled()) LeftClickedSignal(m_system_id); }
 
 void SystemIcon::RClick(const GG::Pt& pt, GG::Flags<GG::ModKey> mod_keys)
-{ if (!Disabled()) RightClickedSignal(m_system_id); }
+{ if (!Disabled()) RightClickedSignal(m_system_id, mod_keys); }
 
 void SystemIcon::LDoubleClick(const GG::Pt& pt, GG::Flags<GG::ModKey> mod_keys)
 { if (!Disabled()) LeftDoubleClickedSignal(m_system_id); }
@@ -530,7 +577,7 @@ void SystemIcon::MouseEnter(const GG::Pt& pt, GG::Flags<GG::ModKey> mod_keys) {
 
     PlaySystemIconRolloverSound();
 
-    MouseEnteringSignal(m_system_id);
+    MouseEnteringSignal(m_system_id, mod_keys);
 }
 
 void SystemIcon::MouseLeave() {
@@ -559,10 +606,10 @@ void SystemIcon::Refresh() {
     std::string name;
     m_system_connection.disconnect();
 
-    const System* system = GetSystem(m_system_id);
+    TemporaryPtr<const System> system = GetSystem(m_system_id);
     if (system) {
         name = system->Name();
-        m_system_connection = GG::Connect(system->StateChangedSignal,   &SystemIcon::Refresh,   this,   boost::signals::at_front);
+        m_system_connection = GG::Connect(system->StateChangedSignal,   &SystemIcon::Refresh,   this,   boost::signals2::at_front);
     }
 
     SetName(name);   // sets GG::Control name.  doesn't affect displayed system name
