@@ -40,13 +40,13 @@ namespace {
     GG::Y   TechPanelHeight()
     { return GG::Y(ClientUI::Pts()*6); }
 
-
-
     const double ZOOM_STEP_SIZE = 1.12;
     const double MIN_SCALE = std::pow(ZOOM_STEP_SIZE, -25.0);
     const double MAX_SCALE = std::pow(ZOOM_STEP_SIZE, 10.0);
     const double INITIAL_SCALE = std::pow(ZOOM_STEP_SIZE, -5.0);
 
+    const double PI = 3.1415926535897932384626433;
+
     struct ToggleCategoryFunctor {
         ToggleCategoryFunctor(TechTreeWnd* tree_wnd, const std::string& category) : m_tree_wnd(tree_wnd), m_category(category) {}
         void operator()() {m_tree_wnd->ToggleCategory(m_category);}
@@ -68,6 +68,102 @@ namespace {
     };
 }
 
+///////////////////////////
+// TechPanelRowBrowseWnd //
+///////////////////////////
+boost::shared_ptr<GG::BrowseInfoWnd> TechPanelRowBrowseWnd(const std::string& tech_name, int empire_id) {
+    const Empire* empire = Empires().Lookup(empire_id);
+    const Tech* tech = GetTech(tech_name);
+    if (!tech) {
+        boost::shared_ptr<GG::BrowseInfoWnd> browse_wnd;
+        return browse_wnd;
+    }
+
+    std::string main_text;
+
+    main_text += UserString(tech->Category()) + " ";
+    main_text += UserString(boost::lexical_cast<std::string>(tech->Type())) + "  :  ";
+    main_text += UserString(tech->ShortDescription()) + "\n";
+
+    if (empire) {
+        TechStatus tech_status = empire->GetTechStatus(tech_name);
+        if (!tech->Researchable()) {
+            main_text += UserString("TECH_WND_UNRESEARCHABLE") + "\n";
+
+        } else if (tech_status == TS_UNRESEARCHABLE) {
+            main_text += UserString("TECH_WND_STATUS_LOCKED") + "\n";
+
+            const std::set<std::string>& prereqs = tech->Prerequisites();
+            std::vector<std::string> unresearched_prereqs;
+            for (std::set<std::string>::const_iterator it = prereqs.begin(); it != prereqs.end(); ++it) {
+                TechStatus prereq_status = empire->GetTechStatus(*it);
+                if (prereq_status != TS_COMPLETE)
+                    unresearched_prereqs.push_back(*it);
+            }
+            if (!unresearched_prereqs.empty()) {
+                main_text += UserString("TECH_WND_UNRESEARCHED_PREREQUISITES");
+                for (std::vector<std::string>::const_iterator it = unresearched_prereqs.begin();
+                        it != unresearched_prereqs.end(); ++it)
+                { main_text += UserString(*it) + "  "; }
+                main_text += "\n";
+            }
+
+        } else if (tech_status == TS_RESEARCHABLE) {
+            main_text += UserString("TECH_WND_STATUS_RESEARCHABLE") + "\n";
+
+        } else if (tech_status == TS_COMPLETE) {
+            main_text += UserString("TECH_WND_STATUS_COMPLETED") + "\n";
+        }
+
+        const ResearchQueue& queue = empire->GetResearchQueue();
+        ResearchQueue::const_iterator queue_it = queue.find(tech_name);
+        if (queue_it != queue.end()) {
+            main_text += UserString("TECH_WND_ENQUEUED") + "\n";
+
+            float progress = empire->ResearchProgress(tech_name);
+            float total_cost = tech->ResearchCost(empire_id);
+            float allocation = queue_it->allocated_rp;
+            float max_allocation = tech->PerTurnCost(empire_id);
+
+            // %1% / %2%  +  %3% / %4% RP/turn
+            main_text += boost::io::str(FlexibleFormat(UserString("TECH_WND_PROGRESS"))
+                    % DoubleToString(progress, 3, false)
+                    % DoubleToString(total_cost, 3, false)
+                    % DoubleToString(allocation, 3, false)
+                    % DoubleToString(max_allocation, 3, false)) + "\n";
+
+            int ETA = queue_it->turns_left;
+            if (ETA != -1)
+                main_text += boost::io::str(FlexibleFormat(UserString("TECH_WND_ETA"))
+                    % ETA);
+
+        } else if (tech->Researchable()) {
+            int turns = tech->ResearchTime(empire_id);
+            float cost = tech->ResearchCost(empire_id);
+            const std::string& cost_units = UserString("ENC_RP");
+
+            main_text += boost::io::str(FlexibleFormat(UserString("ENC_COST_AND_TURNS_STR"))
+                % DoubleToString(cost, 3, false)
+                % cost_units
+                % turns);
+        }
+
+    } else if (tech->Researchable()) {
+        int turns = tech->ResearchTime(empire_id);
+        float cost = tech->ResearchCost(empire_id);
+        const std::string& cost_units = UserString("ENC_RP");
+
+        main_text += boost::io::str(FlexibleFormat(UserString("ENC_COST_AND_TURNS_STR"))
+            % DoubleToString(cost, 3, false)
+            % cost_units
+            % turns);
+    }
+
+    boost::shared_ptr<GG::BrowseInfoWnd> browse_wnd(new IconTextBrowseWnd(
+        ClientUI::TechIcon(tech_name), UserString(tech_name), main_text));
+    return browse_wnd;
+}
+
 
 //////////////////////////////////////////////////
 // TechTreeWnd::TechTreeControls                //
@@ -123,31 +219,31 @@ TechTreeWnd::TechTreeControls::TechTreeControls(GG::X x, GG::Y y, GG::X w) :
     // create a button for each tech category...
     const std::vector<std::string>& cats = GetTechManager().CategoryNames();
     for (unsigned int i = 0; i < cats.size(); ++i) {
-        m_category_buttons.push_back(new CUIButton(GG::X0, GG::Y0, GG::X(20), UserString(cats[i])));
+        m_category_buttons.push_back(new CUIButton(UserString(cats[i])));
         AttachChild(m_category_buttons.back());
         m_category_buttons.back()->MarkNotSelected();
     }
     // and one for "ALL"
-    m_category_buttons.push_back(new CUIButton(GG::X0, GG::Y0, GG::X(20), UserString("ALL")));
+    m_category_buttons.push_back(new CUIButton(UserString("ALL")));
     AttachChild(m_category_buttons.back());
     m_category_buttons.back()->MarkNotSelected();
 
     // create a button for each tech status
-    m_tech_status_buttons[TS_UNRESEARCHABLE] = new CUIButton(GG::X0, GG::Y0, GG::X(20), UserString("TECH_WND_STATUS_LOCKED"));
+    m_tech_status_buttons[TS_UNRESEARCHABLE] = new CUIButton(UserString("TECH_WND_STATUS_LOCKED"));
     AttachChild(m_tech_status_buttons[TS_UNRESEARCHABLE]);
-    m_tech_status_buttons[TS_RESEARCHABLE] = new CUIButton(GG::X0, GG::Y0, GG::X(20), UserString("TECH_WND_STATUS_RESEARCHABLE"));
+    m_tech_status_buttons[TS_RESEARCHABLE] = new CUIButton(UserString("TECH_WND_STATUS_RESEARCHABLE"));
     AttachChild(m_tech_status_buttons[TS_RESEARCHABLE]);
-    m_tech_status_buttons[TS_COMPLETE] = new CUIButton(GG::X0, GG::Y0, GG::X(20), UserString("TECH_WND_STATUS_COMPLETED"));
+    m_tech_status_buttons[TS_COMPLETE] = new CUIButton(UserString("TECH_WND_STATUS_COMPLETED"));
     AttachChild(m_tech_status_buttons[TS_COMPLETE]);
     // colour
     for (std::map<TechStatus, CUIButton*>::iterator it = m_tech_status_buttons.begin(); it != m_tech_status_buttons.end(); ++it)
         it->second->MarkNotSelected();
 
     // create buttons to switch between tree and list views
-    m_list_view_button = new CUIButton(GG::X0, GG::Y0, GG::X(80), UserString("TECH_WND_LIST_VIEW"));
+    m_list_view_button = new CUIButton(UserString("TECH_WND_LIST_VIEW"));
     m_list_view_button->MarkNotSelected();
     AttachChild(m_list_view_button);
-    m_tree_view_button = new CUIButton(GG::X0, GG::Y(30), GG::X(80), UserString("TECH_WND_TREE_VIEW"));
+    m_tree_view_button = new CUIButton(UserString("TECH_WND_TREE_VIEW"));
     m_tree_view_button->MarkNotSelected();
     AttachChild(m_tree_view_button);
 
@@ -254,8 +350,6 @@ void TechTreeWnd::TechTreeControls::Render() {
 
     // use GL to draw the lines
     glDisable(GL_TEXTURE_2D);
-    GLint initial_modes[2];
-    glGetIntegerv(GL_POLYGON_MODE, initial_modes);
 
     glBegin(GL_LINES);
         glColor(ClientUI::WndOuterBorderColor());
@@ -279,9 +373,6 @@ void TechTreeWnd::TechTreeControls::Render() {
         }
     glEnd();
 
-    // reset this to whatever it was initially
-    glPolygonMode(GL_BACK, initial_modes[1]);
-
     glEnable(GL_TEXTURE_2D);
 }
 
@@ -289,7 +380,7 @@ void TechTreeWnd::TechTreeControls::LDrag(const GG::Pt& pt, const GG::Pt& move,
     if (m_drag_offset != GG::Pt(-GG::X1, -GG::Y1)) {  // resize-dragging
         GG::Pt new_lr = pt - m_drag_offset;
 
-        new_lr.y = LowerRight().y;    // ignore y-resizes
+        new_lr.y = Bottom();    // ignore y-resizes
 
         // constrain to within parent
         if (GG::Wnd* parent = Parent()) {
@@ -338,7 +429,8 @@ public:
     std::set<TechStatus>    GetTechStatusesShown() const;
 
     mutable TechTreeWnd::TechSignalType         TechBrowsedSignal;
-    mutable TechTreeWnd::TechClickSignalType    TechClickedSignal;
+    mutable TechTreeWnd::TechClickSignalType    TechLeftClickedSignal;
+    mutable TechTreeWnd::TechClickSignalType    TechRightClickedSignal;
     mutable TechTreeWnd::TechClickSignalType    TechDoubleClickedSignal;
     //@}
 
@@ -386,10 +478,10 @@ private:
         { ButtonUpSignal(pt); }
         virtual void MouseWheel(const GG::Pt& pt, int move, GG::Flags<GG::ModKey> mod_keys)
         { ZoomedSignal(move); }
-        mutable boost::signal<void (int)>           ZoomedSignal;
-        mutable boost::signal<void (const GG::Pt&)> DraggedSignal;
-        mutable boost::signal<void (const GG::Pt&)> ButtonDownSignal;
-        mutable boost::signal<void (const GG::Pt&)> ButtonUpSignal;
+        mutable boost::signals2::signal<void (int)>           ZoomedSignal;
+        mutable boost::signals2::signal<void (const GG::Pt&)> DraggedSignal;
+        mutable boost::signals2::signal<void (const GG::Pt&)> ButtonDownSignal;
+        mutable boost::signals2::signal<void (const GG::Pt&)> ButtonUpSignal;
     };
 
     void Layout(bool keep_position);    // lays out tech panels
@@ -401,8 +493,10 @@ private:
     void ScrolledSlot(int, int, int, int);
 
     void TechBrowsedSlot(const std::string& tech_name);
-    void TechClickedSlot(const std::string& tech_name,
-                         const GG::Flags<GG::ModKey>& modkeys);
+    void TechLeftClickedSlot(const std::string& tech_name,
+                             const GG::Flags<GG::ModKey>& modkeys);
+    void TechRightClickedSlot(const std::string& tech_name,
+                              const GG::Flags<GG::ModKey>& modkeys);
     void TechDoubleClickedSlot(const std::string& tech_name,
                                const GG::Flags<GG::ModKey>& modkeys);
 
@@ -442,6 +536,7 @@ private:
 class TechTreeWnd::LayoutPanel::TechPanel : public GG::Wnd {
 public:
     TechPanel(const std::string& tech_name, const TechTreeWnd::LayoutPanel* panel);
+    virtual         ~TechPanel();
 
     virtual bool    InWindow(const GG::Pt& pt) const;
     virtual void    Render();
@@ -452,6 +547,7 @@ public:
     virtual void    LButtonUp(const GG::Pt& pt, GG::Flags<GG::ModKey> mod_keys)
     { ForwardEventToParent(); }
     virtual void    LClick(const GG::Pt& pt, GG::Flags<GG::ModKey> mod_keys);
+    virtual void    RClick(const GG::Pt& pt, GG::Flags<GG::ModKey> mod_keys);
     virtual void    LDoubleClick(const GG::Pt& pt, GG::Flags<GG::ModKey> mod_keys);
     virtual void    MouseEnter(const GG::Pt& pt, GG::Flags<GG::ModKey> mod_keys);
     virtual void    MouseLeave();
@@ -462,7 +558,8 @@ public:
     int             FontSize() const;
 
     mutable TechTreeWnd::TechSignalType         TechBrowsedSignal;
-    mutable TechTreeWnd::TechClickSignalType    TechClickedSignal;
+    mutable TechTreeWnd::TechClickSignalType    TechLeftClickedSignal;
+    mutable TechTreeWnd::TechClickSignalType    TechRightClickedSignal;
     mutable TechTreeWnd::TechClickSignalType    TechDoubleClickedSignal;
 
 private:
@@ -504,6 +601,9 @@ TechTreeWnd::LayoutPanel::TechPanel::TechPanel(const std::string& tech_name, con
     Update();
 }
 
+TechTreeWnd::LayoutPanel::TechPanel::~TechPanel()
+{ delete m_icon; }
+
 int TechTreeWnd::LayoutPanel::TechPanel::FontSize() const
 { return ClientUI::Pts() * 3 / 2; }
 
@@ -597,9 +697,9 @@ void TechTreeWnd::LayoutPanel::TechPanel::Render() {
         GG::Pt eta_lr = eta_ul + GG::Pt(panel_size.x/2, panel_size.y/2) + GG::Pt(GG::X(2), GG::Y(2));
 
         glColor(GG::CLR_BLACK);
-        CircleArc(eta_ul, eta_lr, 0, 6.28, true);
+        CircleArc(eta_ul, eta_lr, 0, 2*PI, true);
         glColor(border_colour);
-        CircleArc(eta_ul, eta_lr, 0, 6.28, true);
+        CircleArc(eta_ul, eta_lr, 0, 2*PI, true);
 
         boost::shared_ptr<GG::Font> font = ClientUI::GetFont(FontSize());
 
@@ -646,7 +746,35 @@ void TechTreeWnd::LayoutPanel::TechPanel::Render() {
 
 void TechTreeWnd::LayoutPanel::TechPanel::LClick(const GG::Pt& pt, GG::Flags<GG::ModKey> mod_keys) {
     if (m_layout_panel->m_selected_tech_name != m_tech_name)
-        TechClickedSignal(m_tech_name, mod_keys);
+        TechLeftClickedSignal(m_tech_name, mod_keys);
+}
+
+void TechTreeWnd::LayoutPanel::TechPanel::RClick(const GG::Pt& pt,
+                                                 GG::Flags<GG::ModKey> mod_keys)
+{
+    TechRightClickedSignal(m_tech_name, mod_keys);
+
+    if (m_enqueued)
+        return;
+    if (m_status == TS_COMPLETE)
+        return;
+
+    GG::MenuItem menu_contents;
+    menu_contents.next_level.push_back(GG::MenuItem(UserString("PRODUCTION_DETAIL_ADD_TO_QUEUE"),   1, false, false));
+
+    GG::PopupMenu popup(pt.x, pt.y, ClientUI::GetFont(), menu_contents, ClientUI::TextColor(),
+                        ClientUI::WndOuterBorderColor(), ClientUI::WndColor(), ClientUI::EditHiliteColor());
+
+    if (popup.Run()) {
+        switch (popup.MenuID()) {
+        case 1: {
+            LDoubleClick(pt, mod_keys);
+            break;
+        }
+        default:
+            break;
+        }
+    }
 }
 
 void TechTreeWnd::LayoutPanel::TechPanel::LDoubleClick(const GG::Pt& pt, GG::Flags<GG::ModKey> mod_keys)
@@ -665,103 +793,6 @@ void TechTreeWnd::LayoutPanel::TechPanel::MouseLeave() {
 void TechTreeWnd::LayoutPanel::TechPanel::Select(bool select)
 { m_selected = select; }
 
-namespace {
-    boost::shared_ptr<GG::BrowseInfoWnd> TechPanelRowBrowseWnd(const std::string& tech_name,
-                                                               int empire_id)
-    {
-        const Empire* empire = Empires().Lookup(empire_id);
-        const Tech* tech = GetTech(tech_name);
-        if (!tech) {
-            boost::shared_ptr<GG::BrowseInfoWnd> browse_wnd;
-            return browse_wnd;
-        }
-
-        std::string main_text;
-
-        main_text += UserString(tech->Category()) + " ";
-        main_text += UserString(boost::lexical_cast<std::string>(tech->Type())) + "  :  ";
-        main_text += UserString(tech->ShortDescription()) + "\n";
-
-        if (empire) {
-            TechStatus tech_status = empire->GetTechStatus(tech_name);
-            if (!tech->Researchable()) {
-                main_text += UserString("TECH_WND_UNRESEARCHABLE") + "\n";
-
-            } else if (tech_status == TS_UNRESEARCHABLE) {
-                main_text += UserString("TECH_WND_STATUS_LOCKED") + "\n";
-
-                const std::set<std::string>& prereqs = tech->Prerequisites();
-                std::vector<std::string> unresearched_prereqs;
-                for (std::set<std::string>::const_iterator it = prereqs.begin(); it != prereqs.end(); ++it) {
-                    TechStatus prereq_status = empire->GetTechStatus(*it);
-                    if (prereq_status != TS_COMPLETE)
-                        unresearched_prereqs.push_back(*it);
-                }
-                if (!unresearched_prereqs.empty()) {
-                    main_text += UserString("TECH_WND_UNRESEARCHED_PREREQUISITES");
-                    for (std::vector<std::string>::const_iterator it = unresearched_prereqs.begin();
-                            it != unresearched_prereqs.end(); ++it)
-                    { main_text += UserString(*it) + "  "; }
-                    main_text += "\n";
-                }
-
-            } else if (tech_status == TS_RESEARCHABLE) {
-                main_text += UserString("TECH_WND_STATUS_RESEARCHABLE") + "\n";
-
-            } else if (tech_status == TS_COMPLETE) {
-                main_text += UserString("TECH_WND_STATUS_COMPLETED") + "\n";
-            }
-
-            const ResearchQueue& queue = empire->GetResearchQueue();
-            ResearchQueue::const_iterator queue_it = queue.find(tech_name);
-            if (queue_it != queue.end()) {
-                main_text += UserString("TECH_WND_ENQUEUED") + "\n";
-
-                double progress = empire->ResearchProgress(tech_name);
-                double total_cost = tech->ResearchCost(empire_id);
-                double allocation = queue_it->allocated_rp;
-                double max_allocation = tech->PerTurnCost(empire_id);
-
-                // %1% / %2%  +  %3% / %4% [[ENC_RP]] / turn
-                main_text += boost::io::str(FlexibleFormat(UserString("TECH_WND_PROGRESS"))
-                        % DoubleToString(progress, 3, false)
-                        % DoubleToString(total_cost, 3, false)
-                        % DoubleToString(allocation, 3, false)
-                        % DoubleToString(max_allocation, 3, false)) + "\n";
-
-                int ETA = queue_it->turns_left;
-                if (ETA != -1)
-                    main_text += boost::io::str(FlexibleFormat(UserString("TECH_WND_ETA"))
-                        % ETA);
-
-            } else if (tech->Researchable()) {
-                int turns = tech->ResearchTime(empire_id);
-                double cost = tech->ResearchCost(empire_id);
-                const std::string& cost_units = UserString("ENC_RP");
-
-                main_text += boost::io::str(FlexibleFormat(UserString("ENC_COST_AND_TURNS_STR"))
-                    % DoubleToString(cost, 3, false)
-                    % cost_units
-                    % turns);
-            }
-
-        } else if (tech->Researchable()) {
-            int turns = tech->ResearchTime(empire_id);
-            double cost = tech->ResearchCost(empire_id);
-            const std::string& cost_units = UserString("ENC_RP");
-
-            main_text += boost::io::str(FlexibleFormat(UserString("ENC_COST_AND_TURNS_STR"))
-                % DoubleToString(cost, 3, false)
-                % cost_units
-                % turns);
-        }
-
-        boost::shared_ptr<GG::BrowseInfoWnd> browse_wnd(new IconTextBrowseWnd(
-            ClientUI::TechIcon(tech_name), UserString(tech_name), main_text));
-        return browse_wnd;
-    }
-}
-
 void TechTreeWnd::LayoutPanel::TechPanel::Update() {
     Select(m_layout_panel->m_selected_tech_name == m_tech_name);
 
@@ -830,17 +861,12 @@ TechTreeWnd::LayoutPanel::LayoutPanel(GG::X w, GG::Y h) :
     m_layout_surface = new LayoutSurface();
 
     m_vscroll = new CUIScroll(GG::X1, GG::Y0, GG::X1, GG::Y1, GG::VERTICAL);
-    m_hscroll = new CUIScroll(GG::X1, GG::Y0, GG::X1, GG::Y1, GG::VERTICAL);
+    m_hscroll = new CUIScroll(GG::X1, GG::Y0, GG::X1, GG::Y1, GG::HORIZONTAL);
 
-    boost::shared_ptr<GG::Font> font = ClientUI::GetFont();
     const GG::X ZBSIZE(ClientUI::ScrollWidth() * 2);
 
-    m_zoom_in_button = new CUIButton(GG::X1, GG::Y1, ZBSIZE, "+", font, ClientUI::WndColor(),
-                                     ClientUI::CtrlBorderColor(), 1, ClientUI::TextColor(),
-                                     GG::INTERACTIVE | GG::ONTOP);
-    m_zoom_out_button = new CUIButton(GG::X1, GG::Y1, ZBSIZE, "-", font, ClientUI::WndColor(),
-                                      ClientUI::CtrlBorderColor(), 1, ClientUI::TextColor(),
-                                      GG::INTERACTIVE | GG::ONTOP);
+    m_zoom_in_button = new CUIButton("+", GG::X1, GG::Y1, ZBSIZE, ClientUI::WndColor());
+    m_zoom_out_button = new CUIButton("-", GG::X1, GG::Y1, ZBSIZE, ClientUI::WndColor());
 
     DoLayout();
 
@@ -890,11 +916,8 @@ void TechTreeWnd::LayoutPanel::Render() {
     GG::Pt lr = LowerRight();
 
     glDisable(GL_TEXTURE_2D);
-    GLint initial_modes[2];
-    glGetIntegerv(GL_POLYGON_MODE, initial_modes);
 
     // draw background
-    glPolygonMode(GL_BACK, GL_FILL);
     glBegin(GL_POLYGON);
         glColor(ClientUI::CtrlColor());
         glVertex(ul.x, ul.y);
@@ -904,9 +927,6 @@ void TechTreeWnd::LayoutPanel::Render() {
         glVertex(ul.x, ul.y);
     glEnd();
 
-    // reset this to whatever it was initially
-    glPolygonMode(GL_BACK, initial_modes[1]);
-
     BeginClipping();
     glEnable(GL_LINE_SMOOTH);
 
@@ -1081,7 +1101,7 @@ void TechTreeWnd::LayoutPanel::HideStatus(TechStatus status) {
 }
 
 void TechTreeWnd::LayoutPanel::ShowTech(const std::string& tech_name)
-{ TechClickedSlot(tech_name, GG::Flags<GG::ModKey>()); }
+{ TechLeftClickedSlot(tech_name, GG::Flags<GG::ModKey>()); }
 
 void TechTreeWnd::LayoutPanel::CenterOnTech(const std::string& tech_name) {
     std::map<std::string, TechPanel*>::const_iterator it = m_techs.find(tech_name);
@@ -1205,7 +1225,8 @@ void TechTreeWnd::LayoutPanel::Layout(bool keep_position) {
         tech_panel->MoveTo(GG::Pt(node->GetX(), node->GetY()));
         m_layout_surface->AttachChild(tech_panel);
         GG::Connect(tech_panel->TechBrowsedSignal,          &TechTreeWnd::LayoutPanel::TechBrowsedSlot,         this);
-        GG::Connect(tech_panel->TechClickedSignal,          &TechTreeWnd::LayoutPanel::TechClickedSlot,         this);
+        GG::Connect(tech_panel->TechLeftClickedSignal,      &TechTreeWnd::LayoutPanel::TechLeftClickedSlot,     this);
+        GG::Connect(tech_panel->TechRightClickedSignal,     &TechTreeWnd::LayoutPanel::TechRightClickedSlot,    this);
         GG::Connect(tech_panel->TechDoubleClickedSignal,    &TechTreeWnd::LayoutPanel::TechDoubleClickedSlot,   this);
 
         const std::vector<TechTreeLayout::Edge*> edges = m_graph.GetOutEdges(tech_name);
@@ -1327,8 +1348,9 @@ void TechTreeWnd::LayoutPanel::ScrolledSlot(int, int, int, int) {
 void TechTreeWnd::LayoutPanel::TechBrowsedSlot(const std::string& tech_name)
 { TechBrowsedSignal(tech_name); }
 
-void TechTreeWnd::LayoutPanel::TechClickedSlot(const std::string& tech_name,
-                                               const GG::Flags<GG::ModKey>& modkeys) {
+void TechTreeWnd::LayoutPanel::TechLeftClickedSlot(const std::string& tech_name,
+                                                   const GG::Flags<GG::ModKey>& modkeys)
+{
     // deselect previously-selected tech panel
     if (m_techs.find(m_selected_tech_name) != m_techs.end())
         m_techs[m_selected_tech_name]->Select(false);
@@ -1336,9 +1358,13 @@ void TechTreeWnd::LayoutPanel::TechClickedSlot(const std::string& tech_name,
     if (m_techs.find(tech_name) != m_techs.end())
         m_techs[tech_name]->Select(true);
     m_selected_tech_name = tech_name;
-    TechClickedSignal(tech_name, modkeys);
+    TechLeftClickedSignal(tech_name, modkeys);
 }
 
+void TechTreeWnd::LayoutPanel::TechRightClickedSlot(const std::string& tech_name,
+                                                    const GG::Flags<GG::ModKey>& modkeys)
+{ TechRightClickedSignal(tech_name, modkeys); }
+
 void TechTreeWnd::LayoutPanel::TechDoubleClickedSlot(const std::string& tech_name,
                                                      const GG::Flags<GG::ModKey>& modkeys)
 { TechDoubleClickedSignal(tech_name, modkeys); }
@@ -1382,6 +1408,7 @@ class TechTreeWnd::TechListBox : public CUIListBox {
 public:
     /** \name Structors */ //@{
     TechListBox(GG::X x, GG::Y y, GG::X w, GG::Y h);
+    virtual ~TechListBox();
     //@}
 
     /** \name Accessors */ //@{
@@ -1403,7 +1430,8 @@ public:
     //@}
 
     mutable TechSignalType      TechBrowsedSignal;      ///< emitted when the mouse rolls over a technology
-    mutable TechClickSignalType TechClickedSignal;      ///< emitted when a technology is single-clicked
+    mutable TechClickSignalType TechLeftClickedSignal;  ///< emitted when a technology is single-left-clicked
+    mutable TechClickSignalType TechRightClickedSignal; ///< emitted when a technology is single-right-clicked
     mutable TechClickSignalType TechDoubleClickedSignal;///< emitted when a technology is double-clicked
 
 private:
@@ -1419,8 +1447,9 @@ private:
     };
 
     void    Populate();
-    void    PropagateDoubleClickSignal(GG::ListBox::iterator it);
-    void    PropagateLeftClickSignal(GG::ListBox::iterator it, const GG::Pt& pt);
+    void    TechDoubleClicked(GG::ListBox::iterator it);
+    void    TechLeftClicked(GG::ListBox::iterator it, const GG::Pt& pt);
+    void    TechRightClicked(GG::ListBox::iterator it, const GG::Pt& pt);
 
     std::set<std::string>                   m_categories_shown;
     std::set<TechStatus>                    m_tech_statuses_shown;
@@ -1513,8 +1542,9 @@ TechTreeWnd::TechListBox::TechRow::TechRow(GG::X w, const std::string& tech_name
 TechTreeWnd::TechListBox::TechListBox(GG::X x, GG::Y y, GG::X w, GG::Y h) :
     CUIListBox(x, y, w, h)
 {
-    GG::Connect(DoubleClickedSignal,    &TechListBox::PropagateDoubleClickSignal,   this);
-    GG::Connect(LeftClickedSignal,      &TechListBox::PropagateLeftClickSignal,     this);
+    GG::Connect(DoubleClickedSignal,    &TechListBox::TechDoubleClicked,    this);
+    GG::Connect(LeftClickedSignal,      &TechListBox::TechLeftClicked,      this);
+    GG::Connect(RightClickedSignal,     &TechListBox::TechRightClicked,     this);
 
     SetStyle(GG::LIST_NOSORT | GG::LIST_NOSEL);
 
@@ -1539,6 +1569,13 @@ TechTreeWnd::TechListBox::TechListBox(GG::X x, GG::Y y, GG::X w, GG::Y h) :
     }
 }
 
+TechTreeWnd::TechListBox::~TechListBox() {
+    for (std::multimap<std::string, TechRow*>::iterator it = m_all_tech_rows.begin();
+         it != m_all_tech_rows.end(); ++it)
+    { delete it->second; }
+    m_all_tech_rows.clear();
+}
+
 std::set<std::string> TechTreeWnd::TechListBox::GetCategoriesShown() const
 { return m_categories_shown; }
 
@@ -1643,17 +1680,22 @@ void TechTreeWnd::TechListBox::HideStatus(TechStatus status) {
 }
 
 bool TechTreeWnd::TechListBox::TechVisible(const std::string& tech_name) {
-    const Empire* empire = Empires().Lookup(HumanClientApp::GetApp()->EmpireID());
-    if (!empire)
-        return true;
     const Tech* tech = ::GetTech(tech_name);
     if (!tech)
         return false;
 
+    // Unresearchable techs are never to be shown on tree
+    if (!tech->Researchable())
+        return false;
+
     // check that category and status are visible
     if (m_categories_shown.find(tech->Category()) == m_categories_shown.end())
         return false;
 
+    // check tech status
+    const Empire* empire = Empires().Lookup(HumanClientApp::GetApp()->EmpireID());
+    if (!empire)
+        return true;    // if no empire, techs have no status, so just return true
     if (m_tech_statuses_shown.find(empire->GetTechStatus(tech->Name())) == m_tech_statuses_shown.end())
         return false;
 
@@ -1661,17 +1703,51 @@ bool TechTreeWnd::TechListBox::TechVisible(const std::string& tech_name) {
     return true;
 }
 
-void TechTreeWnd::TechListBox::PropagateLeftClickSignal(GG::ListBox::iterator it, const GG::Pt& pt) {
+void TechTreeWnd::TechListBox::TechLeftClicked(GG::ListBox::iterator it, const GG::Pt& pt) {
     // determine type of row that was clicked, and emit appropriate signal
-
-    TechRow* tech_row = dynamic_cast<TechRow*>(*it);
-    if (tech_row)
-        TechClickedSignal(tech_row->GetTech(), GG::Flags<GG::ModKey>());
+    if (TechRow* tech_row = dynamic_cast<TechRow*>(*it))
+        TechLeftClickedSignal(tech_row->GetTech(), GG::Flags<GG::ModKey>());
 }
 
-void TechTreeWnd::TechListBox::PropagateDoubleClickSignal(GG::ListBox::iterator it) {
-    // determine type of row that was clicked, and emit appropriate signal
+void TechTreeWnd::TechListBox::TechRightClicked(GG::ListBox::iterator it, const GG::Pt& pt) {
+    if ((*it)->Disabled())
+        return;
+    const Empire* empire = Empires().Lookup(HumanClientApp::GetApp()->EmpireID());
+    if (!empire)
+        return;
 
+    TechRow* tech_row = dynamic_cast<TechRow*>(*it);
+    if (!tech_row)
+        return;
+    const std::string& tech_name = tech_row->GetTech();
+
+    if (empire->TechResearched(tech_name))
+        return;
+
+    const ResearchQueue& rq = empire->GetResearchQueue();
+    if (rq.find(tech_name) != rq.end())
+        return;
+
+    GG::MenuItem menu_contents;
+    menu_contents.next_level.push_back(GG::MenuItem(UserString("PRODUCTION_DETAIL_ADD_TO_QUEUE"),   1, false, false));
+
+    GG::PopupMenu popup(pt.x, pt.y, ClientUI::GetFont(), menu_contents, ClientUI::TextColor(),
+                        ClientUI::WndOuterBorderColor(), ClientUI::WndColor(), ClientUI::EditHiliteColor());
+
+    if (popup.Run()) {
+        switch (popup.MenuID()) {
+        case 1: {
+            TechDoubleClicked(it);
+            break;
+        }
+        default:
+            break;
+        }
+    }
+}
+
+void TechTreeWnd::TechListBox::TechDoubleClicked(GG::ListBox::iterator it) {
+    // determine type of row that was clicked, and emit appropriate signal
     TechRow* tech_row = dynamic_cast<TechRow*>(*it);
     if (tech_row)
         TechDoubleClickedSignal(tech_row->GetTech(), GG::Flags<GG::ModKey>());
@@ -1690,20 +1766,22 @@ TechTreeWnd::TechTreeWnd(GG::X w, GG::Y h) :
     Sound::TempUISoundDisabler sound_disabler;
 
     m_layout_panel = new LayoutPanel(w, h);
-    GG::Connect(m_layout_panel->TechBrowsedSignal,          &TechTreeWnd::TechBrowsedSlot, this);
-    GG::Connect(m_layout_panel->TechClickedSignal,          &TechTreeWnd::TechClickedSlot, this);
-    GG::Connect(m_layout_panel->TechDoubleClickedSignal,    &TechTreeWnd::TechDoubleClickedSlot, this);
+    GG::Connect(m_layout_panel->TechBrowsedSignal,      &TechTreeWnd::TechBrowsedSlot,          this);
+    GG::Connect(m_layout_panel->TechLeftClickedSignal,  &TechTreeWnd::TechLeftClickedSlot,      this);
+    GG::Connect(m_layout_panel->TechRightClickedSignal, &TechTreeWnd::TechRightClickedSlot,     this);
+    GG::Connect(m_layout_panel->TechDoubleClickedSignal,&TechTreeWnd::TechDoubleClickedSlot,    this);
     AttachChild(m_layout_panel);
 
     m_tech_list = new TechListBox(GG::X0, GG::Y0, w, h);
-    GG::Connect(m_tech_list->TechBrowsedSignal,             &TechTreeWnd::TechBrowsedSlot, this);
-    GG::Connect(m_tech_list->TechClickedSignal,             &TechTreeWnd::TechClickedSlot, this);
-    GG::Connect(m_tech_list->TechDoubleClickedSignal,       &TechTreeWnd::TechDoubleClickedSlot, this);
+    GG::Connect(m_tech_list->TechBrowsedSignal,         &TechTreeWnd::TechBrowsedSlot,          this);
+    GG::Connect(m_tech_list->TechLeftClickedSignal,     &TechTreeWnd::TechLeftClickedSlot,      this);
+    GG::Connect(m_tech_list->TechRightClickedSignal,    &TechTreeWnd::TechRightClickedSlot,     this);
+    GG::Connect(m_tech_list->TechDoubleClickedSignal,   &TechTreeWnd::TechDoubleClickedSlot,    this);
 
     GG::X ENC_WIDTH(480);
     GG::Y ENC_HEIGHT(240);
 
-    m_enc_detail_panel = new EncyclopediaDetailPanel(ENC_WIDTH, ENC_HEIGHT);
+    m_enc_detail_panel = new EncyclopediaDetailPanel(ENC_WIDTH, ENC_HEIGHT, GG::ONTOP | GG::INTERACTIVE | GG::DRAGABLE | GG::RESIZABLE | PINABLE );
     AttachChild(m_enc_detail_panel);
 
     m_tech_tree_controls = new TechTreeControls(GG::X1, GG::Y1, m_layout_panel->Width() - ClientUI::ScrollWidth());
@@ -1911,7 +1989,7 @@ void TechTreeWnd::SelectTech(const std::string& tech_name)
 void TechTreeWnd::TechBrowsedSlot(const std::string& tech_name)
 { TechBrowsedSignal(tech_name); }
 
-void TechTreeWnd::TechClickedSlot(const std::string& tech_name,
+void TechTreeWnd::TechLeftClickedSlot(const std::string& tech_name,
                                   const GG::Flags<GG::ModKey>& modkeys)
 {
     if (modkeys & GG::MOD_KEY_SHIFT) {
@@ -1922,6 +2000,10 @@ void TechTreeWnd::TechClickedSlot(const std::string& tech_name,
     }
 }
 
+void TechTreeWnd::TechRightClickedSlot(const std::string& tech_name,
+                                       const GG::Flags<GG::ModKey>& modkeys)
+{}
+
 void TechTreeWnd::TechDoubleClickedSlot(const std::string& tech_name,
                                         const GG::Flags<GG::ModKey>& modkeys)
 {
