@@ -14,13 +14,16 @@
 #include "../../UI/IntroScreen.h"
 #include "../../UI/GalaxySetupWnd.h"
 #include "../../UI/MultiplayerLobbyWnd.h"
+#include "../../UI/SaveFileDialog.h"
 #include "../../UI/ServerConnectWnd.h"
 #include "../../UI/Sound.h"
 #include "../../network/Message.h"
 #include "../../network/Networking.h"
 #include "../../util/i18n.h"
+#include "../../util/MultiplayerCommon.h"
 #include "../../util/OptionsDB.h"
 #include "../../util/Process.h"
+#include "../../util/SaveGamePreviewUtils.h"
 #include "../../util/Serialize.h"
 #include "../../util/SitRepEntry.h"
 #include "../../util/Directories.h"
@@ -33,11 +36,12 @@
 #include <GG/BrowseInfoWnd.h>
 #include <GG/Cursor.h>
 #include <GG/utf8/checked.h>
+#include <GG/Ogre/Plugins/OISInput.h>
 
-#include <OgreRoot.h>
-#include <OgreVector3.h>
 #include <OgreRenderWindow.h>
-#include <OgreRenderSystem.h>
+#include <OgreRoot.h>
+
+#include <OIS/OISMouse.h>
 
 #include <boost/lexical_cast.hpp>
 #include <boost/filesystem/operations.hpp>
@@ -90,11 +94,15 @@ namespace {
 
     // command-line options
     void AddOptions(OptionsDB& db) {
-        db.Add("autosave.single-player",    UserStringNop("OPTIONS_DB_AUTOSAVE_SINGLE_PLAYER"),    true,   Validator<bool>());
-        db.Add("autosave.multiplayer",      UserStringNop("OPTIONS_DB_AUTOSAVE_MULTIPLAYER"),      false,  Validator<bool>());
-        db.Add("autosave.turns",            UserStringNop("OPTIONS_DB_AUTOSAVE_TURNS"),            1,      RangedValidator<int>(1, 50));
-        db.Add("autosave.limit",            UserStringNop("OPTIONS_DB_AUTOSAVE_LIMIT"),            10,     RangedValidator<int>(1, 100));
-        db.Add("UI.swap-mouse-lr",          UserStringNop("OPTIONS_DB_UI_MOUSE_LR_SWAP"),          false);
+        db.Add("autosave.single-player",        UserStringNop("OPTIONS_DB_AUTOSAVE_SINGLE_PLAYER"),     true,   Validator<bool>());
+        db.Add("autosave.multiplayer",          UserStringNop("OPTIONS_DB_AUTOSAVE_MULTIPLAYER"),       true,   Validator<bool>());
+        db.Add("autosave.turns",                UserStringNop("OPTIONS_DB_AUTOSAVE_TURNS"),             1,      RangedValidator<int>(1, 50));
+        db.Add("autosave.limit",                UserStringNop("OPTIONS_DB_AUTOSAVE_LIMIT"),             10,     RangedValidator<int>(1, 100));
+        db.Add("UI.swap-mouse-lr",              UserStringNop("OPTIONS_DB_UI_MOUSE_LR_SWAP"),           false);
+        db.Add("UI.keypress-repeat-delay",      UserStringNop("OPTIONS_DB_KEYPRESS_REPEAT_DELAY"),      360,    RangedValidator<int>(0, 1000));
+        db.Add("UI.keypress-repeat-interval",   UserStringNop("OPTIONS_DB_KEYPRESS_REPEAT_INTERVAL"),   20,     RangedValidator<int>(0, 1000));
+        db.Add("UI.mouse-click-repeat-delay",   UserStringNop("OPTIONS_DB_MOUSE_REPEAT_DELAY"),         360,    RangedValidator<int>(0, 1000));
+        db.Add("UI.mouse-click-repeat-interval",UserStringNop("OPTIONS_DB_MOUSE_REPEAT_INTERVAL"),      15,     RangedValidator<int>(0, 1000));
     }
     bool temp_bool = RegisterOptions(&AddOptions);
 
@@ -187,7 +195,6 @@ HumanClientApp::HumanClientApp(Ogre::Root* root,
     SetStyleFactory(style);
 
     SetMinDragTime(0);
-    EnableMouseButtonDownRepeat(250, 15);
 
     m_ui = boost::shared_ptr<ClientUI>(new ClientUI());
 
@@ -211,6 +218,11 @@ HumanClientApp::HumanClientApp(Ogre::Root* root,
     SetCursor(boost::shared_ptr<GG::TextureCursor>(new GG::TextureCursor(cursor_texture, GG::Pt(GG::X(6), GG::Y(3)))));
     RenderCursor(true);
 
+    EnableKeyPressRepeat(GetOptionsDB().Get<int>("UI.keypress-repeat-delay"),
+                         GetOptionsDB().Get<int>("UI.keypress-repeat-interval"));
+    EnableMouseButtonDownRepeat(GetOptionsDB().Get<int>("UI.mouse-click-repeat-delay"),
+                                GetOptionsDB().Get<int>("UI.mouse-click-repeat-interval"));
+
     GG::Connect(WindowMovedSignal,      &HumanClientApp::HandleWindowMove,      this);
     GG::Connect(WindowResizedSignal,    &HumanClientApp::HandleWindowResize,    this);
     GG::Connect(WindowClosingSignal,    &HumanClientApp::HandleWindowClosing,   this);
@@ -269,6 +281,33 @@ HumanClientApp::~HumanClientApp() {
 bool HumanClientApp::SinglePlayerGame() const
 { return m_single_player_game; }
 
+bool HumanClientApp::CanSaveNow() const {
+    // only host can save in multiplayer
+    if (!SinglePlayerGame() && !Networking().PlayerIsHost(PlayerID()))
+        return false;
+
+    // can't save while AIs are playing their turns...
+    for (std::map<int, PlayerInfo>::const_iterator player_it = this->m_player_info.begin();
+         player_it != this->m_player_info.end(); ++player_it)
+    {
+        const PlayerInfo& info = player_it->second;
+        if (info.client_type != Networking::CLIENT_TYPE_AI_PLAYER)
+            continue;   // only care about AIs
+
+        std::map<int, Message::PlayerStatus>::const_iterator
+            status_it = this->m_player_status.find(player_it->first);
+
+        if (status_it == this->m_player_status.end()) {
+            return false;  // missing status for AI; can't assume it's ready
+        }
+        if (status_it->second != Message::WAITING) {
+            return false;
+        }
+    }
+
+    return true;
+}
+
 void HumanClientApp::SetSinglePlayerGame(bool sp/* = true*/)
 { m_single_player_game = sp; }
 
@@ -282,9 +321,23 @@ namespace {
     }
 }
 
+#ifdef FREEORION_MACOSX
+#include <stdlib.h>
+#endif
+
 void HumanClientApp::StartServer() {
     std::string SERVER_CLIENT_EXE = ServerClientExe();
     Logger().debugStream() << "HumanClientApp::StartServer: " << SERVER_CLIENT_EXE;
+
+#ifdef FREEORION_MACOSX
+    // On OSX set environment variable DYLD_LIBRARY_PATH to python framework folder
+    // bundled with app, so the dynamic linker uses the bundled python library.
+    // Otherwise the dynamic linker will look for a correct python lib in system
+    // paths, and if it can't find it, throw an error and terminate!
+    // Setting environment variable here, spawned child processes will inherit it.
+    setenv("DYLD_LIBRARY_PATH", GetPythonHome().string().c_str(), 1);
+#endif
+
     std::vector<std::string> args;
     args.push_back("\"" + SERVER_CLIENT_EXE + "\"");
     args.push_back("--resource-dir");
@@ -513,13 +566,10 @@ void HumanClientApp::LoadSinglePlayerGame(std::string filename/* = ""*/) {
         }
     } else {
         try {
-            std::vector<std::pair<std::string, std::string> > save_file_types;
-            save_file_types.push_back(std::pair<std::string, std::string>(UserString("GAME_MENU_SAVE_FILES"), "*.sav"));
-            std::string path_string = PathString(GetSaveDir());
-            FileDlg dlg(path_string, "", false, false, save_file_types);
-            dlg.Run();
-            if (!dlg.Result().empty())
-                filename = *dlg.Result().begin();
+            SaveFileDialog sfd(SP_SAVE_FILE_EXTENSION, true);
+            sfd.Run();
+            if (!sfd.Result().empty())
+                filename = sfd.Result();
         } catch (const std::exception& e) {
             ClientUI::MessageBox(e.what(), true);
         }
@@ -576,6 +626,37 @@ void HumanClientApp::LoadSinglePlayerGame(std::string filename/* = ""*/) {
     m_fsm->process_event(HostSPGameRequested());
 }
 
+void HumanClientApp::RequestSavePreviews(const std::string& directory, PreviewInformation& previews){
+    bool server_just_for_us = false;
+    std::string  generic_directory = fs::path(directory).generic_string();
+    if(!m_networking.Connected()){
+        Logger().debugStream() << "HumanClientApp::RequestSavePreviews: No game running. Start a server for savegame queries.";
+        StartServer();
+        server_just_for_us = true;
+        
+        Logger().debugStream() << "HumanClientApp::RequestSavePreviews Connecting to server";
+        unsigned int start_time = Ticks();
+        while (!m_networking.ConnectToLocalHostServer()) {
+            if (SERVER_CONNECT_TIMEOUT < Ticks() - start_time) {
+                Logger().errorStream() << "HumanClientApp::LoadSinglePlayerGame() server connecting timed out";
+                ClientUI::MessageBox(UserString("ERR_CONNECT_TIMED_OUT"), true);
+                KillServer();
+                return;
+            }
+        }
+        m_connected = true;
+    }
+    Logger().debugStream() << "HumanClientApp::RequestSavePreviews Requesting previews for " << generic_directory;
+    Message response;
+    m_networking.SendSynchronousMessage(RequestSavePreviewsMessage(PlayerID(), generic_directory), response);
+    if(response.Type() == Message::DISPATCH_SAVE_PREVIEWS){
+        ExtractMessageData(response, previews);
+        Logger().debugStream() << "HumanClientApp::RequestSavePreviews Got " << previews.previews.size() << " previews.";
+    }else{
+        Logger().errorStream() << "HumanClientApp::RequestSavePreviews: Wrong response type from server: " << EnumToString(response.Type());
+    }
+}
+
 Ogre::SceneManager* HumanClientApp::SceneManager()
 { return m_scene_manager; }
 
@@ -782,7 +863,7 @@ void HumanClientApp::RenderBegin() {
 
 void HumanClientApp::HandleMessage(Message& msg) {
     if (INSTRUMENT_MESSAGE_HANDLING)
-        std::cerr << "HumanClientApp::HandleMessage(" << MessageTypeStr(msg.Type()) << ")\n";
+        std::cerr << "HumanClientApp::HandleMessage(" << msg.Type() << ")\n";
 
     switch (msg.Type()) {
     case Message::ERROR_MSG:            m_fsm->process_event(Error(msg));                   break;
@@ -797,7 +878,7 @@ void HumanClientApp::HandleMessage(Message& msg) {
     case Message::TURN_UPDATE:          m_fsm->process_event(TurnUpdate(msg));              break;
     case Message::TURN_PARTIAL_UPDATE:  m_fsm->process_event(TurnPartialUpdate(msg));       break;
     case Message::TURN_PROGRESS:        m_fsm->process_event(TurnProgress(msg));            break;
-    case Message::PLAYER_STATUS:        m_fsm->process_event(PlayerStatus(msg));            break;
+    case Message::PLAYER_STATUS:        m_fsm->process_event(::PlayerStatus(msg));          break;
     case Message::COMBAT_START:         m_fsm->process_event(CombatStart(msg));             break;
     case Message::COMBAT_TURN_UPDATE:   m_fsm->process_event(CombatRoundUpdate(msg));       break;
     case Message::COMBAT_END:           m_fsm->process_event(CombatEnd(msg));               break;
@@ -815,7 +896,7 @@ void HumanClientApp::HandleMessage(Message& msg) {
 
 void HumanClientApp::HandleSaveGameDataRequest() {
     if (INSTRUMENT_MESSAGE_HANDLING)
-        std::cerr << "HumanClientApp::HandleSaveGameDataRequest(" << MessageTypeStr(Message::SAVE_GAME) << ")\n";
+        std::cerr << "HumanClientApp::HandleSaveGameDataRequest(" << Message::SAVE_GAME << ")\n";
     SaveGameUIData ui_data;
     m_ui->GetSaveGameUIData(ui_data);
     m_networking.SendMessage(ClientSaveDataMessage(PlayerID(), Orders(), ui_data));
@@ -890,12 +971,51 @@ void HumanClientApp::HandleWindowClosing()
 
 void HumanClientApp::HandleWindowClose() {
     Logger().debugStream() << "HumanClientApp::HandleWindowClose()";
-    EndGame(true);
+    EndGame();
     Exit(0);
 }
 
-void HumanClientApp::HandleFocusChange()
-{ Logger().debugStream() << "HumanClientApp::HandleFocusChange()"; }
+void HumanClientApp::HandleFocusChange() {
+    Logger().debugStream() << "HumanClientApp::HandleFocusChange()";
+    const Ogre::Root::PluginInstanceList& ogre_plugins_list = m_root->getInstalledPlugins();
+
+    // get OIS plugin
+    OISInput* ois;
+    for (Ogre::Root::PluginInstanceList::const_iterator it = ogre_plugins_list.begin();
+         it != ogre_plugins_list.end(); ++it)
+    {
+        if (ois = dynamic_cast<OISInput*>(*it))
+            break;
+    }
+    OIS::Mouse* mouse = 0;
+    if (ois)
+        mouse = ois->GetMouse();
+    if (mouse) {
+        //OIS::MouseState mouse_state = mouse->getMouseState();
+        //OIS::MouseEvent mouse_event(mouse, mouse_state);
+        //OIS::MouseListener* ml = ois;
+        //for (OIS::MouseButtonID id = OIS::MB_Left; id <= OIS::MB_Button7; id = OIS::MouseButtonID(id + 1))
+        //    ml->mouseReleased(mouse_event, id);
+        mouse->capture();
+    }
+
+    OIS::Keyboard* keyboard = 0;
+    if (ois)
+        keyboard = ois->GetKeyboard();
+    if (keyboard) {
+        //OIS::KeyListener* kl = ois;
+        //for (OIS::KeyCode key_code = OIS::KC_ESCAPE; key_code <= OIS::KC_MEDIASELECT;
+        //     key_code = OIS::KeyCode(key_code + 1))
+        //{
+        //    OIS::KeyEvent key_event(keyboard, key_code, '\0');
+        //    kl->keyReleased(key_event);
+        //}
+        keyboard->capture();
+    }
+
+    CancelDragDrop();
+    ClearEventState();
+}
 
 void HumanClientApp::StartGame() {
     m_game_started = true;
@@ -965,20 +1085,28 @@ void HumanClientApp::Autosave() {
     const char* legal_chars = "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ1234567890_-";
 
     // get empire name, filtered for filename acceptability
-    std::string empire_name = Empires().Lookup(EmpireID())->Name();
+    int client_empire_id = EmpireID();
+    const Empire* empire = Empires().Lookup(client_empire_id);
+    std::string empire_name;
+    if (empire)
+        empire_name = empire->Name();
+    else
+        empire_name = UserString("OBSERVER");
     std::string::size_type first_good_empire_char = empire_name.find_first_of(legal_chars);
     if (first_good_empire_char == std::string::npos) {
-        empire_name = "";
+        empire_name.clear();
     } else {
         std::string::size_type first_bad_empire_char = empire_name.find_first_not_of(legal_chars, first_good_empire_char);
         empire_name = empire_name.substr(first_good_empire_char, first_bad_empire_char - first_good_empire_char);
     }
 
     // get player name, also filtered
-    std::string player_name = Empires().Lookup(EmpireID())->PlayerName();
+    std::string player_name;
+    if (empire)
+        player_name = empire->PlayerName();
     std::string::size_type first_good_player_char = player_name.find_first_of(legal_chars);
     if (first_good_player_char == std::string::npos) {
-        player_name = "";
+        player_name.clear();
     } else {
         std::string::size_type first_bad_player_char = player_name.find_first_not_of(legal_chars, first_good_player_char);
         player_name = player_name.substr(first_good_player_char, first_bad_player_char - first_good_player_char);
@@ -992,11 +1120,7 @@ void HumanClientApp::Autosave() {
         extension = MP_SAVE_FILE_EXTENSION;
 
     // Add timestamp to autosave generated files
-    boost::posix_time::time_facet* facet = new boost::posix_time::time_facet("%Y%m%d_%H%M%S");
-    std::stringstream date_stream;
-    date_stream.imbue(std::locale(date_stream.getloc(), facet));
-    date_stream << boost::posix_time::microsec_clock::local_time();
-    std::string datetime_str=date_stream.str();
+    std::string datetime_str = FilenameTimestamp();
 
     boost::filesystem::path autosave_dir_path(GetSaveDir() / "auto");
 
@@ -1027,6 +1151,19 @@ void HumanClientApp::Autosave() {
     }
 }
 
+std::string HumanClientApp::SelectLoadFile(){
+    SaveFileDialog sfd(true);
+    sfd.Run();
+    return sfd.Result();
+}
+
+std::string HumanClientApp::SelectSaveFile() {
+    SaveFileDialog sfd(false);
+    sfd.Run();
+    return sfd.Result();
+}
+
+
 void HumanClientApp::EndGame(bool suppress_FSM_reset) {
     Logger().debugStream() << "HumanClientApp::EndGame";
     m_game_started = false;
