@@ -21,17 +21,6 @@ CombatLogManager&   GetCombatLogManager();
 
 namespace {
     const bool TRACE_EXECUTION = true;
-
-    void FreeCombatData(CombatData* combat_data) {
-        if (!combat_data)
-            return;
-        delete combat_data->m_system;
-        for (std::map<int, UniverseObject*>::iterator it = combat_data->m_combat_universe.begin();
-             it != combat_data->m_combat_universe.end();
-             ++it) {
-            delete it->second;
-        }
-    }
 }
 
 ////////////////////////////////////////////////////////////
@@ -70,35 +59,39 @@ IntroMenu::IntroMenu(my_context ctx) :
     Base(ctx)
 {
     if (TRACE_EXECUTION) Logger().debugStream() << "(HumanClientFSM) IntroMenu";
+    Client().KillServer();
+
+    Client().Remove(Client().GetClientUI()->GetMapWnd());
+    Client().GetClientUI()->GetMapWnd()->Hide();
+
     if (GetOptionsDB().Get<bool>("tech-demo"))
         Client().Register(Client().GetClientUI()->GetCombatWnd());
     else {
         Client().Register(Client().GetClientUI()->GetIntroScreen());
-        Client().Remove(Client().GetClientUI()->GetMessageWnd());
-        Client().Remove(Client().GetClientUI()->GetPlayerListWnd());
+        Client().GetClientUI()->GetMapWnd()->HideMessages();
+        Client().GetClientUI()->GetMapWnd()->HideEmpires();
     }
 }
 
 IntroMenu::~IntroMenu() {
     if (TRACE_EXECUTION) Logger().debugStream() << "(HumanClientFSM) ~IntroMenu";
-    if (GetOptionsDB().Get<bool>("tech-demo"))
-        Client().Remove(Client().GetClientUI()->GetCombatWnd());
-
-    Client().Remove(Client().GetClientUI()->GetIntroScreen());
 }
 
 boost::statechart::result IntroMenu::react(const HostSPGameRequested& a) {
     if (TRACE_EXECUTION) Logger().debugStream() << "(HumanClientFSM) IntroMenu.HostSPGameRequested";
+    Client().Remove(Client().GetClientUI()->GetIntroScreen());
     return transit<WaitingForSPHostAck>();
 }
 
 boost::statechart::result IntroMenu::react(const HostMPGameRequested& a) {
     if (TRACE_EXECUTION) Logger().debugStream() << "(HumanClientFSM) IntroMenu.HostMPGameRequested";
+    Client().Remove(Client().GetClientUI()->GetIntroScreen());
     return transit<WaitingForMPHostAck>();
 }
 
 boost::statechart::result IntroMenu::react(const JoinMPGameRequested& a) {
     if (TRACE_EXECUTION) Logger().debugStream() << "(HumanClientFSM) IntroMenu.JoinMPGameRequested";
+    Client().Remove(Client().GetClientUI()->GetIntroScreen());
     return transit<WaitingForMPJoinAck>();
 }
 
@@ -120,8 +113,7 @@ boost::statechart::result WaitingForSPHostAck::react(const HostSPGame& msg) {
     Client().Networking().SetHostPlayerID(msg.m_message.ReceivingPlayer());
 
     Client().GetClientUI()->GetMapWnd()->Sanitize();
-    Client().GetClientUI()->GetMessageWnd()->Show();
-    Client().GetClientUI()->GetPlayerListWnd()->Show();
+
     return transit<PlayingGame>();
 }
 
@@ -229,13 +221,12 @@ MPLobby::MPLobby(my_context ctx) :
 
 MPLobby::~MPLobby() {
     if (TRACE_EXECUTION) Logger().debugStream() << "(HumanClientFSM) ~MPLobby";
-
-    Client().Remove(Client().GetClientUI()->GetMultiPlayerLobbyWnd());
 }
 
 boost::statechart::result MPLobby::react(const Disconnection& d) {
     if (TRACE_EXECUTION) Logger().debugStream() << "(HumanClientFSM) MPLobby.Disconnection";
     ClientUI::MessageBox(UserString("SERVER_LOST"), true);
+    Client().Remove(Client().GetClientUI()->GetMultiPlayerLobbyWnd());
     return transit<IntroMenu>();
 }
 
@@ -276,6 +267,7 @@ boost::statechart::result MPLobby::react(const CancelMPGameClicked& a)
 {
     if (TRACE_EXECUTION) Logger().debugStream() << "(HumanClientFSM) MPLobby.CancelMPGameClicked";
     HumanClientApp::GetApp()->Networking().DisconnectFromServer();
+    Client().Remove(Client().GetClientUI()->GetMultiPlayerLobbyWnd());
     return transit<IntroMenu>();
 }
 
@@ -298,9 +290,7 @@ boost::statechart::result MPLobby::react(const GameStart& msg) {
     post_event(msg);
 
     Client().GetClientUI()->GetMapWnd()->Sanitize();
-    Client().GetClientUI()->GetMessageWnd()->Show();
-    Client().GetClientUI()->GetPlayerListWnd()->Show();
-
+    Client().Remove(Client().GetClientUI()->GetMultiPlayerLobbyWnd());
     return transit<WaitingForGameStart>();
 }
 
@@ -314,6 +304,7 @@ boost::statechart::result MPLobby::react(const Error& msg) {
     ClientUI::MessageBox(UserString(problem), true);
 
     if (fatal) {
+        Client().Remove(Client().GetClientUI()->GetMultiPlayerLobbyWnd());
         return transit<IntroMenu>();
     } else {
         return discard_event();
@@ -324,15 +315,17 @@ boost::statechart::result MPLobby::react(const Error& msg) {
 ////////////////////////////////////////////////////////////
 // PlayingGame
 ////////////////////////////////////////////////////////////
-PlayingGame::PlayingGame() :
-    Base()
-{ if (TRACE_EXECUTION) Logger().debugStream() << "(HumanClientFSM) PlayingGame"; }
+PlayingGame::PlayingGame(my_context ctx) :
+    Base(ctx)
+{
+    if (TRACE_EXECUTION) Logger().debugStream() << "(HumanClientFSM) PlayingGame";
+
+    Client().Register(Client().GetClientUI()->GetMapWnd());
+    Client().GetClientUI()->GetMapWnd()->Show();
+}
 
 PlayingGame::~PlayingGame() {
     if (TRACE_EXECUTION) Logger().debugStream() << "(HumanClientFSM) ~PlayingGame";
-    Client().Remove(Client().GetClientUI()->GetMapWnd());
-    Client().Networking().SetHostPlayerID(Networking::INVALID_PLAYER_ID);
-    Client().Networking().SetPlayerID(Networking::INVALID_PLAYER_ID);
 }
 
 boost::statechart::result PlayingGame::react(const HostID& msg) {
@@ -357,7 +350,7 @@ boost::statechart::result PlayingGame::react(const HostID& msg) {
 }
 
 boost::statechart::result PlayingGame::react(const PlayerChat& msg) {
-    if (TRACE_EXECUTION) Logger().debugStream() << "(HumanClientFSM) PlayingGame.PlayerChat";
+    if (TRACE_EXECUTION) Logger().debugStream() << "(HumanClientFSM) PlayingGame.PlayerChat: " << msg.m_message.Text();
     const std::string& text = msg.m_message.Text();
     int sending_player_id = msg.m_message.SendingPlayer();
     int recipient_player_id = msg.m_message.ReceivingPlayer();
@@ -371,6 +364,9 @@ boost::statechart::result PlayingGame::react(const Disconnection& d) {
     if (TRACE_EXECUTION) Logger().debugStream() << "(HumanClientFSM) PlayingGame.Disconnection";
     Client().EndGame(true);
     ClientUI::MessageBox(UserString("SERVER_LOST"), true);
+    Client().Remove(Client().GetClientUI()->GetMapWnd());
+    Client().Networking().SetHostPlayerID(Networking::INVALID_PLAYER_ID);
+    Client().Networking().SetPlayerID(Networking::INVALID_PLAYER_ID);
     return transit<IntroMenu>();
 }
 
@@ -380,6 +376,7 @@ boost::statechart::result PlayingGame::react(const PlayerStatus& msg) {
     Message::PlayerStatus status;
     ExtractMessageData(msg.m_message, about_player_id, status);
 
+    Client().SetPlayerStatus(about_player_id, status);
     Client().GetClientUI()->GetMessageWnd()->HandlePlayerStatusUpdate(status, about_player_id);
     Client().GetClientUI()->GetPlayerListWnd()->HandlePlayerStatusUpdate(status, about_player_id);
     // TODO: tell the map wnd or something else as well?
@@ -457,6 +454,9 @@ boost::statechart::result PlayingGame::react(const EndGame& msg) {
         break;
     }
     ClientUI::MessageBox(reason_message, error);
+    Client().Remove(Client().GetClientUI()->GetMapWnd());
+    Client().Networking().SetHostPlayerID(Networking::INVALID_PLAYER_ID);
+    Client().Networking().SetPlayerID(Networking::INVALID_PLAYER_ID);
     return transit<IntroMenu>();
 }
 
@@ -464,7 +464,9 @@ boost::statechart::result PlayingGame::react(const ResetToIntroMenu& msg) {
     if (TRACE_EXECUTION) Logger().debugStream() << "(HumanClientFSM) PlayingGame.ResetToIntroMenu";
 
     Client().GetClientUI()->GetMessageWnd()->HandleGameStatusUpdate(UserString("RETURN_TO_INTRO") + "\n");
-
+    Client().Remove(Client().GetClientUI()->GetMapWnd());
+    Client().Networking().SetHostPlayerID(Networking::INVALID_PLAYER_ID);
+    Client().Networking().SetPlayerID(Networking::INVALID_PLAYER_ID);
     return transit<IntroMenu>();
 }
 
@@ -479,6 +481,9 @@ boost::statechart::result PlayingGame::react(const Error& msg) {
 
     if (fatal) {
         Client().GetClientUI()->GetMessageWnd()->HandleGameStatusUpdate(UserString("RETURN_TO_INTRO") + "\n");
+        Client().Remove(Client().GetClientUI()->GetMapWnd());
+        Client().Networking().SetHostPlayerID(Networking::INVALID_PLAYER_ID);
+        Client().Networking().SetPlayerID(Networking::INVALID_PLAYER_ID);
         return transit<IntroMenu>();
     } else {
         return discard_event();
@@ -514,8 +519,9 @@ WaitingForGameStart::WaitingForGameStart(my_context ctx) :
 {
     if (TRACE_EXECUTION) Logger().debugStream() << "(HumanClientFSM) WaitingForGameStart";
 
-    Client().Register(Client().GetClientUI()->GetMessageWnd());
-    Client().Register(Client().GetClientUI()->GetPlayerListWnd());
+    Client().GetClientUI()->GetMapWnd()->ShowMessages();
+    Client().GetClientUI()->GetMapWnd()->ShowEmpires();
+
     Client().GetClientUI()->GetMapWnd()->EnableOrderIssuing(false);
 }
 
@@ -534,12 +540,14 @@ boost::statechart::result WaitingForGameStart::react(const GameStart& msg) {
     bool single_player_game = false;
     int empire_id = ALL_EMPIRES;
     int current_turn = INVALID_GAME_TURN;
+    Client().PlayerStatus().clear();
 
     ExtractMessageData(msg.m_message,       single_player_game,             empire_id,
                        current_turn,        Empires(),                      GetUniverse(),
                        GetSpeciesManager(), GetCombatLogManager(),          Client().Players(),
                        orders,              loaded_game_data,               ui_data_available,
-                       ui_data,             save_state_string_available,    save_state_string);
+                       ui_data,             save_state_string_available,    save_state_string,
+                       Client().GetGalaxySetupData());
 
     Logger().debugStream() << "Extracted GameStart message for turn: " << current_turn << " with empire: " << empire_id;
 
@@ -569,9 +577,6 @@ WaitingForTurnData::WaitingForTurnData(my_context ctx) :
     Base(ctx)
 {
     if (TRACE_EXECUTION) Logger().debugStream() << "(HumanClientFSM) WaitingForTurnData";
-
-    Client().Register(Client().GetClientUI()->GetMessageWnd());
-    Client().Register(Client().GetClientUI()->GetPlayerListWnd());
     Client().GetClientUI()->GetMapWnd()->EnableOrderIssuing(false);
 }
 
@@ -619,6 +624,10 @@ boost::statechart::result WaitingForTurnData::react(const TurnUpdate& msg) {
 ////////////////////////////////////////////////////////////
 // PlayingTurn
 ////////////////////////////////////////////////////////////
+namespace {
+    static bool once = true;
+}
+
 PlayingTurn::PlayingTurn(my_context ctx) :
     Base(ctx)
 {
@@ -635,18 +644,19 @@ PlayingTurn::PlayingTurn(my_context ctx) :
     if (Client().GetApp()->GetClientType() != Networking::CLIENT_TYPE_HUMAN_OBSERVER)
         Client().GetClientUI()->GetMapWnd()->EnableOrderIssuing(true);
 
-    // observers can't do anything but wait for the next update, and need to
-    // be back in WaitingForTurnData, so posting TurnEnded here has the effect
-    // of keeping observers in the WaitingForTurnData state so they can receive
-    // updates from the server.
-    if (Client().GetApp()->GetClientType() == Networking::CLIENT_TYPE_HUMAN_OBSERVER)
+    if (Client().GetApp()->GetClientType() == Networking::CLIENT_TYPE_HUMAN_OBSERVER) {
+        // observers can't do anything but wait for the next update, and need to
+        // be back in WaitingForTurnData, so posting TurnEnded here has the effect
+        // of keeping observers in the WaitingForTurnData state so they can receive
+        // updates from the server.
         post_event(TurnEnded());
 
-    else if (GetOptionsDB().Get<bool>("auto-advance-first-turn")) {
-        static bool once = true;
-        if (once) {
-            post_event(AutoAdvanceFirstTurn());
+    } else if (Client().GetApp()->GetClientType() == Networking::CLIENT_TYPE_HUMAN_PLAYER) {
+        if ((once && GetOptionsDB().Get<bool>("auto-advance-first-turn")) ||
+            Client().GetClientUI()->GetMapWnd()->AutoEndTurnEnabled())
+        {
             once = false;
+            post_event(AdvanceTurn());
         }
     }
 }
@@ -660,8 +670,8 @@ boost::statechart::result PlayingTurn::react(const SaveGame& msg) {
     return discard_event();
 }
 
-boost::statechart::result PlayingTurn::react(const AutoAdvanceFirstTurn& d) {
-    Client().GetClientUI()->GetMapWnd()->m_turn_update->LeftClickedSignal();
+boost::statechart::result PlayingTurn::react(const AdvanceTurn& d) {
+    Client().GetClientUI()->GetMapWnd()->m_btn_turn->LeftClickedSignal();
     return discard_event();
 }
 
@@ -682,6 +692,50 @@ boost::statechart::result PlayingTurn::react(const TurnEnded& msg) {
     return transit<WaitingForTurnData>();
 }
 
+boost::statechart::result PlayingTurn::react(const PlayerStatus& msg) {
+    if (TRACE_EXECUTION) Logger().debugStream() << "(HumanClientFSM) PlayingTurn.PlayerStatus";
+    int about_player_id;
+    Message::PlayerStatus status;
+    ExtractMessageData(msg.m_message, about_player_id, status);
+
+    Client().SetPlayerStatus(about_player_id, status);
+    Client().GetClientUI()->GetMessageWnd()->HandlePlayerStatusUpdate(status, about_player_id);
+    Client().GetClientUI()->GetPlayerListWnd()->HandlePlayerStatusUpdate(status, about_player_id);
+
+    if (Client().GetApp()->GetClientType() == Networking::CLIENT_TYPE_HUMAN_MODERATOR &&
+        Client().GetClientUI()->GetMapWnd()->AutoEndTurnEnabled())
+    {
+        // check status of all non-mod non-obs players: are they all done their turns?
+        bool all_participants_waiting = true;
+        const std::map<int, Message::PlayerStatus>& player_status = Client().PlayerStatus();
+        const std::map<int, PlayerInfo>& players = Client().Players();
+        for (std::map<int, PlayerInfo>::const_iterator it = players.begin();
+                it != players.end(); ++it)
+        {
+            int player_id = it->first;
+
+            if (it->second.client_type != Networking::CLIENT_TYPE_AI_PLAYER &&
+                it->second.client_type != Networking::CLIENT_TYPE_HUMAN_PLAYER)
+            { continue; }   // only active participants matter...
+
+            std::map<int, Message::PlayerStatus>::const_iterator status_it = player_status.find(player_id);
+            if (status_it == player_status.end()) {
+                all_participants_waiting = false;
+                break;
+            }
+            if (status_it->second != Message::WAITING) {
+                all_participants_waiting = false;
+                break;
+            }
+        }
+
+        // if all participants waiting, can end turn immediately
+        if (all_participants_waiting)
+            post_event(AdvanceTurn());
+    }
+
+    return discard_event();
+}
 
 ////////////////////////////////////////////////////////////
 // ResolvingCombat
@@ -699,9 +753,6 @@ ResolvingCombat::ResolvingCombat(my_context ctx) :
 
 ResolvingCombat::~ResolvingCombat() {
     if (TRACE_EXECUTION) Logger().debugStream() << "(HumanClientFSM) ~ResolvingCombat";
-    Client().GetClientUI()->GetMapWnd()->Show();
-    FreeCombatData(m_previous_combat_data.get());
-    FreeCombatData(m_combat_data.get());
 }
 
 boost::statechart::result ResolvingCombat::react(const CombatStart& msg) {
@@ -719,7 +770,6 @@ boost::statechart::result ResolvingCombat::react(const CombatStart& msg) {
 boost::statechart::result ResolvingCombat::react(const CombatRoundUpdate& msg) {
     if (TRACE_EXECUTION) Logger().debugStream() << "(HumanClientFSM) ResolvingCombat.CombatRoundUpdate";
     if (m_previous_combat_data.get()) {
-        FreeCombatData(m_previous_combat_data.get());
         m_previous_combat_data.release();
     }
     m_previous_combat_data = m_combat_data;
@@ -731,5 +781,6 @@ boost::statechart::result ResolvingCombat::react(const CombatRoundUpdate& msg) {
 
 boost::statechart::result ResolvingCombat::react(const CombatEnd& msg) {
     if (TRACE_EXECUTION) Logger().debugStream() << "(HumanClientFSM) ResolvingCombat.CombatEnd";
+    Client().GetClientUI()->GetMapWnd()->Show();
     return transit<WaitingForTurnData>();
 }
