@@ -8,6 +8,7 @@
 #include "../util/Logger.h"
 #include "../util/MultiplayerCommon.h"
 #include "../util/ModeratorAction.h"
+#include "../util/SaveGamePreviewUtils.h"
 #include "../universe/Meter.h"
 #include "../universe/System.h"
 #include "../universe/Universe.h"
@@ -35,88 +36,16 @@
 
 
 namespace {
-    const std::string MESSAGE_SCOPE_PREFIX = "Message::";
     const std::string DUMMY_EMPTY_MESSAGE = "Lathanda";
     const std::string ACKNOWLEDGEMENT = "ACK";
-    std::string StripMessageScoping(const std::string& str) {
-        return boost::algorithm::starts_with(str, MESSAGE_SCOPE_PREFIX) ?
-            boost::algorithm::erase_first_copy(str, MESSAGE_SCOPE_PREFIX) :
-            str;
-    }
 }
 
 ////////////////////////////////////////////////
 // Free Functions
 ////////////////////////////////////////////////
-namespace GG {
-    GG_ENUM_MAP_BEGIN(Message::MessageType)
-    GG_ENUM_MAP_INSERT(Message::UNDEFINED)
-    GG_ENUM_MAP_INSERT(Message::DEBUG)
-    GG_ENUM_MAP_INSERT(Message::HOST_SP_GAME)
-    GG_ENUM_MAP_INSERT(Message::HOST_MP_GAME)
-    GG_ENUM_MAP_INSERT(Message::JOIN_GAME)
-    GG_ENUM_MAP_INSERT(Message::HOST_ID)
-    GG_ENUM_MAP_INSERT(Message::LOBBY_UPDATE)
-    GG_ENUM_MAP_INSERT(Message::LOBBY_CHAT)
-    GG_ENUM_MAP_INSERT(Message::LOBBY_EXIT)
-    GG_ENUM_MAP_INSERT(Message::START_MP_GAME)
-    GG_ENUM_MAP_INSERT(Message::SAVE_GAME)
-    GG_ENUM_MAP_INSERT(Message::LOAD_GAME)
-    GG_ENUM_MAP_INSERT(Message::GAME_START)
-    GG_ENUM_MAP_INSERT(Message::TURN_UPDATE)
-    GG_ENUM_MAP_INSERT(Message::TURN_PARTIAL_UPDATE)
-    GG_ENUM_MAP_INSERT(Message::TURN_ORDERS)
-    GG_ENUM_MAP_INSERT(Message::TURN_PROGRESS)
-    GG_ENUM_MAP_INSERT(Message::PLAYER_STATUS)
-    GG_ENUM_MAP_INSERT(Message::CLIENT_SAVE_DATA)
-    GG_ENUM_MAP_INSERT(Message::COMBAT_START)
-    GG_ENUM_MAP_INSERT(Message::COMBAT_TURN_UPDATE)
-    GG_ENUM_MAP_INSERT(Message::COMBAT_TURN_ORDERS)
-    GG_ENUM_MAP_INSERT(Message::COMBAT_END)
-    GG_ENUM_MAP_INSERT(Message::PLAYER_CHAT)
-    GG_ENUM_MAP_INSERT(Message::DIPLOMACY)
-    GG_ENUM_MAP_INSERT(Message::DIPLOMATIC_STATUS)
-    GG_ENUM_MAP_INSERT(Message::PLAYER_ELIMINATED)
-    GG_ENUM_MAP_INSERT(Message::REQUEST_NEW_OBJECT_ID)
-    GG_ENUM_MAP_INSERT(Message::DISPATCH_NEW_OBJECT_ID)
-    GG_ENUM_MAP_INSERT(Message::REQUEST_NEW_DESIGN_ID)
-    GG_ENUM_MAP_INSERT(Message::DISPATCH_NEW_DESIGN_ID)
-    GG_ENUM_MAP_INSERT(Message::VICTORY_DEFEAT)
-    GG_ENUM_MAP_INSERT(Message::END_GAME)
-    GG_ENUM_MAP_INSERT(Message::MODERATOR_ACTION)
-    GG_ENUM_MAP_INSERT(Message::SHUT_DOWN_SERVER)
-    GG_ENUM_MAP_END
-}
-
-std::string MessageTypeStr(Message::MessageType type)
-{ return StripMessageScoping(GG::GetEnumMap<Message::MessageType>().FromEnum(type)); }
-
-namespace GG {
-    GG_ENUM_MAP_BEGIN(Message::TurnProgressPhase)
-    GG_ENUM_MAP_INSERT(Message::FLEET_MOVEMENT)
-    GG_ENUM_MAP_INSERT(Message::COMBAT)
-    GG_ENUM_MAP_INSERT(Message::EMPIRE_PRODUCTION)
-    GG_ENUM_MAP_INSERT(Message::WAITING_FOR_PLAYERS)
-    GG_ENUM_MAP_INSERT(Message::PROCESSING_ORDERS)
-    GG_ENUM_MAP_INSERT(Message::COLONIZE_AND_SCRAP)
-    GG_ENUM_MAP_INSERT(Message::DOWNLOADING)
-    GG_ENUM_MAP_INSERT(Message::LOADING_GAME)
-    GG_ENUM_MAP_INSERT(Message::GENERATING_UNIVERSE)
-    GG_ENUM_MAP_INSERT(Message::STARTING_AIS)
-    GG_ENUM_MAP_END
-}
-
-namespace GG {
-    GG_ENUM_MAP_BEGIN(Message::PlayerStatus)
-    GG_ENUM_MAP_INSERT(Message::PLAYING_TURN)
-    GG_ENUM_MAP_INSERT(Message::RESOLVING_COMBAT)
-    GG_ENUM_MAP_INSERT(Message::WAITING)
-    GG_ENUM_MAP_END
-}
-
 std::ostream& operator<<(std::ostream& os, const Message& msg) {
     os << "Message: "
-       << MessageTypeStr(msg.Type()) << " "
+       << msg.Type() << " "
        << msg.SendingPlayer();
 
     if (msg.SendingPlayer() == Networking::INVALID_PLAYER_ID)
@@ -216,7 +145,7 @@ void BufferToHeader(const int* header_buf, Message& message) {
     message.m_type = static_cast<Message::MessageType>(header_buf[0]);
     message.m_sending_player = header_buf[1];
     message.m_receiving_player = header_buf[2];
-    message.m_synchronous_response = header_buf[3];
+    message.m_synchronous_response = (header_buf[3] != 0);
     message.m_message_size = header_buf[4];
 }
 
@@ -282,7 +211,8 @@ Message GameStartMessage(int player_id, bool single_player_game, int empire_id,
                          int current_turn, const EmpireManager& empires,
                          const Universe& universe, const SpeciesManager& species,
                          const CombatLogManager& combat_logs,
-                         const std::map<int, PlayerInfo>& players)
+                         const std::map<int, PlayerInfo>& players,
+                         const GalaxySetupData& galaxy_setup_data)
 {
     std::ostringstream os;
     {
@@ -298,6 +228,7 @@ Message GameStartMessage(int player_id, bool single_player_game, int empire_id,
         bool loaded_game_data = false;
         oa << BOOST_SERIALIZATION_NVP(players)
            << BOOST_SERIALIZATION_NVP(loaded_game_data);
+        oa << BOOST_SERIALIZATION_NVP(galaxy_setup_data);
     }
     return Message(Message::GAME_START, Networking::INVALID_PLAYER_ID, player_id, os.str());
 }
@@ -307,7 +238,8 @@ Message GameStartMessage(int player_id, bool single_player_game, int empire_id,
                          const Universe& universe, const SpeciesManager& species,
                          const CombatLogManager& combat_logs,
                          const std::map<int, PlayerInfo>& players,
-                         const OrderSet& orders, const SaveGameUIData* ui_data)
+                         const OrderSet& orders, const SaveGameUIData* ui_data,
+                         const GalaxySetupData& galaxy_setup_data)
 {
     std::ostringstream os;
     {
@@ -324,12 +256,13 @@ Message GameStartMessage(int player_id, bool single_player_game, int empire_id,
         oa << BOOST_SERIALIZATION_NVP(players)
            << BOOST_SERIALIZATION_NVP(loaded_game_data);
         Serialize(oa, orders);
-        bool ui_data_available = ui_data;
+        bool ui_data_available = (ui_data != 0);
         oa << BOOST_SERIALIZATION_NVP(ui_data_available);
         if (ui_data_available)
             oa << boost::serialization::make_nvp("ui_data", *ui_data);
         bool save_state_string_available = false;
         oa << BOOST_SERIALIZATION_NVP(save_state_string_available);
+        oa << BOOST_SERIALIZATION_NVP(galaxy_setup_data);
     }
     return Message(Message::GAME_START, Networking::INVALID_PLAYER_ID, player_id, os.str());
 }
@@ -339,7 +272,8 @@ Message GameStartMessage(int player_id, bool single_player_game, int empire_id,
                          const Universe& universe, const SpeciesManager& species,
                          const CombatLogManager& combat_logs,
                          const std::map<int, PlayerInfo>& players,
-                         const OrderSet& orders, const std::string* save_state_string)
+                         const OrderSet& orders, const std::string* save_state_string,
+                         const GalaxySetupData& galaxy_setup_data)
 {
     std::ostringstream os;
     {
@@ -358,10 +292,11 @@ Message GameStartMessage(int player_id, bool single_player_game, int empire_id,
         Serialize(oa, orders);
         bool ui_data_available = false;
         oa << BOOST_SERIALIZATION_NVP(ui_data_available);
-        bool save_state_string_available = save_state_string;   // save_state_string pointer may be 0
+        bool save_state_string_available = (save_state_string != 0);
         oa << BOOST_SERIALIZATION_NVP(save_state_string_available);
         if (save_state_string_available)
             oa << boost::serialization::make_nvp("save_state_string", *save_state_string);
+        oa << BOOST_SERIALIZATION_NVP(galaxy_setup_data);
     }
     return Message(Message::GAME_START, Networking::INVALID_PLAYER_ID, player_id, os.str());
 }
@@ -569,6 +504,19 @@ Message ModeratorActionMessage(int sender, const Moderator::ModeratorAction& act
 Message ShutdownServerMessage(int sender)
 { return Message(Message::SHUT_DOWN_SERVER, sender, Networking::INVALID_PLAYER_ID, DUMMY_EMPTY_MESSAGE); }
 
+/** requests previews of savefiles from server synchronously */
+Message RequestSavePreviewsMessage(int sender, std::string directory)
+{ return Message(Message::REQUEST_SAVE_PREVIEWS, sender, Networking::INVALID_PLAYER_ID, directory); }
+
+/** returns the savegame previews to the client */
+Message DispatchSavePreviewsMessage(int receiver, const PreviewInformation& previews) {
+    std::ostringstream os;
+    {
+        freeorion_oarchive oa(os);
+        oa << BOOST_SERIALIZATION_NVP(previews);
+    }
+    return Message(Message::DISPATCH_SAVE_PREVIEWS, Networking::INVALID_PLAYER_ID, receiver, os.str(), true);
+}
 
 ////////////////////////////////////////////////
 // Multiplayer Lobby Message named ctors
@@ -676,7 +624,7 @@ void ExtractMessageData(const Message& msg, bool& single_player_game, int& empir
                         std::map<int, PlayerInfo>& players, OrderSet& orders,
                         bool& loaded_game_data, bool& ui_data_available,
                         SaveGameUIData& ui_data, bool& save_state_string_available,
-                        std::string& save_state_string)
+                        std::string& save_state_string, GalaxySetupData& galaxy_setup_data)
 {
     try {
         std::istringstream is(msg.Text());
@@ -712,6 +660,8 @@ void ExtractMessageData(const Message& msg, bool& single_player_game, int& empir
             ui_data_available = false;
             save_state_string_available = false;
         }
+        ia >> BOOST_SERIALIZATION_NVP(galaxy_setup_data);
+
     } catch (const std::exception& err) {
         Logger().errorStream() << "ExtractMessageData(const Message& msg, bool& single_player_game, int& empire_id, "
                                << "int& current_turn, EmpireManager& empires, Universe& universe, "
@@ -1013,8 +963,8 @@ void ExtractMessageData(const Message& msg, CombatData& combat_data) {
     }
 }
 
-void ExtractMessageData(const Message& msg, System*& system,
-                        std::map<int, UniverseObject*>& combat_universe)
+void ExtractMessageData(const Message& msg, TemporaryPtr<System> system,
+                        std::map<int, TemporaryPtr<UniverseObject> >& combat_universe)
 {
     try {
         std::istringstream is(msg.Text());
@@ -1030,3 +980,22 @@ void ExtractMessageData(const Message& msg, System*& system,
         throw err;
     }
 }
+
+void ExtractMessageData(const Message& msg, std::string& directory) {
+    directory = msg.Text();
+}
+
+void ExtractMessageData(const Message& msg, PreviewInformation& previews) {
+    try{
+        std::istringstream is(msg.Text());
+        freeorion_iarchive ia(is);
+        ia >> BOOST_SERIALIZATION_NVP(previews);
+    }catch(const std::exception& err){
+        Logger().errorStream() << "ExtractMessageData(const Message& msg, PreviewInformation& previews"
+        << ") failed!  "
+        << "Message:\n"
+        << msg.Text() << "\n"
+        << "Error: " << err.what();
+        throw err;
+    }
+}
\ No newline at end of file
