@@ -4,6 +4,7 @@
 
 #include "Networking.h"
 #include "../util/Export.h"
+#include <GG/Enum.h>
 
 #include <boost/shared_array.hpp>
 
@@ -26,6 +27,8 @@ struct MultiplayerLobbyData;
 class OrderSet;
 struct PlayerInfo;
 struct SaveGameUIData;
+struct PreviewInformation;
+struct GalaxySetupData;
 struct SinglePlayerSetupData;
 class ShipDesign;
 class System;
@@ -54,7 +57,7 @@ FO_COMMON_API void HeaderToBuffer(const Message& message, int* header_buf);
 class FO_COMMON_API Message {
 public:
     /** Represents the type of the message */
-    enum MessageType {
+    GG_CLASS_ENUM(MessageType,
         UNDEFINED = 0,
         DEBUG,                  ///< used to send special messages used for debugging purposes
         ERROR_MSG,              ///< used to communicate errors between server and clients
@@ -90,10 +93,12 @@ public:
         PLAYER_ELIMINATED,      ///< sent by server to all clients (except the eliminated player) when a player is eliminated
         END_GAME,               ///< sent by the server when the current game is to ending (see EndGameReason for the possible reasons this message is sent out)
         MODERATOR_ACTION,       ///< sent by client to server when a moderator edits the universe
-        SHUT_DOWN_SERVER        ///< sent by host client to server to kill the server process
-    };
+        SHUT_DOWN_SERVER,       ///< sent by host client to server to kill the server process
+        REQUEST_SAVE_PREVIEWS,  ///< sent by client to request previews of available savegames
+        DISPATCH_SAVE_PREVIEWS  ///< sent by host to client to provide the savegame previews
+    )
 
-    enum TurnProgressPhase {
+    GG_CLASS_ENUM(TurnProgressPhase,
         FLEET_MOVEMENT,         ///< fleet movement turn progress message
         COMBAT,                 ///< combat turn progress message
         EMPIRE_PRODUCTION,      ///< empire production turn progress message
@@ -104,24 +109,24 @@ public:
         LOADING_GAME,           ///< loading gamestate from save
         GENERATING_UNIVERSE,    ///< creating new universe
         STARTING_AIS            ///< creating AI clients
-    };
+    )
 
-    enum PlayerStatus {
+    GG_CLASS_ENUM(PlayerStatus,
         PLAYING_TURN,           ///< player is playing a turn, on the galax map
         RESOLVING_COMBAT,       ///< player is resolving a combat interactively
         WAITING                 ///< player is waiting for others to submit orders, to resolve combats, or for turn processing to complete
-    };
+    )
 
-    enum EndGameReason {
+    GG_CLASS_ENUM(EndGameReason,
         LOCAL_CLIENT_DISCONNECT,///< the local player's client networking detected a disconnection from the server
         PLAYER_DISCONNECT,      ///< an active player (not an observer) was disconnected
         YOU_ARE_ELIMINATED      ///< the receiving player is eliminated from the game
-    };
+    )
 
-    enum VictoryOrDefeat {
+    GG_CLASS_ENUM(VictoryOrDefeat,
         VICTORY,                ///< a player or players have met a victory condition
         DEFEAT                  ///< a player or players have met a defeat condition
-    };
+    )
 
     /** \name Structors */ //@{
     Message(); ///< Default ctor.
@@ -172,9 +177,6 @@ FO_COMMON_API void swap(Message& lhs, Message& rhs); ///< Swaps the contents of
 // Message stringification
 ////////////////////////////////////////////////
 
-/** Returns a string representation of \a type. */
-FO_COMMON_API std::string MessageTypeStr(Message::MessageType type);
-
 /** Writes \a msg to \a os.  The format of the output is designed for debugging purposes. */
 FO_COMMON_API std::ostream& operator<<(std::ostream& os, const Message& msg);
 
@@ -201,25 +203,28 @@ FO_COMMON_API Message HostIDMessage(int host_player_id);
 
 /** creates a GAME_START message.  Contains the initial game state visible to player \a player_id.*/
 FO_COMMON_API Message GameStartMessage(int player_id, bool single_player_game, int empire_id, int current_turn,
-                         const EmpireManager& empires, const Universe& universe,
-                         const SpeciesManager& species, const CombatLogManager& combat_logs,
-                         const std::map<int, PlayerInfo>& players);
+                                       const EmpireManager& empires, const Universe& universe,
+                                       const SpeciesManager& species, const CombatLogManager& combat_logs,
+                                       const std::map<int, PlayerInfo>& players,
+                                       const GalaxySetupData& galaxy_setup_data);
 
 /** creates a GAME_START message.  Contains the initial game state visible to
   * player \a player_id.  Also includes data loaded from a saved game. */
 FO_COMMON_API Message GameStartMessage(int player_id, bool single_player_game, int empire_id, int current_turn,
-                         const EmpireManager& empires, const Universe& universe,
-                         const SpeciesManager& species, const CombatLogManager& combat_logs,
-                         const std::map<int, PlayerInfo>& players, const OrderSet& orders,
-                         const SaveGameUIData* ui_data);
+                                       const EmpireManager& empires, const Universe& universe,
+                                       const SpeciesManager& species, const CombatLogManager& combat_logs,
+                                       const std::map<int, PlayerInfo>& players, const OrderSet& orders,
+                                       const SaveGameUIData* ui_data,
+                                       const GalaxySetupData& galaxy_setup_data);
 
 /** creates a GAME_START message.  Contains the initial game state visible to
   * player \a player_id.  Also includes state string loaded from a saved game. */
 FO_COMMON_API Message GameStartMessage(int player_id, bool single_player_game, int empire_id, int current_turn,
-                         const EmpireManager& empires, const Universe& universe,
-                         const SpeciesManager& species, const CombatLogManager& combat_logs,
-                         const std::map<int, PlayerInfo>& players, const OrderSet& orders,
-                         const std::string* save_state_string);
+                                       const EmpireManager& empires, const Universe& universe,
+                                       const SpeciesManager& species, const CombatLogManager& combat_logs,
+                                       const std::map<int, PlayerInfo>& players, const OrderSet& orders,
+                                       const std::string* save_state_string,
+                                       const GalaxySetupData& galaxy_setup_data);
 
 /** creates a HOST_SP_GAME acknowledgement message.  The \a player_id is the ID
   * of the receiving player.  This message should only be sent by the server.*/
@@ -243,9 +248,11 @@ FO_COMMON_API Message TurnProgressMessage(Message::TurnProgressPhase phase_id, i
 FO_COMMON_API Message PlayerStatusMessage(int player_id, int about_player_id, Message::PlayerStatus player_status);
 
 /** creates a TURN_UPDATE message. */
-FO_COMMON_API Message TurnUpdateMessage(int player_id, int empire_id, int current_turn, const EmpireManager& empires,
-                          const Universe& universe, const SpeciesManager& species,
-                          const CombatLogManager& combat_logs, const std::map<int, PlayerInfo>& players);
+FO_COMMON_API Message TurnUpdateMessage(int player_id, int empire_id, int current_turn,
+                                        const EmpireManager& empires, const Universe& universe,
+                                        const SpeciesManager& species,
+                                        const CombatLogManager& combat_logs,
+                                        const std::map<int, PlayerInfo>& players);
 
 /** create a TURN_PARTIAL_UPDATE message. */
 FO_COMMON_API Message TurnPartialUpdateMessage(int player_id, int empire_id, const Universe& universe);
@@ -325,6 +332,12 @@ FO_COMMON_API Message ModeratorActionMessage(int sender, const Moderator::Modera
 /** tells server to shut down. */
 FO_COMMON_API Message ShutdownServerMessage(int sender);
 
+/** requests previews of savefiles from server */
+FO_COMMON_API Message RequestSavePreviewsMessage(int sender, std::string directory);
+
+/** returns the savegame previews to the client */
+FO_COMMON_API Message DispatchSavePreviewsMessage(int receiver, const PreviewInformation& preview);
+
 ////////////////////////////////////////////////
 // Multiplayer Lobby Message named ctors
 ////////////////////////////////////////////////
@@ -373,12 +386,13 @@ FO_COMMON_API void ExtractMessageData(const Message& msg, std::string& problem,
 
 FO_COMMON_API void ExtractMessageData(const Message& msg, MultiplayerLobbyData& lobby_data);
 
-FO_COMMON_API void ExtractMessageData(const Message& msg, bool& single_player_game, int& empire_id, int& current_turn,
-                        EmpireManager& empires, Universe& universe, SpeciesManager& species,
-                        CombatLogManager& combat_logs, std::map<int, PlayerInfo>& players,
-                        OrderSet& orders, bool& loaded_game_data, bool& ui_data_available,
-                        SaveGameUIData& ui_data, bool& save_state_string_available,
-                        std::string& save_state_string);
+FO_COMMON_API void ExtractMessageData(const Message& msg, bool& single_player_game, int& empire_id,
+                                      int& current_turn, EmpireManager& empires, Universe& universe,
+                                      SpeciesManager& species, CombatLogManager& combat_logs,
+                                      std::map<int, PlayerInfo>& players, OrderSet& orders,
+                                      bool& loaded_game_data, bool& ui_data_available,
+                                      SaveGameUIData& ui_data, bool& save_state_string_available,
+                                      std::string& save_state_string, GalaxySetupData& galaxy_setup_data);
 
 FO_COMMON_API void ExtractMessageData(const Message& msg, std::string& player_name, Networking::ClientType& client_type);
 
@@ -424,4 +438,8 @@ FO_COMMON_API void ExtractMessageData(const Message& msg, CombatData& combat_dat
 void ExtractMessageData(const Message& msg, System*& system,
                         std::map<int, UniverseObject*>& combat_universe);
 
+FO_COMMON_API void ExtractMessageData(const Message& msg, std::string& directory);
+
+FO_COMMON_API void ExtractMessageData(const Message& msg, PreviewInformation& previews);
+
 #endif // _Message_h_
