@@ -97,6 +97,7 @@ namespace parse {
             |    tok.PopulationCenter_ [ _val = OBJ_POP_CENTER ]
             |    tok.ProductionCenter_ [ _val = OBJ_PROD_CENTER ]
             |    tok.System_ [ _val = OBJ_SYSTEM ]
+            |    tok.Field_ [ _val = OBJ_FIELD ]
             ;
         static bool once = true;
         if (once) {
@@ -204,6 +205,7 @@ namespace parse {
         static enum_parser_rule<ShipSlotType>::type retval
             =    tok.External_ [ _val = SL_EXTERNAL ]
             |    tok.Internal_ [ _val = SL_INTERNAL ]
+            |    tok.Core_     [ _val = SL_CORE ]
             ;
         static bool once = true;
         if (once) {
@@ -221,20 +223,25 @@ namespace parse {
     {
         const parse::lexer& tok = parse::lexer::instance();
         static enum_parser_rule<ShipPartClass>::type retval
-            =    tok.ShortRange_ [ _val = PC_SHORT_RANGE ]
-            |    tok.Missiles_ [ _val = PC_MISSILES ]
-            |    tok.Fighters_ [ _val = PC_FIGHTERS ]
-            |    tok.PointDefense_ [ _val = PC_POINT_DEFENSE ]
-            |    tok.Shield_ [ _val = PC_SHIELD ]
-            |    tok.Armour_ [ _val = PC_ARMOUR ]
-            |    tok.Troops_ [ _val = PC_TROOPS ]
-            |    tok.Detection_ [ _val = PC_DETECTION ]
-            |    tok.Stealth_ [ _val = PC_STEALTH ]
-            |    tok.Fuel_ [ _val = PC_FUEL ]
-            |    tok.Colony_ [ _val = PC_COLONY ]
-            |    tok.BattleSpeed_ [ _val = PC_BATTLE_SPEED ]
-            |    tok.StarlaneSpeed_ [ _val = PC_STARLANE_SPEED ]
-            |    tok.General_ [ _val = PC_GENERAL ]
+            =    tok.ShortRange_        [ _val = PC_SHORT_RANGE ]
+            |    tok.Missiles_          [ _val = PC_MISSILES ]
+            |    tok.Fighters_          [ _val = PC_FIGHTERS ]
+            |    tok.PointDefense_      [ _val = PC_POINT_DEFENSE ]
+            |    tok.Shield_            [ _val = PC_SHIELD ]
+            |    tok.Armour_            [ _val = PC_ARMOUR ]
+            |    tok.Troops_            [ _val = PC_TROOPS ]
+            |    tok.Detection_         [ _val = PC_DETECTION ]
+            |    tok.Stealth_           [ _val = PC_STEALTH ]
+            |    tok.Fuel_              [ _val = PC_FUEL ]
+            |    tok.Colony_            [ _val = PC_COLONY ]
+            |    tok.BattleSpeed_       [ _val = PC_BATTLE_SPEED ]
+            |    tok.StarlaneSpeed_     [ _val = PC_STARLANE_SPEED ]
+            |    tok.General_           [ _val = PC_GENERAL ]
+            |    tok.Bombard_           [ _val = PC_BOMBARD ]
+            |    tok.Research_          [ _val = PC_RESEARCH ]
+            |    tok.Industry_          [ _val = PC_INDUSTRY ]
+            |    tok.Trade_             [ _val = PC_TRADE ]
+            |    tok.ProductionLocation_[ _val = PC_PRODICTION_LOCATION ]
             ;
         static bool once = true;
         if (once) {
@@ -330,6 +337,7 @@ namespace parse {
             |    tok.MaxShield_             [ _val = METER_MAX_SHIELD ]
             |    tok.MaxStructure_          [ _val = METER_MAX_STRUCTURE ]
             |    tok.MaxTroops_             [ _val = METER_MAX_TROOPS ]
+            |    tok.MaxSupply_             [ _val = METER_MAX_SUPPLY ]
 
             |    tok.Construction_          [ _val = METER_CONSTRUCTION ]
             |    tok.Industry_              [ _val = METER_INDUSTRY ]
@@ -343,9 +351,9 @@ namespace parse {
             |    tok.Shield_                [ _val = METER_SHIELD ]
             |    tok.Structure_             [ _val = METER_STRUCTURE ]
             |    tok.Troops_                [ _val = METER_TROOPS ]
+            |    tok.Supply_                [ _val = METER_SUPPLY ]
 
             |    tok.RebelTroops_           [ _val = METER_REBEL_TROOPS ]
-            |    tok.Supply_                [ _val = METER_SUPPLY ]
             |    tok.Stealth_               [ _val = METER_STEALTH ]
             |    tok.Detection_             [ _val = METER_DETECTION ]
             |    tok.BattleSpeed_           [ _val = METER_BATTLE_SPEED ]
@@ -406,6 +414,7 @@ namespace parse {
             |    tok.SetMaxShield_          [ _val = METER_MAX_SHIELD ]
             |    tok.SetMaxStructure_       [ _val = METER_MAX_STRUCTURE ]
             |    tok.SetMaxTroops_          [ _val = METER_MAX_TROOPS ]
+            |    tok.SetMaxSupply_          [ _val = METER_MAX_SUPPLY ]
 
             |    tok.SetConstruction_       [ _val = METER_CONSTRUCTION ]
             |    tok.SetIndustry_           [ _val = METER_INDUSTRY ]
@@ -419,9 +428,9 @@ namespace parse {
             |    tok.SetShield_             [ _val = METER_SHIELD ]
             |    tok.SetStructure_          [ _val = METER_STRUCTURE ]
             |    tok.SetTroops_             [ _val = METER_TROOPS ]
+            |    tok.SetSupply_             [ _val = METER_SUPPLY ]
 
             |    tok.SetRebelTroops_        [ _val = METER_REBEL_TROOPS ]
-            |    tok.SetSupply_             [ _val = METER_SUPPLY ]
             |    tok.SetStealth_            [ _val = METER_STEALTH ]
             |    tok.SetDetection_          [ _val = METER_DETECTION ]
             |    tok.SetBattleSpeed_        [ _val = METER_BATTLE_SPEED ]
