@@ -4,8 +4,9 @@
 #include "../universe/Predicates.h"
 #include "../universe/UniverseObject.h"
 #include "../universe/Planet.h"
-
 #include "../universe/Tech.h"
+#include "../util/SitRepEntry.h"
+#include "../util/AppInterface.h"
 
 #include <GG/Clr.h>
 
@@ -50,8 +51,8 @@ namespace {
     const std::string&  NameFromProductionQueueElement(const ProductionQueue::Element& element)         { return element.item.name; }
     int                 DesignIDFromProductionQueueElement(const ProductionQueue::Element& element)     { return element.item.design_id; }
 
-    bool                (Empire::*BuildableItemBuilding)(BuildType, const std::string&, int) const =    &Empire::BuildableItem;
-    bool                (Empire::*BuildableItemShip)(BuildType, int, int) const =                       &Empire::BuildableItem;
+    bool                (Empire::*BuildableItemBuilding)(BuildType, const std::string&, int) const =    &Empire::ProducibleItem;
+    bool                (Empire::*BuildableItemShip)(BuildType, int, int) const =                       &Empire::ProducibleItem;
 
     const ProductionQueue::Element&
                             (ProductionQueue::*ProductionQueueOperatorSquareBrackets)(int) const =      &ProductionQueue::operator[];
@@ -94,33 +95,139 @@ namespace {
         return retval;
     }
     boost::function<std::vector<IntPair>(const Empire&)> obstructedStarlanesFunc =      &obstructedStarlanesP;
+    
+void CalculateSupplyUpdate(const std::map<int, std::set<int> >& starlanes,
+                    const std::map<int, int>&           supply_system_ranges,
+                    const std::set<int>&                supply_unobstructed_systems,
+                    std::map<int, int>&                 propegating_supply_ranges,
+                    int                                 min_tracked_supply = 0,
+                    bool                                obstructed = true ) // Note: must be called with min_tracked_supply = 0 to give the standard result
+{
 
-    typedef std::pair<double, int> DoubleIntPair;
+    // store supply range in jumps of all unobstructed systems before
+    // propegation, and add to list of systems to propegate from.
+    std::list<int> propegating_systems_list;
+    for (std::map<int, int>::const_iterator it = supply_system_ranges.begin();
+         it != supply_system_ranges.end(); ++it)
+    {
+        if (!obstructed || (supply_unobstructed_systems.find(it->first) != supply_unobstructed_systems.end()))
+            propegating_supply_ranges.insert(*it);
+        else
+            propegating_supply_ranges[it->first] = min_tracked_supply;
 
-    typedef PairToTupleConverter<double, int> DoubleIntPairConverter;
+        // add system to list of systems to popegate supply from
+        propegating_systems_list.push_back(it->first);
+    }
 
-    //boost::mpl::vector<DoubleIntPair, const Empire&, const ProductionQueue::Element&>()
-    DoubleIntPair ProductionCostAndTimeP(const Empire& empire, const ProductionQueue::Element& element)
+    // iterate through list of accessible systems, processing each in order it
+    // was added (like breadth first search) until no systems are left able to
+    // further propregate
+    std::list<int>::iterator sys_list_it = propegating_systems_list.begin();
+    std::list<int>::iterator sys_list_end = propegating_systems_list.end();
+    while (sys_list_it != sys_list_end) {
+        int cur_sys_id = *sys_list_it;
+        int cur_sys_range = propegating_supply_ranges[cur_sys_id];    // range away from this system that supplies can be transported
+
+        if (cur_sys_range <= min_tracked_supply) {
+            // can't propegate supply out a system that has no range if min_tracked_supply is zero;
+            // if min_tracked_supply is negative, then may be able to continue propagating. A negative supply 
+            // number indicates number of jumps to nearest supply
+            ++sys_list_it;
+            continue;
+        }
+
+        // can propegate further, if adjacent systems have smaller supply range
+        // than one less than this system's range
+        std::map<int, std::set<int> >::const_iterator system_it = starlanes.find(cur_sys_id);
+        if (system_it == starlanes.end()) {
+            // no starlanes out of this system
+            ++sys_list_it;
+            continue;
+        }
+
+        const std::set<int>& starlane_ends = system_it->second;
+        for (std::set<int>::const_iterator lane_it = starlane_ends.begin();
+             lane_it != starlane_ends.end(); ++lane_it)
+        {
+            int lane_end_sys_id = *lane_it;
+
+            if (obstructed && (supply_unobstructed_systems.find(lane_end_sys_id) == supply_unobstructed_systems.end())) {
+                // can't propegate here
+                continue;
+            }
+
+            // compare next system's supply range to this system's supply range.  propegate if necessary.
+            std::map<int, int>::const_iterator lane_end_sys_it = propegating_supply_ranges.find(lane_end_sys_id);
+            if (lane_end_sys_it == propegating_supply_ranges.end() || lane_end_sys_it->second <= cur_sys_range) {
+                // next system has no supply yet, or its range equal to or smaller than this system's
+
+                // update next system's range, if propegating from this system would make it larger
+                if (lane_end_sys_it == propegating_supply_ranges.end() || lane_end_sys_it->second < cur_sys_range - 1) {
+                    // update with new range
+                    propegating_supply_ranges[lane_end_sys_id] = cur_sys_range - 1;
+                    // add next system to list of systems to propegate further
+                    propegating_systems_list.push_back(lane_end_sys_id);
+                }
+            }
+        }
+        ++sys_list_it;
+        sys_list_end = propegating_systems_list.end();
+    }
+}
+    
+
+    std::map<int,int> supplyProjectionsP(const Empire& empire, int min_tracked_supply, bool obstructed) {
+        std::map< int, std::set< int > >    starlanes = empire.KnownStarlanes();
+        const std::map<int, int>&           supply_system_ranges = empire.SystemSupplyRanges();
+        std::map<int, int>                  propegating_supply_ranges;
+
+        if (obstructed) {
+            const std::set<int>&                supply_unobstructed_systems = empire.SupplyUnobstructedSystems();
+            CalculateSupplyUpdate(              starlanes, 
+                                                supply_system_ranges, 
+                                                supply_unobstructed_systems,
+                                                propegating_supply_ranges,
+                                                min_tracked_supply,
+                                                true ); // Note: must be called with min_tracked_supply = 0 to give the standard result
+        } else {
+            std::set<int> known_systems;
+            CalculateSupplyUpdate(              starlanes, 
+                                                supply_system_ranges, 
+                                                known_systems,
+                                                propegating_supply_ranges,
+                                                min_tracked_supply,
+                                                false); // Note: must be called with min_tracked_supply = 0 to give the standard result
+        }
+        return propegating_supply_ranges;
+    }
+    boost::function<std::map<int,int>(const Empire&, int min_tracked_supply, bool obstructed)> supplyProjectionsFunc =      &supplyProjectionsP;
+
+    typedef std::pair<float, int> FloatIntPair;
+
+    typedef PairToTupleConverter<float, int> FloatIntPairConverter;
+
+    //boost::mpl::vector<FloatIntPair, const Empire&, const ProductionQueue::Element&>()
+    FloatIntPair ProductionCostAndTimeP(const Empire& empire, const ProductionQueue::Element& element)
     { return empire.ProductionCostAndTime(element); }
-    boost::function<DoubleIntPair(const Empire&,const ProductionQueue::Element& element )> ProductionCostAndTimeFunc = &ProductionCostAndTimeP;
+    boost::function<FloatIntPair(const Empire&,const ProductionQueue::Element& element )> ProductionCostAndTimeFunc = &ProductionCostAndTimeP;
 
     //.def("availablePP",                     make_function(&ProductionQueue::AvailablePP,          return_value_policy<return_by_value>()))
     //.add_property("allocatedPP",            make_function(&ProductionQueue::AllocatedPP,          return_internal_reference<>()))
     //.def("objectsWithWastedPP",             make_function(&ProductionQueue::ObjectsWithWastedPP,  return_value_policy<return_by_value>()))
 
-    std::map<std::set<int>, double> PlanetsWithAvailablePP_P(const Empire& empire) {
+    std::map<std::set<int>, float> PlanetsWithAvailablePP_P(const Empire& empire) {
         const boost::shared_ptr<ResourcePool>& industry_pool = empire.GetResourcePool(RE_INDUSTRY);
         const ProductionQueue& prodQueue = empire.GetProductionQueue();
-        std::map<std::set<int>, double> planetsWithAvailablePP;
-        std::map<std::set<int>, double> objectsWithAvailablePP = prodQueue.AvailablePP(industry_pool);
-        for (std::map<std::set<int>, double>::iterator map_it = objectsWithAvailablePP.begin(); 
+        std::map<std::set<int>, float> planetsWithAvailablePP;
+        std::map<std::set<int>, float> objectsWithAvailablePP = prodQueue.AvailablePP(industry_pool);
+        for (std::map<std::set<int>, float>::iterator map_it = objectsWithAvailablePP.begin(); 
              map_it != objectsWithAvailablePP.end(); map_it++)
         {
             std::set<int> planetSet;
             std::set<int> objSet = map_it->first;
             for (std::set<int>::iterator obj_it = objSet.begin(); obj_it != objSet.end(); obj_it++) {
-                UniverseObject* location = GetUniverseObject(*obj_it);
-                if (/* const Planet* planet = */ universe_object_cast<const Planet*>(location))
+                TemporaryPtr<UniverseObject> location = GetUniverseObject(*obj_it);
+                if (/* TemporaryPtr<const Planet> planet = */ boost::dynamic_pointer_cast<const Planet>(location))
                     planetSet.insert(*obj_it);
             }
             if (!planetSet.empty())
@@ -128,20 +235,20 @@ namespace {
         }
         return planetsWithAvailablePP;
     }
-    boost::function<std::map<std::set<int>, double>(const Empire& )> PlanetsWithAvailablePP_Func = &PlanetsWithAvailablePP_P;
+    boost::function<std::map<std::set<int>, float>(const Empire& )> PlanetsWithAvailablePP_Func = &PlanetsWithAvailablePP_P;
 
-    std::map<std::set<int>, double> PlanetsWithAllocatedPP_P(const Empire& empire) {
+    std::map<std::set<int>, float> PlanetsWithAllocatedPP_P(const Empire& empire) {
         const ProductionQueue& prodQueue = empire.GetProductionQueue();
-        std::map<std::set<int>, double> planetsWithAllocatedPP;
-        std::map<std::set<int>, double> objectsWithAllocatedPP = prodQueue.AllocatedPP();
-        for (std::map<std::set<int>, double>::iterator map_it = objectsWithAllocatedPP.begin(); 
+        std::map<std::set<int>, float> planetsWithAllocatedPP;
+        std::map<std::set<int>, float> objectsWithAllocatedPP = prodQueue.AllocatedPP();
+        for (std::map<std::set<int>, float>::iterator map_it = objectsWithAllocatedPP.begin(); 
              map_it != objectsWithAllocatedPP.end(); map_it++)
              {
                  std::set<int> planetSet;
                  std::set<int> objSet = map_it->first;
                  for (std::set<int>::iterator obj_it = objSet.begin(); obj_it != objSet.end(); obj_it++) {
-                     UniverseObject* location = GetUniverseObject(*obj_it);
-                     if (/* const Planet* planet = */ universe_object_cast<const Planet*>(location))
+                     TemporaryPtr<UniverseObject> location = GetUniverseObject(*obj_it);
+                     if (/* TemporaryPtr<const Planet> planet = */ boost::dynamic_pointer_cast<const Planet>(location))
                          planetSet.insert(*obj_it);
                  }
                  if (!planetSet.empty())
@@ -149,7 +256,7 @@ namespace {
              }
              return planetsWithAllocatedPP;
     }
-    boost::function<std::map<std::set<int>, double>(const Empire& )> PlanetsWithAllocatedPP_Func = &PlanetsWithAllocatedPP_P;
+    boost::function<std::map<std::set<int>, float>(const Empire& )> PlanetsWithAllocatedPP_Func = &PlanetsWithAllocatedPP_P;
 
     std::set<std::set<int> > PlanetsWithWastedPP_P(const Empire& empire) {
         const boost::shared_ptr<ResourcePool>& industry_pool = empire.GetResourcePool(RE_INDUSTRY);
@@ -162,8 +269,8 @@ namespace {
                  std::set<int> planetSet;
                  std::set<int> objSet = *sets_it;
                  for (std::set<int>::iterator obj_it = objSet.begin(); obj_it != objSet.end(); obj_it++) {
-                     UniverseObject* location = GetUniverseObject(*obj_it);
-                     if (/* const Planet* planet = */ universe_object_cast<const Planet*>(location))
+                     TemporaryPtr<UniverseObject> location = GetUniverseObject(*obj_it);
+                     if (/* TemporaryPtr<const Planet> planet = */ boost::dynamic_pointer_cast<const Planet>(location))
                          planetSet.insert(*obj_it);
                  }
                  if (!planetSet.empty())
@@ -217,13 +324,13 @@ namespace FreeOrionPython {
         class_<std::vector<ItemSpec> >("ItemSpecVec")
             .def(boost::python::vector_indexing_suite<std::vector<ItemSpec>, true>())
         ;
-        boost::python::to_python_converter<DoubleIntPair, DoubleIntPairConverter>();
+        boost::python::to_python_converter<FloatIntPair, FloatIntPairConverter>();
 
         class_<ResourcePool, boost::shared_ptr<ResourcePool>, boost::noncopyable >("resPool", boost::python::no_init);
         //FreeOrionPython::SetWrapper<int>::Wrap("IntSet");
         FreeOrionPython::SetWrapper<IntSet>::Wrap("IntSetSet");
-        class_<std::map<std::set<int>, double> > ("resPoolMap")
-            .def(boost::python::map_indexing_suite< std::map<std::set<int>, double>, true>() )
+        class_<std::map<std::set<int>, float> > ("resPoolMap")
+            .def(boost::python::map_indexing_suite< std::map<std::set<int>, float>, true>() )
         ;
         
         ///////////////////
@@ -247,17 +354,17 @@ namespace FreeOrionPython {
             .def("productionCostAndTime",           make_function(
                                                         ProductionCostAndTimeFunc,      
                                                         return_value_policy<return_by_value>(),
-                                                        boost::mpl::vector<DoubleIntPair, const Empire&, const ProductionQueue::Element& >()
+                                                        boost::mpl::vector<FloatIntPair, const Empire&, const ProductionQueue::Element& >()
                                                     ))
             .add_property("planetsWithAvailablePP", make_function(
                                                         PlanetsWithAvailablePP_Func,
                                                         return_value_policy<return_by_value>(),
-                                                        boost::mpl::vector<std::map<std::set<int>, double>, const Empire& >()
+                                                        boost::mpl::vector<std::map<std::set<int>, float>, const Empire& >()
                                                     ))
             .add_property("planetsWithAllocatedPP", make_function(
                                                         PlanetsWithAllocatedPP_Func,
                                                         return_value_policy<return_by_value>(),
-                                                        boost::mpl::vector<std::map<std::set<int>, double>, const Empire& >()
+                                                        boost::mpl::vector<std::map<std::set<int>, float>, const Empire& >()
                                                     ))
             .add_property("planetsWithWastedPP",    make_function(
                                                         PlanetsWithWastedPP_Func,
@@ -288,6 +395,7 @@ namespace FreeOrionPython {
 
             .add_property("fleetSupplyableSystemIDs",   make_function(&Empire::FleetSupplyableSystemIDs,    return_internal_reference<>()))
             .add_property("supplyUnobstructedSystems",  make_function(&Empire::SupplyUnobstructedSystems,   return_internal_reference<>()))
+            .add_property("systemSupplyRanges",         make_function(&Empire::SystemSupplyRanges,          return_internal_reference<>()))
 
             .add_property("numSitReps",             &Empire::NumSitRepEntries)
             .def("getSitRep",                       make_function(
@@ -299,7 +407,11 @@ namespace FreeOrionPython {
             .def("obstructedStarlanes",             make_function(obstructedStarlanesFunc,
                                                     return_value_policy<return_by_value>(),
                                                     boost::mpl::vector<std::vector<IntPair>, const Empire&>()
-            ))
+                                                    ))
+            .def("supplyProjections",               make_function(supplyProjectionsFunc,
+                                                    return_value_policy<return_by_value>(),
+                                                    boost::mpl::vector<std::map<int, int>, const Empire&, int, bool>()
+                                                    ))
         ;
 
         ////////////////////
