@@ -12,6 +12,7 @@
 #include "../universe/Special.h"
 #include "../universe/Species.h"
 #include "../util/Logger.h"
+#include "../util/MultiplayerCommon.h"
 
 #include <boost/mpl/vector.hpp>
 #include <boost/python.hpp>
@@ -21,20 +22,23 @@ namespace {
     void                    DumpObjects(const Universe& universe)
     { Logger().debugStream() << universe.Objects().Dump(); }
 
+    // We're returning the result of operator-> here so that python doesn't
+    // need to deal with our TemporaryPtr class.
+    // Please don't use this trick elsewhere to grab a raw UniverseObject*!
     const UniverseObject*   GetUniverseObjectP(const Universe& universe, int id)
-    { return ::GetUniverseObject(id); }
-    const Fleet*            GetFleetP(const Universe& universe, int id)
-    { return ::GetFleet(id); }
+    { return ::GetUniverseObject(id).operator->(); }
     const Ship*             GetShipP(const Universe& universe, int id)
-    { return ::GetShip(id); }
+    { return ::GetShip(id).operator->(); }
+    const Fleet*            GetFleetP(const Universe& universe, int id)
+    { return ::GetFleet(id).operator->(); }
     const Planet*           GetPlanetP(const Universe& universe, int id)
-    { return ::GetPlanet(id); }
+    { return ::GetPlanet(id).operator->(); }
     const System*           GetSystemP(const Universe& universe, int id)
-    { return ::GetSystem(id); }
+    { return ::GetSystem(id).operator->(); }
     const Field*            GetFieldP(const Universe& universe, int id)
-    { return ::GetField(id);  }
+    { return ::GetField(id).operator->();  }
     const Building*         GetBuildingP(const Universe& universe, int id)
-    { return ::GetBuilding(id); }
+    { return ::GetBuilding(id).operator->(); }
 
     std::vector<int>        ObjectIDs(const Universe& universe)
     { return Objects().FindObjectIDs(); }
@@ -51,7 +55,7 @@ namespace {
     std::vector<int>        BuildingIDs(const Universe& universe)
     { return Objects().FindObjectIDs<Building>(); }
 
-    std::vector<std::string>    SpeciesFoci(const Species& species) {
+    std::vector<std::string>SpeciesFoci(const Species& species) {
         std::vector<std::string> retval;
         const std::vector<FocusType>& foci = species.Foci();
         for (std::vector<FocusType>::const_iterator f_it = foci.begin(); f_it != foci.end(); f_it++ )
@@ -199,6 +203,20 @@ namespace {
                                                        const std::vector<std::string>& parts) = &ShipDesign::ValidDesign;
     bool                    (*ValidDesignDesign)(const ShipDesign&) =                           &ShipDesign::ValidDesign;
 
+    std::vector<int>        DirectFireStatsP(const ShipDesign& ship_design) {
+        const std::vector<std::string>& partslist = ship_design.Parts();
+        std::vector<int> results;
+        for (std::vector<std::string>::const_iterator part_it = partslist.begin(); part_it!=partslist.end(); part_it++){
+            const PartType* part = GetPartType(*part_it);
+            if (part && part->Class() == PC_SHORT_RANGE) { // TODO: handle other weapon classes when they are implemented
+                const DirectFireStats& stats = boost::get<DirectFireStats>(part->Stats());
+                results.push_back(stats.m_damage);
+            }
+        }
+        return results;
+    }
+    boost::function<std::vector<int> (const ShipDesign&)> DirectFireStatsFunc =                 &DirectFireStatsP;
+
     const std::vector<std::string>& (ShipDesign::*PartsVoid)(void) const =                      &ShipDesign::Parts;
     std::vector<std::string>        (ShipDesign::*PartsSlotType)(ShipSlotType) const =          &ShipDesign::Parts;
 
@@ -371,6 +389,8 @@ namespace FreeOrionPython {
             .def("specialAddedOnTurn",          &UniverseObject::SpecialAddedOnTurn)
             .def("contains",                    &UniverseObject::Contains)
             .def("containedBy",                 &UniverseObject::ContainedBy)
+            .add_property("containedObjects",   make_function(&UniverseObject::ContainedObjectIDs,  return_value_policy<return_by_value>()))
+            .add_property("containerObject",    &UniverseObject::ContainerObjectID)
             .def("currentMeterValue",           &UniverseObject::CurrentMeterValue)
             .def("initialMeterValue",           &UniverseObject::InitialMeterValue)
             .def("nextTurnCurrentMeterValue",   &UniverseObject::NextTurnCurrentMeterValue)
@@ -448,6 +468,12 @@ namespace FreeOrionPython {
             .def("perTurnCost",                 &ShipDesign::PerTurnCost)
             .add_property("hull",               make_function(&ShipDesign::Hull,            return_value_policy<return_by_value>()))
             .add_property("parts",              make_function(PartsVoid,                    return_internal_reference<>()))
+            .add_property("directFireStats",    make_function(
+                                                    DirectFireStatsFunc,
+                                                    return_value_policy<return_by_value>(),
+                                                    boost::mpl::vector<std::vector<int>, const ShipDesign&>()
+                                                ))
+
             .def("partsInSlotType",             PartsSlotType,                              return_value_policy<return_by_value>())
             .def("productionLocationForEmpire", &ShipDesign::ProductionLocation)
         ;
@@ -493,7 +519,6 @@ namespace FreeOrionPython {
             .def("productionCost",              &BuildingType::ProductionCost)
             .def("productionTime",              &BuildingType::ProductionTime)
             .def("perTurnCost",                 &BuildingType::PerTurnCost)
-            .add_property("maintenanceCost",    &BuildingType::MaintenanceCost)
             .def("captureResult",               &BuildingType::GetCaptureResult)
             .def("canBeProduced",               &BuildingType::ProductionLocation)  //(int empire_id, int location_id)
         ;
@@ -502,8 +527,9 @@ namespace FreeOrionPython {
         // ResourceCenter //
         ////////////////////
         class_<ResourceCenter, noncopyable>("resourceCenter", no_init)
-            .add_property("focus",              make_function(&ResourceCenter::Focus,       return_value_policy<copy_const_reference>()))
-            .add_property("availableFoci",      &ResourceCenter::AvailableFoci)
+            .add_property("focus",                  make_function(&ResourceCenter::Focus,       return_value_policy<copy_const_reference>()))
+            .add_property("turnsSinceFocusChange" , &ResourceCenter::TurnsSinceFocusChange)
+            .add_property("availableFoci",          &ResourceCenter::AvailableFoci)
         ;
 
         ///////////////////
@@ -533,7 +559,7 @@ namespace FreeOrionPython {
             .def("OrbitalPositionOnTurn",                   &Planet::OrbitalPositionOnTurn)
             .add_property("RotationalPeriod",               &Planet::RotationalPeriod)
             .add_property("AxialTilt",                      &Planet::AxialTilt)
-            .add_property("buildingIDs",                    make_function(&Planet::Buildings,           return_internal_reference<>()))
+            .add_property("buildingIDs",                    make_function(&Planet::BuildingIDs,     return_internal_reference<>()))
         ;
 
         //////////////////
@@ -541,15 +567,16 @@ namespace FreeOrionPython {
         //////////////////
         class_<System, bases<UniverseObject>, noncopyable>("system", no_init)
             .add_property("starType",           &System::GetStarType)
-            .add_property("numOrbits",          &System::Orbits)
             .add_property("numStarlanes",       &System::NumStarlanes)
             .add_property("numWormholes",       &System::NumWormholes)
             .def("HasStarlaneToSystemID",       &System::HasStarlaneTo)
             .def("HasWormholeToSystemID",       &System::HasWormholeTo)
-            .add_property("starlanesWormholes", make_function(&System::StarlanesWormholes,              return_value_policy<return_by_value>()))
-            .add_property("allObjectIDs",       make_function(&System::FindObjectIDs<UniverseObject>,   return_value_policy<return_by_value>()))
-            .add_property("planetIDs",          make_function(&System::FindObjectIDs<Planet>,           return_value_policy<return_by_value>()))
-            .add_property("fleetIDs",           make_function(&System::FindObjectIDs<Fleet>,            return_value_policy<return_by_value>()))
+            .add_property("starlanesWormholes", make_function(&System::StarlanesWormholes,  return_value_policy<return_by_value>()))
+            .add_property("planetIDs",          make_function(&System::PlanetIDs,           return_value_policy<return_by_value>()))
+            .add_property("buildingIDs",        make_function(&System::BuildingIDs,         return_value_policy<return_by_value>()))
+            .add_property("fleetIDs",           make_function(&System::FleetIDs,            return_value_policy<return_by_value>()))
+            .add_property("shipIDs",            make_function(&System::ShipIDs,             return_value_policy<return_by_value>()))
+            .add_property("fieldIDs",           make_function(&System::FieldIDs,            return_value_policy<return_by_value>()))
             .add_property("lastTurnBattleHere", &System::LastTurnBattleHere)
         ;
 
@@ -589,6 +616,7 @@ namespace FreeOrionPython {
             .add_property("description",        make_function(&Species::Description,    return_value_policy<copy_const_reference>()))
             .add_property("homeworlds",         make_function(&Species::Homeworlds,     return_value_policy<copy_const_reference>()))
             .add_property("foci",               &SpeciesFoci)
+            .add_property("preferredFocus",     make_function(&Species::PreferredFocus, return_value_policy<copy_const_reference>()))
             .add_property("canColonize",        make_function(&Species::CanColonize,    return_value_policy<return_by_value>()))
             .add_property("canProduceShips",    make_function(&Species::CanProduceShips,return_value_policy<return_by_value>()))
             .add_property("tags",               make_function(&Species::Tags,           return_value_policy<return_by_value>()))
@@ -597,4 +625,18 @@ namespace FreeOrionPython {
         ;
         def("getSpecies",                       &GetSpecies,                            return_value_policy<reference_existing_object>());
     }
+
+    void WrapGalaxySetupData() {
+        class_<GalaxySetupData>("galaxySetupData")
+            .def_readonly ("seed",              &GalaxySetupData::m_seed)
+            .def_readwrite("size",              &GalaxySetupData::m_size)
+            .def_readwrite("shape",             &GalaxySetupData::m_shape)
+            .def_readonly ("age",               &GalaxySetupData::m_age)
+            .def_readonly ("starlaneFrequency", &GalaxySetupData::m_starlane_freq)
+            .def_readonly ("planetDensity",     &GalaxySetupData::m_planet_density)
+            .def_readonly ("specialsFrequency", &GalaxySetupData::m_specials_freq)
+            .def_readonly ("monsterFrequency",  &GalaxySetupData::m_monster_freq)
+            .def_readonly ("nativeFrequency",   &GalaxySetupData::m_native_freq)
+            .def_readonly ("maxAIAggression",   &GalaxySetupData::m_ai_aggr);
+    }
 }
