@@ -118,21 +118,11 @@ namespace {
         }
     }
 
-    void LoadEmpireNames(std::list<std::string>& names) {
-        boost::filesystem::ifstream ifs(GetResourceDir() / "empire_names.txt");
-        while (ifs) {
-            std::string latest_name;
-            std::getline(ifs, latest_name);
-            if (!latest_name.empty())
-                names.push_back(latest_name.substr(0, latest_name.find_last_not_of(" \t") + 1)); // strip off trailing whitespace
-        }
-    }
-
     std::string GenerateEmpireName(std::list<std::pair<int, PlayerSetupData> >& players) {
         // load default empire names
         static std::list<std::string> empire_names;
         if (empire_names.empty())
-            LoadEmpireNames(empire_names);
+            UserStringList("EMPIRE_NAMES", empire_names);
         std::set<std::string> validNames(empire_names.begin(), empire_names.end());
         for (std::list<std::pair<int, PlayerSetupData> >::iterator player_setup_it = players.begin(); player_setup_it != players.end(); player_setup_it++) {
             std::set<std::string>::iterator name_it = validNames.find(player_setup_it->second.m_empire_name);
@@ -150,18 +140,13 @@ namespace {
             return UserString("EMPIRE");
         }
     }
-
-    std::string GenerateEmpireName() {
-        std::list<std::pair<int, PlayerSetupData> > empty_data;
-        return GenerateEmpireName(empty_data);
-    }
 }
 
 
 ////////////////////////////////////////////////////////////
 // ResolveCombat
 ////////////////////////////////////////////////////////////
-ResolveCombat::ResolveCombat(System* system, std::set<int>& empire_ids) :
+ResolveCombat::ResolveCombat(TemporaryPtr<System> system, std::set<int>& empire_ids) :
     m_system(system),
     m_empire_ids()
 { std::swap(m_empire_ids, empire_ids); }
@@ -327,7 +312,7 @@ sc::result Idle::react(const HostSPGame& msg) {
 ////////////////////////////////////////////////////////////
 MPLobby::MPLobby(my_context c) :
     my_base(c),
-    m_lobby_data(new MultiplayerLobbyData(true)),
+    m_lobby_data(new MultiplayerLobbyData()),
     m_server_save_game_data(new ServerSaveGameData())
 {
     if (TRACE_EXECUTION) Logger().debugStream() << "(ServerFSM) MPLobby";
@@ -502,6 +487,7 @@ sc::result MPLobby::react(const LobbyUpdate& msg) {
     // or the galaxy setup.
 
     // GalaxySetupData
+    m_lobby_data->m_seed =          incoming_lobby_data.m_seed;
     m_lobby_data->m_size =          incoming_lobby_data.m_size;
     m_lobby_data->m_shape =         incoming_lobby_data.m_shape;
     m_lobby_data->m_age =           incoming_lobby_data.m_age;
@@ -598,8 +584,15 @@ sc::result MPLobby::react(const LobbyUpdate& msg) {
             psd.m_client_type == Networking::CLIENT_TYPE_HUMAN_MODERATOR)
         {
             psd.m_empire_color = GG::Clr(0, 0, 0, 0);
+            // On OSX the following two lines must not be included.
+            // Clearing empire name and starting species name from
+            // PlayerSetupData causes a weird crash (bus error) deep
+            // in GG code on OSX in the Multiplayer Lobby when selecting
+            // Observer or Moderator as client type.
+#ifndef FREEORION_MACOSX
             psd.m_empire_name.clear();
             psd.m_starting_species_name.clear();
+#endif
             psd.m_save_game_empire_id = ALL_EMPIRES;
 
         } else if (psd.m_client_type == Networking::CLIENT_TYPE_AI_PLAYER) {
@@ -628,14 +621,13 @@ sc::result MPLobby::react(const LobbyUpdate& msg) {
     // to determine if a new save file was selected, check if the selected file
     // index is different, and the new file index is in the valid range
     bool new_save_file_selected = false;
-    int new_file_index = incoming_lobby_data.m_save_file_index;
-    int old_file_index = m_lobby_data->m_save_file_index;
-    const int NUM_FILE_INDICES = static_cast<int>(m_lobby_data->m_save_games.size());
-    if (new_file_index != old_file_index  &&  new_file_index >= 0  &&  new_file_index < NUM_FILE_INDICES) {
+    std::string new_file = incoming_lobby_data.m_save_game;
+    std::string old_file = m_lobby_data->m_save_game;
+    if (new_file != old_file) {
         new_save_file_selected = true;
 
         // update selected file index
-        m_lobby_data->m_save_file_index = new_file_index;
+        m_lobby_data->m_save_game = new_file;
 
         // reset assigned empires in save game for all players.  new loaded game may not have the same set of empire IDs to choose from
         for (std::list<std::pair<int, PlayerSetupData> >::iterator player_setup_it = m_lobby_data->m_players.begin();
@@ -646,16 +638,15 @@ sc::result MPLobby::react(const LobbyUpdate& msg) {
 
         // refresh save game empire data
         boost::filesystem::path save_dir(GetSaveDir());
-        const std::string& save_filename = m_lobby_data->m_save_games[new_file_index];
         try {
-            LoadEmpireSaveGameData((save_dir / save_filename).string(),
+            LoadEmpireSaveGameData((save_dir / m_lobby_data->m_save_game).string(),
                                    m_lobby_data->m_save_game_empire_data);
         } catch (const std::exception&) {
             // inform player who attempted to change the save file that there was a problem
             PlayerConnectionPtr& player_connection = msg.m_player_connection;
             player_connection->SendMessage(ErrorMessage(UserStringNop("UNABLE_TO_READ_SAVE_FILE"), false));
             // revert to old save file
-            m_lobby_data->m_save_file_index = old_file_index;
+            m_lobby_data->m_save_game = old_file;
         }
     }
 
@@ -713,13 +704,26 @@ sc::result MPLobby::react(const StartMPGame& msg) {
 
         } else {
             // Load game...
-            boost::filesystem::path save_dir(GetSaveDir());
-            std::string save_filename = m_lobby_data->m_save_games[m_lobby_data->m_save_file_index];
+            std::string save_filename = (GetSaveDir() / m_lobby_data->m_save_game).string();
 
             try {
-                LoadGame((save_dir / save_filename).string(),   *m_server_save_game_data,
+                LoadGame(save_filename,   *m_server_save_game_data,
                          m_player_save_game_data,               GetUniverse(),
-                         Empires(),     GetSpeciesManager(),    GetCombatLogManager());
+                         Empires(),     GetSpeciesManager(),    GetCombatLogManager(),
+                         server.m_galaxy_setup_data);
+                int seed = 0;
+                try {
+                    Logger().debugStream() << "Seeding with loaded galaxy seed: " << server.m_galaxy_setup_data.m_seed;
+                    seed = boost::lexical_cast<unsigned int>(server.m_galaxy_setup_data.m_seed);
+                } catch (...) {
+                    try {
+                        boost::hash<std::string> string_hash;
+                        std::size_t h = string_hash(server.m_galaxy_setup_data.m_seed);
+                        seed = static_cast<unsigned int>(h);
+                    } catch (...) {}
+                }
+                Seed(seed);
+
             } catch (const std::exception&) {
                 SendMessageToAllPlayers(ErrorMessage(UserStringNop("UNABLE_TO_READ_SAVE_FILE"), true));
                 return discard_event();
@@ -925,7 +929,8 @@ sc::result WaitingForSPGameJoiners::react(const CheckStartConditions& u) {
             try {
                 LoadGame(m_single_player_setup_data->m_filename,            *m_server_save_game_data,
                          m_player_save_game_data,   GetUniverse(),          Empires(),
-                         GetSpeciesManager(),       GetCombatLogManager());
+                         GetSpeciesManager(),       GetCombatLogManager(),  server.m_galaxy_setup_data);
+
             } catch (const std::exception&) {
                 SendMessageToHost(ErrorMessage(UserStringNop("UNABLE_TO_READ_SAVE_FILE"), true));
                 return transit<Idle>();
@@ -1017,7 +1022,8 @@ sc::result WaitingForMPGameJoiners::react(const JoinGame& msg) {
 
     } else if (client_type == Networking::CLIENT_TYPE_HUMAN_PLAYER) {
         // verify that there is room left for this player
-        int already_connected_players = m_expected_ai_player_names.size() + server.m_networking.NumEstablishedPlayers();
+        int already_connected_players = m_expected_ai_player_names.size() +
+                                        server.m_networking.NumEstablishedPlayers();
         if (already_connected_players >= m_num_expected_players) {
             // too many human players
             Logger().errorStream() << "WaitingForSPGameJoiners.JoinGame : A human player attempted to join the game but there was not enough room.  Terminating connection.";
@@ -1048,9 +1054,7 @@ sc::result WaitingForMPGameJoiners::react(const CheckStartConditions& u) {
         if (m_player_save_game_data.empty())
             server.NewMPGameInit(*m_lobby_data);
         else
-            server.LoadMPGameInit(*m_lobby_data,
-                                  m_player_save_game_data,
-                                  m_server_save_game_data);
+            server.LoadMPGameInit(*m_lobby_data, m_player_save_game_data, m_server_save_game_data);
         return transit<PlayingGame>();
     }
 
@@ -1071,9 +1075,16 @@ PlayingGame::~PlayingGame()
 sc::result PlayingGame::react(const PlayerChat& msg) {
     if (TRACE_EXECUTION) Logger().debugStream() << "(ServerFSM) PlayingGame.PlayerChat";
     ServerApp& server = Server();
-    for (ServerNetworking::const_established_iterator it = server.m_networking.established_begin(); it != server.m_networking.established_end(); ++it) {
-        if (msg.m_message.ReceivingPlayer() == Networking::INVALID_PLAYER_ID || msg.m_message.ReceivingPlayer() == (*it)->PlayerID())
-            (*it)->SendMessage(SingleRecipientChatMessage(msg.m_message.SendingPlayer(), (*it)->PlayerID(), msg.m_message.Text()));
+    for (ServerNetworking::const_established_iterator it = server.m_networking.established_begin();
+         it != server.m_networking.established_end(); ++it)
+    {
+        if (msg.m_message.ReceivingPlayer() == Networking::INVALID_PLAYER_ID ||
+            msg.m_message.ReceivingPlayer() == (*it)->PlayerID())
+        {
+            (*it)->SendMessage(SingleRecipientChatMessage(msg.m_message.SendingPlayer(),
+                                                          (*it)->PlayerID(),
+                                                          msg.m_message.Text()));
+        }
     }
     return discard_event();
 }
@@ -1109,7 +1120,8 @@ sc::result PlayingGame::react(const ModeratorAct& msg) {
 
         // update player(s) of changed gamestate as result of action
         server.m_networking.SendMessage(TurnProgressMessage(Message::DOWNLOADING, player_id));
-        server.m_networking.SendMessage(TurnPartialUpdateMessage(player_id, server.PlayerEmpireID(player_id),
+        server.m_networking.SendMessage(TurnPartialUpdateMessage(player_id,
+                                                                 server.PlayerEmpireID(player_id),
                                                                  GetUniverse()));
     }
 
@@ -1370,9 +1382,7 @@ sc::result WaitingForSaveData::react(const ClientSaveData& msg) {
     m_player_save_game_data.push_back(
         PlayerSaveGameData(player_connection->PlayerName(),
                            server.PlayerEmpireID(message.SendingPlayer()),
-                           order_set,
-                           ui_data,
-                           save_state_string,
+                           order_set,       ui_data,    save_state_string,
                            client_type));
 
 
@@ -1389,15 +1399,21 @@ sc::result WaitingForSaveData::react(const ClientSaveData& msg) {
         try {
             SaveGame(save_filename,     server_data,    m_player_save_game_data,
                      GetUniverse(),     Empires(),      GetSpeciesManager(),
-                     GetCombatLogManager());
+                     GetCombatLogManager(),             server.m_galaxy_setup_data,
+                     !server.m_single_player_game
+                    );
+
         } catch (const std::exception&) {
+            Logger().debugStream() << "Catch std::exception&";
             SendMessageToAllPlayers(ErrorMessage(UserStringNop("UNABLE_TO_WRITE_SAVE_FILE"), false));
         }
-
+        
+        Logger().debugStream() << "Finished ClientSaveData from within if.";
         context<WaitingForTurnEnd>().m_save_filename = "";
         return transit<WaitingForTurnEndIdle>();
     }
-
+    
+    Logger().debugStream() << "Finished ClientSaveData from outside of if.";
     return discard_event();
 }
 
@@ -1407,7 +1423,6 @@ sc::result WaitingForSaveData::react(const ClientSaveData& msg) {
 ////////////////////////////////////////////////////////////
 ProcessingTurn::ProcessingTurn(my_context c) :
     my_base(c),
-    m_combat_system(0),
     m_combat_empire_ids()
 { if (TRACE_EXECUTION) Logger().debugStream() << "(ServerFSM) ProcessingTurn"; }
 
@@ -1491,7 +1506,7 @@ ResolvingCombat::ResolvingCombat(my_context c) :
     std::map<int, std::vector<CombatSetupGroup> > setup_groups;
     server.m_current_combat =
         new CombatData(context<ProcessingTurn>().m_combat_system, setup_groups);
-    context<ProcessingTurn>().m_combat_system = 0;
+    context<ProcessingTurn>().m_combat_system = TemporaryPtr<System>();
 
     // TODO: For now, we're just sending all designs to everyone.  Reconsider
     // this later.
@@ -1565,7 +1580,7 @@ sc::result ResolvingCombat::react(const CombatTurnOrders& msg) {
         int owner_id = ALL_EMPIRES;
         if (order.Type() == CombatOrder::SHIP_ORDER ||
             order.Type() == CombatOrder::SETUP_PLACEMENT_ORDER) {
-            if (UniverseObject* object = GetUniverseObject(order.ID())) {
+            if (TemporaryPtr<UniverseObject> object = GetUniverseObject(order.ID())) {
                 assert(!object->Unowned());
                 owner_id = object->Owner();
             }
