@@ -16,7 +16,7 @@ struct ScriptingContext;
 
 namespace Condition {
     struct ConditionBase;
-    typedef std::vector<const UniverseObject*> ObjectSet;
+    typedef std::vector<TemporaryPtr<const UniverseObject> > ObjectSet;
 }
 namespace Effect {
     class EffectsGroup;
@@ -98,21 +98,11 @@ public:
     void    GetTargetSet(int source_id, TargetSet& targets, TargetSet& potential_targets) const;
 
     /** execute all effects in group */
-    void    Execute(int source_id, const TargetSet& targets) const;
-    void    Execute(int source_id, const TargetsAndCause& targets_and_cause,
-                    AccountingMap& accounting_map) const;
-
-    /** execute all SetMeter effects in group.  This is useful for doing meter
-      * estimate updates and effect accounting, for which executing non-meter
-      * effects is neither needed nor useful. */
-    void    ExecuteSetMeter(int source_id, const TargetSet& targets) const;
-    void    ExecuteSetMeter(int source_id, const TargetsAndCause& targets_and_cause,
-                            AccountingMap& accounting_map) const;
-
-    void    ExecuteSetEmpireMeter(int source_id, const TargetSet& targets) const;
-
-    /** execute all appearance modifying effects in group. */
-    void    ExecuteAppearanceModifications(int source_id, const TargetSet& targets) const;
+    void    Execute(const Effect::TargetsCauses& targets_causes,
+                    AccountingMap* accounting_map = 0,
+                    bool only_meter_effects = false,
+                    bool only_appearance_effects = false,
+                    bool include_empire_meter_effects = false) const;
 
     const std::string&              StackingGroup() const       { return m_stacking_group; }
     const Condition::ConditionBase* Scope() const               { return m_scope; }
@@ -151,6 +141,12 @@ public:
 
     virtual void        Execute(const ScriptingContext& context) const = 0;
     virtual void        Execute(const ScriptingContext& context, const TargetSet& targets) const;
+    virtual void        Execute(const Effect::TargetsCauses& targets_causes,
+                                bool stacking,
+                                AccountingMap* accounting_map = 0,
+                                bool only_meter_effects = false,
+                                bool only_appearance_effects = false,
+                                bool include_empire_meter_effects = false) const;
     virtual std::string Description() const = 0;
     virtual std::string Dump() const = 0;
 
@@ -197,22 +193,19 @@ public:
         there is a ship meter (i.e. specifying PC_FUEL would be illegal). */
     SetShipPartMeter(MeterType meter_type,
                      ShipPartClass part_class,
-                     const ValueRef::ValueRefBase<double>* value,
-                     ShipSlotType slot_type = INVALID_SHIP_SLOT_TYPE);
+                     const ValueRef::ValueRefBase<double>* value);
 
     /** Affects the \a meter_type meters that belong to all PC_FIGHTERS parts
         of type \a fighter_type. */
     SetShipPartMeter(MeterType meter_type,
                      CombatFighterType fighter_type,
-                     const ValueRef::ValueRefBase<double>* value,
-                     ShipSlotType slot_type = INVALID_SHIP_SLOT_TYPE);
+                     const ValueRef::ValueRefBase<double>* value);
 
     /** Affects the \a meter_type meters that belong to all parts named \a
         part_name. */
     SetShipPartMeter(MeterType meter_type,
                      const std::string& part_name,
-                     const ValueRef::ValueRefBase<double>* value,
-                     ShipSlotType slot_type = INVALID_SHIP_SLOT_TYPE);
+                     const ValueRef::ValueRefBase<double>* value);
 
     virtual ~SetShipPartMeter();
 
@@ -220,14 +213,12 @@ public:
     virtual std::string Description() const;
     virtual std::string Dump() const;
     const std::string&  GetPartName() const {return m_part_name;}
-    ShipSlotType        GetShipSlotType() const {return m_slot_type;}
     MeterType           GetMeterType() const {return m_meter;};
 
 private:
     ShipPartClass                         m_part_class;
     CombatFighterType                     m_fighter_type;
     std::string                           m_part_name;
-    ShipSlotType                          m_slot_type;
     MeterType                             m_meter;
     const ValueRef::ValueRefBase<double>* m_value;
 
@@ -495,7 +486,7 @@ public:
     virtual std::string Description() const;
     virtual std::string Dump() const;
 private:
-    const ValueRef::ValueRefBase< ::StarType>*   m_type;
+    const ValueRef::ValueRefBase< ::StarType>*  m_type;
     const ValueRef::ValueRefBase<double>*       m_x;
     const ValueRef::ValueRefBase<double>*       m_y;
 
@@ -892,7 +883,6 @@ void Effect::SetShipPartMeter::serialize(Archive& ar, const unsigned int version
         & BOOST_SERIALIZATION_NVP(m_part_class)
         & BOOST_SERIALIZATION_NVP(m_fighter_type)
         & BOOST_SERIALIZATION_NVP(m_part_name)
-        & BOOST_SERIALIZATION_NVP(m_slot_type)
         & BOOST_SERIALIZATION_NVP(m_meter)
         & BOOST_SERIALIZATION_NVP(m_value);
 }
