@@ -13,6 +13,7 @@
 #include "../util/OptionsDB.h"
 #include "../util/Random.h"
 #include "../util/Directories.h"
+#include "../util/SitRepEntry.h"
 #include "../Empire/Empire.h"
 #include "../Empire/EmpireManager.h"
 
@@ -20,19 +21,19 @@
 
 namespace {
     // high tilt is arbitrarily taken to mean 35 degrees or more
-    const double HIGH_TILT_THERSHOLD = 35.0;
+    const float HIGH_TILT_THERSHOLD = 35.0f;
 
-    double SizeRotationFactor(PlanetSize size) {
+    float SizeRotationFactor(PlanetSize size) {
         switch (size) {
-        case SZ_TINY:     return 1.5;
-        case SZ_SMALL:    return 1.25;
-        case SZ_MEDIUM:   return 1.0;
-        case SZ_LARGE:    return 0.75;
-        case SZ_HUGE:     return 0.5;
-        case SZ_GASGIANT: return 0.25;
-        default:          return 1.0;
+        case SZ_TINY:     return 1.5f;
+        case SZ_SMALL:    return 1.25f;
+        case SZ_MEDIUM:   return 1.0f;
+        case SZ_LARGE:    return 0.75f;
+        case SZ_HUGE:     return 0.5f;
+        case SZ_GASGIANT: return 0.25f;
+        default:          return 1.0f;
         }
-        return 1.0;
+        return 1.0f;
     }
 
     static const std::string EMPTY_STRING;
@@ -43,7 +44,7 @@ namespace {
 // Year
 ////////////////////////////////////////////////////////////
 Year::Year(Day d) :
-    TypesafeDouble(d / 360.0)
+    TypesafeFloat(d / 360.0f)
 {}
 
 
@@ -57,13 +58,15 @@ Planet::Planet() :
     m_type(PT_TERRAN),
     m_original_type(PT_TERRAN),
     m_size(SZ_MEDIUM),
-    m_orbital_period(1.0),
-    m_initial_orbital_position(0.0),
-    m_rotational_period(1.0),
-    m_axial_tilt(23.0),
+    m_orbital_period(1.0f),
+    m_initial_orbital_position(0.0f),
+    m_rotational_period(1.0f),
+    m_axial_tilt(23.0f),
     m_just_conquered(false),
     m_is_about_to_be_colonized(false),
     m_is_about_to_be_invaded(false),
+    m_is_about_to_be_bombarded(false),
+    m_ordered_given_to_empire_id(ALL_EMPIRES),
     m_last_turn_attacked_by_ship(-1),
     m_surface_texture()
 {
@@ -79,13 +82,15 @@ Planet::Planet(PlanetType type, PlanetSize size) :
     m_type(type),
     m_original_type(type),
     m_size(size),
-    m_orbital_period(1.0),
-    m_initial_orbital_position(RandZeroToOne() * 2 * 3.14159),
-    m_rotational_period(1.0),
+    m_orbital_period(1.0f),
+    m_initial_orbital_position(RandZeroToOne() * 2 * 3.14159f),
+    m_rotational_period(1.0f),
     m_axial_tilt(RandZeroToOne() * HIGH_TILT_THERSHOLD),
     m_just_conquered(false),
     m_is_about_to_be_colonized(false),
     m_is_about_to_be_invaded(false),
+    m_is_about_to_be_bombarded(false),
+    m_ordered_given_to_empire_id(ALL_EMPIRES),
     m_last_turn_attacked_by_ship(-1),
     m_surface_texture()
 {
@@ -109,14 +114,14 @@ Planet* Planet::Clone(int empire_id) const {
         return 0;
 
     Planet* retval = new Planet();
-    retval->Copy(this, empire_id);
+    retval->Copy(TemporaryFromThis(), empire_id);
     return retval;
 }
 
-void Planet::Copy(const UniverseObject* copied_object, int empire_id) {
+void Planet::Copy(TemporaryPtr<const UniverseObject> copied_object, int empire_id) {
     if (copied_object == this)
         return;
-    const Planet* copied_planet = universe_object_cast<Planet*>(copied_object);
+    TemporaryPtr<const Planet> copied_planet = boost::dynamic_pointer_cast<const Planet>(copied_object);
     if (!copied_planet) {
         Logger().errorStream() << "Planet::Copy passed an object that wasn't a Planet";
         return;
@@ -133,7 +138,7 @@ void Planet::Copy(const UniverseObject* copied_object, int empire_id) {
     if (vis >= VIS_BASIC_VISIBILITY) {
         this->m_name =                      copied_planet->m_name;
 
-        this->m_buildings =                 copied_planet->VisibleContainedObjects(empire_id);
+        this->m_buildings =                 copied_planet->VisibleContainedObjectIDs(empire_id);
         this->m_type =                      copied_planet->m_type;
         this->m_original_type =             copied_planet->m_original_type;
         this->m_size =                      copied_planet->m_size;
@@ -147,6 +152,8 @@ void Planet::Copy(const UniverseObject* copied_object, int empire_id) {
             if (vis >= VIS_FULL_VISIBILITY) {
                 this->m_is_about_to_be_colonized =  copied_planet->m_is_about_to_be_colonized;
                 this->m_is_about_to_be_invaded   =  copied_planet->m_is_about_to_be_invaded;
+                this->m_is_about_to_be_bombarded =  copied_planet->m_is_about_to_be_bombarded;
+                this->m_ordered_given_to_empire_id =copied_planet->m_ordered_given_to_empire_id;
                 this->m_last_turn_attacked_by_ship= copied_planet->m_last_turn_attacked_by_ship;
             } else {
                 // copy system name if at partial visibility, as it won't be copied
@@ -173,9 +180,6 @@ bool Planet::HasTag(const std::string& name) const {
     return species && species->Tags().count(name);
 }
 
-const std::string& Planet::TypeName() const
-{ return UserString("PLANET"); }
-
 UniverseObjectType Planet::ObjectType() const
 { return OBJ_PLANET; }
 
@@ -184,9 +188,9 @@ std::string Planet::Dump() const {
     os << UniverseObject::Dump();
     os << PopCenter::Dump();
     os << ResourceCenter::Dump();
-    os << " type: " << UserString(GG::GetEnumMap<PlanetType>().FromEnum(m_type))
-       << " original type: " << UserString(GG::GetEnumMap<PlanetType>().FromEnum(m_original_type))
-       << " size: " << UserString(GG::GetEnumMap<PlanetSize>().FromEnum(m_size))
+    os << " type: " << UserString(EnumToString(m_type))
+       << " original type: " << UserString(EnumToString(m_original_type))
+       << " size: " << UserString(EnumToString(m_size))
        << " rot period: " << m_rotational_period
        << " axis tilt: " << m_axial_tilt
        << " buildings: ";
@@ -201,6 +205,10 @@ std::string Planet::Dump() const {
         os << " (About to be Invaded)";
     if (m_just_conquered)
         os << " (Just Conquered)";
+    if (m_is_about_to_be_bombarded)
+        os << " (About to be Bombarded)";
+    if (m_ordered_given_to_empire_id != ALL_EMPIRES)
+        os << " (Ordered to be given to empire with id: " << m_ordered_given_to_empire_id << ")";
     os << " last attacked on turn: " << m_last_turn_attacked_by_ship;
 
     return os.str();
@@ -221,6 +229,7 @@ int Planet::SizeAsInt() const {
 
 void Planet::Init() {
     AddMeter(METER_SUPPLY);
+    AddMeter(METER_MAX_SUPPLY);
     AddMeter(METER_SHIELD);
     AddMeter(METER_MAX_SHIELD);
     AddMeter(METER_DEFENSE);
@@ -413,26 +422,8 @@ Day Planet::RotationalPeriod() const
 Degree Planet::AxialTilt() const
 { return m_axial_tilt; }
 
-bool Planet::Contains(int object_id) const
-{ return m_buildings.find(object_id) != m_buildings.end(); }
-
-std::vector<UniverseObject*> Planet::FindObjects() const {
-    std::vector<UniverseObject*> retval;
-    // add buildings on this planet
-    for (std::set<int>::const_iterator it = m_buildings.begin(); it != m_buildings.end(); ++it)
-        retval.push_back(GetUniverseObject(*it));
-    return retval;
-}
-
-std::vector<int> Planet::FindObjectIDs() const {
-    std::vector<int> retval;
-    // add buildings on this planet
-    std::copy(m_buildings.begin(), m_buildings.end(), std::back_inserter(retval));
-    return retval;
-}
-
-UniverseObject* Planet::Accept(const UniverseObjectVisitor& visitor) const
-{ return visitor.Visit(const_cast<Planet* const>(this)); }
+TemporaryPtr<UniverseObject> Planet::Accept(const UniverseObjectVisitor& visitor) const
+{ return visitor.Visit(boost::const_pointer_cast<Planet>(boost::static_pointer_cast<const Planet>(TemporaryFromThis()))); }
 
 Meter* Planet::GetMeter(MeterType type)
 { return UniverseObject::GetMeter(type); }
@@ -451,6 +442,8 @@ float Planet::NextTurnCurrentMeterValue(MeterType type) const {
     switch (type) {
     case METER_TARGET_POPULATION:
     case METER_POPULATION:
+    case METER_TARGET_HAPPINESS:
+    case METER_HAPPINESS:
         return PopCenterNextTurnMeterValue(type);
         break;
     case METER_TARGET_INDUSTRY:
@@ -466,6 +459,7 @@ float Planet::NextTurnCurrentMeterValue(MeterType type) const {
     case METER_SHIELD:      max_meter_type = METER_MAX_SHIELD;          break;
     case METER_TROOPS:      max_meter_type = METER_MAX_TROOPS;          break;
     case METER_DEFENSE:     max_meter_type = METER_MAX_DEFENSE;         break;
+    case METER_SUPPLY:      max_meter_type = METER_MAX_SUPPLY;          break;
         break;
     default:
         return UniverseObject::NextTurnCurrentMeterValue(type);
@@ -491,19 +485,35 @@ float Planet::NextTurnCurrentMeterValue(MeterType type) const {
     return std::min(current_meter_value + 1.0f, max_meter_value);
 }
 
+int Planet::ContainerObjectID() const
+{ return this->SystemID(); }
+
+const std::set<int>& Planet::ContainedObjectIDs() const
+{ return m_buildings; }
+
+bool Planet::Contains(int object_id) const
+{ return object_id != INVALID_OBJECT_ID && m_buildings.find(object_id) != m_buildings.end(); }
+
+bool Planet::ContainedBy(int object_id) const
+{ return object_id != INVALID_OBJECT_ID && this->SystemID() == object_id; }
+
 std::vector<std::string> Planet::AvailableFoci() const {
     std::vector<std::string> retval;
-    ScriptingContext context(this);
-    if (const Species* species = GetSpecies(this->SpeciesName())) {
+    TemporaryPtr<const Planet> this_planet = boost::dynamic_pointer_cast<const Planet>(TemporaryFromThis());
+    if (!this_planet)
+        return retval;
+    ScriptingContext context(this_planet);
+    if (const Species* species = GetSpecies(this_planet->SpeciesName())) {
         const std::vector<FocusType>& foci = species->Foci();
         for (std::vector<FocusType>::const_iterator it = foci.begin(); it != foci.end(); ++it) {
             const FocusType& focus_type = *it;
             if (const Condition::ConditionBase* location = focus_type.Location()) {
-                if (location->Eval(context, this))
+                if (location->Eval(context, this_planet))
                     retval.push_back(focus_type.Name());
             }
         }
     }
+
     return retval;
 }
 
@@ -558,25 +568,11 @@ void Planet::SetHighAxialTilt() {
 }
 
 void Planet::AddBuilding(int building_id) {
-    if (this->Contains(building_id)) {
-        Logger().debugStream() << "Planet::AddBuilding this planet " << this->Name() << " already contained building " << building_id;
-        return;
-    }
-    //Logger().debugStream() << "Planet::AddBuilding " << this->Name() << " adding building: " << building_id;
-    if (Building* building = GetBuilding(building_id)) {
-        if (System* system = GetSystem(this->SystemID())) {
-            system->Insert(building);
-        } else {
-            Logger().errorStream() << "... planet is not located in a system?!?!";
-            building->MoveTo(X(), Y());
-            building->SetSystem(SystemID());
-        }
-        building->SetPlanetID(ID());
-        m_buildings.insert(building_id);
-    } else {
-        Logger().errorStream() << "Planet::AddBuilding() : Attempted to add an id of a non-building object to a planet.";
-    }
-    StateChangedSignal();
+    size_t buildings_size = m_buildings.size();
+    m_buildings.insert(building_id);
+    if (buildings_size != m_buildings.size())
+        StateChangedSignal();
+    // expect calling code to set building's planet
 }
 
 bool Planet::RemoveBuilding(int building_id) {
@@ -589,11 +585,11 @@ bool Planet::RemoveBuilding(int building_id) {
 }
 
 void Planet::Reset() {
-    SetOwner(ALL_EMPIRES);
     PopCenter::Reset();
     ResourceCenter::Reset();
 
     GetMeter(METER_SUPPLY)->Reset();
+    GetMeter(METER_MAX_SUPPLY)->Reset();
     GetMeter(METER_SHIELD)->Reset();
     GetMeter(METER_MAX_SHIELD)->Reset();
     GetMeter(METER_DEFENSE)->Reset();
@@ -601,13 +597,17 @@ void Planet::Reset() {
     GetMeter(METER_DETECTION)->Reset();
     GetMeter(METER_REBEL_TROOPS)->Reset();
 
-    for (std::set<int>::const_iterator it = m_buildings.begin(); it != m_buildings.end(); ++it)
-        if (Building* building = GetBuilding(*it))
-            building->Reset();
+    if (m_is_about_to_be_colonized && !OwnedBy(ALL_EMPIRES)) {
+        for (std::set<int>::const_iterator it = m_buildings.begin(); it != m_buildings.end(); ++it)
+            if (TemporaryPtr<Building> building = GetBuilding(*it))
+                building->Reset();
+    }
 
     m_just_conquered = false;
     m_is_about_to_be_colonized = false;
     m_is_about_to_be_invaded = false;
+    m_is_about_to_be_bombarded = false;
+    SetOwner(ALL_EMPIRES);
 }
 
 void Planet::Depopulate() {
@@ -618,7 +618,7 @@ void Planet::Depopulate() {
     GetMeter(METER_TRADE)->Reset();
     GetMeter(METER_CONSTRUCTION)->Reset();
 
-    SetFocus("");
+    ClearFocus();
 }
 
 void Planet::Conquer(int conquerer) {
@@ -628,35 +628,114 @@ void Planet::Conquer(int conquerer) {
     Empire::ConquerProductionQueueItemsAtLocation(ID(), conquerer);
 
     // deal with UniverseObjects (eg. buildings) located on this planet
-    std::vector<UniverseObject*> contained_objects = this->FindObjects();
-    for (std::vector<UniverseObject*>::iterator it = contained_objects.begin(); it != contained_objects.end(); ++it) {
-        UniverseObject* obj = *it;
+    std::vector<TemporaryPtr<Building> > buildings = Objects().FindObjects<Building>(m_buildings);
+    for (std::vector<TemporaryPtr<Building> >::iterator it = buildings.begin();
+         it != buildings.end(); ++it)
+    {
+        TemporaryPtr<Building> building = *it;
+        const BuildingType* type = GetBuildingType(building->BuildingTypeName());
 
-        // Buildings:
-        if (Building* building = universe_object_cast<Building*>(obj)) {
-            const BuildingType* type = GetBuildingType(building->BuildingTypeName());
+        // determine what to do with building of this type...
+        const CaptureResult cap_result = type->GetCaptureResult(building->Owner(), conquerer, this->ID(), false);
 
-            // determine what to do with building of this type...
-            const CaptureResult cap_result = type->GetCaptureResult(obj->Owner(), conquerer, this->ID(), false);
-
-            if (cap_result == CR_CAPTURE) {
-                // replace ownership
-                obj->SetOwner(conquerer);
-            } else if (cap_result == CR_DESTROY) {
-                // destroy object
-                Logger().debugStream() << "Planet::Conquer destroying object: " << obj->Name();
-                GetUniverse().Destroy(obj->ID());
-                obj = 0;
-            } else if (cap_result == CR_RETAIN) {
-                // do nothing
-            }
+        if (cap_result == CR_CAPTURE) {
+            // replace ownership
+            building->SetOwner(conquerer);
+        } else if (cap_result == CR_DESTROY) {
+            // destroy object
+            //Logger().debugStream() << "Planet::Conquer destroying object: " << building->Name();
+            this->RemoveBuilding(building->ID());
+            if (TemporaryPtr<System> system = GetSystem(this->SystemID()))
+                system->Remove(building->ID());
+            GetUniverse().Destroy(building->ID());
+        } else if (cap_result == CR_RETAIN) {
+            // do nothing
         }
-
-        // TODO: deal with any other UniverseObject subclasses...?
     }
 
     // replace ownership
     SetOwner(conquerer);
+
+    GetMeter(METER_SUPPLY)->SetCurrent(0.0f);
+    GetMeter(METER_SUPPLY)->BackPropegate();
+    GetMeter(METER_INDUSTRY)->SetCurrent(0.0f);
+    GetMeter(METER_INDUSTRY)->BackPropegate();
+    GetMeter(METER_RESEARCH)->SetCurrent(0.0f);
+    GetMeter(METER_RESEARCH)->BackPropegate();
+    GetMeter(METER_TRADE)->SetCurrent(0.0f);
+    GetMeter(METER_TRADE)->BackPropegate();
+    GetMeter(METER_CONSTRUCTION)->SetCurrent(0.0f);
+    GetMeter(METER_CONSTRUCTION)->BackPropegate();
+    GetMeter(METER_DEFENSE)->SetCurrent(0.0f);
+    GetMeter(METER_DEFENSE)->BackPropegate();
+    GetMeter(METER_SHIELD)->SetCurrent(0.0f);
+    GetMeter(METER_SHIELD)->BackPropegate();
+}
+
+bool Planet::Colonize(int empire_id, const std::string& species_name, double population) {
+    const Species* species = 0;
+
+    // if desired pop > 0, we want a colony, not an outpost, so we need to do some checks
+    if (population > 0.0) {
+        // check if specified species exists and get reference
+        species = GetSpecies(species_name);
+        if (!species) {
+            Logger().errorStream() << "Planet::Colonize couldn't get species already on planet with name: " << species_name;
+            return false;
+        }
+        // check if specified species can colonize this planet
+        if (EnvironmentForSpecies(species_name) < PE_HOSTILE) {
+            Logger().errorStream() << "Planet::Colonize: can't colonize planet already populated by species " << species_name;
+            return false;
+        }
+    }
+
+    // reset the planet to unowned/unpopulated
+    Reset();
+
+    // if desired pop > 0, we want a colony, not an outpost, so we have to set the colony species
+    if (population > 0.0)
+        SetSpecies(species_name);
+
+    // find a default focus. use first defined available focus.
+    // AvailableFoci function should return a vector of all names of
+    // available foci.
+    std::vector<std::string> available_foci = AvailableFoci();
+    if (species && !available_foci.empty()) {
+        bool found_preference = false;
+        for (std::vector<std::string>::const_iterator it = available_foci.begin();
+             it != available_foci.end(); ++it)
+        {
+            if (!it->empty() && *it == species->PreferredFocus()) {
+                SetFocus(*it);
+                found_preference = true;
+                break;
+            }
+        }
+
+        if (!found_preference)
+            SetFocus(*available_foci.begin());
+    } else {
+        Logger().debugStream() << "Planet::Colonize unable to find a focus to set for species " << species_name;
+    }
+
+    // set colony population
+    GetMeter(METER_POPULATION)->SetCurrent(population);
+    GetMeter(METER_TARGET_POPULATION)->SetCurrent(population);
+    GetMeter(METER_HAPPINESS)->SetCurrent(20.0f);
+    BackPropegateMeters();
+
+
+    // set specified empire as owner
+    SetOwner(empire_id);
+
+    // if there are buildings on the planet, set the specified empire as their owner too
+    std::vector<TemporaryPtr<Building> > buildings = Objects().FindObjects<Building>(BuildingIDs());
+    for (std::vector<TemporaryPtr<Building> >::iterator building_it = buildings.begin();
+         building_it != buildings.end(); ++building_it)
+    { (*building_it)->SetOwner(empire_id); }
+
+    return true;
 }
 
 void Planet::SetIsAboutToBeColonized(bool b) {
@@ -679,6 +758,25 @@ void Planet::SetIsAboutToBeInvaded(bool b) {
 void Planet::ResetIsAboutToBeInvaded()
 { SetIsAboutToBeInvaded(false); }
 
+void Planet::SetIsAboutToBeBombarded(bool b) {
+    bool initial_status = m_is_about_to_be_bombarded;
+    if (b == initial_status) return;
+    m_is_about_to_be_bombarded = b;
+    StateChangedSignal();
+}
+
+void Planet::ResetIsAboutToBeBombarded()
+{ SetIsAboutToBeBombarded(false); }
+
+void Planet::SetGiveToEmpire(int empire_id) {
+    if (empire_id == m_ordered_given_to_empire_id) return;
+    m_ordered_given_to_empire_id = empire_id;
+    StateChangedSignal();
+}
+
+void Planet::ClearGiveToEmpire()
+{ SetGiveToEmpire(ALL_EMPIRES); }
+
 void Planet::SetLastTurnAttackedByShip(int turn)
 { m_last_turn_attacked_by_ship = turn; }
 
@@ -687,41 +785,14 @@ void Planet::SetSurfaceTexture(const std::string& texture) {
     StateChangedSignal();
 }
 
-void Planet::SetSystem(int sys) {
-    //Logger().debugStream() << "Planet::MoveTo(UniverseObject* object)";
-    UniverseObject::SetSystem(sys);
-    for (std::set<int>::const_iterator it = m_buildings.begin(); it != m_buildings.end(); ++it) {
-        UniverseObject* obj = GetUniverseObject(*it);
-        if (!obj) {
-            Logger().errorStream() << "Planet::SetSystem couldn't get building object with id " << *it;
-            continue;
-        }
-        obj->SetSystem(sys);
-    }
-}
-
-void Planet::MoveTo(double x, double y) {
-    //Logger().debugStream() << "Planet::MoveTo(double x, double y)";
-    // move planet itself
-    UniverseObject::MoveTo(x, y);
-    // move buildings
-    for (std::set<int>::const_iterator it = m_buildings.begin(); it != m_buildings.end(); ++it) {
-        UniverseObject* obj = GetUniverseObject(*it);
-        if (!obj) {
-            Logger().errorStream() << "Planet::MoveTo couldn't get building object with id " << *it;
-            continue;
-        }
-        obj->UniverseObject::MoveTo(x, y);
-    }
-}
-
 void Planet::PopGrowthProductionResearchPhase() {
     UniverseObject::PopGrowthProductionResearchPhase();
 
+    bool just_conquered = m_just_conquered;
     // do not do production if planet was just conquered
-    if (m_just_conquered)
-        m_just_conquered = false;
-    else
+    m_just_conquered = false;
+
+    if (!just_conquered)
         ResourceCenterPopGrowthProductionResearchPhase();
 
     PopCenterPopGrowthProductionResearchPhase();
@@ -729,17 +800,29 @@ void Planet::PopGrowthProductionResearchPhase() {
     // check for planets with zero population.  If they have a species set, then
     // they probably just starved
     if (!SpeciesName().empty() && GetMeter(METER_POPULATION)->Current() == 0.0) {
-        // generate starvation sitrep for empire that owns this depopulated planet
-        if (Empire* empire = Empires().Lookup(this->Owner()))
+        if (Empire* empire = Empires().Lookup(this->Owner())) {
+            // generate starvation sitrep for empire that owns this depopulated planet
             empire->AddSitRepEntry(CreatePlanetStarvedToDeathSitRep(this->ID()));
+
+            // record depopulation of planet with species while owned by this empire
+            std::map<std::string, int>::iterator species_it =
+                empire->SpeciesShipsLost().find(SpeciesName());
+            if (species_it == empire->SpeciesPlanetsDepoped().end())
+                empire->SpeciesPlanetsDepoped()[SpeciesName()] = 1;
+            else
+                species_it->second++;
+        }
         // remove species
         SetSpecies("");
     }
 
-    GetMeter(METER_SHIELD)->SetCurrent(Planet::NextTurnCurrentMeterValue(METER_SHIELD));
-    GetMeter(METER_DEFENSE)->SetCurrent(Planet::NextTurnCurrentMeterValue(METER_DEFENSE));
-    GetMeter(METER_TROOPS)->SetCurrent(Planet::NextTurnCurrentMeterValue(METER_TROOPS));
-    GetMeter(METER_REBEL_TROOPS)->SetCurrent(Planet::NextTurnCurrentMeterValue(METER_REBEL_TROOPS));
+    if (!just_conquered) {
+        GetMeter(METER_SHIELD)->SetCurrent(Planet::NextTurnCurrentMeterValue(METER_SHIELD));
+        GetMeter(METER_DEFENSE)->SetCurrent(Planet::NextTurnCurrentMeterValue(METER_DEFENSE));
+        GetMeter(METER_TROOPS)->SetCurrent(Planet::NextTurnCurrentMeterValue(METER_TROOPS));
+        GetMeter(METER_REBEL_TROOPS)->SetCurrent(Planet::NextTurnCurrentMeterValue(METER_REBEL_TROOPS));
+        GetMeter(METER_SUPPLY)->SetCurrent(Planet::NextTurnCurrentMeterValue(METER_SUPPLY));
+    }
 
     StateChangedSignal();
 }
@@ -756,7 +839,7 @@ void Planet::ResetTargetMaxUnpairedMeters() {
         stealth->AddToCurrent(0.01f);
     }
 
-    GetMeter(METER_SUPPLY)->ResetCurrent();
+    GetMeter(METER_MAX_SUPPLY)->ResetCurrent();
     GetMeter(METER_MAX_SHIELD)->ResetCurrent();
     GetMeter(METER_MAX_DEFENSE)->ResetCurrent();
     GetMeter(METER_MAX_TROOPS)->ResetCurrent();
@@ -775,42 +858,32 @@ void Planet::ClampMeters() {
     UniverseObject::GetMeter(METER_DEFENSE)->ClampCurrentToRange(Meter::DEFAULT_VALUE, UniverseObject::GetMeter(METER_MAX_DEFENSE)->Current());
     UniverseObject::GetMeter(METER_MAX_TROOPS)->ClampCurrentToRange();
     UniverseObject::GetMeter(METER_TROOPS)->ClampCurrentToRange(Meter::DEFAULT_VALUE, UniverseObject::GetMeter(METER_MAX_TROOPS)->Current());
+    UniverseObject::GetMeter(METER_MAX_SUPPLY)->ClampCurrentToRange();
+    UniverseObject::GetMeter(METER_SUPPLY)->ClampCurrentToRange(Meter::DEFAULT_VALUE, UniverseObject::GetMeter(METER_MAX_SUPPLY)->Current());
 
     UniverseObject::GetMeter(METER_REBEL_TROOPS)->ClampCurrentToRange();
     UniverseObject::GetMeter(METER_DETECTION)->ClampCurrentToRange();
-    UniverseObject::GetMeter(METER_SUPPLY)->ClampCurrentToRange();
-}
-
-std::set<int> Planet::VisibleContainedObjects(int empire_id) const {
-    std::set<int> retval;
-    const Universe& universe = GetUniverse();
-    for (std::set<int>::const_iterator it = m_buildings.begin(); it != m_buildings.end(); ++it) {
-        int object_id = *it;
-        if (universe.GetObjectVisibilityByEmpire(object_id, empire_id) >= VIS_BASIC_VISIBILITY)
-            retval.insert(object_id);
-    }
-    return retval;
 }
 
 // free functions
 
-double PlanetRadius(PlanetSize size) {
-    double retval = 0.0;
+float PlanetRadius(PlanetSize size) {
+    float retval = 0.0f;
     switch (size) {
-    case INVALID_PLANET_SIZE: retval = 0.0; break;
-    case SZ_NOWORLD:          retval = 0.0; break;
-    case SZ_TINY:             retval = 2.0; break;
-    case SZ_SMALL:            retval = 3.5; break;
+    case INVALID_PLANET_SIZE: retval = 0.0f; break;
+    case SZ_NOWORLD:          retval = 0.0f; break;
+    case SZ_TINY:             retval = 2.0f; break;
+    case SZ_SMALL:            retval = 3.5f; break;
     default:
-    case SZ_MEDIUM:           retval = 5.0; break;
-    case SZ_LARGE:            retval = 7.0; break;
-    case SZ_HUGE:             retval = 9.0; break;
-    case SZ_ASTEROIDS:        retval = 0.0; break;
-    case SZ_GASGIANT:         retval = 11.0; break; // this one goes to eleven
-    case NUM_PLANET_SIZES:    retval = 0.0; break;
+    case SZ_MEDIUM:           retval = 5.0f; break;
+    case SZ_LARGE:            retval = 7.0f; break;
+    case SZ_HUGE:             retval = 9.0f; break;
+    case SZ_ASTEROIDS:        retval = 0.0f; break;
+    case SZ_GASGIANT:         retval = 11.0f; break; // this one goes to eleven
+    case NUM_PLANET_SIZES:    retval = 0.0f; break;
     };
     return retval;
 }
 
-double AsteroidBeltRadius()
-{ return 12.5; }
+float AsteroidBeltRadius()
+{ return 12.5f; }
