@@ -2,6 +2,8 @@
 
 #include "../util/i18n.h"
 #include "../util/Logger.h"
+#include "../util/Random.h"
+#include "../util/AppInterface.h"
 #include "Fleet.h"
 #include "Predicates.h"
 #include "ShipDesign.h"
@@ -19,6 +21,7 @@ Ship::Ship() :
     m_ordered_scrapped(false),
     m_ordered_colonize_planet_id(INVALID_OBJECT_ID),
     m_ordered_invade_planet_id(INVALID_OBJECT_ID),
+    m_ordered_bombard_planet_id(INVALID_OBJECT_ID),
     m_last_turn_active_in_combat(INVALID_GAME_TURN),
     m_produced_by_empire_id(ALL_EMPIRES)
 {}
@@ -30,6 +33,7 @@ Ship::Ship(int empire_id, int design_id, const std::string& species_name,
     m_ordered_scrapped(false),
     m_ordered_colonize_planet_id(INVALID_OBJECT_ID),
     m_ordered_invade_planet_id(INVALID_OBJECT_ID),
+    m_ordered_bombard_planet_id(INVALID_OBJECT_ID),
     m_last_turn_active_in_combat(INVALID_GAME_TURN),
     m_species_name(species_name),
     m_produced_by_empire_id(produced_by_empire_id)
@@ -53,6 +57,12 @@ Ship::Ship(int empire_id, int design_id, const std::string& species_name,
     AddMeter(METER_MAX_STRUCTURE);
     AddMeter(METER_BATTLE_SPEED);
     AddMeter(METER_STARLANE_SPEED);
+    AddMeter(METER_TARGET_INDUSTRY);
+    AddMeter(METER_INDUSTRY);
+    AddMeter(METER_TARGET_RESEARCH);
+    AddMeter(METER_RESEARCH);
+    AddMeter(METER_TARGET_TRADE);
+    AddMeter(METER_TRADE);
 
     const std::vector<std::string>& part_names = Design()->Parts();
     for (std::size_t i = 0; i < part_names.size(); ++i) {
@@ -115,14 +125,14 @@ Ship* Ship::Clone(int empire_id) const {
         return 0;
 
     Ship* retval = new Ship();
-    retval->Copy(this, empire_id);
+    retval->Copy(TemporaryFromThis(), empire_id);
     return retval;
 }
 
-void Ship::Copy(const UniverseObject* copied_object, int empire_id) {
+void Ship::Copy(TemporaryPtr<const UniverseObject> copied_object, int empire_id) {
     if (copied_object == this)
         return;
-    const Ship* copied_ship = universe_object_cast<Ship*>(copied_object);
+    TemporaryPtr<const Ship> copied_ship = boost::dynamic_pointer_cast<const Ship>(copied_object);
     if (!copied_ship) {
         Logger().errorStream() << "Ship::Copy passed an object that wasn't a Ship";
         return;
@@ -137,7 +147,7 @@ void Ship::Copy(const UniverseObject* copied_object, int empire_id) {
     if (vis >= VIS_BASIC_VISIBILITY) {
         if (this->m_fleet_id != copied_ship->m_fleet_id) {
             // as with other containers, removal from the old container is triggered by the contained Object; removal from System is handled by UniverseObject::Copy
-            if (Fleet* oldFleet = GetFleet(this->m_fleet_id)) 
+            if (TemporaryPtr<Fleet> oldFleet = GetFleet(this->m_fleet_id)) 
                 oldFleet->RemoveShip(this->ID());
             this->m_fleet_id =              copied_ship->m_fleet_id; // as with other containers (Systems), actual insertion into fleet ships set is handled by the fleet
         }
@@ -158,6 +168,7 @@ void Ship::Copy(const UniverseObject* copied_object, int empire_id) {
                 this->m_ordered_scrapped =          copied_ship->m_ordered_scrapped;
                 this->m_ordered_colonize_planet_id= copied_ship->m_ordered_colonize_planet_id;
                 this->m_ordered_invade_planet_id  = copied_ship->m_ordered_invade_planet_id;
+                this->m_ordered_bombard_planet_id = copied_ship->m_ordered_bombard_planet_id;
                 this->m_last_turn_active_in_combat= copied_ship->m_last_turn_active_in_combat;
                 this->m_part_meters =               copied_ship->m_part_meters;
                 this->m_produced_by_empire_id =     copied_ship->m_produced_by_empire_id;
@@ -215,12 +226,15 @@ bool Ship::HasTag(const std::string& name) const {
     return false;
 }
 
-const std::string& Ship::TypeName() const
-{ return UserString("SHIP"); }
-
 UniverseObjectType Ship::ObjectType() const
 { return OBJ_SHIP; }
 
+bool Ship::ContainedBy(int object_id) const {
+    return object_id != INVALID_OBJECT_ID
+        && (    object_id == m_fleet_id
+            ||  object_id == this->SystemID());
+}
+
 std::string Ship::Dump() const {
     std::stringstream os;
     os << UniverseObject::Dump();
@@ -251,7 +265,7 @@ std::string Ship::Dump() const {
         const Meter& meter = it->second;
         ++it;
         os << UserString(part_name) << " "
-           << UserString(GG::GetEnumMap<MeterType>().FromEnum(meter_type))
+           << UserString(EnumToString(meter_type))
            << ": " << meter.Current() << "  ";
     }
     return os.str();
@@ -299,6 +313,11 @@ bool Ship::HasTroops() const {
     return design && design->HasTroops();
 }
 
+bool Ship::CanBombard() const {
+    const ShipDesign* design = Design();
+    return design && design->CanBombard();
+}
+
 float Ship::Speed() const
 { return CurrentMeterValue(METER_STARLANE_SPEED); }
 
@@ -306,7 +325,8 @@ const std::string& Ship::PublicName(int empire_id) const {
     // Disclose real ship name only to fleet owners. Rationale: a player who
     // doesn't know the design for a particular ship can easily guess it if the
     // ship's name is "Scout"
-    if (GetUniverse().AllObjectsVisible() || empire_id == ALL_EMPIRES || OwnedBy(empire_id))
+    // An exception is made for unowned monsters.
+    if (GetUniverse().AllObjectsVisible() || empire_id == ALL_EMPIRES || OwnedBy(empire_id) || (IsMonster() && Owner() == ALL_EMPIRES))
         return Name();
     const ShipDesign* design = Design();
     if (design)
@@ -318,30 +338,61 @@ const std::string& Ship::PublicName(int empire_id) const {
     else if (Unowned() && GetVisibility(empire_id) > VIS_NO_VISIBILITY)
         return UserString("FW_ROGUE_SHIP");
     else
-        return UserString("SHIP");
+        return UserString("OBJ_SHIP");
 }
 
-UniverseObject* Ship::Accept(const UniverseObjectVisitor& visitor) const {
-    return visitor.Visit(const_cast<Ship* const>(this));
-}
+TemporaryPtr<UniverseObject> Ship::Accept(const UniverseObjectVisitor& visitor) const
+{ return visitor.Visit(boost::const_pointer_cast<Ship>(boost::static_pointer_cast<const Ship>(TemporaryFromThis()))); }
 
 float Ship::NextTurnCurrentMeterValue(MeterType type) const {
+    const Meter* meter = UniverseObject::GetMeter(type);
+    if (!meter)
+        throw std::invalid_argument("Ship::NextTurnCurrentMeterValue passed meter type that the Ship does not have.");
+    float current_meter_value = meter->Current();
+
     //if (type == METER_FUEL) {
     //    // todo: consider fleet movement or being stationary, which may partly replenish fuel
     //    // todo: consider fleet passing through or being in a supplied system, which replenishes fuel
-    //} else if (type == METER_SUPPLY) {
-    //    // todo: consider fleet passing through or being in a supplied system, which replenishes supplies
     //}
+
     if (type == METER_SHIELD) {
         if (m_last_turn_active_in_combat >= CurrentTurn())
-            return std::max(0.0f,
-                            std::min(UniverseObject::GetMeter(METER_SHIELD)->Current(),
+            return std::max(0.0f,   // battle just happened. shields limited to max shield, but don't regen
+                            std::min(current_meter_value,
                                      UniverseObject::GetMeter(METER_MAX_SHIELD)->Current()));
-        else
+        else                        // shields regneerate to max shield
             return UniverseObject::GetMeter(METER_MAX_SHIELD)->Current();
     }
 
-    return UniverseObject::NextTurnCurrentMeterValue(type);
+
+    // ResourceCenter-like resource meter growth...
+
+    MeterType target_meter_type = INVALID_METER_TYPE;
+    switch (type) {
+    case METER_TARGET_INDUSTRY:
+    case METER_TARGET_RESEARCH:
+    case METER_TARGET_TRADE:
+        return current_meter_value;
+        break;
+    case METER_INDUSTRY:    target_meter_type = METER_TARGET_INDUSTRY;      break;
+    case METER_RESEARCH:    target_meter_type = METER_TARGET_RESEARCH;      break;
+    case METER_TRADE:       target_meter_type = METER_TARGET_TRADE;         break;
+    default:
+        return UniverseObject::NextTurnCurrentMeterValue(type);
+    }
+
+    const Meter* target_meter = UniverseObject::GetMeter(target_meter_type);
+    if (!target_meter)
+        throw std::runtime_error("Ship::NextTurnCurrentMeterValue dealing with invalid meter type");
+    float target_meter_value = target_meter->Current();
+
+    // meter growth or decay towards target is one per turn.
+    if (target_meter_value > current_meter_value)
+        return std::min(current_meter_value + 1.0f, target_meter_value);
+    else if (target_meter_value < current_meter_value)
+        return std::max(target_meter_value, current_meter_value - 1.0f);
+    else
+        return current_meter_value;
 }
 
 const Meter* Ship::GetPartMeter(MeterType type, const std::string& part_name) const
@@ -409,8 +460,10 @@ std::vector<float> Ship::AllWeaponsDamage() const {
 }
 
 void Ship::SetFleetID(int fleet_id) {
-    m_fleet_id = fleet_id;
-    StateChangedSignal();
+    if (m_fleet_id != fleet_id) {
+        m_fleet_id = fleet_id;
+        StateChangedSignal();
+    }
 }
 
 void Ship::Resupply() {
@@ -467,67 +520,64 @@ void Ship::SetSpecies(const std::string& species_name) {
     m_species_name = species_name;
 }
 
-void Ship::MoveTo(double x, double y) {
-    UniverseObject::MoveTo(x, y);
-
-    // if ship is being moved away from its fleet, remove from the fleet.  otherwise, keep ship in fleet.
-    if (Fleet* fleet = GetFleet(this->FleetID())) {
-        //Logger().debugStream() << "Ship::MoveTo removing " << this->ID() << " from fleet " << fleet->Name();
-        fleet->RemoveShip(this->ID());
-    }
-}
-
 void Ship::SetOrderedScrapped(bool b) {
     if (b == m_ordered_scrapped) return;
     m_ordered_scrapped = b;
     StateChangedSignal();
-    if (Fleet* fleet = GetFleet(this->FleetID()))
-        fleet->StateChangedSignal();
 }
 
 void Ship::SetColonizePlanet(int planet_id) {
     if (planet_id == m_ordered_colonize_planet_id) return;
     m_ordered_colonize_planet_id = planet_id;
     StateChangedSignal();
-    if (Fleet* fleet = GetFleet(this->FleetID()))
-        fleet->StateChangedSignal();
 }
 
-void Ship::ClearColonizePlanet() {
-    SetColonizePlanet(INVALID_OBJECT_ID);
-}
+void Ship::ClearColonizePlanet()
+{ SetColonizePlanet(INVALID_OBJECT_ID); }
 
 void Ship::SetInvadePlanet(int planet_id) {
     if (planet_id == m_ordered_invade_planet_id) return;
     m_ordered_invade_planet_id = planet_id;
     StateChangedSignal();
-    if (Fleet* fleet = GetFleet(this->FleetID()))
-        fleet->StateChangedSignal();
 }
 
-void Ship::ClearInvadePlanet() {
-    SetInvadePlanet(INVALID_OBJECT_ID);
+void Ship::ClearInvadePlanet()
+{ SetInvadePlanet(INVALID_OBJECT_ID); }
+
+void Ship::SetBombardPlanet(int planet_id) {
+    if (planet_id == m_ordered_bombard_planet_id) return;
+    m_ordered_bombard_planet_id = planet_id;
+    StateChangedSignal();
 }
 
+void Ship::ClearBombardPlanet()
+{ SetBombardPlanet(INVALID_OBJECT_ID); }
+
 void Ship::ResetTargetMaxUnpairedMeters() {
     UniverseObject::ResetTargetMaxUnpairedMeters();
 
     UniverseObject::GetMeter(METER_MAX_FUEL)->ResetCurrent();
     UniverseObject::GetMeter(METER_MAX_SHIELD)->ResetCurrent();
     UniverseObject::GetMeter(METER_MAX_STRUCTURE)->ResetCurrent();
+    UniverseObject::GetMeter(METER_TARGET_INDUSTRY)->ResetCurrent();
+    UniverseObject::GetMeter(METER_TARGET_RESEARCH)->ResetCurrent();
+    UniverseObject::GetMeter(METER_TARGET_TRADE)->ResetCurrent();
 
     UniverseObject::GetMeter(METER_DETECTION)->ResetCurrent();
     UniverseObject::GetMeter(METER_BATTLE_SPEED)->ResetCurrent();
     UniverseObject::GetMeter(METER_STARLANE_SPEED)->ResetCurrent();
 
     for (PartMeterMap::iterator it = m_part_meters.begin(); it != m_part_meters.end(); ++it)
-        it->second.ResetCurrent();
+    { it->second.ResetCurrent(); }
 }
 
 void Ship::PopGrowthProductionResearchPhase() {
     UniverseObject::PopGrowthProductionResearchPhase();
 
     UniverseObject::GetMeter(METER_SHIELD)->SetCurrent(Ship::NextTurnCurrentMeterValue(METER_SHIELD));
+    UniverseObject::GetMeter(METER_INDUSTRY)->SetCurrent(Ship::NextTurnCurrentMeterValue(METER_INDUSTRY));
+    UniverseObject::GetMeter(METER_RESEARCH)->SetCurrent(Ship::NextTurnCurrentMeterValue(METER_RESEARCH));
+    UniverseObject::GetMeter(METER_TRADE)->SetCurrent(Ship::NextTurnCurrentMeterValue(METER_TRADE));
 
     StateChangedSignal();
 }
@@ -541,6 +591,12 @@ void Ship::ClampMeters() {
     UniverseObject::GetMeter(METER_SHIELD)->ClampCurrentToRange(Meter::DEFAULT_VALUE, UniverseObject::GetMeter(METER_MAX_SHIELD)->Current());
     UniverseObject::GetMeter(METER_MAX_STRUCTURE)->ClampCurrentToRange();
     UniverseObject::GetMeter(METER_STRUCTURE)->ClampCurrentToRange(Meter::DEFAULT_VALUE, UniverseObject::GetMeter(METER_MAX_STRUCTURE)->Current());
+    UniverseObject::GetMeter(METER_TARGET_INDUSTRY)->ClampCurrentToRange();
+    UniverseObject::GetMeter(METER_INDUSTRY)->ClampCurrentToRange();
+    UniverseObject::GetMeter(METER_TARGET_RESEARCH)->ClampCurrentToRange();
+    UniverseObject::GetMeter(METER_RESEARCH)->ClampCurrentToRange();
+    UniverseObject::GetMeter(METER_TARGET_TRADE)->ClampCurrentToRange();
+    UniverseObject::GetMeter(METER_TRADE)->ClampCurrentToRange();
 
     UniverseObject::GetMeter(METER_DETECTION)->ClampCurrentToRange();
     UniverseObject::GetMeter(METER_BATTLE_SPEED)->ClampCurrentToRange();
@@ -549,3 +605,29 @@ void Ship::ClampMeters() {
     for (PartMeterMap::iterator it = m_part_meters.begin(); it != m_part_meters.end(); ++it)
         it->second.ClampCurrentToRange();
 }
+
+////////////////////
+// Free Functions //
+////////////////////
+std::string NewMonsterName() {
+    static std::vector<std::string> monster_names;
+    static std::map<std::string, int> monster_names_used;
+    if (monster_names.empty()) {
+        // load monster names from stringtable
+        std::list<std::string> monster_names_list;
+        UserStringList("MONSTER_NAMES", monster_names_list);
+
+        monster_names.reserve(monster_names_list.size());
+        std::copy(monster_names_list.begin(), monster_names_list.end(), std::back_inserter(monster_names));
+        if (monster_names.empty()) // safety check to ensure not leaving list empty in case of stringtable failure
+            monster_names.push_back(UserString("MONSTER"));
+    }
+
+    // select name randomly from list
+    int monster_name_index = RandSmallInt(0, static_cast<int>(monster_names.size()) - 1);
+    std::string result = monster_names[monster_name_index];
+    if (monster_names_used[result]++) {
+        result += " " + RomanNumber(monster_names_used[result]);
+    }
+    return result;
+}
