@@ -3,6 +3,7 @@
 #include "../util/OptionsDB.h"
 #include "../util/Directories.h"
 #include "../util/Logger.h"
+#include "../util/AppInterface.h"
 #include "../parse/Parse.h"
 #include "../Empire/Empire.h"
 #include "../Empire/EmpireManager.h"
@@ -28,7 +29,7 @@ namespace {
         PartTypeStringVisitor(std::string& str) :
             m_str(str)
         {}
-        void operator()(const double& d) const
+        void operator()(const float& d) const
         { m_str = "capacity stat"; }
         void operator()(const DirectFireStats& stats) const
         { m_str = "direct-fire weapon stats"; }
@@ -54,7 +55,7 @@ namespace {
         ValueRef::ValueRefBase<double>* vr =
             new ValueRef::Operation<double>(
                 ValueRef::PLUS,
-                new ValueRef::Variable<double>(std::vector<adobe::name_t>(1, Value_name)),
+                new ValueRef::Variable<double>(ValueRef::EFFECT_TARGET_VALUE_REFERENCE, std::vector<std::string>()),
                 new ValueRef::Constant<double>(increase)
             );
         return EffectsGroupPtr(
@@ -71,7 +72,7 @@ namespace {
         ValueRef::ValueRefBase<double>* vr =
             new ValueRef::Operation<double>(
                 ValueRef::PLUS,
-                new ValueRef::Variable<double>(std::vector<adobe::name_t>(1, Value_name)),
+                new ValueRef::Variable<double>(ValueRef::EFFECT_TARGET_VALUE_REFERENCE, std::vector<std::string>()),
                 new ValueRef::Constant<double>(increase)
             );
         std::string stacking_group = (allow_stacking ? "" :
@@ -83,56 +84,6 @@ namespace {
                 part_name, stacking_group));
     }
 
-    struct DescriptionVisitor : public boost::static_visitor<>
-    {
-        DescriptionVisitor(ShipPartClass part_class, std::string& description) :
-            m_class(part_class),
-            m_description(description)
-        {}
-        void operator()(const float& d) const {
-            const std::string& desc_string =
-                m_class == PC_FUEL || m_class == PC_COLONY
-                    ? UserString("PART_DESC_CAPACITY")
-                    : UserString("PART_DESC_STRENGTH");
-            m_description +=
-                str(FlexibleFormat(desc_string) % d);
-        }
-        void operator()(const DirectFireStats& stats) const {
-            m_description +=
-                str(FlexibleFormat(UserString("PART_DESC_DIRECT_FIRE_STATS"))
-                    % stats.m_damage
-                    % stats.m_ROF
-                    % stats.m_range);
-        }
-        void operator()(const LRStats& stats) const {
-            m_description +=
-                str(FlexibleFormat(UserString("PART_DESC_LR_STATS"))
-                    % stats.m_damage
-                    % stats.m_ROF
-                    % stats.m_range
-                    % stats.m_speed
-                    % stats.m_structure
-                    % stats.m_stealth
-                    % stats.m_capacity);
-        }
-        void operator()(const FighterStats& stats) const {
-            m_description +=
-                str(FlexibleFormat(UserString("PART_DESC_FIGHTER_STATS"))
-                    % (stats.m_type == BOMBER ? UserString("BOMBER") : UserString("INTERCEPTOR"))
-                    % stats.m_anti_ship_damage
-                    % stats.m_anti_fighter_damage
-                    % stats.m_launch_rate
-                    % stats.m_speed
-                    % stats.m_stealth
-                    % stats.m_structure
-                    % stats.m_detection
-                    % stats.m_capacity);
-        }
-
-        const ShipPartClass m_class;
-        std::string& m_description;
-    };
-
     bool DesignsTheSame(const ShipDesign& one, const ShipDesign& two) {
         return (
             one.Name()              == two.Name() &&
@@ -187,7 +138,7 @@ PartTypeManager::PartTypeManager() {
         Logger().debugStream() << "Part Types:";
         for (iterator it = begin(); it != end(); ++it) {
             const PartType* p = it->second;
-            Logger().debugStream() << " ... " << p->Name() << " class: " << p->Class() << " stats: " << p->StatDescription();
+            Logger().debugStream() << " ... " << p->Name() << " class: " << p->Class();
         }
     }
 }
@@ -309,17 +260,19 @@ FighterStats::FighterStats(CombatFighterType type,
 
 
 namespace {
-    const UniverseObject* SourceForEmpire(int empire_id) {
+    // Looks like there are at least 3 SourceForEmpire functions lying around - one in ShipDesign, one in Tech, and one in Building
+    // TODO: Eliminate duplication
+    TemporaryPtr<const UniverseObject> SourceForEmpire(int empire_id) {
         const Empire* empire = Empires().Lookup(empire_id);
         if (!empire) {
             Logger().debugStream() << "SourceForEmpire: Unable to get empire with ID: " << empire_id;
-            return 0;
+            return TemporaryPtr<const UniverseObject>();
         }
         // get a source object, which is owned by the empire with the passed-in
         // empire id.  this is used in conditions to reference which empire is
         // doing the building.  Ideally this will be the capital, but any object
         // owned by the empire will work.
-        const UniverseObject* source = GetUniverseObject(empire->CapitalID());
+        TemporaryPtr<const UniverseObject> source = GetUniverseObject(empire->CapitalID());
         // no capital?  scan through all objects to find one owned by this empire
         if (!source) {
             for (ObjectMap::const_iterator<> obj_it = Objects().const_begin(); obj_it != Objects().const_end(); ++obj_it) {
@@ -435,24 +388,31 @@ void PartType::Init(const std::vector<boost::shared_ptr<const Effect::EffectsGro
     case PC_STARLANE_SPEED:
         if (boost::get<float>(m_stats) != 0)
             m_effects.push_back(IncreaseMeter(METER_STARLANE_SPEED, boost::get<float>(m_stats)));
+        break;
+    case PC_RESEARCH:
+        if (boost::get<float>(m_stats) != 0)
+            m_effects.push_back(IncreaseMeter(METER_TARGET_RESEARCH,boost::get<float>(m_stats)));
+        break;
+    case PC_INDUSTRY:
+        if (boost::get<float>(m_stats) != 0)
+            m_effects.push_back(IncreaseMeter(METER_TARGET_INDUSTRY,boost::get<float>(m_stats)));
+        break;
+    case PC_TRADE:
+        if (boost::get<float>(m_stats) != 0)
+            m_effects.push_back(IncreaseMeter(METER_TARGET_TRADE,   boost::get<float>(m_stats)));
+        break;
     default:
         break;
     }
 
-    for (std::vector<boost::shared_ptr<const Effect::EffectsGroup> >::const_iterator it = effects.begin();
-         it != effects.end(); ++it)
+    for (std::vector<boost::shared_ptr<const Effect::EffectsGroup> >::const_iterator
+         it = effects.begin(); it != effects.end(); ++it)
     { m_effects.push_back(*it); }
 }
 
 PartType::~PartType()
 { delete m_location; }
 
-std::string PartType::StatDescription() const {
-    std::string retval;
-    boost::apply_visitor(DescriptionVisitor(m_class, retval), m_stats);
-    return retval;
-}
-
 bool PartType::CanMountInSlotType(ShipSlotType slot_type) const {
     if (INVALID_SHIP_SLOT_TYPE == slot_type)
         return false;
@@ -462,21 +422,34 @@ bool PartType::CanMountInSlotType(ShipSlotType slot_type) const {
     return false;
 }
 
-double PartType::ProductionCost(int empire_id, int location_id) const {
+bool PartType::ProductionCostTimeLocationInvariant() const {
+    if (CHEAP_AND_FAST_SHIP_PRODUCTION)
+        return true;
+    if (m_production_cost && !m_production_cost->TargetInvariant())
+        return false;
+    if (m_production_time && !m_production_time->TargetInvariant())
+        return false;
+    return true;
+}
+
+float PartType::ProductionCost(int empire_id, int location_id) const {
     if (CHEAP_AND_FAST_SHIP_PRODUCTION || !m_production_cost) {
-        return 1.0;
+        return 1.0f;
     } else {
         if (ValueRef::ConstantExpr(m_production_cost))
-            return m_production_cost->Eval();
+            return static_cast<float>(m_production_cost->Eval());
 
-        UniverseObject* location = GetUniverseObject(location_id);
+        TemporaryPtr<UniverseObject> location = GetUniverseObject(location_id);
         if (!location)
-            return 999999.9;    // arbitrary large number
+            return 999999.9f;    // arbitrary large number
+
+        TemporaryPtr<const UniverseObject> source = SourceForEmpire(empire_id);
+        if (!source && !m_production_cost->SourceInvariant())
+            return 999999.9f;
 
-        const UniverseObject* source = SourceForEmpire(empire_id);
         ScriptingContext context(source, location);
 
-        return m_production_cost->Eval(context);
+        return static_cast<float>(m_production_cost->Eval(context));
     }
 }
 
@@ -487,11 +460,14 @@ int PartType::ProductionTime(int empire_id, int location_id) const {
         if (ValueRef::ConstantExpr(m_production_time))
             return m_production_time->Eval();
 
-        UniverseObject* location = GetUniverseObject(location_id);
+        TemporaryPtr<UniverseObject> location = GetUniverseObject(location_id);
         if (!location)
             return 9999;    // arbitrary large number
 
-        const UniverseObject* source = SourceForEmpire(empire_id);
+        TemporaryPtr<const UniverseObject> source = SourceForEmpire(empire_id);
+        if (!source && !m_production_time->SourceInvariant())
+            return 9999;
+
         ScriptingContext context(source, location);
 
         return m_production_time->Eval(context);
@@ -522,16 +498,6 @@ void HullType::Init(const std::vector<boost::shared_ptr<const Effect::EffectsGro
 HullType::~HullType()
 { delete m_location; }
 
-std::string HullType::StatDescription() const {
-    std::string retval = 
-        str(FlexibleFormat(UserString("HULL_DESC"))
-            % m_starlane_speed
-            % m_fuel
-            % m_battle_speed
-            % m_structure);
-    return retval;
-}
-
 unsigned int HullType::NumSlots(ShipSlotType slot_type) const {
     unsigned int count = 0;
     for (std::vector<Slot>::const_iterator it = m_slots.begin(); it != m_slots.end(); ++it)
@@ -540,21 +506,37 @@ unsigned int HullType::NumSlots(ShipSlotType slot_type) const {
     return count;
 }
 
-double HullType::ProductionCost(int empire_id, int location_id) const {
+// HullType:: and PartType::ProductionCost and ProductionTime are almost identical.
+// Chances are, the same is true of buildings and techs as well.
+// TODO: Eliminate duplication
+bool HullType::ProductionCostTimeLocationInvariant() const {
+    if (CHEAP_AND_FAST_SHIP_PRODUCTION)
+        return true;
+    if (m_production_cost && !m_production_cost->LocalCandidateInvariant())
+        return false;
+    if (m_production_time && !m_production_time->LocalCandidateInvariant())
+        return false;
+    return true;
+}
+
+float HullType::ProductionCost(int empire_id, int location_id) const {
     if (CHEAP_AND_FAST_SHIP_PRODUCTION || !m_production_cost) {
-        return 1.0;
+        return 1.0f;
     } else {
         if (ValueRef::ConstantExpr(m_production_cost))
-            return m_production_cost->Eval();
+            return static_cast<float>(m_production_cost->Eval());
 
-        UniverseObject* location = GetUniverseObject(location_id);
+        TemporaryPtr<UniverseObject> location = GetUniverseObject(location_id);
         if (!location)
-            return 999999.9;    // arbitrary large number
+            return 999999.9f;    // arbitrary large number
+
+        TemporaryPtr<const UniverseObject> source = SourceForEmpire(empire_id);
+        if (!source && !m_production_cost->SourceInvariant())
+            return 999999.9f;
 
-        const UniverseObject* source = SourceForEmpire(empire_id);
         ScriptingContext context(source, location);
 
-        return m_production_cost->Eval(context);
+        return static_cast<float>(m_production_cost->Eval(context));
     }
 }
 
@@ -565,11 +547,14 @@ int HullType::ProductionTime(int empire_id, int location_id) const {
         if (ValueRef::ConstantExpr(m_production_time))
             return m_production_time->Eval();
 
-        UniverseObject* location = GetUniverseObject(location_id);
+        TemporaryPtr<UniverseObject> location = GetUniverseObject(location_id);
         if (!location)
             return 9999;    // arbitrary large number
 
-        const UniverseObject* source = SourceForEmpire(empire_id);
+        TemporaryPtr<const UniverseObject> source = SourceForEmpire(empire_id);
+        if (!source && !m_production_time->SourceInvariant())
+            return 999999;
+
         ScriptingContext context(source, location);
 
         return m_production_time->Eval(context);
@@ -594,7 +579,7 @@ HullTypeManager::HullTypeManager() {
         Logger().debugStream() << "Hull Types:";
         for (iterator it = begin(); it != end(); ++it) {
             const HullType* h = it->second;
-            Logger().debugStream() << " ... " << h->Name();// << " stats: " << h->StatDescription();
+            Logger().debugStream() << " ... " << h->Name();
         }
     }
 }
@@ -640,6 +625,7 @@ ShipDesign::ShipDesign() :
     m_3D_model(),
     m_name_desc_in_stringtable(false),
     m_is_armed(false),
+    m_can_bombard(false),
     m_detection(0.0),
     m_colony_capacity(0.0),
     m_troop_capacity(0.0),
@@ -649,6 +635,10 @@ ShipDesign::ShipDesign() :
     m_structure(0.0),
     m_battle_speed(0.0),
     m_starlane_speed(0.0),
+    m_research_generation(0.0),
+    m_industry_generation(0.0),
+    m_trade_generation(0.0),
+    m_is_production_location(false),
     m_min_SR_range(FLT_MAX),
     m_max_SR_range(0.0),
     m_min_LR_range(FLT_MAX),
@@ -662,13 +652,15 @@ ShipDesign::ShipDesign() :
 {}
 
 ShipDesign::ShipDesign(const std::string& name, const std::string& description,
-                       int designed_on_turn, const std::string& hull, const std::vector<std::string>& parts,
+                       int designed_on_turn, int designed_by_empire, const std::string& hull,
+                       const std::vector<std::string>& parts,
                        const std::string& icon, const std::string& model,
                        bool name_desc_in_stringtable, bool monster) :
     m_id(INVALID_OBJECT_ID),
     m_name(name),
     m_description(description),
     m_designed_on_turn(designed_on_turn),
+    m_designed_by_empire(designed_by_empire),
     m_hull(hull),
     m_parts(parts),
     m_is_monster(monster),
@@ -676,6 +668,7 @@ ShipDesign::ShipDesign(const std::string& name, const std::string& description,
     m_3D_model(model),
     m_name_desc_in_stringtable(name_desc_in_stringtable),
     m_is_armed(false),
+    m_can_bombard(false),
     m_detection(0.0),
     m_colony_capacity(0.0),
     m_troop_capacity(0.0),
@@ -685,6 +678,10 @@ ShipDesign::ShipDesign(const std::string& name, const std::string& description,
     m_structure(0.0),
     m_battle_speed(0.0),
     m_starlane_speed(0.0),
+    m_research_generation(0.0),
+    m_industry_generation(0.0),
+    m_trade_generation(0.0),
+    m_is_production_location(false),
     m_min_SR_range(FLT_MAX),
     m_max_SR_range(0.0),
     m_min_LR_range(FLT_MAX),
@@ -716,6 +713,12 @@ const std::string& ShipDesign::Name(bool stringtable_lookup /* = true */) const
         return m_name;
 }
 
+void ShipDesign::SetName(const std::string& name) {
+    if (m_name != "") {
+        m_name = name;
+    }
+}
+
 const std::string& ShipDesign::Description(bool stringtable_lookup /* = true */) const {
     if (m_name_desc_in_stringtable && stringtable_lookup)
         return UserString(m_description);
@@ -723,21 +726,46 @@ const std::string& ShipDesign::Description(bool stringtable_lookup /* = true */)
         return m_description;
 }
 
-double ShipDesign::ProductionCost(int empire_id, int location_id) const {
+void ShipDesign::SetDescription(const std::string& description) {
+    if (m_description != "") {
+        m_description = description;
+    }
+}
+
+bool ShipDesign::ProductionCostTimeLocationInvariant() const {
+    if (CHEAP_AND_FAST_SHIP_PRODUCTION)
+        return true;
+    // as seen in ShipDesign::ProductionCost, the location is passed as the
+    // local candidate in the ScriptingContext
+
+    // check hull and all parts
+    if (const HullType* hull = GetHullType(m_hull))
+        if (!hull->ProductionCostTimeLocationInvariant())
+            return false;
+    for (std::vector<std::string>::const_iterator it = m_parts.begin(); it != m_parts.end(); ++it)
+        if (const PartType* part = GetPartType(*it))
+            if (!part->ProductionCostTimeLocationInvariant())
+                return false;
+
+    // if hull and all parts are invariant, so is whole design
+    return true;
+}
+
+float ShipDesign::ProductionCost(int empire_id, int location_id) const {
     if (CHEAP_AND_FAST_SHIP_PRODUCTION) {
-        return 1.0;
+        return 1.0f;
     } else {
-        double cost_accumulator = 0.0;
+        float cost_accumulator = 0.0f;
         if (const HullType* hull = GetHullType(m_hull))
             cost_accumulator += hull->ProductionCost(empire_id, location_id);
         for (std::vector<std::string>::const_iterator it = m_parts.begin(); it != m_parts.end(); ++it)
             if (const PartType* part = GetPartType(*it))
                 cost_accumulator += part->ProductionCost(empire_id, location_id);
-        return std::max(0.0, cost_accumulator);
+        return std::max(0.0f, cost_accumulator);
     }
 }
 
-double ShipDesign::PerTurnCost(int empire_id, int location_id) const
+float ShipDesign::PerTurnCost(int empire_id, int location_id) const
 { return ProductionCost(empire_id, location_id) / std::max(1, ProductionTime(empire_id, location_id)); }
 
 int ShipDesign::ProductionTime(int empire_id, int location_id) const {
@@ -772,7 +800,7 @@ bool ShipDesign::CanColonize() const {
 //// TEMPORARY
 float ShipDesign::Defense() const {
     // accumulate defense from defensive parts in design.
-    double total_defense = 0.0;
+    float total_defense = 0.0f;
     const PartTypeManager& part_manager = GetPartTypeManager();
     std::vector<std::string> all_parts = Parts();
     for (std::vector<std::string>::const_iterator it = all_parts.begin(); it != all_parts.end(); ++it) {
@@ -823,13 +851,13 @@ std::vector<std::string> ShipDesign::Parts(ShipSlotType slot_type) const {
 }
 
 bool ShipDesign::ProductionLocation(int empire_id, int location_id) const {
-    const UniverseObject* location = GetUniverseObject(location_id);
+    TemporaryPtr<const UniverseObject> location = GetUniverseObject(location_id);
     if (!location)
         return false;
 
     // currently ships can only be built at planets, and by species that are
     // not planetbound
-    const Planet* planet = universe_object_cast<const Planet*>(location);
+    TemporaryPtr<const Planet> planet = boost::dynamic_pointer_cast<const Planet>(location);
     if (!planet)
         return false;
     const std::string& species_name = planet->SpeciesName();
@@ -854,7 +882,7 @@ bool ShipDesign::ProductionLocation(int empire_id, int location_id) const {
     // empire id.  this is used in conditions to reference which empire is
     // doing the producing.  Ideally this will be the capital, but any object
     // owned by the empire will work.
-    const UniverseObject* source = SourceForEmpire(empire_id);
+    TemporaryPtr<const UniverseObject> source = SourceForEmpire(empire_id);
     // if this empire doesn't own ANYTHING, then how is it producing anyway?
     if (!source)
         return false;
@@ -890,7 +918,7 @@ void ShipDesign::SetID(int id)
 { m_id = id; }
 
 bool ShipDesign::ValidDesign(const std::string& hull, const std::vector<std::string>& parts) {
-    // ensure hull type exists and has exactly enough slots for passed parts
+    // ensure hull type exists and has at least enough slots for passed parts
     const HullType* hull_type = GetHullTypeManager().GetHullType(hull);
     if (!hull_type) {
         Logger().debugStream() << "ShipDesign::ValidDesign: hull not found: " << hull;
@@ -1026,6 +1054,21 @@ void ShipDesign::BuildStatCaches() {
         case PC_DETECTION:
             m_detection += boost::get<float>(part->Stats());
             break;
+        case PC_BOMBARD:
+            m_can_bombard = true;
+            break;
+        case PC_RESEARCH:
+            m_research_generation += boost::get<float>(part->Stats());
+            break;
+        case PC_INDUSTRY:
+            m_industry_generation += boost::get<float>(part->Stats());
+            break;
+        case PC_TRADE:
+            m_trade_generation += boost::get<float>(part->Stats());
+            break;
+        case PC_PRODICTION_LOCATION:
+            m_is_production_location = true;
+            break;
         default:
             break;
         }
@@ -1048,7 +1091,6 @@ std::string ShipDesign::Dump() const {
     ++g_indent;
     retval += DumpIndent() + "name = \"" + m_name + "\"\n";
     retval += DumpIndent() + "description = \"" + m_description + "\"\n";
-    retval += DumpIndent() + "lookup_strings = " + (m_name_desc_in_stringtable ? "true" : "false") + "\n";
     retval += DumpIndent() + "hull = \"" + m_hull + "\"\n";
     retval += DumpIndent() + "parts = ";
     if (m_parts.empty()) {
@@ -1070,6 +1112,24 @@ std::string ShipDesign::Dump() const {
     return retval; 
 }
 
+bool operator ==(const ShipDesign& first, const ShipDesign& second) {
+    if (first.Hull() != second.Hull())
+        return false;
+
+    std::map<std::string, int> first_parts;
+    std::map<std::string, int> second_parts;
+
+    for (std::vector<std::string>::const_iterator it = first.Parts().begin();
+         it != first.Parts().end(); ++it)
+    { ++first_parts[*it]; }
+
+    for (std::vector<std::string>::const_iterator it = second.Parts().begin();
+         it != second.Parts().end(); ++it)
+    { ++second_parts[*it]; }
+
+    return first_parts == second_parts;
+}
+
 /////////////////////////////////////
 // PredefinedShipDesignManager     //
 /////////////////////////////////////
@@ -1180,7 +1240,7 @@ namespace {
 
         // duplicate design to add to Universe
         ShipDesign* copy = new ShipDesign(design->Name(false), design->Description(false),
-                                          design->DesignedOnTurn(), design->Hull(), design->Parts(),
+                                          design->DesignedOnTurn(), design->DesignedByEmpire(), design->Hull(), design->Parts(),
                                           design->Icon(), design->Model(), true, monster);
         if (!copy) {
             Logger().errorStream() << "PredefinedShipDesignManager::AddShipDesignsToUniverse() couldn't duplicate the design with name " << design->Name();
