@@ -98,7 +98,7 @@ struct FO_COMMON_API FighterStats {
     float               m_anti_ship_damage;
     float               m_anti_fighter_damage;
     float               m_launch_rate;
-    float              m_fighter_weapon_range;
+    float               m_fighter_weapon_range;
     float               m_speed;
     float               m_stealth;
     float               m_structure;
@@ -122,7 +122,7 @@ struct FO_COMMON_API FighterStats {
 
 /** A variant type containing all ShipPartClass-specific stats for a PartType.
   * Note that most parts need only a single value to represent their
-  * capabilities.  This is represented by the double variant. */
+  * capabilities.  This is represented by the float variant. */
 typedef boost::variant<float, DirectFireStats, LRStats, FighterStats> PartTypeStats;
 
 /** Common parameters for PartType and HullType constructors.  Used as temporary
@@ -203,17 +203,21 @@ public:
     /** \name Accessors */ //@{
     const std::string&      Name() const            { return m_name; };             ///< returns name of part
     const std::string&      Description() const     { return m_description; }       ///< returns description, including a description of the stats and effects of this part
-    std::string             StatDescription() const;                                ///< returns autogenerated list of various stats associated with part, such as capacity or power, or more type specific stats like range or firing rate
     ShipPartClass           Class() const           { return m_class; }             ///< returns that class of part that this is.
     const PartTypeStats&    Stats() const           { return m_stats; }             ///< returns how good the part is at its function.  might be weapon or shield strength, or cargo hold capacity
     bool                    CanMountInSlotType(ShipSlotType slot_type) const;       ///< returns true if this part can be placed in a slot of the indicated type
-    double                  ProductionCost(int empire_id, int location_id) const;   ///< returns the number of production points required to produce this part
+    const std::vector<ShipSlotType>&
+                            MountableSlotTypes() const  { return m_mountable_slot_types; }
+
+    bool                    ProductionCostTimeLocationInvariant() const;            ///< returns true if the production cost and time are invariant (does not depend on) the location
+    float                   ProductionCost(int empire_id, int location_id) const;   ///< returns the number of production points required to produce this part
     int                     ProductionTime(int empire_id, int location_id) const;   ///< returns the number of turns required to produce this part
     bool                    Producible() const      { return m_producible; }        ///< returns whether this part type is producible by players and appears on the design screen
-    const std::set<std::string>& Tags() const    { return m_tags; }
+
+    const std::set<std::string>& Tags() const       { return m_tags; }
     const Condition::ConditionBase* Location() const{ return m_location; }          ///< returns the condition that determines the locations where ShipDesign containing part can be produced
     const std::vector<boost::shared_ptr<const Effect::EffectsGroup> >& Effects() const
-    { return m_effects; }                                                           ///< returns the EffectsGroups that encapsulate the effects this part has
+                                                    { return m_effects; }           ///< returns the EffectsGroups that encapsulate the effects this part has
     const std::string&      Icon() const            { return m_icon; }              ///< returns icon graphic that represents part in UI
     //@}
 
@@ -278,18 +282,18 @@ FO_COMMON_API const PartType* GetPartType(const std::string& name);
   * parsed main item. */
 struct HullTypeStats {
     HullTypeStats() :
-        fuel(0.0),
-        battle_speed(0.0),
-        starlane_speed(0.0),
-        stealth(0.0),
-        structure(0.0)
+        fuel(0.0f),
+        battle_speed(0.0f),
+        starlane_speed(0.0f),
+        stealth(0.0f),
+        structure(0.0f)
     {}
 
-    HullTypeStats(double fuel_,
-                  double battle_speed_,
-                  double starlane_speed_,
-                  double stealth_,
-                  double structure_) :
+    HullTypeStats(float fuel_,
+                  float battle_speed_,
+                  float starlane_speed_,
+                  float stealth_,
+                  float structure_) :
         fuel(fuel_),
         battle_speed(battle_speed_),
         starlane_speed(starlane_speed_),
@@ -297,11 +301,11 @@ struct HullTypeStats {
         structure(structure_)
     {}
 
-    double  fuel;
-    double  battle_speed;
-    double  starlane_speed;
-    double  stealth;
-    double  structure;
+    float   fuel;
+    float   battle_speed;
+    float   starlane_speed;
+    float   stealth;
+    float   structure;
 
     template <class Archive>
     void serialize(Archive& ar, const unsigned int) {
@@ -333,11 +337,11 @@ public:
     HullType() :
         m_name("generic hull type"),
         m_description("indescribable"),
-        m_battle_speed(1.0),
-        m_starlane_speed(1.0),
-        m_fuel(0.0),
-        m_stealth(0.0),
-        m_structure(0.0),
+        m_battle_speed(1.0f),
+        m_starlane_speed(1.0f),
+        m_fuel(0.0f),
+        m_stealth(0.0f),
+        m_structure(0.0f),
         m_production_cost(0),
         m_production_time(0),
         m_producible(false),
@@ -350,8 +354,8 @@ public:
     {}
 
     HullType(const std::string& name, const std::string& description,
-             double fuel, double battle_speed, double starlane_speed,
-             double stealth, double structure,
+             float fuel, float battle_speed, float starlane_speed,
+             float stealth, float structure,
              const PartHullCommonParams& common_params,
              const std::vector<Slot>& slots,
              const std::string& graphic) :
@@ -402,19 +406,19 @@ public:
     /** \name Accessors */ //@{
     const std::string&  Name() const            { return m_name; }              ///< returns name of hull
     const std::string&  Description() const     { return m_description; }       ///< returns description, including a description of the stats and effects of this hull
-    std::string         StatDescription() const;                                ///< returns autogenerated list of various stats associated with part, such as speeds or fuel capacity
 
-    double              BattleSpeed() const     { return m_battle_speed; }      ///< returns battle speed of hull
-    double              StarlaneSpeed() const   { return m_starlane_speed; }    ///< returns starlane speed of hull
-    double              Fuel() const            { return m_fuel; }              ///< returns fuel capacity of hull
-    double              Stealth() const         { return m_stealth; }           ///< returns stealth of hull
-    double              Structure() const       { return m_structure; }         ///< returns structure of hull
-    double              Shields() const         { return 0.0; }                 ///< returns shields of hull
-    double              ColonyCapacity() const  { return 0.0; }                 ///< returns colonist capacity of hull
-    double              TroopCapacity() const   { return 0.0; }                 ///< returns the troop capacity of hull
-    double              Detection() const       { return 0.0; }                 ///< returns detection ability of hull
+    float               BattleSpeed() const     { return m_battle_speed; }      ///< returns battle speed of hull
+    float               StarlaneSpeed() const   { return m_starlane_speed; }    ///< returns starlane speed of hull
+    float               Fuel() const            { return m_fuel; }              ///< returns fuel capacity of hull
+    float               Stealth() const         { return m_stealth; }           ///< returns stealth of hull
+    float               Structure() const       { return m_structure; }         ///< returns structure of hull
+    float               Shields() const         { return 0.0f; }                ///< returns shields of hull
+    float               ColonyCapacity() const  { return 0.0f; }                ///< returns colonist capacity of hull
+    float               TroopCapacity() const   { return 0.0f; }                ///< returns the troop capacity of hull
+    float               Detection() const       { return 0.0f; }                ///< returns detection ability of hull
 
-    double              ProductionCost(int empire_id, int location_id) const;   ///< returns the number of production points required to produce this hull
+    bool                ProductionCostTimeLocationInvariant() const;            ///< returns true if the production cost and time are invariant (does not depend on) the location
+    float               ProductionCost(int empire_id, int location_id) const;   ///< returns the number of production points required to produce this hull
     int                 ProductionTime(int empire_id, int location_id) const;   ///< returns the number of turns required to produce this hull
     bool                Producible() const      { return m_producible; }        ///< returns whether this hull type is producible by players and appears on the design screen
 
@@ -438,11 +442,11 @@ private:
 
     std::string                             m_name;
     std::string                             m_description;
-    double                                  m_battle_speed;
-    double                                  m_starlane_speed;
-    double                                  m_fuel;
-    double                                  m_stealth;
-    double                                  m_structure;
+    float                                   m_battle_speed;
+    float                                   m_starlane_speed;
+    float                                   m_fuel;
+    float                                   m_stealth;
+    float                                   m_structure;
     const ValueRef::ValueRefBase<double>*   m_production_cost;
     const ValueRef::ValueRefBase<int>*      m_production_time;
     bool                                    m_producible;
@@ -493,13 +497,13 @@ FO_COMMON_API const HullTypeManager& GetHullTypeManager();
   * 0 is returned instead. */
 FO_COMMON_API const HullType* GetHullType(const std::string& name);
 
-
 class FO_COMMON_API ShipDesign {
 public:
     /** \name Structors */ //@{
     ShipDesign();
     ShipDesign(const std::string& name, const std::string& description,
-               int designed_on_turn, const std::string& hull, const std::vector<std::string>& parts,
+               int designed_on_turn, int designed_by_empire, const std::string& hull,
+               const std::vector<std::string>& parts,
                const std::string& icon, const std::string& model,
                bool name_desc_in_stringtable = false, bool monster = false);
     //@}
@@ -511,22 +515,26 @@ public:
       * the name string is looked up in the stringtable before being returned.
       * otherwise, the raw name string is returned. */
     const std::string&              Name(bool stringtable_lookup = true) const;
+    void                            SetName(const std::string& name);
 
     /** returns description of design.  if \a stringtable_lookup is true and
       * the design was constructed specifying name_desc_in_stringtable true,
       * the description string is looked up in the stringtable before being
       * returned.  otherwise, the raw name string is returned. */
     const std::string&              Description(bool stringtable_lookup = true) const;
+    void                            SetDescription(const std::string& description);
 
     int                             DesignedOnTurn() const  { return m_designed_on_turn; };     ///< returns turn on which design was created
+    int                             DesignedByEmpire() const { return m_designed_by_empire; };  ///< returns id of empire that created this design
 
-    double                          ProductionCost(int empire_id, int location_id) const;       ///< returns the total cost to build a ship of this design
-    double                          PerTurnCost(int empire_id, int location_id) const;          ///< returns the maximum per-turn number of production points that can be spent on building a ship of this design
+    bool                            ProductionCostTimeLocationInvariant() const;                ///< returns true if the production cost and time are invariant (does not depend on) the location
+    float                           ProductionCost(int empire_id, int location_id) const;       ///< returns the total cost to build a ship of this design
+    float                           PerTurnCost(int empire_id, int location_id) const;          ///< returns the maximum per-turn number of production points that can be spent on building a ship of this design
     int                             ProductionTime(int empire_id, int location_id) const;       ///< returns the time in turns it takes to build a ship of this design
     bool                            Producible() const      { return m_producible; }            ///< returns whether this design is producible by players and appears on the production screen list
 
-    double                          BattleSpeed() const     { return m_battle_speed; }          ///< returns design speed on the battle map
-    double                          StarlaneSpeed() const   { return m_starlane_speed; }        ///< returns design speed along starlanes
+    float                           BattleSpeed() const     { return m_battle_speed; }          ///< returns design speed on the battle map
+    float                           StarlaneSpeed() const   { return m_starlane_speed; }        ///< returns design speed along starlanes
 
     float                           Structure() const       { return m_structure; }             ///< returns the max structure of this design
     float                           Shields() const         { return m_shields; }               ///< returns the max shields of this design
@@ -536,8 +544,14 @@ public:
     float                           TroopCapacity() const   { return m_troop_capacity; }        ///< returns the troop capacity of this design
     float                           Stealth() const         { return m_stealth; }               ///< returns the stealth of this design
 
+    float                           IndustryGeneration() const { return m_industry_generation; }///< returns the industry output from this ship design
+    float                           ResearchGeneration() const { return m_research_generation; }///< returns the research output from this ship design
+    float                           TradeGeneration() const    { return m_trade_generation; }   ///< returns the trade output from this ship design
+    bool                            IsProductionLocation() const { return m_is_production_location;}///< returns true if this ship design can be a production location
+
     bool                            CanColonize() const;
     bool                            HasTroops() const       { return (m_troop_capacity > 0.0f); }
+    bool                            CanBombard() const      { return m_can_bombard; }
     bool                            IsArmed() const         { return m_is_armed; }
     bool                            IsMonster() const       { return m_is_monster; }
 
@@ -579,6 +593,8 @@ public:
     bool                            LookupInStringtable() const { return m_name_desc_in_stringtable; }
 
     std::string                     Dump() const;                                   ///< returns a data file format representation of this object
+
+    friend bool operator ==(const ShipDesign& first, const ShipDesign& second);
     //@}
 
     bool                            ProductionLocation(int empire_id, int location_id) const;   ///< returns true iff the empire with ID empire_id can produce this design at the location with location_id
@@ -608,6 +624,7 @@ private:
     std::string                 m_description;
 
     int                         m_designed_on_turn;
+    int                         m_designed_by_empire;
 
     std::string                 m_hull;
     std::vector<std::string>    m_parts;
@@ -621,6 +638,7 @@ private:
     // Note that these are fine to compute on demand and cache here -- it is
     // not necessary to serialize them.
     bool    m_is_armed;
+    bool    m_can_bombard;
     float   m_detection;
     float   m_colony_capacity;
     float   m_troop_capacity;
@@ -630,11 +648,16 @@ private:
     float   m_structure;
     float   m_battle_speed;
     float   m_starlane_speed;
+    float   m_research_generation;
+    float   m_industry_generation;
+    float   m_trade_generation;
+    bool    m_is_production_location;
+
     bool    m_producible;
     std::multimap<float, const PartType*>  m_SR_weapons;
     std::multimap<float, const PartType*>  m_LR_weapons;
     std::multimap<float, const PartType*>  m_PD_weapons;
-    std::vector<const PartType*>            m_F_weapons;
+    std::vector<const PartType*>           m_F_weapons;
     float   m_min_SR_range;
     float   m_max_SR_range;
     float   m_min_LR_range;
@@ -651,6 +674,9 @@ private:
     void serialize(Archive& ar, const unsigned int version);
 };
 
+///< Returns true if the two designs have the same hull and parts.
+FO_COMMON_API bool operator ==(const ShipDesign& first, const ShipDesign& second);
+
 /** Returns the ShipDesign specification object with id \a ship_design_id.  If
   * no such ShipDesign is present in the Universe (because it doesn't exist,
   * or isn't know to this client), 0 is returned instead. */
