@@ -7,6 +7,7 @@
 #include "../util/OptionsDB.h"
 #include "../util/Directories.h"
 #include "../util/Logger.h"
+#include "../util/AppInterface.h"
 #include "../Empire/Empire.h"
 #include "../Empire/EmpireManager.h"
 #include "ValueRef.h"
@@ -56,10 +57,10 @@ namespace {
         if (next_techs.empty())
             return 0;
 
-        double min_price = next_techs[0]->ResearchCost(empire_id);
+        float min_price = next_techs[0]->ResearchCost(empire_id);
         int min_index = 0;
         for (unsigned int i = 0; i < next_techs.size(); ++i) {
-            double price = next_techs[i]->ResearchCost(empire_id);
+            float price = next_techs[i]->ResearchCost(empire_id);
             if (price < min_price) {
                 min_price = price;
                 min_index = i;
@@ -143,17 +144,17 @@ std::string Tech::Dump() const {
 }
 
 namespace {
-    const UniverseObject* SourceForEmpire(int empire_id) {
+    TemporaryPtr<const UniverseObject> SourceForEmpire(int empire_id) {
         const Empire* empire = Empires().Lookup(empire_id);
         if (!empire) {
             Logger().debugStream() << "SourceForEmpire: Unable to get empire with ID: " << empire_id;
-            return 0;
+            return TemporaryPtr<const UniverseObject>();
         }
         // get a source object, which is owned by the empire with the passed-in
         // empire id.  this is used in conditions to reference which empire is
         // doing the building.  Ideally this will be the capital, but any object
         // owned by the empire will work.
-        const UniverseObject* source = GetUniverseObject(empire->CapitalID());
+        TemporaryPtr<const UniverseObject> source = GetUniverseObject(empire->CapitalID());
         // no capital?  scan through all objects to find one owned by this empire
         if (!source) {
             for (ObjectMap::const_iterator<> obj_it = Objects().const_begin(); obj_it != Objects().const_end(); ++obj_it) {
@@ -167,27 +168,23 @@ namespace {
     }
 }
 
-double Tech::ResearchCost(int empire_id) const {
+float Tech::ResearchCost(int empire_id) const {
     if (CHEAP_AND_FAST_TECH_RESEARCH || !m_research_cost) {
         return 1.0;
     } else {
         if (ValueRef::ConstantExpr(m_research_cost))
             return m_research_cost->Eval();
 
-        const UniverseObject* source = SourceForEmpire(empire_id);
+        TemporaryPtr<const UniverseObject> source = SourceForEmpire(empire_id);
+        if (!source && !m_research_cost->SourceInvariant())
+            return 999999.9f;
 
-        if (source || m_research_cost->SourceInvariant()) {
-            ScriptingContext context(source);
-            return m_research_cost->Eval(context);
-
-        } else {
-            // if cost depends on a source object, but no such object is present, default to arbitrary large value
-            return 9999999.9;
-        }
+        ScriptingContext context(source);
+        return m_research_cost->Eval(context);
     }
 }
 
-double Tech::PerTurnCost(int empire_id) const
+float Tech::PerTurnCost(int empire_id) const
 { return ResearchCost(empire_id) / std::max(1, ResearchTime(empire_id)); }
 
 int Tech::ResearchTime(int empire_id) const {
@@ -197,7 +194,10 @@ int Tech::ResearchTime(int empire_id) const {
         if (ValueRef::ConstantExpr(m_research_turns))
             return m_research_turns->Eval();
 
-        const UniverseObject* source = SourceForEmpire(empire_id);
+        TemporaryPtr<const UniverseObject> source = SourceForEmpire(empire_id);
+        if (!source && !m_research_turns->SourceInvariant())
+            return 9999;
+
         ScriptingContext context(source);
 
         return m_research_turns->Eval(context);
@@ -530,7 +530,7 @@ TechManager& TechManager::GetTechManager() {
     return manager;
 }
 
-std::vector<std::string> TechManager::RecursivePrereqs(const std::string& tech_name, int empire_id) const {
+std::vector<std::string> TechManager::RecursivePrereqs(const std::string& tech_name, int empire_id, bool min_required /*= true*/) const {
     const Tech* tech = this->GetTech(tech_name);
     if (!tech)
         return std::vector<std::string>();
@@ -538,11 +538,12 @@ std::vector<std::string> TechManager::RecursivePrereqs(const std::string& tech_n
     // compile set of recursive prereqs
     std::list<std::string> prereqs_list;                    // working list of prereqs as being processed.  may contain duplicates
     std::set<std::string> prereqs_set;                      // set of (unique) prereqs leading to tech
-    std::multimap<double, std::string> techs_to_add_map;    // indexed and sorted by cost per turn
+    std::multimap<float, std::string> techs_to_add_map;    // indexed and sorted by cost per turn
 
     // initialize working list with 1st order prereqs
     std::set<std::string> cur_prereqs = tech->Prerequisites();
     std::copy(cur_prereqs.begin(), cur_prereqs.end(), std::back_inserter(prereqs_list));
+    const Empire* empire = Empires().Lookup(empire_id);
 
     // traverse list, appending new prereqs to it, and putting unique prereqs into set
     for (std::list<std::string>::iterator it = prereqs_list.begin(); it != prereqs_list.end(); ++it) {
@@ -552,10 +553,14 @@ std::vector<std::string> TechManager::RecursivePrereqs(const std::string& tech_n
         // check if this tech is already in the map of prereqs.  If so, it has already been processed, and can be skipped.
         if (prereqs_set.find(cur_name) != prereqs_set.end()) continue;
 
+        // if this tech is already known and min_required==true, can skip.
+        if (min_required && empire && (empire->GetTechStatus(cur_name) == TS_COMPLETE))
+            continue;
+
         // tech is new, so put it into the set of already-processed prereqs
         prereqs_set.insert(cur_name);
         // and the map of techs, sorted by cost
-        techs_to_add_map.insert(std::pair<double, std::string>(cur_tech->ResearchCost(empire_id), cur_name));
+        techs_to_add_map.insert(std::pair<float, std::string>(cur_tech->ResearchCost(empire_id), cur_name));
 
         // get prereqs of new tech, append to list
         cur_prereqs = cur_tech->Prerequisites();
@@ -564,7 +569,7 @@ std::vector<std::string> TechManager::RecursivePrereqs(const std::string& tech_n
 
     // extract sorted techs into vector, to be passed to signal...
     std::vector<std::string> retval;
-    for (std::multimap<double, std::string>::const_iterator it = techs_to_add_map.begin();
+    for (std::multimap<float, std::string>::const_iterator it = techs_to_add_map.begin();
          it != techs_to_add_map.end(); ++it)
     { retval.push_back(it->second); }
 
