@@ -4,11 +4,13 @@
 
 #include "Enums.h"
 #include "ObjectMap.h"
+#include "TemporaryPtr.h"
 
-#include <boost/signal.hpp>
+#include <boost/signals2/signal.hpp>
 #include <boost/unordered_map.hpp>
 #include <boost/numeric/ublas/symmetric.hpp>
 #include <boost/serialization/access.hpp>
+#include <boost/thread/shared_mutex.hpp>
 
 #include <vector>
 #include <list>
@@ -24,7 +26,6 @@
 #  undef GetObjectA
 #endif
 
-struct PlayerSetupData;
 class Empire;
 struct UniverseObjectVisitor;
 class XMLElement;
@@ -33,7 +34,7 @@ class UniverseObject;
 class System;
 namespace Condition {
     struct ConditionBase;
-    typedef std::vector<const UniverseObject*> ObjectSet;
+    typedef std::vector<TemporaryPtr<const UniverseObject> > ObjectSet;
 }
 
 namespace Effect {
@@ -41,7 +42,7 @@ namespace Effect {
     struct TargetsAndCause;
     struct SourcedEffectsGroup;
     class EffectsGroup;
-    typedef std::vector<UniverseObject*> TargetSet;
+    typedef std::vector<TemporaryPtr<UniverseObject> > TargetSet;
     typedef std::map<int, std::map<MeterType, std::vector<AccountingInfo> > > AccountingMap;
     typedef std::vector<std::pair<SourcedEffectsGroup, TargetsAndCause> > TargetsCauses;
     typedef std::map<int, std::map<MeterType, double> > DiscrepancyMap;
@@ -78,7 +79,8 @@ public:
     typedef ShipDesignMap::const_iterator           ship_design_iterator;           ///< const iterator over ship designs created by players that are known by this client
 
     /** \name Signal Types */ //@{
-    typedef boost::signal<void (const UniverseObject *)> UniverseObjectDeleteSignalType; ///< emitted just before the UniverseObject is deleted
+    /** emitted just before the UniverseObject is deleted */
+    typedef boost::signals2::signal<void (TemporaryPtr<const UniverseObject>)> UniverseObjectDeleteSignalType;
     //@}
 
     /** \name Structors */ //@{
@@ -119,6 +121,7 @@ public:
     const std::set<int>&    EmpireStaleKnowledgeObjectIDs(int empire_id) const;
 
     const ShipDesign*       GetShipDesign(int ship_design_id) const;                    ///< returns the ship design with id \a ship_design id, or 0 if non exists
+    void                    RenameShipDesign(int design_id, const std::string& name = "", const std::string& description = "");
     ship_design_iterator    beginShipDesigns() const   {return m_ship_designs.begin();} ///< returns the begin iterator for ship designs
     ship_design_iterator    endShipDesigns() const     {return m_ship_designs.end();}   ///< returns the end iterator for ship designs
 
@@ -216,25 +219,13 @@ public:
       * to grant their owners victory. */
     const std::multimap<int, std::string>&  GetMarkedForVictory() const {return m_marked_for_victory;}
 
+    const std::map<std::string, std::map<int, std::map<int, double> > >&
+                                            GetStatRecords() const { return m_stat_records; }
+
     mutable UniverseObjectDeleteSignalType UniverseObjectDeleteSignal; ///< the state changed signal object for this UniverseObject
     //@}
 
     /** \name Mutators */ //@{
-    /** Inserts object \a obj into the universe; returns the ID number
-      * assigned to the object, or -1 on failure.
-      * \note Universe gains ownership of \a obj once it is inserted; the
-      * caller should \a never delete \a obj after passing it to Insert(). */
-    int             Insert(UniverseObject* obj);
-
-    /** Inserts object \a obj of given ID into the universe; returns true
-      * on proper insert, or false on failure.
-      * \note Universe gains ownership of \a obj once it is inserted; the
-      * caller should \a never delete \a obj after
-      * passing it to InsertID().
-      * Useful mostly for times when ID needs to be consistent on client
-      * and server */
-    bool            InsertID(UniverseObject* obj, int id);
-
     /** Inserts \a ship_design into the universe; returns the ship design ID
       * assigned to it, or -1 on failure.
       * \note Universe gains ownership of \a ship_design once inserted. */
@@ -244,15 +235,9 @@ public:
       * true on success, or false on failure.
       * \note Universe gains ownership of \a ship_design once inserted. */
     bool            InsertShipDesignID(ShipDesign* ship_design, int id);
-
-    /** Generates systems and planets, assigns homeworlds and populates them
-      * with people, industry and bases, and places starting fleets.  Uses
-      * predefined galaxy shapes. */
-    void            CreateUniverse(unsigned int seed, int size, Shape shape,
-                                   GalaxySetupOption age, GalaxySetupOption starlane_freq,
-                                   GalaxySetupOption planet_density, GalaxySetupOption specials_freq,
-                                   GalaxySetupOption monster_freq, GalaxySetupOption native_freq,
-                                   const std::map<int, PlayerSetupData>& player_setup_data);
+    
+    // Resets the universe object to prepare for generation of a new universe
+    void            ResetUniverse();
 
     /** Clears main ObjectMap, empires' latest known objects map, and
       * ShipDesign map. */
@@ -273,6 +258,8 @@ public:
 
     /** Calls above ApplyMeterEffectsAndUpdateMeters() function on all objects.*/
     void            ApplyMeterEffectsAndUpdateMeters();
+    /** Like ApplyMeterEffectsAndUpdateMeters(), but only target, max and unpaired meters are updated.*/
+    void            ApplyMeterEffectsAndUpdateTargetMaxUnpairedMeters();
 
     /** Executes effects that modify objects' appearance in the human client. */
     void            ApplyAppearanceEffects(const std::vector<int>& object_ids);
@@ -329,10 +316,9 @@ public:
       * be seen by the empire, updates the latest known state to note this. */
     void            UpdateEmpireStaleObjectKnowledge();
 
-    /** Resizes the system graph to the appropriate size and populates
-      * m_system_distances.  Uses the Universe latest known set of objects for
-      * the empire with id \a for_empire_id or uses the main / true / visible
-      * objects if \a for_empire_id is ALL_EMPIRES*/
+    /** Fills pathfinding data structure and determines least jumps distances
+      * between systems for the empire with id \a for_empire_id or uses the
+      * main / true / visible objects if \a for_empire_id is ALL_EMPIRES*/
     void            InitializeSystemGraph(int for_empire_id = ALL_EMPIRES);
 
     /** Regenerates per-empire system view graphs by filtering the complete
@@ -361,8 +347,8 @@ public:
     /** Destroys object with ID \a object_id, and destroys any associted
       * objects, such as contained buildings of planets, contained anything of
       * systems, or fleets if their last ship has id \a object_id and the fleet
-      * is thus empty. */
-    void            RecursiveDestroy(int object_id);
+      * is thus empty. Returns the ids of all destroyed objects. */
+    std::set<int>   RecursiveDestroy(int object_id);
 
     /** Used by the Destroy effect to mark an object for destruction later
       * during turn processing. (objects can't be destroyed immediately as
@@ -388,18 +374,18 @@ public:
     /** Sets whether to inhibit UniverseObjectSignals.  Inhibits if \a inhibit
       * is true, and (re)enables UniverseObjectSignals if \a inhibit is false. */
     void            InhibitUniverseObjectSignals(bool inhibit = true);
+
+    void            UpdateStatRecords();
     //@}
 
 
     /** Generates an object ID for a future object. Usually used by the server
       * to service new ID requests. */
-    int             GenerateObjectID() { return ++m_last_allocated_object_id; }
+    int             GenerateObjectID();
 
     /** Generates design ID for a new (ship) design. Usually used by the
       * server to service new ID requests. */
-    int             GenerateDesignID() { return ++m_last_allocated_design_id; }
-
-    typedef std::vector<std::vector<std::set<System*> > > AdjacencyGrid;
+    int             GenerateDesignID() { return ++m_last_allocated_design_id; } // TODO: See GenerateObjectID()
 
     /** Returns true if UniverseOjbectSignals are inhibited, false otherwise. */
     const bool&     UniverseObjectSignalsInhibited();
@@ -412,55 +398,72 @@ public:
     int&            EncodingEmpire();
 
     double          UniverseWidth() const;
+    void            SetUniverseWidth(double width) { m_universe_width = width; }
     bool            AllObjectsVisible() const { return m_all_objects_visible; }
 
+    /** \name Generators */ //@{
+    TemporaryPtr<Ship> CreateShip(int id = INVALID_OBJECT_ID);
+    TemporaryPtr<Ship> CreateShip(int empire_id, int design_id, const std::string& species_name,
+                                  int produced_by_empire_id = ALL_EMPIRES, int id = INVALID_OBJECT_ID);
+
+    TemporaryPtr<Fleet> CreateFleet(int id = INVALID_OBJECT_ID);
+    TemporaryPtr<Fleet> CreateFleet(const std::string& name, double x, double y, int owner, int id = INVALID_OBJECT_ID);
+
+    TemporaryPtr<Planet> CreatePlanet(int id = INVALID_OBJECT_ID);
+    TemporaryPtr<Planet> CreatePlanet(PlanetType type, PlanetSize size, int id = INVALID_OBJECT_ID);
+
+    TemporaryPtr<System> CreateSystem(int id = INVALID_OBJECT_ID);
+    TemporaryPtr<System> CreateSystem(StarType star, const std::string& name, double x, double y, int id = INVALID_OBJECT_ID);
+    TemporaryPtr<System> CreateSystem(StarType star, const std::map<int, bool>& lanes_and_holes,
+                                      const std::string& name, double x, double y, int id = INVALID_OBJECT_ID);
+
+    TemporaryPtr<Building> CreateBuilding(int id = INVALID_OBJECT_ID);
+    TemporaryPtr<Building> CreateBuilding(int empire_id, const std::string& building_type,
+                                          int produced_by_empire_id = ALL_EMPIRES, int id = INVALID_OBJECT_ID);
+
+    TemporaryPtr<Field> CreateField(int id = INVALID_OBJECT_ID);
+    TemporaryPtr<Field> CreateField(const std::string& field_type, double x, double y, double radius, int id = INVALID_OBJECT_ID);
+    //@}
+
 private:
-    template <typename T>
-    class distance_matrix
-    {
-    public:
-        typedef boost::numeric::ublas::symmetric_matrix<
-            T,
-            boost::numeric::ublas::lower
-        > storage_type;
+    /// minimal public interface for distance caches
+    template <class T> struct distance_matrix_storage {
+        typedef T value_type;
+        typedef std::vector<T>& row_ref;
 
-        struct const_row_ref
-        {
-            const_row_ref(std::size_t i, const storage_type& m) : m_i(i), m_m(m) {}
-            T operator[](std::size_t j) const { return m_m(m_i, j); }
-        private:
-            std::size_t m_i;
-            const storage_type& m_m;
-        };
+        distance_matrix_storage() {};
+        distance_matrix_storage(const distance_matrix_storage<T>& src)
+        { resize(src.m_data.size()); };
 
-        struct row_ref
-        {
-            row_ref(std::size_t i, storage_type& m) : m_i(i), m_m(m) {}
-            T& operator[](std::size_t j) { return m_m(m_i, j); }
-        private:
-            std::size_t m_i;
-            storage_type& m_m;
-        };
+        size_t size() 
+        { return m_data.size(); }
 
-        distance_matrix() :
-            m_m()
-        {}
+        void resize(size_t a_size) {
+            const size_t old_size = size();
 
-        const_row_ref operator[](std::size_t i) const
-        { return const_row_ref(i, m_m); }
+            m_data.clear();
+            m_data.resize(a_size);
+            m_row_mutexes.resize(a_size);
+            for (size_t i = old_size; i < a_size; ++i)
+                m_row_mutexes[i] = boost::shared_ptr<boost::shared_mutex>(new boost::shared_mutex());
+        }
 
-        row_ref operator[](std::size_t i)
-        { return row_ref(i, m_m); }
-
-        void resize(std::size_t rows, std::size_t columns)
-        { m_m.resize(rows, columns); }
-
-    private:
-        storage_type m_m;
+        std::vector< std::vector<T> > m_data;
+        std::vector< boost::shared_ptr<boost::shared_mutex> > m_row_mutexes;
+        boost::shared_mutex m_mutex;
     };
 
-    typedef distance_matrix<double> DistanceMatrix;
-    typedef distance_matrix<short> JumpsMatrix;
+    /** Inserts object \a obj into the universe; returns a TemporaryPtr
+      * to the inserted object. */
+    template <class T>
+    TemporaryPtr<T>             Insert(T* obj);
+
+    /** Inserts object \a obj of given ID into the universe; returns a
+      * TemporaryPtr to the inserted object on proper insert, or a null
+      * TemporaryPtr on failure.  Useful mostly for times when ID needs
+      * to be consistent on client and server */
+    template <class T>
+    TemporaryPtr<T>            InsertID(T* obj, int id);
 
     struct GraphImpl;
 
@@ -470,24 +473,11 @@ private:
 
     /** Removes entries in \a targets_causes about effects groups acting
       * on objects in \a target_objects, and then repopulates for EffectsGroups
-      * that act on at least one of the objects in \a target_objects. */
+      * that act on at least one of the objects in \a target_objects. If 
+      * \a target_objects is empty then default target candidates will be used. */
     void    GetEffectsAndTargets(Effect::TargetsCauses& targets_causes,
                                  const std::vector<int>& target_objects);
 
-    /** Used by GetEffectsAndTargets to process a vector of effects groups.
-      * Stores target set of specified \a effects_groups and \a source_object_id
-      * in \a targets_causes
-      * NOTE: this method will modify target_objects temporarily, but restore
-      * its contents before returning. */
-    void    StoreTargetsAndCausesOfEffectsGroups(const std::vector<boost::shared_ptr<const Effect::EffectsGroup> >& effects_groups,
-                                                 const UniverseObject* source,
-                                                 EffectsCauseType effect_cause_type,
-                                                 const std::string& specific_cause_name,
-                                                 Effect::TargetSet& target_objects,
-                                                 Effect::TargetsCauses& targets_causes,
-                                                 std::map<const Condition::ConditionBase*, Effect::TargetSet>& source_cached_condition_matches,
-                                                 std::map<const Condition::ConditionBase*, Effect::TargetSet>& invariant_cached_condition_matches);
-
     /** Executes all effects.  For use on server when processing turns.
       * If \a only_meter_effects is true, then only SetMeter effects are
       * executed.  This is useful on server or clients to update meter
@@ -501,42 +491,10 @@ private:
 
     /** Does actual updating of meter estimates after the public function have
       * processed objects_vec or whatever they were passed and cleared the
-      * relevant effect accounting for those objects and meters. */
+      * relevant effect accounting for those objects and meters. If an empty 
+      * vector is passed, it will instead update all existing objects. */
     void    UpdateMeterEstimatesImpl(const std::vector<int>& objects_vec);
 
-    /** Generates planets for all systems that have empty object maps (ie those
-      * that aren't homeworld systems).*/
-    void    PopulateSystems(GalaxySetupOption density);
-
-    /** Adds start-of-game specials to objects. */
-    void    AddStartingSpecials(GalaxySetupOption specials_freq);
-
-    /** Adds non-empire-affiliated native populations to planets. */
-    void    GenerateNatives(GalaxySetupOption freq);
-
-    /** Adds space monsters to systems. */
-    void    GenerateSpaceMonsters(GalaxySetupOption freq);
-
-    /** Creates starlanes and adds them systems already generated. */
-    void    GenerateStarlanes(GalaxySetupOption freq, const AdjacencyGrid& adjacency_grid);
-
-    /** Picks systems to host homeworlds, generates planets for them, stores
-      * the ID's of the homeworld planets into the homeworld vector. */
-    void    GenerateHomeworlds(int players, std::vector<int>& homeworld_planet_ids);
-
-    /** Names the planets in each system, based on the system's name. */
-    void    NamePlanets();
-
-    /** Creates some initial fields in universe. */
-    void    GenerateFields(GalaxySetupOption freq);
-
-    /** Creates Empires objects for each entry in \a player_setup_data with
-      * empire id equal to the specified player ids (so that the calling code
-      * can know which empire belongs to which player).  Homeworlds are
-      * associated with the empires, and starting buildings and fleets are
-      * created, and empire starting ship designs are created and added. */
-    void    GenerateEmpires(std::vector<int>& homeworld_planet_ids, const std::map<int, PlayerSetupData>& player_setup_data);
-
     ObjectMap                       m_objects;                          ///< map from object id to UniverseObjects in the universe.  for the server: all of them, up to date and true information about object is stored;  for clients, only limited information based on what the client knows about is sent.
     EmpireObjectMap                 m_empire_latest_known_objects;      ///< map from empire id to (map from object id to latest known information about each object by that empire)
 
@@ -553,10 +511,10 @@ private:
     ShipDesignMap                   m_ship_designs;                     ///< ship designs in the universe
     std::map<int, std::set<int> >   m_empire_known_ship_design_ids;     ///< ship designs known to each empire
 
-    DistanceMatrix                  m_system_distances;                 ///< the straight-line distances between all the systems
-    JumpsMatrix                     m_system_jumps;                     ///< the least-jumps distances between all the systems
-    GraphImpl*                      m_graph_impl;                       ///< a graph in which the systems are vertices and the starlanes are edges
-    boost::unordered_map<int, int>  m_system_id_to_graph_index;
+    mutable distance_matrix_storage<short>
+                                    m_system_jumps;                     ///< indexed by system graph index (not system id), caches the smallest number of jumps to travel between all the systems
+    boost::shared_ptr<GraphImpl>    m_graph_impl;                       ///< a graph in which the systems are vertices and the starlanes are edges
+    boost::unordered_map<int, size_t>  m_system_id_to_graph_index;
 
     Effect::AccountingMap           m_effect_accounting_map;            ///< map from target object id, to map from target meter, to orderered list of structs with details of an effect and what it does to the meter
     Effect::DiscrepancyMap          m_effect_discrepancy_map;           ///< map from target object id, to map from target meter, to discrepancy between meter's actual initial value, and the initial value that this meter should have as far as the client can tell: the unknown factor affecting the meter
@@ -569,8 +527,11 @@ private:
 
     double                          m_universe_width;
     bool                            m_inhibit_universe_object_signals;
-    int                             m_encoding_empire;
-    bool                            m_all_objects_visible;
+    int                             m_encoding_empire;                  ///< used during serialization to globally set what empire knowledge to use
+    bool                            m_all_objects_visible;              ///< flag set to skip visibility tests and make everything visible to all players
+
+    std::map<std::string, std::map<int, std::map<int, double> > >
+                                    m_stat_records;                     ///< storage for statistics calculated for empires. Indexed by stat name (string), contains a map indexed by empire id, contains a map from turn number (int) to stat value (double).
 
     /** Fills \a designs_to_serialize with ShipDesigns known to the empire with
       * the ID \a encoding empire.  If encoding_empire is ALL_EMPIRES, then all
@@ -604,64 +565,13 @@ private:
     /***/
     void    GetEmpireStaleKnowledgeObjects(ObjectKnowledgeMap& empire_stale_knowledge_object_ids, int encoding_empire) const;
 
+    template <class T>
+    TemporaryPtr<T> InsertNewObject(T* object);
+
     friend class boost::serialization::access;
     template <class Archive>
     void serialize(Archive& ar, const unsigned int version);
 };
 
-/** A combination of names of ShipDesign that can be put together to make a
-  * fleet of ships, and a name for such a fleet, loaded from starting_fleets.txt
-  * ShipDesign names refer to designs listed in premade_ship_designs.txt.
-  * Useful for saving or specifying prearranged combinations of prearranged
-  * ShipDesigns to automatically put together, such as during universe creation.*/
-class FleetPlan {
-public:
-    FleetPlan(const std::string& fleet_name, const std::vector<std::string>& ship_design_names,
-              bool lookup_name_userstring = false) :
-        m_name(fleet_name),
-        m_ship_designs(ship_design_names),
-        m_name_in_stringtable(lookup_name_userstring)
-    {}
-    FleetPlan() :
-        m_name(""),
-        m_ship_designs(),
-        m_name_in_stringtable(false)
-    {}
-    virtual ~FleetPlan() {};
-    const std::string&              Name() const;
-    const std::vector<std::string>& ShipDesigns() const { return m_ship_designs; }
-protected:
-    std::string                     m_name;
-    std::vector<std::string>        m_ship_designs;
-    bool                            m_name_in_stringtable;
-};
-
-/** The combination of a FleetPlan and spawning instructions for start of game
-  * monsters. */
-class FO_COMMON_API MonsterFleetPlan : public FleetPlan {
-public:
-    MonsterFleetPlan(const std::string& fleet_name, const std::vector<std::string>& ship_design_names,
-                     double spawn_rate = 1.0, int spawn_limit = 9999, const Condition::ConditionBase* location = 0,
-                     bool lookup_name_userstring = false) :
-        FleetPlan(fleet_name, ship_design_names, lookup_name_userstring),
-        m_spawn_rate(spawn_rate),
-        m_spawn_limit(spawn_limit),
-        m_location(location)
-    {}
-    MonsterFleetPlan() :
-        FleetPlan(),
-        m_spawn_rate(1.0),
-        m_spawn_limit(9999),
-        m_location(0)
-    {}
-    virtual ~MonsterFleetPlan();
-    double                          SpawnRate() const   { return m_spawn_rate; }
-    int                             SpawnLimit() const  { return m_spawn_limit; }
-    const Condition::ConditionBase* Location() const    { return m_location; }
-protected:
-    double                          m_spawn_rate;
-    int                             m_spawn_limit;
-    const Condition::ConditionBase* m_location;
-};
 
 #endif // _Universe_h_
