@@ -1,7 +1,6 @@
 #include "MultiplayerCommon.h"
 
 #include "OptionsDB.h"
-#include "Obstacle.h"
 #include "../combat/AsteroidBeltObstacle.h"
 #include "../util/Directories.h"
 #include "../util/i18n.h"
@@ -16,11 +15,12 @@
 #undef int64_t
 #endif
 
-#include <GG/utf8/checked.h>
 
 #include <boost/filesystem/fstream.hpp>
 #include <boost/filesystem/operations.hpp>
 #include <boost/system/system_error.hpp>
+#include <OpenSteer/Obstacle.h>
+#include <GG/utf8/checked.h>
 
 
 const std::string MP_SAVE_FILE_EXTENSION = ".mps";
@@ -41,6 +41,7 @@ namespace {
 
     const double TWO_PI = 8.0 * std::atan(1.0);
 
+#ifdef DEBUG_XML_TO_CLR
     std::string ClrToString(GG::Clr clr) {
         unsigned int r = static_cast<int>(clr.r);
         unsigned int g = static_cast<int>(clr.g);
@@ -52,24 +53,29 @@ namespace {
             boost::lexical_cast<std::string>(a) + ")";
         return retval;
     }
+#endif
 
     // overload for arriving fleets
-    void FindPlacementAreas(const System* system,
-                            const std::map<int, std::vector<Fleet*> >& arriving_fleets_by_starlane,
+    void FindPlacementAreas(const TemporaryPtr<System> system,
+                            const std::map<int, std::vector<TemporaryPtr<Fleet> > >& arriving_fleets_by_starlane,
                             const std::map<int, std::set<int> >& empires_by_starlane,
                             std::vector<CombatSetupGroup>& setup_groups)
     {
         assert(!arriving_fleets_by_starlane.begin()->second.back()->Unowned());
         int owner = arriving_fleets_by_starlane.begin()->second.back()->Owner();
-        for (std::map<int, std::vector<Fleet*> >::const_iterator it = arriving_fleets_by_starlane.begin();
-             it != arriving_fleets_by_starlane.end();
-             ++it)
+        for (std::map<int, std::vector<TemporaryPtr<Fleet> > >::const_iterator it = arriving_fleets_by_starlane.begin();
+             it != arriving_fleets_by_starlane.end(); ++it)
         {
             setup_groups.push_back(CombatSetupGroup());
             CombatSetupGroup& setup_group = setup_groups.back();
 
-            for (std::size_t i = 0; i < it->second.size(); ++i) {
-                setup_group.m_ships.insert(it->second[i]->begin(), it->second[i]->end());
+            std::vector<TemporaryPtr<Fleet> > fleets = it->second;
+            for (std::vector<TemporaryPtr<Fleet> >::const_iterator fleet_it = fleets.begin();
+                 fleet_it != fleets.end(); ++fleet_it)
+            {
+                TemporaryPtr<Fleet> fleet = *fleet_it;
+                const std::set<int> ship_ids = fleet->ShipIDs();
+                setup_group.m_ships.insert(ship_ids.begin(), ship_ids.end());
             }
 
             double rads = StarlaneEntranceOrbitalPosition(it->first, system->ID());
@@ -98,25 +104,34 @@ namespace {
     }
 
     // overload for present fleets
-    void FindPlacementAreas(const System* system,
-                            const std::vector<Fleet*>& fleets,
+    void FindPlacementAreas(TemporaryPtr<const System> system,
+                            const std::vector<TemporaryPtr<Fleet> >& fleets,
                             CombatSetupGroup& setup_group)
     {
-        for (std::size_t i = 0; i < fleets.size(); ++i) {
-            setup_group.m_ships.insert(fleets[i]->begin(), fleets[i]->end());
+        for (std::vector<TemporaryPtr<Fleet> >::const_iterator fleet_it = fleets.begin();
+                fleet_it != fleets.end(); ++fleet_it)
+        {
+            TemporaryPtr<Fleet> fleet = *fleet_it;
+            const std::set<int> ship_ids = fleet->ShipIDs();
+            setup_group.m_ships.insert(ship_ids.begin(), ship_ids.end());
         }
 
-        for (System::const_orbit_iterator it = system->begin(); it != system->end(); ++it) {
-            if (const Planet* planet = GetPlanet(it->second)) {
-                double orbit_r = OrbitalRadius(it->first);
-                double rads = planet->OrbitalPositionOnTurn(CurrentTurn());
-                float planet_r = PlanetRadius(planet->Size());
-                setup_group.m_regions.push_back(
-                    CombatSetupRegion(orbit_r * std::cos(rads), orbit_r * std::sin(rads), planet_r * 1.5));
-            }
+        std::vector<TemporaryPtr<const Planet> > planets = Objects().FindObjects<const Planet>(system->PlanetIDs());
+        for (std::vector<TemporaryPtr<const Planet> >::const_iterator it = planets.begin();
+             it != planets.end(); ++it)
+        {
+            TemporaryPtr<const Planet> planet = *it;
+
+            int orbit = system->OrbitOfPlanet(planet->ID());
+            double orbit_r = OrbitalRadius(orbit);
+
+            float rads = planet->OrbitalPositionOnTurn(CurrentTurn());
+            float planet_r = PlanetRadius(planet->Size());
+            setup_group.m_regions.push_back(
+                CombatSetupRegion(orbit_r * std::cos(rads), orbit_r * std::sin(rads), planet_r * 1.5));
         }
 
-        setup_group.m_regions.push_back(CombatSetupRegion(0.0, 0.0, StarRadius() / 2.0));
+        setup_group.m_regions.push_back(CombatSetupRegion(0.0f, 0.0f, StarRadius() / 2.0f));
 
         // provide a gap between the nearest point on the ellipse and the
         // allowed placement area.
@@ -129,10 +144,10 @@ namespace {
         setup_group.m_allow = false;
     }
 
-    void PlaceShips(const System* system,
+    void PlaceShips(TemporaryPtr<const System> system,
                     const CombatSetupGroup& setup_group,
-                    const std::map<int, UniverseObject*>& combat_universe,
-                    std::map<Ship*, std::pair<OpenSteer::Vec3, OpenSteer::Vec3> >& placements)
+                    const std::map<int, TemporaryPtr<UniverseObject> >& combat_universe,
+                    std::map<TemporaryPtr<Ship>, std::pair<OpenSteer::Vec3, OpenSteer::Vec3> >& placements)
     {
         Logger().debugStream() << "Placing combat ships in " << system->Name();
         // TODO: Do something other than just random placement.  Take into
@@ -141,7 +156,7 @@ namespace {
              it != setup_group.m_ships.end(); ++it)
         {
             assert(combat_universe.find(*it) != combat_universe.end());
-            Ship* ship = static_cast<Ship*>(combat_universe.find(*it)->second);
+            TemporaryPtr<Ship> ship = boost::dynamic_pointer_cast<Ship>(combat_universe.find(*it)->second);
 #define LIMIT_ITERATIONS 1
 #if LIMIT_ITERATIONS
             const unsigned int MAX_PLACEMENT_ITERATIONS = 50;
@@ -211,10 +226,10 @@ namespace {
         }
     }
 
-    void PlaceShips(const System* system,
+    void PlaceShips(TemporaryPtr<const System> system,
                     const std::map<int, std::vector<CombatSetupGroup> >& setup_groups,
-                    const std::map<int, UniverseObject*>& combat_universe,
-                    std::map<Ship*, std::pair<OpenSteer::Vec3, OpenSteer::Vec3> >& placements)
+                    const std::map<int, TemporaryPtr<UniverseObject> >& combat_universe,
+                    std::map<TemporaryPtr<Ship>, std::pair<OpenSteer::Vec3, OpenSteer::Vec3> >& placements)
     {
         for (std::map<int, std::vector<CombatSetupGroup> >::const_iterator it = setup_groups.begin();
              it != setup_groups.end(); ++it)
@@ -258,7 +273,9 @@ GG::Clr XMLToClr(const XMLElement& clr) {
                 retval.b = (rgba >> 8)  & 0xFF;
                 retval.a = rgba         & 0xFF;
             }
-            //std::cout << "hex colour: " << hex_colour << " int: " << rgba << " RGBA: " << ClrToString(retval) << std::endl;
+#ifdef DEBUG_XML_TO_CLR
+            std::cout << "hex colour: " << hex_colour << " int: " << rgba << " RGBA: " << ClrToString(retval) << std::endl;
+#endif
         } else {
             std::cerr << "XMLToClr could not interpret hex colour string \"" << hex_colour << "\"" << std::endl;
         }
@@ -272,7 +289,9 @@ GG::Clr XMLToClr(const XMLElement& clr) {
             retval.b = boost::lexical_cast<int>(clr.Child("blue").Text());
         if (clr.ContainsChild("alpha"))
             retval.a = boost::lexical_cast<int>(clr.Child("alpha").Text());
-        //std::cout << "non hex colour RGBA: " << ClrToString(retval) << std::endl;
+#ifdef DEBUG_XML_TO_CLR
+        std::cout << "non hex colour RGBA: " << ClrToString(retval) << std::endl;
+#endif
     }
     return retval;
 }
@@ -297,44 +316,7 @@ bool operator!=(const PlayerSetupData& lhs, const PlayerSetupData& rhs)
 ////////////////////////////////////////////////////
 // MultiplayerLobbyData
 /////////////////////////////////////////////////////
-MultiplayerLobbyData::MultiplayerLobbyData(bool build_save_game_list) :
-    m_new_game(true),
-    m_save_file_index(-1),
-    m_players(),
-    m_save_games(),
-    m_save_game_empire_data()
-{
-    if (!build_save_game_list)
-        return;
 
-    Logger().debugStream() << "MultiplayerLobbyData::MultiplayerLobbyData(true)";
-
-    // build a list of save files
-    fs::path save_dir(GetSaveDir());
-    Logger().debugStream() << "MultiplayerLobbyData::MultiplayerLobbyData save dir path: " << PathString(save_dir);
-    fs::directory_iterator end_it;
-    for (fs::directory_iterator it(save_dir); it != end_it; ++it) {
-        try {
-#if defined(BOOST_FILESYSTEM_VERSION) && BOOST_FILESYSTEM_VERSION == 3
-            if (fs::exists(*it) && !fs::is_directory(*it) && PathString(it->path().filename())[0] != '.') {
-                std::string filename = PathString(it->path().filename());
-#else
-            if (fs::exists(*it) && !fs::is_directory(*it) && it->path().filename()[0] != '.') {
-                std::string filename = it->path().filename();
-#endif
-                // disallow filenames that begin with a dot, and filenames with spaces in them
-                if (filename.find('.') != 0 && filename.find(' ') == std::string::npos &&
-                    filename.find(MP_SAVE_FILE_EXTENSION) == filename.size() - MP_SAVE_FILE_EXTENSION.size()) {
-                    m_save_games.push_back(filename);
-                }
-            }
-        } catch (const fs::filesystem_error& e) {
-            // ignore files for which permission is denied, and rethrow other exceptions
-            if (e.code() != boost::system::posix_error::permission_denied)
-                throw;
-        }
-    }
-}
 
 std::string MultiplayerLobbyData::Dump() const {
     std::stringstream stream;
@@ -361,7 +343,7 @@ std::string MultiplayerLobbyData::Dump() const {
 ////////////////////////////////////////////////
 // CombatData
 ////////////////////////////////////////////////
-CombatData::CombatData(System* system, std::map<int, std::vector<CombatSetupGroup> >& setup_groups) :
+CombatData::CombatData(TemporaryPtr<System> system, std::map<int, std::vector<CombatSetupGroup> >& setup_groups) :
     m_combat_turn_number(0),
     m_system(system)
 {
@@ -369,15 +351,26 @@ CombatData::CombatData(System* system, std::map<int, std::vector<CombatSetupGrou
     using OpenSteer::Vec3;
     m_pathing_engine.AddObstacle(new SphereObstacle(StarRadius() / 2.0, Vec3()));
 
-    std::map<int, std::vector<Fleet*> > present_fleets_by_empire;
-    std::map<int, std::map<int, std::vector<Fleet*> > > arriving_fleets_by_starlane_by_empire;
+    std::map<int, std::vector<TemporaryPtr<Fleet> > > present_fleets_by_empire;
+    std::map<int, std::map<int, std::vector<TemporaryPtr<Fleet> > > > arriving_fleets_by_starlane_by_empire;
     std::map<int, std::set<int> > empires_by_starlane;
 
-    for (System::const_orbit_iterator it = system->begin(); it != system->end(); ++it) {
-        m_combat_universe[it->second] = GetUniverseObject(it->second);
-        if (const Planet* planet =
-            universe_object_cast<Planet*>(m_combat_universe[it->second])) {
-            double orbit_radius = OrbitalRadius(it->first);
+    std::vector<TemporaryPtr<UniverseObject> > objects =
+        Objects().FindObjects<UniverseObject>(system->ContainedObjectIDs());
+
+    for (std::vector<TemporaryPtr<UniverseObject> >::const_iterator it = objects.begin();
+         it != objects.end(); ++it)
+    {
+        TemporaryPtr<UniverseObject> object = *it;
+        int object_id = object->ID();
+
+        m_combat_universe[object_id] = object;
+
+        if (TemporaryPtr<const Planet> planet =
+                boost::dynamic_pointer_cast<const Planet>(object))
+        {
+            int orbit = system->OrbitOfPlanet(object_id);
+            double orbit_radius = OrbitalRadius(orbit);
             if (planet->Type() == PT_ASTEROIDS) {
                 m_pathing_engine.AddObstacle(
                     new AsteroidBeltObstacle(orbit_radius, AsteroidBeltRadius()));
@@ -390,8 +383,9 @@ CombatData::CombatData(System* system, std::map<int, std::vector<CombatSetupGrou
                 m_pathing_engine.AddObstacle(
                     new SphereObstacle(PlanetRadius(planet->Size()), position));
             }
-        } else if (Fleet* fleet =
-                   universe_object_cast<Fleet*>(m_combat_universe[it->second])) {
+        } else if (TemporaryPtr<Fleet> fleet =
+                   boost::dynamic_pointer_cast<Fleet>(object))
+        {
             assert(!fleet->Unowned());
             int owner = fleet->Owner();
             if (fleet->ArrivedThisTurn()) {
@@ -404,16 +398,15 @@ CombatData::CombatData(System* system, std::map<int, std::vector<CombatSetupGrou
         }
     }
 
-    for (std::map<int, std::vector<Fleet*> >::iterator it = present_fleets_by_empire.begin();
-         it != present_fleets_by_empire.end();
-         ++it)
+    for (std::map<int, std::vector<TemporaryPtr<Fleet> > >::iterator
+         it = present_fleets_by_empire.begin(); it != present_fleets_by_empire.end(); ++it)
     {
         std::vector<CombatSetupGroup>& empire_setup_groups = setup_groups[it->first];
         empire_setup_groups.push_back(CombatSetupGroup());
         FindPlacementAreas(m_system, it->second, empire_setup_groups.back());
     }
 
-    for (std::map<int, std::map<int, std::vector<Fleet*> > >::iterator
+    for (std::map<int, std::map<int, std::vector<TemporaryPtr<Fleet> > > >::iterator
              it = arriving_fleets_by_starlane_by_empire.begin();
          it != arriving_fleets_by_starlane_by_empire.end();
          ++it)
@@ -421,7 +414,7 @@ CombatData::CombatData(System* system, std::map<int, std::vector<CombatSetupGrou
         FindPlacementAreas(m_system, it->second, empires_by_starlane, setup_groups[it->first]);
     }
 
-    std::map<Ship*, std::pair<OpenSteer::Vec3, OpenSteer::Vec3> > placements;
+    std::map<TemporaryPtr<Ship>, std::pair<OpenSteer::Vec3, OpenSteer::Vec3> > placements;
     PlaceShips(m_system, setup_groups, m_combat_universe, placements);
 
     for (std::map<int, std::vector<CombatSetupGroup> >::const_iterator it = setup_groups.begin();
@@ -431,8 +424,8 @@ CombatData::CombatData(System* system, std::map<int, std::vector<CombatSetupGrou
             for (std::set<int>::const_iterator ship_it = it->second[i].m_ships.begin();
                  ship_it != it->second[i].m_ships.end();
                  ++ship_it) {
-                assert(universe_object_cast<Ship*>(GetUniverseObject(*ship_it)));
-                Ship* ship = static_cast<Ship*>(GetUniverseObject(*ship_it));
+                TemporaryPtr<Ship> ship = GetShip(*ship_it);
+                assert(ship);
                 const std::pair<OpenSteer::Vec3, OpenSteer::Vec3>& placement = placements[ship];
                 CombatShipPtr combat_ship(
                     new CombatShip(ship, placement.first, placement.second, m_combat_universe, m_pathing_engine));
