@@ -20,12 +20,6 @@
 #include <fstream>
 #include <vector>
 
-#define DEBUG_CREATE_FLEET_ORDER 0
-#define DEBUG_FLEET_MOVE_ORDER   0
-#if DEBUG_CREATE_FLEET_ORDER || DEBUG_FLEET_MOVE_ORDER
-#  include <iostream>
-#endif
-
 
 /////////////////////////////////////////////////////
 // Order
@@ -54,7 +48,6 @@ bool Order::Executed() const
 bool Order::UndoImpl() const
 { return false; }
 
-
 ////////////////////////////////////////////////
 // RenameOrder
 ////////////////////////////////////////////////
@@ -68,7 +61,7 @@ RenameOrder::RenameOrder(int empire, int object, const std::string& name) :
     m_object(object),
     m_name(name)
 {
-    const UniverseObject* obj = GetUniverseObject(object);
+    TemporaryPtr<const UniverseObject> obj = GetUniverseObject(object);
     if (!obj) {
         Logger().errorStream() << "RenameOrder::RenameOrder() : Attempted to rename nonexistant object with id " << object;
         return;
@@ -85,7 +78,7 @@ RenameOrder::RenameOrder(int empire, int object, const std::string& name) :
 void RenameOrder::ExecuteImpl() const {
     ValidateEmpireID();
 
-    UniverseObject* obj = GetUniverseObject(m_object);
+    TemporaryPtr<UniverseObject> obj = GetUniverseObject(m_object);
 
     if (!obj) {
         Logger().errorStream() << "Attempted to rename nonexistant object with id " << m_object;
@@ -107,7 +100,6 @@ void RenameOrder::ExecuteImpl() const {
     obj->Rename(m_name);
 }
 
-
 ////////////////////////////////////////////////
 // CreateFleetOrder
 ////////////////////////////////////////////////
@@ -115,80 +107,134 @@ NewFleetOrder::NewFleetOrder() :
     Order()
 {}
 
-NewFleetOrder::NewFleetOrder(int empire, const std::string& fleet_name, const int new_id, int system_id, const std::vector<int>& ship_ids) :
+NewFleetOrder::NewFleetOrder(int empire, const std::string& fleet_name, int fleet_id,
+                             int system_id, const std::vector<int>& ship_ids,
+                             bool aggressive) :
     Order(empire),
-    m_fleet_name(fleet_name),
+    m_fleet_names(),
     m_system_id(system_id),
-    m_new_id( new_id ),
-    m_ship_ids(ship_ids)
+    m_fleet_ids(),
+    m_ship_id_groups(),
+    m_aggressives()
 {
-#if DEBUG_CREATE_FLEET_ORDER
-    std::cerr << "NewFleetOrder(int empire, const std::string& fleet_name, const int new_id, int system_id, int ship_id) : \n" << std::endl 
-              << "    m_empire=" << EmpireID() << std::endl
-              << "    m_fleet_name=" << m_fleet_name << std::endl
-              << "    m_system_id=" << m_system_id << std::endl
-              << "    m_position=(" << m_position.first << " " << m_position.second << ")" << std::endl
-              << "    m_new_id=" << m_new_id << std::endl
-              << "    m_ship_ids.size()=" << m_ship_ids.size() << std::endl
-              << std::endl;
-#endif
+    m_fleet_names.push_back(fleet_name);
+    m_fleet_ids.push_back(fleet_id);
+    m_ship_id_groups.push_back(ship_ids);
+    m_aggressives.push_back(aggressive);
 }
 
+
+NewFleetOrder::NewFleetOrder(int empire, const std::vector<std::string>& fleet_names,
+                             const std::vector<int>& fleet_ids, int system_id,
+                             const std::vector<std::vector<int> >& ship_id_groups,
+                             const std::vector<bool>& aggressives) :
+    Order(empire),
+    m_fleet_names(fleet_names),
+    m_system_id(system_id),
+    m_fleet_ids(fleet_ids),
+    m_ship_id_groups(ship_id_groups),
+    m_aggressives(aggressives)
+{}
+
 void NewFleetOrder::ExecuteImpl() const {
     ValidateEmpireID();
 
-    Universe& universe = GetUniverse();
-
     if (m_system_id == INVALID_OBJECT_ID) {
         Logger().errorStream() << "Empire attempted to create a new fleet outside a system";
         return;
     }
-    System* system = GetSystem(m_system_id);
+    TemporaryPtr<System> system = GetSystem(m_system_id);
     if (!system) {
         Logger().errorStream() << "Empire attempted to create a new fleet in a nonexistant system";
         return;
     }
 
-    // verify ship validity
-    std::vector<int> validated_ships;
-    for (unsigned int i = 0; i < m_ship_ids.size(); ++i) {
-        // verify that empire is not trying to take ships from somebody else's fleet
-        const Ship* ship = GetShip(m_ship_ids[i]);
-        if (!ship) {
-            Logger().errorStream() << "Empire attempted to create a new fleet with an invalid ship";
-            continue;
-        }
-        if (!ship->OwnedBy(EmpireID())) {
-            Logger().errorStream() << "Empire attempted to create a new fleet with ships from another's fleet.";
-            continue;
-        }
-        if (ship->SystemID() != m_system_id) {
-            Logger().errorStream() << "Empire attempted to make a new fleet from ship in the wrong system";
-            continue;
-        }
-        validated_ships.push_back(m_ship_ids[i]);
-    }
-    if (validated_ships.empty())
+    if (m_fleet_names.empty())
         return;
+    if (m_fleet_names.size() != m_fleet_ids.size()
+        || m_fleet_names.size() != m_ship_id_groups.size()
+        || m_fleet_names.size() != m_aggressives.size())
+    {
+        Logger().errorStream() << "NewFleetOrder has inconsistent data container sizes...";
+        return;
+    }
+
+    GetUniverse().InhibitUniverseObjectSignals(true);
+    std::vector<TemporaryPtr<Fleet> > created_fleets;
+    created_fleets.reserve(m_fleet_names.size());
 
 
-    // create fleet
-    Fleet* fleet = new Fleet(m_fleet_name, system->X(), system->Y(), EmpireID());
-    fleet->GetMeter(METER_STEALTH)->SetCurrent(Meter::LARGE_VALUE);
-    // an ID is provided to ensure consistancy between server and client universes
-    universe.InsertID(fleet, m_new_id);
-    universe.SetEmpireObjectVisibility(EmpireID(), fleet->ID(), VIS_FULL_VISIBILITY);
-    system->Insert(fleet);
-    // new fleet will get same m_arrival_starlane as fleet of the first ship in the list.
-    Ship* firstShip = GetShip(validated_ships[0]);
-    Fleet* firstFleet = GetFleet(firstShip->FleetID());
-    if (firstFleet)
-        fleet->SetArrivalStarlane(firstFleet->ArrivalStarlane());
-    fleet->SetAggressive(false);
-    fleet->AddShips(validated_ships);
+    // create fleet for each group of ships
+    for (int i = 0; i < static_cast<int>(m_fleet_names.size()); ++i) {
+        const std::string&      fleet_name =    m_fleet_names[i];
+        int                     fleet_id =      m_fleet_ids[i];
+        const std::vector<int>& ship_ids =      m_ship_id_groups[i];
+        bool                    aggressive =    m_aggressives[i];
+
+        if (ship_ids.empty())
+            continue;   // nothing to do...
+
+        // validate specified ships
+        std::vector<TemporaryPtr<Ship> >    validated_ships;
+        std::vector<int>                    validated_ships_ids;
+        for (unsigned int i = 0; i < ship_ids.size(); ++i) {
+            // verify that empire is not trying to take ships from somebody else's fleet
+            TemporaryPtr<Ship> ship = GetShip(ship_ids[i]);
+            if (!ship) {
+                Logger().errorStream() << "Empire attempted to create a new fleet with an invalid ship";
+                continue;
+            }
+            if (!ship->OwnedBy(EmpireID())) {
+                Logger().errorStream() << "Empire attempted to create a new fleet with ships from another's fleet.";
+                continue;
+            }
+            if (ship->SystemID() != m_system_id) {
+                Logger().errorStream() << "Empire attempted to make a new fleet from ship in the wrong system";
+                continue;
+            }
+            validated_ships.push_back(ship);
+            validated_ships_ids.push_back(ship->ID());
+        }
+        if (validated_ships.empty())
+            continue;
+
+        // create fleet
+        TemporaryPtr<Fleet> fleet = GetUniverse().CreateFleet(fleet_name, system->X(), system->Y(),
+                                                              EmpireID(), fleet_id);
+        fleet->GetMeter(METER_STEALTH)->SetCurrent(Meter::LARGE_VALUE);
+        fleet->SetAggressive(aggressive);
+
+        // an ID is provided to ensure consistancy between server and client universes
+        GetUniverse().SetEmpireObjectVisibility(EmpireID(), fleet->ID(), VIS_FULL_VISIBILITY);
+
+        system->Insert(fleet);
+
+        // new fleet will get same m_arrival_starlane as fleet of the first ship in the list.
+        TemporaryPtr<Ship> firstShip = validated_ships[0];
+        TemporaryPtr<Fleet> firstFleet = GetFleet(firstShip->FleetID());
+        if (firstFleet)
+            fleet->SetArrivalStarlane(firstFleet->ArrivalStarlane());
+
+        // remove ships from old fleet(s) and add to new
+        for (std::vector<TemporaryPtr<Ship> >::iterator ship_it = validated_ships.begin();
+             ship_it != validated_ships.end(); ++ship_it)
+        {
+            TemporaryPtr<Ship> ship = *ship_it;
+            if (TemporaryPtr<Fleet> old_fleet = GetFleet(ship->FleetID()))
+                old_fleet->RemoveShip(ship->ID());
+            ship->SetFleetID(fleet->ID());
+        }
+        fleet->AddShips(validated_ships_ids);
+
+        created_fleets.push_back(fleet);
+    }
+
+    GetUniverse().InhibitUniverseObjectSignals(false);
+
+    system->FleetsInsertedSignal(created_fleets);
+    system->StateChangedSignal();
 }
 
-
 ////////////////////////////////////////////////
 // FleetMoveOrder
 ////////////////////////////////////////////////
@@ -196,23 +242,25 @@ FleetMoveOrder::FleetMoveOrder() :
     Order(),
     m_fleet(INVALID_OBJECT_ID),
     m_start_system(INVALID_OBJECT_ID),
-    m_dest_system(INVALID_OBJECT_ID)
+    m_dest_system(INVALID_OBJECT_ID),
+    m_append(false)
 {}
 
-FleetMoveOrder::FleetMoveOrder(int empire, int fleet_id, int start_system_id, int dest_system_id) :
+FleetMoveOrder::FleetMoveOrder(int empire, int fleet_id, int start_system_id, int dest_system_id, bool append) :
     Order(empire),
     m_fleet(fleet_id),
     m_start_system(start_system_id),
-    m_dest_system(dest_system_id)
+    m_dest_system(dest_system_id),
+    m_append(append)
 {
     // perform sanity checks
-    const Fleet* fleet = GetFleet(FleetID());
+    TemporaryPtr<const Fleet> fleet = GetFleet(FleetID());
     if (!fleet) {
         Logger().errorStream() << "Empire with id " << EmpireID() << " ordered fleet with id " << FleetID() << " to move, but no such fleet exists";
         return;
     }
 
-    const System* destination_system = GetSystem(DestinationSystemID());
+    TemporaryPtr<const System> destination_system = GetSystem(DestinationSystemID());
     if (!destination_system) {
         Logger().errorStream() << "Empire with id " << EmpireID() << " ordered fleet to move to system with id " << DestinationSystemID() << " but no such system exists / is known to exist";
         return;
@@ -232,27 +280,18 @@ FleetMoveOrder::FleetMoveOrder(int empire, int fleet_id, int start_system_id, in
     // ensure a zero-length (invalid) route is not requested / sent to a fleet
     if (m_route.empty())
         m_route.push_back(m_start_system);
-
-#if DEBUG_FLEET_MOVE_ORDER
-    std::cerr << "FleetMoveOrder(int empire, int fleet, int start_syste, int dest_system) : " << std::endl
-              << "    m_empire=" << EmpireID() << std::endl
-              << "    m_fleet=" << m_fleet << std::endl
-              << "    m_start_system=" << m_start_system << std::endl
-              << "    m_dest_system=" << m_dest_system << std::endl
-              << std::endl;
-#endif
 }
 
 void FleetMoveOrder::ExecuteImpl() const {
     ValidateEmpireID();
 
-    Fleet* fleet = GetFleet(FleetID());
+    TemporaryPtr<Fleet> fleet = GetFleet(FleetID());
     if (!fleet) {
         Logger().errorStream() << "Empire with id " << EmpireID() << " ordered fleet with id " << FleetID() << " to move, but no such fleet exists";
         return;
     }
 
-    const System* destination_system = GetEmpireKnownSystem(DestinationSystemID(), EmpireID());
+    TemporaryPtr<const System> destination_system = GetEmpireKnownSystem(DestinationSystemID(), EmpireID());
     if (!destination_system) {
         Logger().errorStream() << "Empire with id " << EmpireID() << " ordered fleet to move to system with id " << DestinationSystemID() << " but no such system is known to that empire";
         return;
@@ -273,19 +312,29 @@ void FleetMoveOrder::ExecuteImpl() const {
 
     // verify fleet route first system
     int fleet_sys_id = fleet->SystemID();
-    if (fleet_sys_id != INVALID_OBJECT_ID) {
-        // fleet is in a system.  Its move path should also start from that system.
-        if (fleet_sys_id != m_start_system) {
-            Logger().errorStream() << "Empire with id " << EmpireID() << " ordered a fleet to move from a system with id " << m_start_system <<
-                                     " that it is not at.  Fleet is located at system with id " << fleet_sys_id;
-            return;
+    if (!m_append || fleet->TravelRoute().empty()) {
+        if (fleet_sys_id != INVALID_OBJECT_ID) {
+            // fleet is in a system.  Its move path should also start from that system.
+            if (fleet_sys_id != m_start_system) {
+                Logger().errorStream() << "Empire with id " << EmpireID() << " ordered a fleet to move from a system with id " << m_start_system <<
+                                        " that it is not at.  Fleet is located at system with id " << fleet_sys_id;
+                return;
+            }
+        } else {
+            // fleet is not in a system.  Its move path should start from the next system it is moving to.
+            int next_system = fleet->NextSystemID();
+            if (next_system != m_start_system) {
+                Logger().errorStream() << "Empire with id " << EmpireID() << " ordered a fleet to move starting from a system with id " << m_start_system <<
+                                        ", but the fleet's next destination is system with id " << next_system;
+                return;
+            }
         }
     } else {
-        // fleet is not in a system.  Its move path should start from the next system it is moving to.
-        int next_system = fleet->NextSystemID();
-        if (next_system != m_start_system) {
-            Logger().errorStream() << "Empire with id " << EmpireID() << " ordered a fleet to move starting from a system with id " << m_start_system <<
-                                     ", but the fleet's next destination is system with id " << next_system;
+        // We should append and there is something to append to
+        int last_system = fleet->TravelRoute().back();
+        if (last_system != m_start_system) {
+            Logger().errorStream() << "Empire with id " << EmpireID() << " ordered a fleet to continue from system with id " << m_start_system <<
+            ", but the fleet's current route won't lead there, it leads to system " << last_system;
             return;
         }
     }
@@ -293,8 +342,14 @@ void FleetMoveOrder::ExecuteImpl() const {
 
     // convert list of ids to list of System
     std::list<int> route_list;
+    
+    if(m_append && !fleet->TravelRoute().empty()){
+        route_list = fleet->TravelRoute();
+        route_list.erase(--route_list.end());// Remove the last one since it is the first one of the other
+    }
+    
     std::copy(m_route.begin(), m_route.end(), std::back_inserter(route_list));
-
+    
 
     // validate route.  Only allow travel between systems connected in series by starlanes known to this fleet's owner.
 
@@ -304,91 +359,106 @@ void FleetMoveOrder::ExecuteImpl() const {
         Logger().debugStream() << "FleetMoveOrder::ExecuteImpl rejected out of range move order";
         return;
     }
+    
+    std::string waypoints;
+    for (std::list<int>::iterator it = route_list.begin(); it != route_list.end(); ++it) {
+        waypoints += std::string(" ") + boost::lexical_cast<std::string>(*it);
+    }
+    Logger().debugStream() << "FleetMoveOrder::ExecuteImpl Setting route of fleet " << fleet->ID() << " to " << waypoints;
 
     fleet->SetRoute(route_list);
 }
 
-
 ////////////////////////////////////////////////
 // FleetTransferOrder
 ////////////////////////////////////////////////
 FleetTransferOrder::FleetTransferOrder() :
     Order(),
-    m_fleet_from(INVALID_OBJECT_ID),
-    m_fleet_to(INVALID_OBJECT_ID)
+    m_dest_fleet(INVALID_OBJECT_ID)
 {}
 
-FleetTransferOrder::FleetTransferOrder(int empire, int fleet_from, int fleet_to, const std::vector<int>& ships) : 
+FleetTransferOrder::FleetTransferOrder(int empire, int dest_fleet, const std::vector<int>& ships) :
     Order(empire),
-    m_fleet_from(fleet_from),
-    m_fleet_to(fleet_to),
+    m_dest_fleet(dest_fleet),
     m_add_ships(ships)
 {}
 
 void FleetTransferOrder::ExecuteImpl() const {
     ValidateEmpireID();
 
-    // look up the source fleet and destination fleet
-    Fleet* source_fleet = GetFleet(SourceFleet());
-    Fleet* target_fleet = GetFleet(DestinationFleet());
-
-    if (!source_fleet || !target_fleet) {
-        Logger().errorStream() << "Empire attempted to move ships to or from a nonexistant fleet";
+    // look up the destination fleet
+    TemporaryPtr<Fleet> target_fleet = GetFleet(DestinationFleet());
+    if (!target_fleet) {
+        Logger().errorStream() << "Empire attempted to move ships to a nonexistant fleet";
         return;
     }
-
-    // verify that both fleets are in the same system
-    if (source_fleet->SystemID() == INVALID_OBJECT_ID) {
+    // check that destination fleet is owned by empire
+    if (!target_fleet->OwnedBy(EmpireID())) {
+        Logger().errorStream() << "Empire attempted to move ships to a fleet it does not own";
+        return;
+    }
+    // verify that fleet is in a system
+    if (target_fleet->SystemID() == INVALID_OBJECT_ID) {
         Logger().errorStream() << "Empire attempted to transfer ships to/from fleet(s) not in a system";
         return;
     }
-    if (source_fleet->SystemID() != target_fleet->SystemID()) {
-        Logger().errorStream() << "Empire attempted to transfer ships to/from fleets in different systems";
-        return;
-    }
 
-    // verify that empire is not trying to take ships from somebody else's fleet
-    if (!source_fleet->OwnedBy(EmpireID())) {
-        Logger().errorStream() << "Empire attempted to merge ships from another's fleet.";
-        return;
-    }
+    // check that all ships are in the same system
+    std::vector<TemporaryPtr<Ship> > ships = Objects().FindObjects<Ship>(m_add_ships);
 
-    // verify that empire cannot merge ships into somebody else's fleet.
-    // this is just an additional security measure.  IT could be removed to
-    // allow 'donations' of ships to other players, provided the server
-    // verifies IDs of the Empires issuing the orders.
-    if (!target_fleet->OwnedBy(EmpireID())) {
-        Logger().errorStream() << "Empire attempted to merge ships into another's fleet.";
-        return;
-    }
-
-
-    // iterate down the ship vector check validity of each
-    std::vector<int> validated_ship_ids;
+    std::vector<TemporaryPtr<Ship> > validated_ships;
+    validated_ships.reserve(m_add_ships.size());
+    std::vector<int>                 validated_ship_ids;
     validated_ship_ids.reserve(m_add_ships.size());
-    std::vector<int>::const_iterator itr = m_add_ships.begin();
-    while (itr != m_add_ships.end()) {
-        // find the ship, verify that ID is valid
-        int curr = (*itr);
-        Ship* a_ship = GetShip(curr);
-        if (!a_ship) {
-            Logger().errorStream() << "Illegal ship id specified in fleet merge order.";
-            return;
-        }
 
-        // figure out what fleet this ship is coming from -- verify its the one we
-        // said it comes from
-        if (a_ship->FleetID() != SourceFleet()) {
-            Logger().errorStream() << "Ship in merge order is not in specified source fleet.";
-            return;
-        }
-
-        validated_ship_ids.push_back(curr);
-        itr++;
+    for (std::vector<TemporaryPtr<Ship> >::const_iterator it = ships.begin();
+         it != ships.end(); ++it)
+    {
+        TemporaryPtr<Ship> ship = *it;
+        if (!ship->OwnedBy(EmpireID()))
+            continue;
+        if (ship->SystemID() != target_fleet->SystemID())
+            continue;
+        if (ship->FleetID() == target_fleet->ID())
+            continue;
+        validated_ships.push_back(ship);
+        validated_ship_ids.push_back(ship->ID());
     }
-    target_fleet->AddShips(validated_ship_ids);
-}
+    if (validated_ships.empty())
+        return;
 
+    GetUniverse().InhibitUniverseObjectSignals(true);
+
+    // remove from old fleet(s)
+    std::set<TemporaryPtr<Fleet> > modified_fleets;
+    for (std::vector<TemporaryPtr<Ship> >::iterator it = validated_ships.begin();
+         it != validated_ships.end(); ++it)
+    {
+        TemporaryPtr<Ship> ship = *it;
+        if (TemporaryPtr<Fleet> source_fleet = GetFleet(ship->FleetID())) {
+            source_fleet->RemoveShip(ship->ID());
+            modified_fleets.insert(source_fleet);
+        }
+        ship->SetFleetID(target_fleet->ID());
+    }
+
+    // add to new fleet
+    target_fleet->AddShips(validated_ship_ids);
+
+    GetUniverse().InhibitUniverseObjectSignals(false);
+
+    // signal change to fleet states
+    modified_fleets.insert(target_fleet);
+
+    for (std::set<TemporaryPtr<Fleet> >::iterator it = modified_fleets.begin();
+         it != modified_fleets.end(); ++it)
+    {
+        TemporaryPtr<Fleet> modified_fleet = *it;
+        if (!modified_fleet->Empty())
+            modified_fleet->StateChangedSignal();
+        // if modified fleet is empty, it should be immently destroyed, so that updating it now is redundant
+    }
+}
 
 ////////////////////////////////////////////////
 // ColonizeOrder
@@ -409,7 +479,7 @@ void ColonizeOrder::ExecuteImpl() const {
     ValidateEmpireID();
     int empire_id = EmpireID();
 
-    Ship* ship = GetShip(m_ship);
+    TemporaryPtr<Ship> ship = GetShip(m_ship);
     if (!ship) {
         Logger().errorStream() << "ColonizeOrder::ExecuteImpl couldn't get ship with id " << m_ship;
         return;
@@ -429,7 +499,7 @@ void ColonizeOrder::ExecuteImpl() const {
     }
     double colonist_capacity = design->ColonyCapacity();
 
-    Planet* planet = GetPlanet(m_planet);
+    TemporaryPtr<Planet> planet = GetPlanet(m_planet);
     if (!planet) {
         Logger().errorStream() << "ColonizeOrder::ExecuteImpl couldn't get planet with id " << m_planet;
         return;
@@ -447,8 +517,8 @@ void ColonizeOrder::ExecuteImpl() const {
         return;
     }
     if (GetUniverse().GetObjectVisibilityByEmpire(m_planet, empire_id) < VIS_PARTIAL_VISIBILITY) {
-        Logger().errorStream() << "ColonizeOrder::ExecuteImpl given planet that empire reportedly has insufficient visibility of, but will be allowed to proceed pending investigation";
-        //return;
+        Logger().errorStream() << "ColonizeOrder::ExecuteImpl given planet that empire has insufficient visibility of";
+        return;
     }
     if (colonist_capacity > 0.0 && planet->EnvironmentForSpecies(ship->SpeciesName()) < PE_HOSTILE) {
         Logger().errorStream() << "ColonizeOrder::ExecuteImpl nonzero colonist capacity and planet that ship's species can't colonize";
@@ -472,10 +542,13 @@ void ColonizeOrder::ExecuteImpl() const {
 
     planet->SetIsAboutToBeColonized(true);
     ship->SetColonizePlanet(m_planet);
+
+    if (TemporaryPtr<Fleet> fleet = GetFleet(ship->FleetID()))
+        fleet->StateChangedSignal();
 }
 
 bool ColonizeOrder::UndoImpl() const {
-    Planet* planet = GetPlanet(m_planet);
+    TemporaryPtr<Planet> planet = GetPlanet(m_planet);
     if (!planet) {
         Logger().errorStream() << "ColonizeOrder::UndoImpl couldn't get planet with id " << m_planet;
         return false;
@@ -485,7 +558,7 @@ bool ColonizeOrder::UndoImpl() const {
         return false;
     }
 
-    Ship* ship = GetShip(m_ship);
+    TemporaryPtr<Ship> ship = GetShip(m_ship);
     if (!ship) {
         Logger().errorStream() << "ColonizeOrder::UndoImpl couldn't get ship with id " << m_ship;
         return false;
@@ -498,10 +571,12 @@ bool ColonizeOrder::UndoImpl() const {
     planet->SetIsAboutToBeColonized(false);
     ship->ClearColonizePlanet();
 
+    if (TemporaryPtr<Fleet> fleet = GetFleet(ship->FleetID()))
+        fleet->StateChangedSignal();
+
     return true;
 }
 
-
 ////////////////////////////////////////////////
 // InvadeOrder
 ////////////////////////////////////////////////
@@ -521,7 +596,7 @@ void InvadeOrder::ExecuteImpl() const {
     ValidateEmpireID();
     int empire_id = EmpireID();
 
-    Ship* ship = GetShip(m_ship);
+    TemporaryPtr<Ship> ship = GetShip(m_ship);
     if (!ship) {
         Logger().errorStream() << "InvadeOrder::ExecuteImpl couldn't get ship with id " << m_ship;
         return;
@@ -535,7 +610,7 @@ void InvadeOrder::ExecuteImpl() const {
         return;
     }
 
-    Planet* planet = GetPlanet(m_planet);
+    TemporaryPtr<Planet> planet = GetPlanet(m_planet);
     if (!planet) {
         Logger().errorStream() << "InvadeOrder::ExecuteImpl couldn't get planet with id " << m_planet;
         return;
@@ -573,19 +648,23 @@ void InvadeOrder::ExecuteImpl() const {
     }
 
     // note: multiple ships, from same or different empires, can invade the same planet on the same turn
-    Logger().debugStream() << "InvadeOrder::ExecuteImpl set for ship "<< m_ship<<" "<<ship->Name()<<" to invade planet "<<m_planet<<" "<<planet->Name();    
+    Logger().debugStream() << "InvadeOrder::ExecuteImpl set for ship " << m_ship << " "
+                           << ship->Name() << " to invade planet " << m_planet << " " << planet->Name();
     planet->SetIsAboutToBeInvaded(true);
     ship->SetInvadePlanet(m_planet);
+
+    if (TemporaryPtr<Fleet> fleet = GetFleet(ship->FleetID()))
+        fleet->StateChangedSignal();
 }
 
 bool InvadeOrder::UndoImpl() const {
-    Planet* planet = GetPlanet(m_planet);
+    TemporaryPtr<Planet> planet = GetPlanet(m_planet);
     if (!planet) {
         Logger().errorStream() << "InvadeOrder::UndoImpl couldn't get planet with id " << m_planet;
         return false;
     }
 
-    Ship* ship = GetShip(m_ship);
+    TemporaryPtr<Ship> ship = GetShip(m_ship);
     if (!ship) {
         Logger().errorStream() << "InvadeOrder::UndoImpl couldn't get ship with id " << m_ship;
         return false;
@@ -598,9 +677,109 @@ bool InvadeOrder::UndoImpl() const {
     planet->SetIsAboutToBeInvaded(false);
     ship->ClearInvadePlanet();
 
+    if (TemporaryPtr<Fleet> fleet = GetFleet(ship->FleetID()))
+        fleet->StateChangedSignal();
+
     return true;
 }
 
+////////////////////////////////////////////////
+// BombardOrder
+////////////////////////////////////////////////
+BombardOrder::BombardOrder() :
+    Order(),
+    m_ship(INVALID_OBJECT_ID),
+    m_planet(INVALID_OBJECT_ID)
+{}
+
+BombardOrder::BombardOrder(int empire, int ship, int planet) :
+    Order(empire),
+    m_ship(ship),
+    m_planet(planet)
+{}
+
+void BombardOrder::ExecuteImpl() const {
+    ValidateEmpireID();
+    int empire_id = EmpireID();
+
+    TemporaryPtr<Ship> ship = GetShip(m_ship);
+    if (!ship) {
+        Logger().errorStream() << "BombardOrder::ExecuteImpl couldn't get ship with id " << m_ship;
+        return;
+    }
+    if (!ship->CanBombard()) {
+        Logger().errorStream() << "BombardOrder::ExecuteImpl got ship that can't bombard";
+        return;
+    }
+    if (!ship->OwnedBy(empire_id)) {
+        Logger().errorStream() << "BombardOrder::ExecuteImpl got ship that isn't owned by the order-issuing empire";
+        return;
+    }
+
+    TemporaryPtr<Planet> planet = GetPlanet(m_planet);
+    if (!planet) {
+        Logger().errorStream() << "BombardOrder::ExecuteImpl couldn't get planet with id " << m_planet;
+        return;
+    }
+    if (planet->OwnedBy(empire_id)) {
+        Logger().errorStream() << "BombardOrder::ExecuteImpl given planet that is already owned by the order-issuing empire";
+        return;
+    }
+    if (!planet->Unowned() && Empires().GetDiplomaticStatus(planet->Owner(), empire_id) != DIPLO_WAR) {
+        Logger().errorStream() << "BombardOrder::ExecuteImpl given planet owned by an empire not at war with order-issuing empire";
+        return;
+    }
+    if (GetUniverse().GetObjectVisibilityByEmpire(m_planet, empire_id) < VIS_BASIC_VISIBILITY) {
+        Logger().errorStream() << "BombardOrder::ExecuteImpl given planet that empire reportedly has insufficient visibility of, but will be allowed to proceed pending investigation";
+        //return;
+    }
+
+    int ship_system_id = ship->SystemID();
+    if (ship_system_id == INVALID_OBJECT_ID) {
+        Logger().errorStream() << "BombardOrder::ExecuteImpl given id of ship not in a system";
+        return;
+    }
+    int planet_system_id = planet->SystemID();
+    if (ship_system_id != planet_system_id) {
+        Logger().errorStream() << "BombardOrder::ExecuteImpl given ids of ship and planet not in the same system";
+        return;
+    }
+
+    // note: multiple ships, from same or different empires, can invade the same planet on the same turn
+    Logger().debugStream() << "BombardOrder::ExecuteImpl set for ship " << m_ship << " "
+                           << ship->Name() << " to bombard planet " << m_planet << " " << planet->Name();
+    planet->SetIsAboutToBeBombarded(true);
+    ship->SetBombardPlanet(m_planet);
+
+    if (TemporaryPtr<Fleet> fleet = GetFleet(ship->FleetID()))
+        fleet->StateChangedSignal();
+}
+
+bool BombardOrder::UndoImpl() const {
+    TemporaryPtr<Planet> planet = GetPlanet(m_planet);
+    if (!planet) {
+        Logger().errorStream() << "BombardOrder::UndoImpl couldn't get planet with id " << m_planet;
+        return false;
+    }
+
+    TemporaryPtr<Ship> ship = GetShip(m_ship);
+    if (!ship) {
+        Logger().errorStream() << "BombardOrder::UndoImpl couldn't get ship with id " << m_ship;
+        return false;
+    }
+    if (ship->OrderedBombardPlanet() != m_planet) {
+        Logger().errorStream() << "BombardOrder::UndoImpl ship is not about to bombard planet";
+        return false;
+    }
+
+    planet->SetIsAboutToBeBombarded(false);
+    ship->ClearBombardPlanet();
+
+    if (TemporaryPtr<Fleet> fleet = GetFleet(ship->FleetID()))
+        fleet->StateChangedSignal();
+
+    return true;
+}
 
 ////////////////////////////////////////////////
 // DeleteFleetOrder
@@ -618,10 +797,10 @@ DeleteFleetOrder::DeleteFleetOrder(int empire, int fleet) :
 void DeleteFleetOrder::ExecuteImpl() const {
     ValidateEmpireID();
 
-    Fleet* fleet = GetFleet(FleetID());
+    TemporaryPtr<Fleet> fleet = GetFleet(FleetID());
 
     if (!fleet) {
-        Logger().errorStream() << "Illegal fleet id specified in fleet delete order.";
+        Logger().errorStream() << "Illegal fleet id specified in fleet delete order: " << FleetID();
         return;
     }
 
@@ -631,12 +810,15 @@ void DeleteFleetOrder::ExecuteImpl() const {
     }
 
     if (!fleet->Empty())
-        return;
+        return; // should be no ships to delete
+
+    TemporaryPtr<System> system = GetSystem(fleet->SystemID());
+    if (system)
+        system->Remove(fleet->ID());
 
     GetUniverse().Destroy(FleetID());
 }
 
-
 ////////////////////////////////////////////////
 // ChangeFocusOrder
 ////////////////////////////////////////////////
@@ -655,7 +837,7 @@ ChangeFocusOrder::ChangeFocusOrder(int empire, int planet, const std::string& fo
 void ChangeFocusOrder::ExecuteImpl() const {
     ValidateEmpireID();
 
-    Planet* planet = GetPlanet(PlanetID());
+    TemporaryPtr<Planet> planet = GetPlanet(PlanetID());
 
     if (!planet) {
         Logger().errorStream() << "Illegal planet id specified in change planet focus order.";
@@ -856,6 +1038,7 @@ ShipDesignOrder::ShipDesignOrder() :
 ShipDesignOrder::ShipDesignOrder(int empire, int existing_design_id_to_remember) :
     Order(empire),
     m_design_id(existing_design_id_to_remember),
+    m_update_name_or_description(false),
     m_delete_design_from_empire(false),
     m_create_new_design(false),
     m_designed_on_turn(0),
@@ -866,6 +1049,7 @@ ShipDesignOrder::ShipDesignOrder(int empire, int existing_design_id_to_remember)
 ShipDesignOrder::ShipDesignOrder(int empire, int design_id_to_erase, bool dummy) :
     Order(empire),
     m_design_id(design_id_to_erase),
+    m_update_name_or_description(false),
     m_delete_design_from_empire(true),
     m_create_new_design(false),
     m_designed_on_turn(0),
@@ -876,6 +1060,7 @@ ShipDesignOrder::ShipDesignOrder(int empire, int design_id_to_erase, bool dummy)
 ShipDesignOrder::ShipDesignOrder(int empire, int new_design_id, const ShipDesign& ship_design) :
     Order(empire),
     m_design_id(new_design_id),
+    m_update_name_or_description(false),
     m_delete_design_from_empire(false),
     m_create_new_design(true),
     m_name(ship_design.Name()),
@@ -889,6 +1074,19 @@ ShipDesignOrder::ShipDesignOrder(int empire, int new_design_id, const ShipDesign
     m_name_desc_in_stringtable(ship_design.LookupInStringtable())
 {}
 
+ShipDesignOrder::ShipDesignOrder(int empire, int existing_design_id, const std::string& new_name/* = ""*/, const std::string& new_description/* = ""*/) :
+    Order(empire),
+    m_design_id(existing_design_id),
+    m_update_name_or_description(true),
+    m_delete_design_from_empire(false),
+    m_create_new_design(false),
+    m_name(new_name),
+    m_description(new_description),
+    m_designed_on_turn(0),
+    m_is_monster(false),
+    m_name_desc_in_stringtable(false)
+{}
+
 void ShipDesignOrder::ExecuteImpl() const {
     ValidateEmpireID();
 
@@ -910,7 +1108,7 @@ void ShipDesignOrder::ExecuteImpl() const {
             return;
         }
         ShipDesign* new_ship_design = new ShipDesign(m_name, m_description,
-                                                     m_designed_on_turn, m_hull, m_parts,
+                                                     m_designed_on_turn, EmpireID(), m_hull, m_parts,
                                                      m_icon, m_3D_model, m_name_desc_in_stringtable,
                                                      m_is_monster);
 
@@ -918,8 +1116,27 @@ void ShipDesignOrder::ExecuteImpl() const {
         universe.SetEmpireKnowledgeOfShipDesign(m_design_id, EmpireID());
         empire->AddShipDesign(m_design_id);
 
-    } else if (!m_create_new_design && !m_delete_design_from_empire) {
-        // player is order empire to retain a particular design, so that is can
+    } else if (m_update_name_or_description) {
+        // player is ordering empire to rename a design
+        const std::set<int>& empire_known_design_ids = universe.EmpireKnownShipDesignIDs(EmpireID());
+        std::set<int>::iterator design_it = empire_known_design_ids.find(m_design_id);
+        if (design_it == empire_known_design_ids.end()) {
+            Logger().errorStream() << "Tried to rename/redescribe a ShipDesign that this empire hasn't seen";
+            return;
+        }
+        const ShipDesign* design = GetShipDesign(*design_it);
+        if (!design) {
+            Logger().errorStream() << "Tried to rename/redescribe a ShipDesign that doesn't exist (but this empire has seen it)!";
+            return;
+        }
+        if (design->DesignedByEmpire() != EmpireID()) {
+            Logger().errorStream() << "Tried to rename/redescribe a ShipDesign that isn't owned by this empire!";
+            return;
+        }
+        GetUniverse().RenameShipDesign(m_design_id, m_name, m_description);
+
+    } else {
+        // player is ordering empire to retain a particular design, so that is can
         // be used to construct ships by that empire.
 
         // TODO: consider removing this order, so that an empire needs to use
@@ -941,9 +1158,6 @@ void ShipDesignOrder::ExecuteImpl() const {
             return;
         }
 
-    } else {
-        Logger().errorStream() << "Malformed ShipDesignOrder.";
-        return;
     }
 }
 
@@ -964,12 +1178,16 @@ void ScrapOrder::ExecuteImpl() const {
     ValidateEmpireID();
     int empire_id = EmpireID();
 
-    if (Ship* ship = GetShip(m_object_id)) {
-        if (ship->SystemID() != INVALID_OBJECT_ID && ship->OwnedBy(empire_id))
+    if (TemporaryPtr<Ship> ship = GetShip(m_object_id)) {
+        if (ship->SystemID() != INVALID_OBJECT_ID && ship->OwnedBy(empire_id)) {
             ship->SetOrderedScrapped(true);
-    } else if (Building* building = GetBuilding(m_object_id)) {
+            //Logger().debugStream() << "ScrapOrder::ExecuteImpl empire: " << empire_id
+            //                       << " on ship: " << ship->ID() << " at system: " << ship->SystemID()
+            //                       << " ... ordered scrapped?: " << ship->OrderedScrapped();
+        }
+    } else if (TemporaryPtr<Building> building = GetBuilding(m_object_id)) {
         int planet_id = building->PlanetID();
-        if (const Planet* planet = GetPlanet(planet_id)) {
+        if (TemporaryPtr<const Planet> planet = GetPlanet(planet_id)) {
             if (building->OwnedBy(empire_id) && planet->OwnedBy(empire_id))
                 building->SetOrderedScrapped(true);
         }
@@ -980,10 +1198,10 @@ bool ScrapOrder::UndoImpl() const {
     ValidateEmpireID();
     int empire_id = EmpireID();
 
-    if (Ship* ship = GetShip(m_object_id)) {
+    if (TemporaryPtr<Ship> ship = GetShip(m_object_id)) {
         if (ship->OwnedBy(empire_id))
             ship->SetOrderedScrapped(false);
-    } else if (Building* building = GetBuilding(m_object_id)) {
+    } else if (TemporaryPtr<Building> building = GetBuilding(m_object_id)) {
         if (building->OwnedBy(empire_id))
             building->SetOrderedScrapped(false);
     } else {
@@ -1010,8 +1228,56 @@ AggressiveOrder::AggressiveOrder(int empire, int object_id, bool aggression/* =
 void AggressiveOrder::ExecuteImpl() const {
     ValidateEmpireID();
     int empire_id = EmpireID();
-    if (Fleet* fleet = GetFleet(m_object_id)) {
+    if (TemporaryPtr<Fleet> fleet = GetFleet(m_object_id)) {
         if (fleet->OwnedBy(empire_id))
             fleet->SetAggressive(m_aggression);
     }
 }
+
+/////////////////////////////////////////////////////
+// GiveObjectToEmpireOrder
+/////////////////////////////////////////////////////
+GiveObjectToEmpireOrder::GiveObjectToEmpireOrder() :
+    Order(),
+    m_object_id(INVALID_OBJECT_ID),
+    m_recipient_empire_id(ALL_EMPIRES)
+{}
+
+GiveObjectToEmpireOrder::GiveObjectToEmpireOrder(int empire, int object_id, int recipient) :
+    Order(empire),
+    m_object_id(object_id),
+    m_recipient_empire_id(recipient)
+{}
+
+void GiveObjectToEmpireOrder::ExecuteImpl() const {
+    ValidateEmpireID();
+    int empire_id = EmpireID();
+
+    if (TemporaryPtr<Fleet> fleet = GetFleet(m_object_id)) {
+        if (fleet->OwnedBy(empire_id))
+            fleet->SetGiveToEmpire(m_recipient_empire_id);
+
+    } else if (TemporaryPtr<Planet> planet = GetPlanet(m_object_id)) {
+        if (planet->OwnedBy(empire_id))
+            planet->SetGiveToEmpire(m_recipient_empire_id);
+    }
+}
+
+bool GiveObjectToEmpireOrder::UndoImpl() const {
+    ValidateEmpireID();
+    int empire_id = EmpireID();
+
+    if (TemporaryPtr<Fleet> fleet = GetFleet(m_object_id)) {
+        if (fleet->OwnedBy(empire_id)) {
+            fleet->ClearGiveToEmpire();
+            return true;
+        }
+    } else if (TemporaryPtr<Planet> planet = GetPlanet(m_object_id)) {
+        if (planet->OwnedBy(empire_id)) {
+            planet->ClearGiveToEmpire();
+            return true;
+        }
+    }
+    return false;
+}
+
